/*
 * exe.h
 * (c) 1995, 1996 Onno Hovers, All rights reserved
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 

#ifndef __EXE_H
#define __EXE_H

#include<win32/defines.h>

#define MZEXE_MAGIC                 0x5A4D      /* 'MZ'   */
#define NEEXE_MAGIC                 0x454E      /* 'NE'   */
#define LEEXE_MAGIC                 0x454C      /* 'LE'   */
#define LXEXE_MAGIC                 0x584C      /* 'LX'   */
#define PEEXE_MAGIC                 0x4550      /* 'PE00' */

/*
 * Old MS-Dos Executable Header
 *
 */

typedef struct __MZEXE_HEADER       /* DOS "MZ" .EXE header */
{
    USHORT Magic;                   /* Magic                            */
    USHORT FileSizeRest;          
    USHORT FileSizePages;         
    USHORT RelocCount;            
    USHORT HeaderSizeParas;       
    USHORT MinAlloc;              
    USHORT MaxAlloc;              
    USHORT InitSs;                
    USHORT InitSp;                
    USHORT CheckSum;              
    USHORT InitIp;                
    USHORT InitCs;                
    USHORT Relocations;           
    USHORT OverlayNumber;         
    USHORT reserved1[4];          
    USHORT OemId;                 
    USHORT OemInfo;               
    USHORT reserved2[10];         
    LONG   NewExe;                /* File address of a new exe header */
} MZEXE_HEADER;

typedef struct __MZEXE_HEADER *PMZEXE_HEADER;


/*
 * New Executable Header
 *
 * used for 16 bits segmented executables by OS/2 and Windows
 *
 */

typedef struct __NEEXE_HEADER         /* 16 bit "NE".EXE header */
{
    USHORT Magic;                    
    CHAR   Version;                  
    CHAR   Revision;                 
    USHORT EntryTable;               
    USHORT EntryTableSize;           
    LONG   Checksum;                 
    USHORT Flags;                    
    USHORT AutoData;                 
    USHORT InitHeapSize;             
    USHORT InitStackSize;            
    LONG   InitEntry;                
    LONG   InitStack;                
    USHORT SegmentCount;             
    USHORT ModuleCount;              
    USHORT NonResNameTableSize;      
    USHORT SegmentTable;             
    USHORT ResourceTable;            
    USHORT ResNameTable;             
    USHORT ModuleTable;              
    USHORT ImportTable;              
    LONG   NonResNameTable;         
    USHORT MovableEntriesCount;     
    USHORT AlignShift;               
    USHORT ResourceSegCount;         
    UBYTE  Target;                   
    UBYTE  Flags2;                   
    USHORT ReturnThunks;            
    USHORT SegRefBytes;              
    USHORT SwapAreaSize;             
    USHORT ExpVersion;               
  } NEEXE_HEADER;
  
typedef struct NEEXE_HEADER *PNEEXE_HEADER;



/* Directory format. */

typedef struct __PEEXE_DATA_DIRECTORY 
{
    ULONG   VirtualAddress;
    ULONG   Size;
} PEEXE_DATA_DIRECTORY;

typedef struct __PEEXE_DATA_DIRECTORY *PPEEXE_DATA_DIRECTORY;
typedef struct __PEEXE_DATA_DIRECTORY *LPPEEXE_DATA_DIRECTORY;

#define PEEXE_NUMBEROF_DIRECTORY_ENTRIES    16

/*
 * Portable Executable Header
 *
 * used for 32 bit executables by Windows (W32s/Win95/WinNT)
 */

typedef struct __PEEXE_HEADER
{
    ULONG   Magic;

    /* Standard Header */

    USHORT  Machine;
    USHORT  SectionCount;
    ULONG   TimeDateStamp;
    ULONG   SymbolTable;
    ULONG   SymbolCount;
    USHORT  OptHeaderSize;
    USHORT  Flags;

    /* Optional Header */

    USHORT  CoffMagic;
    UBYTE   MajorLinkerVersion;
    UBYTE   MinorLinkerVersion;
    ULONG   CodeSize;
    ULONG   DataSize;
    ULONG   BssSize;
    ULONG   EntryAddr;
    ULONG   CodeAddr;
    ULONG   DataAddr;

    /* Win32 additional fields. */

    ULONG   ImageBase;
    ULONG   SectionAlignment;
    ULONG   FileAlignment;
    USHORT  MajorOSVersion;
    USHORT  MinorOSVersion;
    USHORT  MajorImageVersion;
    USHORT  MinorImageVersion;
    USHORT  MajorSubsystemVersion;
    USHORT  MinorSubsystemVersion;
    ULONG   reserved1;
    ULONG   SizeOfImage;
    ULONG   SizeOfHeaders;
    ULONG   CheckSum;
    USHORT  Subsystem;
    USHORT  DllCharacteristics;
    ULONG   SizeOfStackReserve;
    ULONG   SizeOfStackCommit;
    ULONG   SizeOfHeapReserve;
    ULONG   SizeOfHeapCommit;
    ULONG   LoaderFlags;
    ULONG   NumberOfRvaAndSizes;
    PEEXE_DATA_DIRECTORY DataDirectory[PEEXE_NUMBEROF_DIRECTORY_ENTRIES];
} PEEXE_HEADER;

typedef struct __PEEXE_HEADER *PPEEXE_HEADER;

#define PEEXE_SIZEOF_STDHEADER          20
#define PEEXE_SIZEOF_STD_OPTHEADER      28
#define PEEXE_SIZEOF_NT_OPTHEADER      224

/* File Flags */

#define PEEXE_FILE_RELOCS_STRIPPED           0x0001  
#define PEEXE_FILE_EXECUTABLE                0x0002  
#define PEEXE_FILE_LINE_NUMS_STRIPPED        0x0004  
#define PEEXE_FILE_LOCAL_SYMS_STRIPPED       0x0008  
#define PEEXE_FILE_MINIMAL_OBJECT            0x0010  
#define PEEXE_FILE_UPDATE_OBJECT             0x0020  
#define PEEXE_FILE_16BIT_MACHINE             0x0040  
#define PEEXE_FILE_BYTES_REVERSED_LO         0x0080  
#define PEEXE_FILE_32BIT_MACHINE             0x0100  
#define PEEXE_FILE_DEBUG_STRIPPED            0x0200  
#define PEEXE_FILE_PATCH                     0x0400  
#define PEEXE_FILE_SYSTEM                    0x1000  
#define PEEXE_FILE_DLL                       0x2000  
#define PEEXE_FILE_BYTES_REVERSED_HI         0x8000  

/* Machines */
#define PEEXE_MACHINE_UNKNOWN           0
#define PEEXE_MACHINE_I860              0x14d
#define PEEXE_MACHINE_I386              0x14c
#define PEEXE_MACHINE_R3000             0x162
#define PEEXE_MACHINE_R4000             0x166
#define PEEXE_MACHINE_ALPHA             0x184


/* Subsystems */

#define PEEXE_SUBSYSTEM_UNKNOWN              0
#define PEEXE_SUBSYSTEM_NATIVE               1
#define PEEXE_SUBSYSTEM_WINDOWS_GUI          2
#define PEEXE_SUBSYSTEM_WINDOWS_CUI          3
#define PEEXE_SUBSYSTEM_OS2_CUI              5
#define PEEXE_SUBSYSTEM_POSIX_CUI            7

/* Dll Flags */

#define PEEXE_LIBRARY_PROCESS_INIT           1   
#define PEEXE_LIBRARY_PROCESS_TERM           2   
#define PEEXE_LIBRARY_THREAD_INIT            4
#define PEEXE_LIBRARY_THREAD_TERM            8

/* Loader Flags */

#define PEEXE_LOADER_BREAK_ON_LOAD    0x00000001
#define PEEXE_LOADER_DEBUG_ON_LOAD    0x00000002


/* Directory Entries */

#define PEEXE_DIRECTORY_ENTRY_EXPORT         0   
#define PEEXE_DIRECTORY_ENTRY_IMPORT         1   
#define PEEXE_DIRECTORY_ENTRY_RESOURCE       2   
#define PEEXE_DIRECTORY_ENTRY_EXCEPTION      3   
#define PEEXE_DIRECTORY_ENTRY_SECURITY       4   
#define PEEXE_DIRECTORY_ENTRY_BASERELOC      5   
#define PEEXE_DIRECTORY_ENTRY_DEBUG          6   
#define PEEXE_DIRECTORY_ENTRY_COPYRIGHT      7   
#define PEEXE_DIRECTORY_ENTRY_GLOBALPTR      8   
#define PEEXE_DIRECTORY_ENTRY_TLS            9   
#define PEEXE_DIRECTORY_ENTRY_LOAD_CONFIG   10   
 
/*
 * union of new executable headers
 */ 
 
typedef struct __EXTEXE_HEADER
{
   MZEXE_HEADER Mz;
   WORD         Magic;
} EXTEXE_HEADER;

typedef union EXTEXE_HEADER *PEXTEXE_HEADER;
 
#endif /* _EXE_H */