/* 
 * kernel/file.h
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#ifndef __KERNEL_FILE_H
#define __KERNEL_FILE_H

#include <stddef.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include "object.h"
#include "imalloc.h"

#ifdef __cplusplus
extern "C" {
#endif
 
typedef struct __FILE_OBJECT
{
   struct KERNEL_OBJECT_PART  Kernel;
   struct FILE_OBJECT_PART
   {
      int       FileDesc;
      long long Pointer;    
   } File;
} FILE_OBJECT;

typedef struct __FILE_OBJECT *PFILE_OBJECT;

typedef struct __FILE_CLASS
{
   struct KERNEL_CLASS_PART  Kernel;
   struct FILE_CLASS_PART
   {
      /* ReadFile/ReadFileEx called */
      BOOL (* Read)( PFILE_OBJECT pobj, PVOID pbuffer, ULONG size, 
                     PULONG pread, POVERLAPPED pol, 
                     POVERLAPPED_COMPLETION_ROUTINE polcr );
                     
      /* WriteFile/WriteFileEx called */
      BOOL (* Write)( PFILE_OBJECT pobj, PCVOID pbuffer, ULONG size,
                      PULONG pwritten, POVERLAPPED pol,
                      POVERLAPPED_COMPLETION_ROUTINE polcr );
                      
      /* SetFilePointer called */
      BOOL (* Seek)( PFILE_OBJECT pobj, PULONG pseeklo, 
                     PULONG pseekhi, ULONG whence );
                     
      /* SetEndOfFile called */
      BOOL (* SetEOF)( PFILE_OBJECT pobj );
      
      /* GetFileByHandleInformation called */
      BOOL (* GetFileInfo)( PFILE_OBJECT pobj,
                            PBY_HANDLE_FILE_INFORMATION pinfo );

      /* GetFileSize called */      
      BOOL (* GetSize)( PFILE_OBJECT pobj, PULONGLONG psize );
      
      /* GetFileType called */
      BOOL (* GetFileType)( PFILE_OBJECT pobj, PULONG ptype );
      
      /* GetFileTime called */
      BOOL (* GetTime)( PFILE_OBJECT pobj, PFILETIME pctime, 
                        PFILETIME patime, PFILETIME pmtime );
                            
      /* SetFileTime called */                            
      BOOL (* SetTime)( PFILE_OBJECT pobj, CONST FILETIME *pctime, 
                        CONST FILETIME *patime, CONST FILETIME *pmtime);
       
      /* FlushFile called */
      BOOL (* Flush)( PFILE_OBJECT pobj );
      
      /* DeviceIoControl called */
      BOOL (* IoControl)( PFILE_OBJECT pobj, ULONG access, 
                          ULONG iocode,
                          PVOID pinbuf,  ULONG insize,
                          PVOID poutbuf, ULONG outsize,
                          PULONG poutsize, POVERLAPPED pol );
      
      /* LockFile/LockFileEx called */
      BOOL (* LockFile)( PFILE_OBJECT pobj, ULONG flags, ULONG res,
                         ULONGLONG start, ULONGLONG size, POVERLAPPED pol );
      
      /* UnlockFile/UnlockFileEx called */
      BOOL (* UnlockFile)( PFILE_OBJECT pobj, ULONG res, 
                           ULONGLONG start, ULONGLONG size, POVERLAPPED pol); 
   } File;
} FILE_CLASS;

typedef struct __FILE_CLASS *PFILE_CLASS;

#define FILE_OBJECT_CALL (a,b,c)\
           ((PFILE_CLASS)((a).Kernel.Class))->File.b c
#define PFILE_OBJECT_CALL(a,b,c)\
           ((PFILE_CLASS)((a)->Kernel.Class))->File.b c
 

extern FILE_CLASS __FileClass;

BOOL  __FileInit(VOID);
BOOL  __FileIsType(PKERNEL_OBJECT pfile, ULONG typeid);
BOOL  __FileClose(PKERNEL_OBJECT pfile);
BOOL  __FileRead(PFILE_OBJECT pfile, LPVOID pbuffer, 
               DWORD toread, LPDWORD pread, LPOVERLAPPED poverlapped,
               LPOVERLAPPED_COMPLETION_ROUTINE polcr);
BOOL  __FileWrite(PFILE_OBJECT pfile, LPCVOID pbuffer, 
                DWORD towrite, LPDWORD pwritten, LPOVERLAPPED poverlapped,
                LPOVERLAPPED_COMPLETION_ROUTINE polcr);
BOOL  __FileSeek(PFILE_OBJECT pobj, PULONG pseeklow, PULONG pseekhigh,
               ULONG whence);
BOOL  __FileSetEOF( PFILE_OBJECT pfile );
BOOL  __FileGetFileInfo(PFILE_OBJECT pfile, PBY_HANDLE_FILE_INFORMATION pinfo);
BOOL  __FileGetFileType( PFILE_OBJECT pfile, PULONG pfiletype );
BOOL  __FileGetSize( PFILE_OBJECT pfile, PULONGLONG psize );
BOOL  __FileGetTime( PFILE_OBJECT pfile, PFILETIME pctime, 
                   PFILETIME patime, PFILETIME pmtime );
BOOL  __FileSetTime( PFILE_OBJECT pfile, CONST FILETIME *pctime,
                   CONST FILETIME *patime, CONST FILETIME *pmtime);
BOOL  __FileFlush( PFILE_OBJECT pfile );
BOOL  __FileIoControl( PFILE_OBJECT pfile, ULONG access, ULONG iocode, 
                     PVOID inbuf, ULONG insize, PVOID outbuf,
                     ULONG outsize, PULONG poutsize, POVERLAPPED pol);
BOOL  __FileLock( PFILE_OBJECT pfile, ULONG flags, ULONG res, ULONGLONG offset,
                ULONGLONG size, POVERLAPPED pol );
BOOL  __FileUnlock( PFILE_OBJECT pfile, ULONG res, ULONGLONG offset, 
                  ULONGLONG size, POVERLAPPED pol );

#ifdef __cplusplus
}
#endif

#endif