/* 
 * kernel/mapping.h
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#ifndef __KERNEL_MAPPING_H
#define __KERNEL_MAPPING_H

#include <stddef.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include "object.h"
#include "imalloc.h"

#ifdef __cplusplus
extern "C" {
#endif
 
typedef struct __MAPPING_OBJECT
{
   struct KERNEL_OBJECT_PART  Kernel;
   struct MAPPING_OBJECT_PART
   {
      PFILE_OBJECT   File;
      int            Descriptor;
      DWORD          Protect;
      ULONGLONG      MaxSize;
      LPCSTR         Name;
   } Mapping;
} MAPPING_OBJECT;

typedef struct __MAPPING_OBJECT *PMAPPING_OBJECT;

typedef struct __MAPPING_CLASS
{
   struct KERNEL_CLASS_PART  Kernel;
   struct MAPPING_CLASS_PART
   {
      /* MapViewOfFile called */
      BOOL (* Map)( PMAPPING_OBJECT pobj, LPVOID base, 
                    DWORD access, ULONGLONG offset, ULONG size)
                     
      /* UnmapViewOfFile called */
      BOOL (* Unmap)( PFILE_OBJECT pobj, LPVOID base )
      
      /* FlushViewOfFile called */
      BOOL (* Flush)( PMAPPING_OBJECT pobj, LPVOID base )
      
   } Mapping;
} MAPPING_CLASS;

typedef struct __MAPPING_CLASS *PMAPPING_CLASS;

#define MAPPING_OBJECT_CALL (a,b,c)\
           ((PMAPPING_CLASS)((a).Kernel.Class))->File.b c
#define PMAPPING_OBJECT_CALL(a,b,c)\
           ((PMAPPING_CLASS)((a)->Kernel.Class))->File.b c
 

extern FILE_CLASS __FileClass;

BOOL  __MapInit(PKERNEL_OBJECT pfile);
BOOL  __MapIsType(PKERNEL_OBJECT pfile, ULONG typeid);
BOOL  __MapClose(PKERNEL_OBJECT pfile);
BOOL  __MapCreate( );
BOOL  __MapOpen( );
BOOL  __MapMap( );
BOOL  __MapUnmap( );
BOOL  __MapFlush( ); 

#ifdef __cplusplus
}
#endif

#endif