/* 
 * kernel/object.h
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * general kernel object definitions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 

#ifndef __OBJECT_H
#define __OBJECT_H

#include <stddef.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include "handle.h"
#include "imalloc.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct __KERNEL_OBJECT
{
   struct KERNEL_OBJECT_PART
   {
      ULONG                  Magic;
      struct __KERNEL_CLASS *Class;
      PVOID                  Security;
      DWORD                  HandleCount;
   } Kernel;
} KERNEL_OBJECT;

typedef struct __KERNEL_OBJECT  *PKERNEL_OBJECT;

typedef struct __KERNEL_CLASS
{
   struct KERNEL_CLASS_PART
   {
      GENERIC_MAPPING  GenericAccess;
      
      /* CloseHandle called */
      BOOL (* Close)(PKERNEL_OBJECT pobj);      
      
      /* ExitProcess called: all handles have to be cleaned up */
      BOOL (* Exit)(PKERNEL_OBJECT pobj);
      
      /* verification of the handle */      
      BOOL (* IsType)(PKERNEL_OBJECT pobj, ULONG typeid); 
      
      /* GetHandleInformation called */
      BOOL (* GetInfoHandle )(PKERNEL_OBJECT pobj, 
                              PHANDLE_DATA pdata, PDWORD presult);
                              
      /* SetHandleInformation called */                              
      BOOL (* SetInfoHandle )(PKERNEL_OBJECT pobj, PHANDLE_DATA pdata, 
                              DWORD mask, DWORD info);
                              
      /* DuplicateHandle called */                              
      BOOL (* Duplicate)(HANDLE hsrcproc, HANDLE hsrc, 
                         HANDLE hdestproc, PHANDLE phdest, 
                         DWORD access, BOOL inherit, DWORD options);
                         
      /* WaitForSingleObject called */
      BOOL (* Wait)(HANDLE hobj, PKERNEL_OBJECT pobj);                         
   } Kernel;
} KERNEL_CLASS;

typedef struct __KERNEL_CLASS  *PKERNEL_CLASS;

#define PKOBJ_CALL(pob,fn,ar) (pob)->Kernel.Class->fn ar
#define KOBJ_CALL(ob,fn,ar)   (ob).Kernel.Class->fn ar

#define PKERNEL_OBJECT_CALL(pob,fn,ar) \
        ((PKERNEL_CLASS)((pob)->Kernel.Class))->Kernel.fn ar
#define KERNEL_OBJECT_CALL(ob,fn,ar) \
        ((PKERNEL_CLASS)((ob).Kernel.Class))->Kernel.fn ar

 
#define  MAGIC(c1,c2,c3,c4)  ((c1) + ((c2)<<8) + ((c3)<<16) + ((c4)<<24))

#define  MAGIC_OBJECT      MAGIC( 'O','B','J',' ' )
#define  MAGIC_HEAP        MAGIC( 'H','E','A','P' )
#define  MAGIC_AREA        MAGIC( 'A','R','E','A' )
#define  MAGIC_THREAD      MAGIC( 'T','H','R','D' )
#define  MAGIC_PROCESS     MAGIC( 'P','R','O','C' )
#define  MAGIC_SESSION     MAGIC( 'S','E','S','S' )
#define  MAGIC_LOCK        MAGIC( 'L','O','C','K' ) 
#define  MAGIC_FILE        MAGIC( 'F','I','L','E' )
#define  MAGIC_CONSOLE     MAGIC( 'C','O','N',' ' )
#define  MAGIC_CONIN       MAGIC( 'C','O','N','I' )
#define  MAGIC_CONOUT      MAGIC( 'C','O','N','O' )
#define  MAGIC_COMM        MAGIC( 'C','O','M','M' )
#define  MAGIC_DISK        MAGIC( 'D','I','S','K' )
#define  MAGIC_SOCKET      MAGIC( 'S','O','C','K' )
#define  MAGIC_MUTEX       MAGIC( 'M','U','T','X' )
#define  MAGIC_SEMAPHORE   MAGIC( 'S','E','M','A' )
#define  MAGIC_EVENT       MAGIC( 'E','V','E','N' )
#define  MAGIC_MODULE      MAGIC( 'M','O','D',' ' )
#define  MAGIC_NAMED_PIPE  MAGIC( 'N','P','I','P' ) 
#define  MAGIC_PIPE        MAGIC( 'P','I','P','E' )
#define  MAGIC_FIND        MAGIC( 'F','I','N','D' )
#define  MAGIC_CHANGE      MAGIC( 'C','H','N','G' )
#define  MAGIC_DIRECTORY   MAGIC( 'D','I','R',' ' )
#define  MAGIC_FILESYSTEM  MAGIC( 'F','S','Y','S' )

#define  TYPEID_OBJECT     MAGIC_OBJECT      
#define  TYPEID_THREAD     MAGIC_THREAD      
#define  TYPEID_PROCESS    MAGIC_PROCESS     
#define  TYPEID_FILE       MAGIC_FILE        
#define  TYPEID_CONSOLE    MAGIC_CONSOLE  
#define  TYPEID_CONIN      MAGIC_CONIN       
#define  TYPEID_CONOUT     MAGIC_CONOUT      
#define  TYPEID_COMM       MAGIC_COMM        
#define  TYPEID_DISK       MAGIC_DISK       
#define  TYPEID_SOCKET     MAGIC_SOCKET    
#define  TYPEID_MUTEX      MAGIC_MUTEX       
#define  TYPEID_SEMAPHORE  MAGIC_SEMAPHORE   
#define  TYPEID_EVENT      MAGIC_EVENT        
#define  TYPEID_NAMED_PIPE MAGIC_NAMED_PIPE  
#define  TYPEID_PIPE       MAGIC_PIPE        
#define  TYPEID_FIND       MAGIC_FIND        
#define  TYPEID_CHANGE     MAGIC_CHANGE      
#define  TYPEID_DIRECTORY  MAGIC_DIRECTORY

BOOL __ObjectExit(PKERNEL_OBJECT pobj);
BOOL __ObjectClose(PKERNEL_OBJECT pobj);
BOOL __ObjectIsType(PKERNEL_OBJECT pobj, ULONG typeid);
BOOL __ObjectGetInfoHandle(PKERNEL_OBJECT pobj, PHANDLE_DATA pdata, 
                      PDWORD presult);
BOOL __ObjectSetInfoHandle(PKERNEL_OBJECT pobj, PHANDLE_DATA pdata, 
                      DWORD mask, DWORD info);
BOOL __ObjectDuplicate (HANDLE hsrcproc, HANDLE hsrc, 
                   HANDLE hdestproc, PHANDLE phdest, 
                   DWORD access, BOOL inherit, DWORD options);                                                 
BOOL __ObjectWait(HANDLE hobj, PKERNEL_OBJECT pobj);     

#ifdef __cplusplus
}
#endif

#endif