/*
 * loader/peload.h
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#ifndef __PELOAD_H
#define __PELOAD_H

#include <sys/types.h>
#include <win32/defines.h>
#include <win32/kernel.h>
#include <win32/ntimage.h>

#ifdef __cplusplus
extern "C" {
#endif

#define PELOAD_BUILTIN   1 

typedef struct __PELOAD_FILE
{
    struct __PELOAD_FILE       *Next;
    ULONG                       Flags;
    LPSTR                       WinName;
    LPSTR                       ShortName;
    LPSTR                       UnixName;   
    INT                         Type;       
    PIMAGE_NT_HEADERS           Headers;
    PIMAGE_SECTION_HEADER       Sections;
    PIMAGE_IMPORT_DESCRIPTOR    Imports;
    PIMAGE_EXPORT_DIRECTORY     Exports;
    PIMAGE_RESOURCE_DIRECTORY   Resources;
    PIMAGE_BASE_RELOCATION      Relocations;
    PIMAGE_TLS_DIRECTORY        Tls;
    DWORD                       BaseAddress;
    DWORD                       LoadAddress;
    DWORD                       LoadSize;
    HMODULE                     HModule;
    ULONG                       Usage;
} PELOAD_FILE;

typedef struct __PELOAD_FILE *PPELOAD_FILE;

extern PPELOAD_FILE __ImageFiles;
extern PPELOAD_FILE __ImageExecutable;
extern INT          __ImageBuiltinModule;

VOID   __ImageInitBuiltin(PIMAGE_EXPORT_DIRECTORY pexports, LPSTR modname);
BOOL   __ImageInit(VOID);
LPVOID __ImageGetExportedAddress(LPVOID loadptr, 
                            PIMAGE_EXPORT_DIRECTORY pexports,
                            LPCSTR funcname);
LPVOID __ImageGetProcAddress(HMODULE hmodule, LPCSTR funcname);
BOOL   __ImageFixupImports(PPELOAD_FILE ppefile);
BOOL   __ImageRelocate(PPELOAD_FILE ppefile);
PPELOAD_FILE __ImageLoadModule(LPCSTR filename);
BOOL   __ImageFreeModule(PPELOAD_FILE ppefile);
PPELOAD_FILE __ImageGetModuleInfo(HMODULE hmod);

#ifdef __cplusplus
}
#endif

#endif /* ___PELOAD_H */
