/*
 * kernel/thread.h 
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * thread support functions
 *  
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 *
 * we enter a global kind of critical section for
 * synchronization, if the system supports kernel threads
 */

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <linux/unistd.h>

#ifdef __cplusplus
extern "{"
#endif

#ifdef NOTYET
/* this exists in shared address space */
typedef struct __PERTHREAD
{
   pid_t                 pid;         /* pid of the thread */
   struct __PERTHREAD   *Next;        /* other threads */
   struct __PERPROCESS  *Process;     /* our process */ 
   ULONG                 Message;     /* for sending messages */
   ULONG                 RefCount;    /* number of handles and refernces */
   UINT                  ExitCode;    /* exit code */
} PERTHREAD, *PPERTHREAD;
#endif

/* lock a thread */
#define THREAD_ENTER   { if( __MultiThreading ) __ThreadEnter();   }
#define THREAD_LEAVE   { if( __MultiThreading ) __ThreadLeave();   }

/* temporary unlock a thread before entering a blocking operation */
#define THREAD_UNBLOCK(p) { if( __MultiThreading ) __ThreadUnblock(p); } 
#define THREAD_BLOCK(p)   { if( __MultiThreading ) __ThreadBlock(p);   }
 
/* actually the number of threads - 1, zero if only one thread runs */
extern INT __MultiThreading;
extern INT __ThreadLockCount; 	
extern INT __ThreadRecurse;
extern INT __ThreadLockId;
extern INT __ThreadSemaphore;

VOID __ThreadInit();
VOID __ThreadDone();
VOID __ThreadEnter(VOID);
VOID __ThreadLeave(VOID);
VOID __ThreadUnblock(PULONG prec);
VOID __ThreadBlock(PULONG prec);
BOOL __ThreadInitTeb(VOID);

typedef struct __THREAD_NEW
{
   LPSECURITY_ATTRIBUTES	Security;
   LPVOID			StackHigh;
   LPVOID			StackLow;
   PTHREAD_START_ROUTINE 	Function;
   LPVOID			Paramater;
   ULONG			Flags;  
} THREAD_NEW, *PTHREAD_NEW;

BOOL ____ThreadCreate(LPSECURITY_ATTRIBUTES psa, ULONG stacksize, 
		LPTHREAD_START_ROUTINE pstart, LPVOID pparam, 
		ULONG creatflags, PULONG pthreadid);

#ifdef __cplusplus
}
#endif