/*
 * kernel/virtual.h
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * virtual memory functions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 

#ifndef __KERNEL_VIRTUAL_H
#define __KERNEL_VIRTUAL_H

#ifdef __cplusplus
extern "C" {
#endif

INT __DevZero;     /* file descriptor for /dev/null */
 
#ifdef linux
#define VM_NO_SECRETS
#endif linux

#define VM_INTERN		1
#define VM_MEMORY		2    /* private memory */ 
#define VM_FILE			3    /* file mapping   */
#define VM_SHARED		4    /* shared memory  */
#define VM_IMAGE		5
#define VM_STACK		6

#define MEM_ALLOC_MASK		( MEM_COMMIT | MEM_RESERVE )
#define MEM_FREE_MASK		( MEM_DECOMMIT | MEM_RELEASE )
#define MEM_TYPE_MASK		( MEM_MAPPED | MEM_PRIVATE | MEM_IMAGE )
#define MEM_STATE_MASK		( MEM_FREE | MEM_COMMIT | MEM_RESERVE )
#define MEM_PROTECT_MASK	0x000003FF

#define MEM_HIGH	0x80000000

typedef struct __VM_REGION
{
   struct __VM_REGION	*Next;
   LPVOID		Base;
   ULONG		Size;
   ULONG		State;
} VM_REGION, *PVM_REGION;

typedef struct __VM_AREA
{
   struct __VM_AREA	*Next;
   LPVOID		Base;
   ULONG		Size; 
   struct __VM_REGION	*First;
   ULONG		Protect;
   ULONG		Type;
   /* unix part */
   ULONG		Flags;	
   ULONG		Descriptor;
   ULONG		Offset;	
} VM_AREA, *PVM_AREA;


BOOL	   	__VirtualInit(VOID);
VOID		__VirtualDump(VOID);
PVM_AREA	__VirtualFindArea(PCVOID addr);
PVM_REGION	__VirtualFindRegion(PVM_AREA area, PCVOID addr);
BOOL		__VirtualIsFree(PCVOID start, DWORD size);
PVOID		__VirtualFindFree(ULONG size, BOOL ftopdown, BOOL fhigh);
BOOL		__VirtualFreeArea(PVM_AREA area);
BOOL		__VirtualInsertArea( PVM_AREA parea );
BOOL		__VirtualSetState( PVM_AREA area, LPVOID newstart, 
				ULONG newsize, ULONG state,
				PULONG poldstate, BOOL fprotect);
ULONG		__VirtualWinProtToUnix(DWORD winprot);
BOOL		__VirtualChangeState(PVM_AREA area, LPVOID base, ULONG size,
				ULONG oldstate, ULONG newstate);
PVOID		__VirtualReserve(PVOID start, ULONG size, ULONG state);		

LPVOID		__VirtualAllocStack(ULONG stacksize);
LPVOID		__VirtualAllocImage(LPVOID base, ULONG size, BOOL fail);
BOOL		__VirtualCommit(PVOID start, ULONG size, ULONG protect);
BOOL		__VirtualDecommit(PVOID start, ULONG size);
BOOL		__VirtualRelease(PVOID start);

#ifdef __cplusplus
}
#endif

#endif