/*
 * wcs.h
 * based on linux/include/asm-i386/string.h
 *
 * gcc and the libc are broken when it comes to wide char strings:
 * who needs 32 bit wide chars? not me!
 */
 
#ifndef __I386_WCSTRING_H_
#define __I386_WCSTRING_H_

/*
 * This string-include defines all string functions as inline
 * functions. Use gcc. It also assumes ds=es=data space, this should be
 * normal. Most of the string-functions are rather heavily hand-optimized,
 * see especially strtok,strstr,str[c]spn. They should work, but are not
 * very easy to understand. Everything is done entirely within the register
 * set, making the functions fast and clean. String instructions have been
 * used through-out, making for "slightly" unclear code :-)
 *
 *		Copyright (C) 1991, 1992 Linus Torvalds
 */

extern inline LPWSTR wcscpy(LPWSTR dest, LPCWSTR src)
{
__asm__ __volatile__(
	"cld\n"
	"1:\tlodsw\n\t"
	"stosw\n\t"
	"testw %%ax,%%ax\n\t"
	"jne 1b"
	: /* no output */
	:"S" (src),"D" (dest):"si","di","ax","memory");
return dest;
}

extern inline LPWSTR wcsncpy(LPWSTR dest,LPCWSTR src, INT count)
{
__asm__ __volatile__(
	"cld\n"
	"1:\tdecl %2\n\t"
	"js 2f\n\t"
	"lodsw\n\t"
	"stosw\n\t"
	"testw %%ax,%%ax\n\t"
	"jne 1b\n\t"
	"rep\n\t"
	"stosw\n"
	"2:"
	: /* no output */
	:"S" (src),"D" (dest),"c" (count):"si","di","ax","cx","memory");
return dest;
}

extern inline LPWSTR wcscat(LPWSTR dest, LPCWSTR  src)
{
__asm__ __volatile__(
	"cld\n\t"
	"repne\n\t"
	"scasw\n\t"
	"subl $2,%1\n"
	"1:\tlodsw\n\t"
	"stosw\n\t"
	"testw %%ax,%%ax\n\t"
	"jne 1b"
	: /* no output */
	:"S" (src),"D" (dest),"a" (0),"c" (0xffffffff):"si","di","ax","cx");
return dest;
}

extern inline LPWSTR wcsncat(LPWSTR  dest,LPCWSTR src, INT count)
{
__asm__ __volatile__(
	"cld\n\t"
	"repne\n\t"
	"scasw\n\t"
	"subl $2,%1\n\t"
	"movl %4,%3\n"
	"1:\tdecl %3\n\t"
	"js 2f\n\t"
	"lodsw\n\t"
	"stosw\n\t"
	"testw %%ax,%%ax\n\t"
	"jne 1b\n"
	"2:\txorl %2,%2\n\t"
	"stosw"
	: /* no output */
	:"S" (src),"D" (dest),"a" (0),"c" (0xffffffff),"g" (count)
	:"si","di","ax","cx","memory");
return dest;
}

extern inline INT wcscmp(LPCWSTR cs, LPCWSTR ct)
{
register INT __res;
__asm__ __volatile__(
	"cld\n"
	"1:\tlodsb\n\t"
	"scasb\n\t"
	"jne 2f\n\t"
	"testb %%al,%%al\n\t"
	"jne 1b\n\t"
	"xorl %%eax,%%eax\n\t"
	"jmp 3f\n"
	"2:\tsbbl %%eax,%%eax\n\t"
	"orb $1,%%eax\n"
	"3:"
	:"=a" (__res):"S" (cs),"D" (ct):"si","di");
return __res;
}

extern inline INT wcsncmp(LPCWSTR cs, LPCWSTR ct, INT count)
{
register INT __res;
__asm__ __volatile__(
	"cld\n"
	"1:\tdecl %3\n\t"
	"js 2f\n\t"
	"lodsw\n\t"
	"scasw\n\t"
	"jne 3f\n\t"
	"testw %%ax,%%ax\n\t"
	"jne 1b\n"
	"2:\txorl %%eax,%%eax\n\t"
	"jmp 4f\n"
	"3:\tsbbl %%eax,%%eax\n\t"
	"orb $1,%%al\n"
	"4:"
	:"=a" (__res):"S" (cs),"D" (ct),"c" (count):"si","di","cx");
return __res;
}

extern inline LPWSTR wcschr(LPCWSTR s, INT c)
{
register LPWSTR __res;
__asm__ __volatile__(
	"cld\n\t"		
	"movw %%ax,%%dx\n"
	"1:\tlodsw\n\t"
	"cmpw %%dx,%%ax\n\t"
	"je 2f\n\t"
	"testw %%ax,%%ax\n\t"
	"jne 1b\n\t"
	"movl $1,%1\n"
	"2:\tmovl %1,%0\n\t"
	"decl %0"
	:"=a" (__res):"S" (s),"0" (c):"si","dx");
return __res;
}

extern inline LPWSTR wcsrchr(LPCWSTR s, INT c)
{
register LPWSTR __res;
__asm__ __volatile__(
	"cld\n\t"
	"movw %%ax,%%di\n"
	"1:\tlodsw\n\t"
	"cmpw %%ax,%%di\n\t"
	"jne 2f\n\t"
	"leal -2(%%esi),%0\n"
	"2:\ttestw %%ax,%%ax\n\t"
	"jne 1b"
	:"=d" (__res):"0" (0),"S" (s),"a" (c):"ax","si","di");
return __res;
}

extern inline LPWSTR wcsstr(LPCWSTR cs, LPCWSTR ct)
{
register LPWSTR __res;
__asm__ __volatile__(
	"cld\n\t" \
	"movl %4,%%edi\n\t"
	"repne\n\t"
	"scasw\n\t"
	"notl %%ecx\n\t"
	"decl %%ecx\n\t"	/* NOTE! This also sets Z if searchstring='' */
	"movl %%ecx,%%edx\n"
	"1:\tmovl %4,%%edi\n\t"
	"movl %%esi,%%eax\n\t"
	"movl %%edx,%%ecx\n\t"
	"repe\n\t"
	"cmpsw\n\t"
 	"je 2f\n\t"		/* also works for empty string, see above */
	"xchgl %%eax,%%esi\n\t"
	"addl $2,%%esi\n\t"
	"cmpw $0,-2(%%eax)\n\t"
	"jne 1b\n\t"
	"xorl %%eax,%%eax\n\t"
	"2:"
	:"=a" (__res):"0" (0),"c" (0xffffffff),"S" (cs),"g" (ct)
	:"cx","dx","di","si");
return __res;
}

extern inline INT wcslen(LPCWSTR s)
{
register int __res;
__asm__ __volatile__(
	"cld\n\t"
	"repne\n\t"
	"scasw\n\t"
	"notl %0\n\t"
	"decl %0"
	:"=c" (__res):"D" (s),"a" (0),"0" (0xffffffff):"di");
return __res;
}

/* Added by Gertjan van Wingerde to make minix and sysv module work */
extern inline INT wcsnlen(LPCWSTR s, INT count)
{
register INT __res;
__asm__ __volatile__(
	"movl %1,%0\n\t"
	"jmp 2f\n"
	"1:\tcmpw $0,(%0)\n\t"
	"je 3f\n\t"
	"incl %0\n"
	"2:\tdecl %2\n\t"
	"cmpl $-1,%2\n\t"
	"jne 1b\n"
	"3:\tsubl %1,%0"
	:"=a" (__res):"c" (s),"d" (count));
return __res;
}
/* end of additional stuff */


#endif