/*
 * win32/console.h
 * Copyright (C) 1996, Free Software Foundation, Inc. 
 *
 * adapted from Scott Cristley's (scottc@net-community.com) Win32 headers.
 * changed are copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * console definitions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
 
#ifndef __WIN32_CONSOLE_H
#define __WIN32_CONSOLE_H

#include <win32/defines.h>

#ifdef __cplusplus
extern "C" {
#endif

#define FOREGROUND_BLUE			0x01
#define FOREGROUND_GREEN		0x02
#define FOREGROUND_RED			0x04
#define FOREGROUND_INTENSITY		0x08
#define BACKGROUND_BLUE			0x10
#define BACKGROUND_GREEN		0x20
#define BACKGROUND_RED			0x40
#define BACKGROUND_INTENSITY		0x80

#define CTRL_C_EVENT			0
#define CTRL_BREAK_EVENT		1
#define CTRL_CLOSE_EVENT		2
#define CTRL_LOGOFF_EVENT		5
#define CTRL_SHUTDOWN_EVENT		6

#define ENABLE_PROCESSED_INPUT		0x01
#define ENABLE_LINE_INPUT		0x02
#define ENABLE_ECHO_INPUT		0x04
#define ENABLE_WINDOW_INPUT		0x08
#define ENABLE_MOUSE_INPUT		0x10

#define ENABLE_PROCESSED_OUTPUT		0x01
#define ENABLE_WRAP_AT_EOL_OUTPUT	0x02

#define KEY_EVENT			0x01
#define MOUSE_EVENT			0x02
#define WINDOW_BUFFER_SIZE_EVENT	0x04
#define MENU_EVENT			0x08
#define FOCUS_EVENT			0x10

#define RIGHT_ALT_PRESSED		0x001
#define LEFT_ALT_PRESSED		0x002
#define RIGHT_CTRL_PRESSED		0x004
#define LEFT_CTRL_PRESSED		0x008
#define SHIFT_PRESSED			0x010
#define NUMLOCK_ON			0x020
#define SCROLLLOCK_ON			0x040
#define CAPSLOCK_ON			0x080
#define ENHANCED_KEY			0x100 

#define FROM_LEFT_1ST_BUTTON_PRESSED	0x01
#define RIGHTMOST_BUTTON_PRESSED	0x02
#define FROM_LEFT_2ND_BUTTON_PRESSED	0x04
#define FROM_LEFT_3RD_BUTTON_PRESSED	0x08
#define FROM_LEFT_4TH_BUTTON_PRESSED	0x10

#define DOUBLE_CLICK			0x02
#define MOUSE_MOVED			0x01

typedef struct __CHAR_INFO 
{
   union 
   {               
      WCHAR UnicodeChar; 
      CHAR AsciiChar; 
   } Char; 
   WORD Attributes;      
} CHAR_INFO, *PCHAR_INFO; 

typedef struct __CONSOLE_CURSOR_INFO
{
   DWORD  dwSize; 
   BOOL   bVisible; 
} CONSOLE_CURSOR_INFO, *PCONSOLE_CURSOR_INFO;

typedef struct __COORD
{
   SHORT X;    
   SHORT Y;    
} COORD; 

typedef struct __SMALL_RECT
{ 
   SHORT Left;      
   SHORT Top;       
   SHORT Right;     
   SHORT Bottom;    
} SMALL_RECT, *PSMALL_RECT; 
 
typedef struct __CONSOLE_SCREEN_BUFFER_INFO
{
   COORD      dwSize; 
   COORD      dwCursorPosition; 
   WORD       wAttributes; 
   SMALL_RECT srWindow; 
   COORD      dwMaximumWindowSize; 
} CONSOLE_SCREEN_BUFFER_INFO, *PCONSOLE_SCREEN_BUFFER_INFO ; 

 typedef struct __KEY_EVENT_RECORD
 { 
    BOOL bKeyDown;             
    WORD wRepeatCount;         
    WORD wVirtualKeyCode;      
    WORD wVirtualScanCode; 
    union { 
      WCHAR UnicodeChar; 
      CHAR  AsciiChar; 
    } uChar;  
    DWORD dwControlKeyState;   
} KEY_EVENT_RECORD; 
 
typedef struct __MOUSE_EVENT_RECORD
{ 
   COORD dwMousePosition; 
   DWORD dwButtonState; 
   DWORD dwControlKeyState; 
   DWORD dwEventFlags; 
} MOUSE_EVENT_RECORD; 
 
typedef struct __WINDOW_BUFFER_SIZE_RECORD
{ 
   COORD dwSize; 
} WINDOW_BUFFER_SIZE_RECORD; 
 
typedef struct __MENU_EVENT_RECORD
{ 
   UINT dwCommandId; 
} MENU_EVENT_RECORD, *PMENU_EVENT_RECORD; 

typedef struct __FOCUS_EVENT_RECORD
{
   BOOL bSetFocus;
} FOCUS_EVENT_RECORD, *PFOCUS_EVENT_RECORD;
 
typedef struct __INPUT_RECORD
{ 
   WORD EventType; 
   union { 
      KEY_EVENT_RECORD KeyEvent; 
      MOUSE_EVENT_RECORD MouseEvent; 
      WINDOW_BUFFER_SIZE_RECORD WindowBufferSizeEvent; 
      MENU_EVENT_RECORD MenuEvent; 
      FOCUS_EVENT_RECORD FocusEvent; 
   } Event; 
} INPUT_RECORD, *PINPUT_RECORD; 

typedef BOOL (WINAPI * PHANDLER_ROUTINE)(DWORD CtrlType);


BOOL
WINAPI
AllocConsole( VOID );


HANDLE
WINAPI
CreateConsoleScreenBuffer(
    DWORD dwDesiredAccess,
    DWORD dwShareMode,
    CONST SECURITY_ATTRIBUTES *lpSecurityAttributes,
    DWORD dwFlags,
    LPVOID lpScreenBufferData
    );


BOOL
WINAPI
FillConsoleOutputAttribute(
    HANDLE hConsoleOutput,
    WORD   wAttribute,
    DWORD  nLength,
    COORD  dwWriteCoord,
    LPDWORD lpNumberOfAttrsWritten
    );


BOOL
WINAPI
FillConsoleOutputCharacterA(
    HANDLE hConsoleOutput,
    CHAR  cCharacter,
    DWORD  nLength,
    COORD  dwWriteCoord,
    LPDWORD lpNumberOfCharsWritten
    );


BOOL
WINAPI
FillConsoleOutputCharacterW(
    HANDLE hConsoleOutput,
    WCHAR  cCharacter,
    DWORD  nLength,
    COORD  dwWriteCoord,
    LPDWORD lpNumberOfCharsWritten
    );


BOOL
WINAPI
FlushConsoleInputBuffer(
    HANDLE hConsoleInput
    );


BOOL
WINAPI
FreeConsole( VOID );


UINT
WINAPI
GetConsoleCP( VOID );


BOOL
WINAPI
GetConsoleCursorInfo(
    HANDLE hConsoleOutput,
    PCONSOLE_CURSOR_INFO lpConsoleCursorInfo
    );


BOOL
WINAPI
GetConsoleMode(
    HANDLE hConsoleHandle,
    LPDWORD lpMode
    );


DWORD
WINAPI
GetConsoleTitleA(
    LPSTR lpConsoleTitle,
    DWORD nSize
    );


DWORD
WINAPI
GetConsoleTitleW(
    LPWSTR lpConsoleTitle,
    DWORD nSize
    );


BOOL
WINAPI
GetNumberOfConsoleInputEvents(
    HANDLE hConsoleInput,
    LPDWORD lpNumberOfEvents
    );


UINT
WINAPI
GetConsoleOutputCP( VOID );


BOOL
WINAPI
GetConsoleScreenBufferInfo(
    HANDLE hConsoleOutput,
    PCONSOLE_SCREEN_BUFFER_INFO lpConsoleScreenBufferInfo
    );


COORD
WINAPI
GetLargestConsoleWindowSize(
    HANDLE hConsoleOutput
    );


BOOL
WINAPI
GetNumberOfConsoleMouseButtons(
    LPDWORD lpNumberOfMouseButtons
    );


BOOL
WINAPI
GenerateConsoleCtrlEvent(
    DWORD dwCtrlEvent,
    DWORD dwProcessGroupId
    );


BOOL
WINAPI
PeekConsoleInputA(
    HANDLE hConsoleInput,
    PINPUT_RECORD lpBuffer,
    DWORD nLength,
    LPDWORD lpNumberOfEventsRead
    );


BOOL
WINAPI
PeekConsoleInputW(
    HANDLE hConsoleInput,
    PINPUT_RECORD lpBuffer,
    DWORD nLength,
    LPDWORD lpNumberOfEventsRead
    );


BOOL
WINAPI
ReadConsoleA(
    HANDLE hConsoleInput,
    LPVOID lpBuffer,
    DWORD nNumberOfCharsToRead,
    LPDWORD lpNumberOfCharsRead,
    LPVOID lpReserved
    );


BOOL
WINAPI
ReadConsoleW(
    HANDLE hConsoleInput,
    LPVOID lpBuffer,
    DWORD nNumberOfCharsToRead,
    LPDWORD lpNumberOfCharsRead,
    LPVOID lpReserved
    );


BOOL
WINAPI
ReadConsoleInputA(
    HANDLE hConsoleInput,
    PINPUT_RECORD lpBuffer,
    DWORD nLength,
    LPDWORD lpNumberOfEventsRead
    );


BOOL
WINAPI
ReadConsoleInputW(
    HANDLE hConsoleInput,
    PINPUT_RECORD lpBuffer,
    DWORD nLength,
    LPDWORD lpNumberOfEventsRead
    );


BOOL
WINAPI
ReadConsoleOutputA(
    HANDLE hConsoleOutput,
    PCHAR_INFO lpBuffer,
    COORD dwBufferSize,
    COORD dwBufferCoord,
    PSMALL_RECT lpReadRegion
    );


BOOL
WINAPI
ReadConsoleOutputW(
    HANDLE hConsoleOutput,
    PCHAR_INFO lpBuffer,
    COORD dwBufferSize,
    COORD dwBufferCoord,
    PSMALL_RECT lpReadRegion
    );


BOOL
WINAPI
ReadConsoleOutputAttribute(
    HANDLE hConsoleOutput,
    LPWORD lpAttribute,
    DWORD nLength,
    COORD dwReadCoord,
    LPDWORD lpNumberOfAttrsRead
    );


BOOL
WINAPI
ReadConsoleOutputCharacterA(
    HANDLE hConsoleOutput,
    LPSTR lpCharacter,
    DWORD nLength,
    COORD dwReadCoord,
    LPDWORD lpNumberOfCharsRead
    );


BOOL
WINAPI
ReadConsoleOutputCharacterW(
    HANDLE hConsoleOutput,
    LPWSTR lpCharacter,
    DWORD nLength,
    COORD dwReadCoord,
    LPDWORD lpNumberOfCharsRead
    );


BOOL
WINAPI
ScrollConsoleScreenBufferA(
    HANDLE hConsoleOutput,
    CONST SMALL_RECT *lpScrollRectangle,
    CONST SMALL_RECT *lpClipRectangle,
    COORD dwDestinationOrigin,
    CONST CHAR_INFO *lpFill
    );


BOOL
WINAPI
ScrollConsoleScreenBufferW(
    HANDLE hConsoleOutput,
    CONST SMALL_RECT *lpScrollRectangle,
    CONST SMALL_RECT *lpClipRectangle,
    COORD dwDestinationOrigin,
    CONST CHAR_INFO *lpFill
    );


BOOL
WINAPI
SetConsoleActiveScreenBuffer(
    HANDLE hConsoleOutput
    );


BOOL
WINAPI
SetConsoleCP(
    UINT wCodePageID
    );


BOOL
WINAPI
SetConsoleCtrlHandler(
    PHANDLER_ROUTINE HandlerRoutine,
    BOOL Add
    );


BOOL
WINAPI
SetConsoleCursorPosition(
    HANDLE hConsoleOutput,
    COORD dwCursorPosition
    );


BOOL
WINAPI
SetConsoleCursorInfo(
    HANDLE hConsoleOutput,
    CONST CONSOLE_CURSOR_INFO *lpConsoleCursorInfo
    );


BOOL
WINAPI
SetConsoleMode(
    HANDLE hConsoleHandle,
    DWORD dwMode
    );


BOOL
WINAPI
SetConsoleOutputCP(
    UINT wCodePageID
    );


BOOL
WINAPI
SetConsoleScreenBufferSize(
    HANDLE hConsoleOutput,
    COORD dwSize
    );


BOOL
WINAPI
SetConsoleTextAttribute(
    HANDLE hConsoleOutput,
    WORD wAttributes
    );


BOOL
WINAPI
SetConsoleTitleA(
    LPCSTR lpConsoleTitle
    );


BOOL
WINAPI
SetConsoleTitleW(
    LPCWSTR lpConsoleTitle
    );


BOOL
WINAPI
SetConsoleWindowInfo(
    HANDLE hConsoleOutput,
    BOOL bAbsolute,
    CONST SMALL_RECT *lpConsoleWindow
    );


BOOL
WINAPI
WriteConsoleA(
    HANDLE hConsoleOutput,
    CONST VOID *lpBuffer,
    DWORD nNumberOfCharsToWrite,
    LPDWORD lpNumberOfCharsWritten,
    LPVOID lpReserved
    );


BOOL
WINAPI
WriteConsoleW(
    HANDLE hConsoleOutput,
    CONST VOID *lpBuffer,
    DWORD nNumberOfCharsToWrite,
    LPDWORD lpNumberOfCharsWritten,
    LPVOID lpReserved
    );


BOOL
WINAPI
WriteConsoleInputA(
    HANDLE hConsoleInput,
    CONST INPUT_RECORD *lpBuffer,
    DWORD nLength,
    LPDWORD lpNumberOfEventsWritten
    );


BOOL
WINAPI
WriteConsoleInputW(
    HANDLE hConsoleInput,
    CONST INPUT_RECORD *lpBuffer,
    DWORD nLength,
    LPDWORD lpNumberOfEventsWritten
    );


BOOL
WINAPI
WriteConsoleOutputA(
    HANDLE hConsoleOutput,
    CONST CHAR_INFO *lpBuffer,
    COORD dwBufferSize,
    COORD dwBufferCoord,
    PSMALL_RECT lpWriteRegion
    );


BOOL
WINAPI
WriteConsoleOutputW(
    HANDLE hConsoleOutput,
    CONST CHAR_INFO *lpBuffer,
    COORD dwBufferSize,
    COORD dwBufferCoord,
    PSMALL_RECT lpWriteRegion
    );


BOOL
WINAPI
WriteConsoleOutputAttribute(
    HANDLE hConsoleOutput,
    CONST WORD *lpAttribute,
    DWORD nLength,
    COORD dwWriteCoord,
    LPDWORD lpNumberOfAttrsWritten
    );


BOOL
WINAPI
WriteConsoleOutputCharacterA(
    HANDLE hConsoleOutput,
    LPCSTR lpCharacter,
    DWORD nLength,
    COORD dwWriteCoord,
    LPDWORD lpNumberOfCharsWritten
    );


BOOL
WINAPI
WriteConsoleOutputCharacterW(
    HANDLE hConsoleOutput,
    LPCWSTR lpCharacter,
    DWORD nLength,
    COORD dwWriteCoord,
    LPDWORD lpNumberOfCharsWritten
    );

    
#ifndef UNICODE
#define FillConsoleOutputCharacter	FillConsoleOutputCharacterA
#define GetConsoleTitle			GetConsoleTitleA
#define PeekConsoleInput		PeekConsoleInputA
#define ReadConsole			ReadConsoleA
#define ReadConsoleInput		ReadConsoleInputA
#define ReadConsoleOutput		ReadConsoleOutputA
#define ReadConsoleOutputCharacter	ReadConsoleOutputCharacterA
#define ScrollConsoleScreenBuffer	ScrollConsoleScreenBufferA
#define SetConsoleTitle			SetConsoleTitleA
#define WriteConsole			WriteConsoleA
#define WriteConsoleInput		WriteConsoleInputA
#define WriteConsoleOutput		WriteConsoleOutputA
#define WriteConsoleOutputCharacter	WriteConsoleOutputCharacterA
#else
#define FillConsoleOutputCharacter	FillConsoleOutputCharacterW
#define GetConsoleTitle			GetConsoleTitleW
#define PeekConsoleInput		PeekConsoleInputW
#define ReadConsole			ReadConsoleW
#define ReadConsoleInput		ReadConsoleInputW
#define ReadConsoleOutput		ReadConsoleOutputW
#define ReadConsoleOutputCharacter	ReadConsoleOutputCharacterW
#define ScrollConsoleScreenBuffer	ScrollConsoleScreenBufferW
#define SetConsoleTitle			SetConsoleTitleW
#define WriteConsole			WriteConsoleW
#define WriteConsoleInput		WriteConsoleInputW
#define WriteConsoleOutput		WriteConsoleOutputW
#define WriteConsoleOutputCharacter	WriteConsoleOutputCharacterW
#endif 

#ifdef __cplusplus
}
#endif

#endif  /* __WIN32_CONSOLE_H */ 