/*
 * win32/defines.h
 * Copyright (C) 1996, Free Software Foundation, Inc.
 *
 * adapted from Scott Christley's (scottc@net-community.com) Win32 Headers
 * changes are copyright 1996, Onno Hovers, All rights reserved
 *
 * basic definitions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
 
#ifndef __WIN32_DEFINES_H
#define __WIN32_DEFINES_H

#include<stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

#define CDECL
#define STDCALL

#if defined(__GNUC__)
# if __GNUC__ > 2 || (__GNUC__ == 2  && __GNUC_MINOR__ >= 7 )
#  undef  STDCALL
#  define STDCALL  __attribute__ ((stdcall))
# endif
#endif

#define WINAPI		STDCALL
#define CALLBACK	STDCALL
#define PASCALL		STDCALL

#define NEAR
#define FAR
#define CONST const

/* base types */
typedef void		VOID;

typedef int		INT;
typedef unsigned int	UINT;
typedef short		SHORT;
typedef long		LONG;

typedef unsigned char	BYTE;
typedef unsigned short	WORD;
typedef unsigned long	DWORD;

/* 64 bit types */
#ifdef __GNUC__
typedef long long           LONGLONG;
typedef signed long long    SLONGLONG;
typedef unsigned long long  DWORDLONG;
typedef unsigned long long  ULONGLONG;
#else
typedef __int64             LONGLONG;
typedef signed   __int64    SLONGLONG;
typedef unsigned __int64    DWORDLONG;
typedef unsigned __int64    ULONGLONG;
#endif

/* these are not all win32 types, but some are included for convenience */
typedef unsigned char	UBYTE;
typedef unsigned short	UWORD;
typedef unsigned short  USHORT;
typedef unsigned long	ULONG;

typedef signed char	SBYTE;
typedef signed short	SWORD;
typedef signed short    SSHORT;
typedef signed long	SLONG;

/* these are the pointer types */
typedef VOID NEAR	*PVOID;
typedef VOID FAR	*LPVOID;
typedef VOID CONST NEAR *PCVOID;
typedef VOID CONST FAR	*LPCVOID;
typedef INT NEAR	*PINT;
typedef INT FAR		*LPINT;
typedef UINT NEAR	*PUINT;
typedef UINT FAR	*LPUINT;
typedef LONG NEAR	*PLONG;
typedef LONG FAR	*LPLONG;
typedef LONGLONG NEAR   *PLONGLONG;
typedef LONGLONG FAR    *LPLONGLONG;

typedef BYTE NEAR	*PBYTE; 
typedef BYTE FAR	*LPBYTE;
typedef WORD NEAR	*PWORD;
typedef WORD FAR 	*LPWORD;
typedef DWORD NEAR	*PDWORD;
typedef DWORD FAR	*LPDWORD;
typedef DWORDLONG NEAR  *PDWORDLONG;
typedef DWORDLONG FAR   *LPDWORDLONG;

/* these are pointer for our own types*/
typedef UBYTE		*PUBYTE;  
typedef UWORD		*PUWORD;
typedef ULONG		*PULONG;
typedef ULONGLONG       *PULONGLONG;
typedef SBYTE		*PSBYTE;
typedef SWORD		*PSWORD;
typedef SLONG		*PSLONG;
typedef SLONGLONG       *PSLONGLONG;

/* return types */
typedef UINT		WPARAM;
typedef LONG		LPARAM;
typedef LONG		LRESULT;

/* handles */
typedef PVOID		HANDLE;
typedef HANDLE		*PHANDLE;
typedef HANDLE		*LPHANDLE;

#ifdef STRICT 
#define DEFINE_HANDLE(name) typedef struct name##__ {int unused;} *name;                      
#else
#define DEFINE_HANDLE(name) typedef HANDLE name;
#endif

/* characters and strings */
typedef char 			CHAR;
typedef CHAR NEAR		*PCHAR;
typedef CHAR NEAR 		*PCH;
typedef CHAR NEAR		*PSTR;
typedef CHAR NEAR		*NPSTR;
typedef CHAR CONST NEAR		*PCSTR;
typedef CHAR FAR 		*LPCHAR;
typedef CHAR FAR		*LPCH;
typedef CHAR FAR		*LPSTR;
typedef CHAR CONST FAR  	*LPCSTR;


typedef short			WCHAR;
typedef WCHAR NEAR		*PWCHAR;
typedef WCHAR NEAR 		*PWCH;
typedef WCHAR NEAR		*PWSTR;
typedef WCHAR NEAR		*NPWSTR;
typedef WCHAR CONST NEAR	*PCWSTR;
typedef WCHAR FAR 		*LPWCHAR;
typedef WCHAR FAR		*LPWCH;
typedef WCHAR FAR		*LPWSTR;
typedef WCHAR CONST FAR		*LPCWSTR;

#ifndef UNICODE
typedef CHAR			 TCHAR;
typedef CHAR			 TBYTE;
#else
typedef WCHAR			 TCHAR;
typedef WCHAR 			 TBYTE;
#endif  UNICODE

typedef TBYTE 			*PTBYTE;
typedef TCHAR NEAR		*PTCHAR;
typedef TCHAR NEAR 		*PTCH;
typedef TCHAR NEAR		*PTSTR;
typedef TCHAR NEAR		*NPTSTR;
typedef TCHAR CONST NEAR	*PCTSTR;
typedef TCHAR FAR 		*LPTCHAR;
typedef TCHAR FAR		*LPTCH;
typedef TCHAR FAR		*LPTSTR;
typedef TCHAR CONST FAR		*LPCTSTR;

/* boolean types */
typedef int             BOOL;
typedef BOOL NEAR	*PBOOL;
typedef BOOL FAR	*LPBOOL;
typedef BYTE		BOOLEAN;
typedef BOOLEAN		*PBOOLEAN;

/* floating point types */
typedef float		FLOAT;
typedef FLOAT NEAR      *PFLOAT;
typedef FLOAT FAR       *LPFLOAT;

/* function pointers */
typedef VOID (WINAPI *FARPROC)();

/* null */
#ifndef NULL
#define NULL	0
#endif

/* boolean values */
#define FALSE	0
#define TRUE	1

/* min/max functions */
#if !defined(NOMINMAX) && !defined(__cplusplus)

#ifndef min
#define min(a,b)	(((a) < (b)) ? (a) : (b))
#endif

#ifndef max
#define max(a,b)	(((a) > (b)) ? (a) : (b))
#endif  

#endif

/* MAKE/HI/LO macros */
#define MAKEWORD(a,b)	     ( (WORD)     (((BYTE)(a)) | (((WORD)    ((BYTE)(b))) << 8 )) )
#define MAKELONG(a,b)        ( (LONG)     (((WORD)(a)) | (((LONG)    ((WORD)(b))) << 16)) )
#define MAKELONGLONG(a, b)   ( (LONGLONG) (((LONG)(a)) | (((LONGLONG)((LONG)(a))) << 32)) )

#define LOLONG(a)       ((LONG)(a))
#define HILONG(a)       ((LONG)(((ULONGLONG)(a) >> 32) & 0xFFFFFFFF))
#define LOWORD(a)       ((WORD)(a))
#define HIWORD(a)	((WORD)(((DWORD)(a) >> 16) & 0xFFFF)) 
#define LOBYTE(a)	((BYTE)(a))
#define HIBYTE(a)	((BYTE)(((WORD )(a) >> 8) & 0xFF))

/* round macros */
#define ROUNDUP(a,b)	((((a)+(b)-1)/(b))*(b))
#define ROUNDDOWN(a,b)	(((a)/(b))*(b))
#define FIELD_OFFSET(type,fld)	((LONG)&(((type *)0)->fld)) 

/* limits */
#define MINCHAR		0x80
#define MAXCHAR		0x7F
#define MINSHORT	0x8000
#define MAXSHORT	0x7FFF
#define MINLONG		0x80000000L
#define MAXLONG		0x7FFFFFFFL
#define MAXBYTE		0xFF
#define MAXWORD		0xFFFF
#define MAXDWORD	0xFFFFFFFFL

#ifdef __GNUC__
#define MINLONGLONG	0x8000000000000000LL
#define MAXLONGLONG	0x7FFFFFFFFFFFFFFFLL
#define MAXDWORDLONG	0xFFFFFFFFFFFFFFFFL
#endif /* __GNUC__ */

/* path and file names */
#define MAX_FILE	256
#define MAX_PATH	260

/* other constants */
#define PAGE_SIZE	4096
#define INVALID_HANDLE_VALUE  (HANDLE) -1

/* array of any size */
#ifdef __GNUC__
#define ANYSIZE_ARRAY	0
#else  /* __GNUC__ */
#define ANYSIZE_ARRAY	1
#endif /* __GNUC__ */

#ifdef __cplusplus
}
#endif

#endif  __WIN32_DEFINES_H