/*
 * win32/kernel.h
 * Copyright 1996, Onno Hovers, All rights reserved
 *
 * base api as in KERNEL32.DLL except:
 *  -console defines (console.h)
 *  -comm defines (comm.h)
 *  -tape defines (tape.h)
 *  -locale api (nls.h)
 */
 
#ifndef __WIN32_KERNEL_H
#define __WIN32_KERNEL_H

#ifndef __WIN32_DEFINES_H
#include <win32/defines.h>
#endif  __WIN32_DEFINES_H

#ifdef __cplusplus
extern "C" {
#endif

/* handle flags */
#define HANDLE_FLAG_INHERIT		1
#define HANDLE_FLAG_PROTECT_FROM_CLOSE	2

/* how to duplicate a handle */
#define DUPLICATE_CLOSE_SOURCE		1
#define DUPLICATE_SAME_ACCESS		2

/* values to pass to GetStdHandle/SetStdHandle */
#define STD_INPUT_HANDLE		(DWORD) -10
#define STD_OUTPUT_HANDLE		(DWORD) -11
#define STD_ERROR_HANDLE		(DWORD) -12

/* seek relative to begin/file pointer/end */
#define FILE_BEGIN			0
#define FILE_CURRENT			1
#define FILE_END			2

/* how to share a file */
#define FILE_SHARE_READ			1
#define FILE_SHARE_WRITE		2

/* file attributes */
#define FILE_ATTRIBUTE_READONLY		0x0001
#define FILE_ATTRIBUTE_HIDDEN		0x0002
#define FILE_ATTRIBUTE_SYSTEM		0x0004
#define FILE_ATTRIBUTE_DIRECTORY	0x0010
#define FILE_ATTRIBUTE_ARCHIVE		0x0020
#define FILE_ATTRIBUTE_NORMAL		0x0080
#define FILE_ATTRIBUTE_TEMPORARY	0x0100
#define FILE_ATTRIBUTE_COMPRESSED	0x0800

/* flags passed to CreateFile with the attributes */
#define FILE_FLAG_POSIX_SEMANTICS	0x01000000
#define FILE_FLAG_BACKUP_SEMANTICS	0x02000000
#define FILE_FLAG_DELETE_ON_CLOSE	0x04000000
#define FILE_FLAG_SEQUENTIAL_SCAN	0x08000000
#define FILE_FLAG_RANDOM_ACCESS		0x10000000
#define FILE_FLAG_NO_BUFFERING		0x20000000
#define FILE_FLAG_OVERLAPPED		0x40000000
#define FILE_FLAG_WRITE_THROUGH		0x80000000

/* security quality of service, also passed to CreateFile with attrs.  */
#define SECURITY_ANONYMOUS		0x00000000
#define SECURITY_IDENTIFICATION		0x00010000
#define SECURITY_IMPERSONATION		0x00020000
#define SECURITY_DELEGATION		0x00030000
#define SECURITY_CONTEXT_TRACKING	0x00040000
#define SECURITY_EFFECTIVE_ONLY		0x00080000
#define SECURITY_SQOS_PRESENT		0x00100000
#define SECURITY_VALID_SQOS_FLAGS	0x001F0000

/* flags passed to CreateFile */
#define CREATE_NEW			1
#define CREATE_ALWAYS			2
#define OPEN_EXISTING			3
#define OPEN_ALWAYS			4
#define TRUNCATE_EXISTING		5

/* how to open a file */
#define OF_READ				0x0000
#define OF_WRITE			0x0001
#define OF_READWRITE			0x0002
#define OF_SHARE_COMPAT			0x0000
#define OF_SHARE_EXCLUSIVE		0x0010
#define OF_SHARE_DENY_WRITE		0x0020
#define OF_SHARE_DENY_READ		0x0030
#define OF_SHARE_DENY_NONE		0x0040
#define OF_PARSE			0x0100
#define OF_DELETE			0x0200
#define OF_VERIFY			0x0400
#define OF_CANCEL			0x0800
#define OF_CREATE			0x1000
#define OF_PROMPT			0x2000
#define OF_EXIST			0x4000
#define OF_REOPEN			0x8000

#define OFS_MAXPATHNAME			128

/* file type */
#define FILE_TYPE_UNKNOWN		0
#define FILE_TYPE_DISK			1
#define FILE_TYPE_CHAR			2
#define FILE_TYPE_PIPE			3

/* flags passed to LockFileEx */
#define LOCKFILE_FAIL_IMMEDIATELY	1
#define LOCKFILE_EXCLUSIVE_LOCK		2

/* flags passed to MoveFileEx */
#define MOVEFILE_REPLACE_EXISTING	1
#define MOVEFILE_COPY_ALLOWED		2
#define MOVEFILE_DELAY_UNTIL_REBOOT	4

/* data type of a stream */
#define BACKUP_DATA			1
#define BACKUP_EA_DATA			2
#define BACKUP_SECURITY_DATA		3
#define BACKUP_ALTERNATE_DATA		4
#define BACKUP_LINK			5

/* attributes ot a stream */
#define STREAM_MODIFIED_WHEN_READ	1
#define STREAM_CONTAINS_SECURITY	2

/* values for FindFirstChangeNotification */
#define FILE_NOTIFY_CHANGE_FILE_NAME	0x01
#define FILE_NOTIFY_CHANGE_DIR_NAME	0x02
#define FILE_NOTIFY_CHANGE_ATTRIBUTES	0x04
#define FILE_NOTIFY_CHANGE_SIZE		0x08
#define FILE_NOTIFY_CHANGE_LAST_WRITE	0x10
#define FILE_NOTIFY_CHANGE_SECURITY	0x100

/* drive types */
#define DRIVE_UNKNOWN			0
#define DRIVE_NO_ROOT_DIR		1
#define DRIVE_REMOVABLE			2
#define DRIVE_FIXED			3
#define DRIVE_REMOTE			4
#define DRIVE_CDROM			5
#define DRIVE_RAMDISK			6

/* information about a filesystem */
#define FS_CASE_SENSITIVE		0x1
#define FS_CASE_IS_PRESERVED		0x2
#define FS_UNICODE_STORED_ON_DISK	0x4
#define FS_PERSISTENT_ACLS		0x8
#define FS_FILE_COMPRESSION		0x10   
#define FS_VOL_IS_COMPRESSED		0x8000

/* flags passed to DefineDosDevice */
#define DDD_RAW_TARGET_PATH		1
#define DDD_REMOVE_DEFINITION		2
#define DDD_EXACT_MATCH_ON_REMOVE	4

/* how long to wait for a named pipe */
#define NMPWAIT_NOWAIT			1
#define NMPWAIT_WAIT_FOREVER		(DWORD) -1
#define NMPWAIT_USE_DEFAULT_WAIT	0

/* access mode of a named pipe */
#define PIPE_ACCESS_INBOUND		1
#define PIPE_ACCESS_OUTBOUND		2
#define PIPE_ACCESS_DUPLEX		3

/* flags passed to CreateNamedPipe */
#define PIPE_WAIT			0
#define PIPE_NOWAIT			1
#define PIPE_READMODE_BYTE		0
#define PIPE_READMODE_MESSAGE		2
#define PIPE_TYPE_BYTE			0
#define PIPE_TYPE_MESSAGE		4

/* GetNamedPipeInfo */
#define PIPE_CLIENT_END			0
#define PIPE_SERVER_END			1
#define PIPE_TYPE_MESSAGE		4

/* GetNamedPipeHandleState */
#define PIPE_NOWAIT			1
#define PIPE_READMODE_MESSAGE		2

/* flags passed to CreateMailslot */
#define MAILSLOT_WAIT_FOREVER		(DWORD) -1
#define MAILSLOT_NO_MESSAGE		(DWORD) -1

/* flags to pass to CreateProcess */
#define DEBUG_PROCESS			0x00000001 
#define DEBUG_ONLY_THIS_PROCESS		0x00000002
#define CREATE_SUSPENDED		0x00000004 
#define DETACHED_PROCESS		0x00000008 
#define CREATE_NEW_CONSOLE		0x00000010 
#define NORMAL_PRIORITY_CLASS		0x00000020 
#define IDLE_PRIORITY_CLASS		0x00000040 
#define HIGH_PRIORITY_CLASS		0x00000080 
#define REALTIME_PRIORITY_CLASS		0x00000100 
#define CREATE_NEW_PROCESS_GROUP	0x00000200 
#define CREATE_UNICODE_ENVIRONMENT	0x00000400 
#define CREATE_SEPARATE_WOW_VDM		0x00000800 
#define CREATE_SHARED_WOW_VDM		0x00001000
#define CREATE_DEFAULT_ERROR_MODE	0x04000000 
#define CREATE_NO_WINDOW		0x08000000

#define PROFILE_USER			0x10000000
#define PROFILE_KERNEL			0x20000000
#define PROFILE_SERVER			0x40000000

/* flags passed in a STARTUPINFO structure */
#define STARTF_USESHOWWINDOW		0x01
#define STARTF_USESIZE			0x02
#define STARTF_USEPOSITION		0x04
#define STARTF_USECOUNTCHARS		0x08
#define STARTF_USEFILLATTRIBUTE		0x10
#define STARTF_FORCEONFEEDBACK		0x40
#define STARTF_FORCEOFFFEEDBACK		0x80
#define STARTF_USESTDHANDLES		0x100

/* return from WinExec */
#define HINSTANCE_ERROR			32

/* */ 
#define SHUTDOWN_NORETRY		1

/* SetProcessWorkingSetSize */
#define PROCESS_SET_QUOTA		0x100

/* thread priorities */
#define THREAD_BASE_PRIORITY_IDLE	-15
#define THREAD_BASE_PRIORITY_MIN	-2
#define	THREAD_BASE_PRIORITY_MAX	 2
#define THREAD_BASE_PRIORITY_LOWRT	 15

#define THREAD_PRIORITY_IDLE		THREAD_BASE_PRIORITY_IDLE
#define THREAD_PRIORITY_LOWEST		THREAD_BASE_PRIORITY_MIN
#define THREAD_PRIORITY_BELOW_NORMAL	(THREAD_BASE_PRIORITY_MIN+1)
#define THREAD_PRIORITY_NORMAL		(0)
#define THREAD_PRIORITY_ABOVE_NORMAL	(THREAD_BASE_PRIORITY_MAX+1)
#define THREAD_PRIORITY_HIGHEST		THREAD_BASE_PRIORITY_MAX
#define THREAD_PRIORITY_TIME_CRITICAL	THREAD_BASE_PRIORITY_LOWRT
#define THREAD_PRIORITY_ERROR_RETURN	MAXLONG

/* minimal available TLS indices */
#define TLS_MINIMUM_AVAILABLE		64

/* maximum for the suspend count of a thread */
#define MAXIMUM_SUSPEND_COUNT		0x7F

/* the reason DllMain is called */
#define DLL_PROCESS_DETACH		0
#define DLL_PROCESS_ATTACH		1
#define DLL_THREAD_ATTACH		2
#define DLL_THREAD_DETACH		3

/* LoadLibraryEx flags */
#define DONT_RESOLVE_DLL_REFERENCES	1
#define LOAD_LIBRARY_AS_DATAFILE	2
#define LOAD_WITH_ALTERED_SEARCH_PATH	8

/* how to format a message */
#define FORMAT_MESSAGE_ALLOCATE_BUFFER	(256)
#define FORMAT_MESSAGE_IGNORE_INSERTS	(512)
#define FORMAT_MESSAGE_FROM_STRING	(1024)
#define FORMAT_MESSAGE_FROM_HMODULE	(2048)
#define FORMAT_MESSAGE_FROM_SYSTEM	(4096)
#define FORMAT_MESSAGE_ARGUMENT_ARRAY	(8192)
#define FORMAT_MESSAGE_MAX_WIDTH_MASK	(255)

/* values for GetBinaryType */
#define SCS_32BIT_BINARY		0
#define SCS_DOS_BINARY			1
#define SCS_WOW_BINARY			2
#define SCS_PIF_BINARY			3
#define SCS_POSIX_BINARY		4
#define SCS_OS216_BINARY		5
	
/* maximum computername length */
#define MAX_COMPUTERNAME_LENGTH		15

/* status codes */
#define STATUS_WAIT_0                   0x00000000
#define STATUS_ABANDONED_WAIT_0         0x00000080
#define STATUS_USER_APC                 0x000000C0
#define STATUS_TIMEOUT                  0x00000102
#define STATUS_PENDING                  0x00000103
#define STATUS_DATATYPE_MISALIGNMENT    0x80000002
#define STATUS_BREAKPOINT               0x80000003
#define STATUS_SINGLE_STEP              0x80000004
#define STATUS_ACCESS_VIOLATION         0xC0000005
#define STATUS_IN_PAGE_ERROR            0xC0000006
#define STATUS_NO_MEMORY                0xC0000017
#define STATUS_ILLEGAL_INSTRUCTION      0xC000001D
#define STATUS_NONCONTINUABLE_EXCEPTION 0xC0000025
#define STATUS_INVALID_DISPOSITION      0xC0000026
#define STATUS_ARRAY_BOUNDS_EXCEEDED    0xC000008C
#define STATUS_FLOAT_DENORMAL_OPERAND   0xC000008D
#define STATUS_FLOAT_DIVIDE_BY_ZERO     0xC000008E
#define STATUS_FLOAT_INEXACT_RESULT     0xC000008F
#define STATUS_FLOAT_INVALID_OPERATION  0xC0000090
#define STATUS_FLOAT_OVERFLOW           0xC0000091
#define STATUS_FLOAT_STACK_CHECK        0xC0000092
#define STATUS_FLOAT_UNDERFLOW          0xC0000093
#define STATUS_INTEGER_DIVIDE_BY_ZERO   0xC0000094
#define STATUS_INTEGER_OVERFLOW         0xC0000095
#define STATUS_PRIVILEGED_INSTRUCTION   0xC0000096
#define STATUS_STACK_OVERFLOW           0xC00000FD
#define STATUS_CONTROL_C_EXIT           0xC000013A
#define STILL_ACTIVE			STATUS_PENDING

/* wait return codes */
#define WAIT_FAILED			(DWORD) -1
#define WAIT_OBJECT_0			STATUS_WAIT0
#define WAIT_ABANDONED			STATUS_ABANDONED_WAIT0
#define WAIT_ABANDONED_0		STATUS_ABANDONED_WAIT0
#define WAIT_TIMEOUT			STATUS_TIMEOUT
#define WAIT_IO_COMPLETION		STATUS_USER_APC

/* timeout */
#define INFINITE			(DWORD) -1

/* maximum wait objects */
#define MAXIMUM_WAIT_OBJECTS		64

/* exception codes */
#define EXCEPTION_ACCESS_VIOLATION      STATUS_ACCESS_VIOLATION
#define EXCEPTION_DATATYPE_MISALIGNMENT STATUS_DATATYPE_MISALIGNMENT
#define EXCEPTION_BREAKPOINT            STATUS_BREAKPOINT
#define EXCEPTION_SINGLE_STEP           STATUS_SINGLE_STEP
#define EXCEPTION_ARRAY_BOUNDS_EXCEEDED STATUS_ARRAY_BOUNDS_EXCEEDED
#define EXCEPTION_FLT_DENORMAL_OPERAND  STATUS_FLOAT_DENORMAL_OPERAND
#define EXCEPTION_FLT_DIVIDE_BY_ZERO    STATUS_FLOAT_DIVIDE_BY_ZERO
#define EXCEPTION_FLT_INEXACT_RESULT    STATUS_FLOAT_INEXACT_RESULT
#define EXCEPTION_FLT_INVALID_OPERATION STATUS_FLOAT_INVALID_OPERATION
#define EXCEPTION_FLT_OVERFLOW          STATUS_FLOAT_OVERFLOW
#define EXCEPTION_FLT_STACK_CHECK       STATUS_FLOAT_STACK_CHECK
#define EXCEPTION_FLT_UNDERFLOW         STATUS_FLOAT_UNDERFLOW
#define EXCEPTION_INT_DIVIDE_BY_ZERO    STATUS_INTEGER_DIVIDE_BY_ZERO
#define EXCEPTION_INT_OVERFLOW          STATUS_INTEGER_OVERFLOW
#define EXCEPTION_PRIV_INSTRUCTION      STATUS_PRIVILEGED_INSTRUCTION
#define EXCEPTION_IN_PAGE_ERROR         STATUS_IN_PAGE_ERROR
#define EXCEPTION_ILLEGEAL_INSTRUCTION	STATUS_ILLEGAL_INSTRUCTION
#define EXCEPTION_NO_MEMORY             STATUS_NO_MEMORY

/* return values from the actual exception handlers */
#define ExceptionContinueExecution	0
#define ExceptionContinueSearch    	1
#define ExceptionNestedException   	2
#define ExceptionCollidedUnwind    	3
 
/* return values from exception filters */
#define EXCEPTION_EXECUTE_HANDLER        1
#define EXCEPTION_CONTINUE_SEARCH        0
#define EXCEPTION_CONTINUE_EXECUTION    -1

/* from OS/2 2.0 exception handling */
#define EH_NONCONTINUABLE		0x01
#define EH_UNWINDING			0x02
#define EH_EXIT_UNWIND			0x04
#define EH_STACK_INVALID		0x08
#define EH_NESTED_CALL			0x10

#define EXCEPTION_CONTINUABLE        	0
#define EXCEPTION_NONCONTINUABLE     	EH_NONCONTINUABLE

/* maximum parameters in an exception record */
#define EXCEPTION_MAXIMUM_PARAMETERS	15

/* virtual memory */
#define PAGE_NOACCESS				0x00000001
#define PAGE_READONLY				0x00000002
#define PAGE_READWRITE				0x00000004
#define PAGE_WRITECOPY				0x00000008	
#define PAGE_EXECUTE				0x00000010
#define PAGE_EXECUTE_READ			0x00000020
#define PAGE_EXECUTE_READWRITE			0x00000040
#define PAGE_EXECUTE_WRITECOPY			0x00000080
#define PAGE_GUARD				0x00000100
#define PAGE_NOCACHE				0x00000200

#define MEM_COMMIT				0x00001000
#define MEM_RESERVE				0x00002000
#define MEM_DECOMMIT				0x00004000
#define MEM_RELEASE				0x00008000
#define MEM_FREE				0x00010000
#define MEM_PRIVATE				0x00020000
#define MEM_MAPPED				0x00040000
#define MEM_TOP_DOWN				0x00100000
#define MEM_IMAGE				0x01000000

#define SEC_FILE				0x00800000
#define SEC_IMAGE				0x01000000
#define SEC_RESERVE				0x02000000
#define SEC_COMMIT				0x04000000
#define SEC_NOCACHE				0x10000000

/* heap flags */
#define HEAP_NO_SERIALIZE		0x01
#define HEAP_GROWABLE			0x02
#define HEAP_GENERATE_EXCEPTIONS	0x04
#define HEAP_ZERO_MEMORY		0x08
#define HEAP_REALLOC_IN_PLACE_ONLY	0x10
#define HEAP_TAIL_CHECKING_ENABLED	0x20
#define HEAP_FREE_CHECKING_ENABLED	0x40
#define HEAP_DISABLE_COALESCE_ON_FREE	0x80
#define HEAP_CREATE_ALIGN_16		0x10000
#define HEAP_CREATE_ENABLE_TRACING 	0x20000
#define HEAP_MAXIMUM_TAG		0x0FFF
#define HEAP_PSEUDO_TAG_FLAG		0x8000
#define HEAP_TAG_SHIFT			16
#define HEAP_MAKE_TAG_FLAGS( f, t )	((DWORD)((f)+((t) << 16)))

/* global heap allocation flags */
#define GMEM_FIXED			0x00
#define GMEM_MOVEABLE			0x02
#define GMEM_NOCOMPACT			0x10
#define GMEM_NODISCARD			0x20
#define GMEM_ZEROINIT			0x40
#define GMEM_MODIFY			0x80
#define GMEM_DISCARDABLE		0x100
#define GMEM_NOT_BANKED			0x1000
#define GMEM_LOWER			0x1000
#define GMEM_SHARE			0x2000
#define GMEM_DDESHARE			0x2000
#define GMEM_NOTIFY			0x4000
#define GMEM_VALID_FLAGS		0x7F72
#define GMEM_INVALID_HANDLE		0x8000

#define GPTR				(GMEM_FIXED | GMEM_ZEROINIT)
#define GHND				(GMEM_MOVABLE | GMEM_ZEROINIT)

#define GMEM_DISCARDED			0x4000
#define GMEM_LOCKCOUNT			0x00FF

/* local heap allocation flags */
#define LMEM_FIXED			0x00
#define LMEM_MOVEABLE			0x02
#define LMEM_NOCOMPACT			0x10
#define LMEM_NODISCARD			0x20
#define LMEM_ZEROINIT			0x40
#define LMEM_MODIFY			0x80
#define LMEM_DISCARDABLE		0x100
#define LMEM_VALID_FLAGS		0x0F72
#define LMEM_INVALID_HANDLE		0x8000

#define LPTR				(LMEM_FIXED | LMEM_ZEROINIT)
#define LHND				(LMEM_MOVABLE | LMEM_ZEROINIT)

#define NONZEROLPTR			(LMEM_FIXED)
#define NONZEROLHND			(LMEM_MOVABLE)

#define LMEM_DISCARDED			0x4000
#define LMEM_LOCKCOUNT			0x00FF

/* GetTimeZoneInformation */
#define TIME_ZONE_ID_UNKNOWN		0
#define TIME_ZONE_ID_STANDARD		1
#define TIME_ZONE_ID_DAYLIGHT		2

#define DBG_CONTINUE			0x00010002
#define DBG_EXCEPTION_NOT_HANDLED	0x80010001

/* debug events */
#define EXCEPTION_DEBUG_EVENT		1
#define CREATE_THREAD_DEBUG_EVENT	2
#define CREATE_PROCESS_DEBUG_EVENT	3
#define EXIT_THREAD_DEBUG_EVENT		4
#define EXIT_PROCESS_DEBUG_EVENT	5
#define LOAD_DLL_DEBUG_EVENT		6
#define UNLOAD_DLL_DEBUG_EVENT		7
#define OUTPUT_DEBUG_STRING_EVENT	8
#define RIP_EVENT			9

/* SetDebugErrorLevel */
#define SLE_ERROR			1
#define SLE_MINORERROR			2
#define SLE_WARNING			3

/* errormode passed to SetErrorMode */
#define SEM_FAILCRITICALERRORS		0x0001
#define SEM_NOGPFAULTERRORBOX		0x0002
#define SEM_NOALIGNMENTFAULTEXCEPT	0x0004
#define SEM_NOOPENFILEERRORBOX		0x8000 

/* PROCESS_HEAP_ENTRY flags */
#define PROCESS_HEAP_REGION		0x01
#define PROCESS_HEAP_UNCOMMITTED_RANGE	0x02
#define PROCESS_HEAP_ENTRY_BUSY		0x04
#define PROCESS_HEAP_ENTRY_MOVEABLE	0x10
#define PROCESS_HEAP_ENTRY_DDESHARE	0x20

/* platforms for GetVersionEx() */
#define VER_PLATFORM_WIN32s		0
#define VER_PLATFORM_WIN32_WINDOWS	1
#define VER_PLATFORM_WIN32_NT		2

/* processor types */
#define PROCESSOR_INTEL_386		386
#define PROCESSOR_INTEL_486		486
#define PROCESSOR_INTEL_PENTIUM		586
#define PROCESSOR_MIPS_R4000		4000
#define PROCESSOR_ALPHA_21064		21064

/* processor architectures */
#define PROCESSOR_ARCHITECTURE_INTEL	0
#define PROCESSOR_ARCHITECTURE_MIPS	1
#define PROCESSOR_ARCHITECTURE_ALPHA	2
#define PROCESSOR_ARCHITECTURE_PPC	3
#define PROCESSOR_ARCHITECTURE_UNKNOWN	0xFFFF

/* standard access rights */
#define DELETE				0x00010000
#define READ_CONTROL			0x00020000
#define WRITE_DAC			0x00040000
#define WRITE_OWNER			0x00080000
#define SYNCHRONIZE			0x00100000

#define STANDARD_RIGHTS_REQUIRED	0x000F0000
#define STANDARD_RIGHTS_READ		READ_CONTROL
#define STANDARD_RIGHTS_WRITE		READ_CONTROL
#define STANDARD_RIGHTS_EXECUTE		READ_CONTROL
#define STANDARD_RIGHTS_ALL		0x001F0000

/* security access rights */
#define ACCESS_SYSTEM_SECURITY		0x01000000
#define MAXIMUM_ALLOWED			0x02000000

/* generic access rights */
#define GENERIC_READ			0x80000000 
#define GENERIC_WRITE			0x40000000
#define GENERIC_EXECUTE			0x20000000
#define GENERIC_ALL			0x10000000

/* file access rights */
#define FILE_READ_DATA			0x0001
#define FILE_WRITE_DATA			0x0002
#define FILE_APPEND_DATA		0x0004
#define FILE_READ_EA			0x0008
#define FILE_WRITE_EA			0x0010
#define FILE_EXECUTE			0x0020
#define FILE_READ_ATTRIBUTES		0x0080
#define FILE_WRITE_ATTRIBUTES		0x0100
#define FILE_ALL_ACCESS            	(STANDARD_RIGHTS_REQUIRED|\
					 SYNCHRONIZE|0x1FF)
#define FILE_GENERIC_READ		(STANDARD_RIGHTS_READ|SYNCHRONIZE|\
					 FILE_READ_DATA|FILE_READ_ATTRIBUTES|\
					 FILE_READ_EA )
#define FILE_GENERIC_WRITE		(STANDARD_RIGHTS_WRITE|SYNCHRONIZE| \
					 FILE_WRITE_DATA|FILE_APPEND_DATA| \
					 FILE_WRITE_ATTRIBUTES|FILE_WRITE_EA )
                                     
#define FILE_GENERIC_EXECUTE		(STANDARD_RIGHTS_EXECUTE|SYNCHRONIZE | \
					 FILE_EXECUTE|FILE_READ_ATTRIBUTES )
					
/* directory access rights */ 
#define FILE_LIST_DIRECTORY		0x0001
#define FILE_ADD_FILE			0x0002
#define FILE_ADD_SUBDIRECTORY		0x0004
#define FILE_TRAVERSE			0x0020
#define FILE_DELETE_CHILD		0x0040

/* named pipe access rights */
#define FILE_CREATE_PIPE_INSTANCE	0x0004

/* thread access rights */
#define THREAD_TERMINATE		0x0001
#define THREAD_SUSPEND_RESUME		0x0002
#define THREAD_GET_CONTEXT		0x0008
#define THREAD_SET_CONTEXT		0x0010
#define THREAD_SET_INFORMATION		0x0020
#define THREAD_QUERY_INFORMATION	0x0040
#define THREAD_SET_THREAD_TOKEN		0x0080 
#define THREAD_IMPERSONATE		0x0100
#define THREAD_DIRECT_IMPERSONATION	0x0200
#define THREAD_ALL_ACCESS		(STANDARD_RIGHTS_REQUIRED |\
					 SYNCHRONIZE | 0x3FF)

/* process access rights */
#define PROCESS_TERMINATE                  0x0001
#define PROCESS_CREATE_THREAD              0x0002
#define PROCESS_VM_OPERATION               0x0008
#define PROCESS_VM_READ                    0x0010
#define PROCESS_VM_WRITE                   0x0020
#define PROCESS_DUP_HANDLE                 0x0040
#define PROCESS_CREATE_PROCESS             0x0090
#define PROCESS_SET_INFORMATION            0x0200
#define PROCESS_QUERY_INFORMATION          0x0400
#define PROCESS_ALL_ACCESS		(STANDARD_RIGHTS_REQUIRED |\
					 SYNCHRONIZE | 0x3FF)

/* memory section access rights */
#define SECTION_QUERY                      0x0001
#define SECTION_MAP_READ                   0x0002
#define SECTION_MAP_WRITE                  0x0004
#define SECTION_MAP_EXECUTE                0x0010
#define SECTION_EXTEND_SIZE                0x0020
#define SECTION_ALL_ACCESS  (STANDARD_RIGHTS_REQUIRED | 0x1F ) 

/* XXX: mutexes, semaphores, etc. */


DEFINE_HANDLE(HRSRC)
DEFINE_HANDLE(HMODULE)
typedef HMODULE	HINSTANCE;

typedef HANDLE	HGLOBAL;
typedef HANDLE	HLOCAL;
typedef HANDLE	GLOBALHANDLE;
typedef HANDLE	LOCALHANDLE;

typedef DWORD	ACCESS_MASK;
typedef WORD	ATOM; 

struct __OVERLAPPED;
typedef VOID (CALLBACK *POVERLAPPED_COMPLETION_ROUTINE) (DWORD, DWORD,
	struct __OVERLAPPED *lpOverlapped);
typedef POVERLAPPED_COMPLETION_ROUTINE LPOVERLAPPED_COMPLETION_ROUTINE;
	
typedef DWORD (WINAPI  *PTHREAD_START_ROUTINE)(LPVOID lpThreadParameter);
typedef PTHREAD_START_ROUTINE	LPTHREAD_START_ROUTINE;

typedef BOOL (CALLBACK *ENUMRESTYPEPROCA)(HMODULE hModule, LPSTR lpType, 
 	LONG lParam);
typedef BOOL (CALLBACK *ENUMRESTYPEPROCW)(HMODULE hModule, LPWSTR lpType, 
 	LONG lParam);
typedef BOOL (CALLBACK *ENUMRESNAMEPROCA)(HMODULE hModule, LPCSTR lpType, 
 	LPSTR lpName,  LONG lParam);
typedef BOOL (CALLBACK *ENUMRESNAMEPROCW)(HMODULE hModule, LPCWSTR lpType, 
 	LPWSTR lpName, LONG lParam);
typedef BOOL (CALLBACK *ENUMRESLANGPROCA)(HMODULE hModule, LPSTR lpType, 
 	LPCSTR lpName, WORD wLanguage, LONG lParam);
typedef BOOL (CALLBACK *ENUMRESLANGPROCW)(HMODULE hModule, LPWSTR lpType, 
 	LPCWSTR lpName, WORD wLanguage, LONG lParam);
                                 
typedef struct __FILETIME 
{ 
   DWORD	dwLowDateTime; 
   DWORD	dwHighDateTime; 
} FILETIME, *LPFILETIME, *PFILETIME; 
 
typedef struct __BY_HANDLE_FILE_INFORMATION 
{
   DWORD	dwFileAttributes; 
   FILETIME	ftCreationTime; 
   FILETIME	ftLastAccessTime; 
   FILETIME	ftLastWriteTime; 
   DWORD	dwVolumeSerialNumber; 
   DWORD	nFileSizeHigh; 
   DWORD	nFileSizeLow; 
   DWORD	nNumberOfLinks; 
   DWORD	nFileIndexHigh; 
   DWORD	nFileIndexLow; 
} BY_HANDLE_FILE_INFORMATION;

typedef struct __BY_HANDLE_FILE_INFORMATION  *PBY_HANDLE_FILE_INFORMATION;
typedef struct __BY_HANDLE_FILE_INFORMATION  *LPBY_HANDLE_FILE_INFORMATION; 

typedef struct FLOATING_SAVE_AREA_I386 
{
   DWORD	ControlWord;
   DWORD	StatusWord;
   DWORD	TagWord;
   DWORD	ErrorOffset;
   DWORD	ErrorSelector;
   DWORD	DataOffset;
   DWORD	DataSelector;
   BYTE		RegisterArea[80];
   DWORD	Cr0NpxState;
} FLOATING_SAVE_AREA_I386;

typedef struct __CONTEXT_I386
{
   DWORD	ContextFlags;
   DWORD	Dr0;
   DWORD	Dr1;
   DWORD	Dr2;
   DWORD	Dr3;
   DWORD	Dr6;
   DWORD	Dr7;

   FLOATING_SAVE_AREA_I386	FloatSave;

   DWORD	SegGs;
   DWORD	SegFs;
   DWORD	SegEs;
   DWORD	SegDs;

   DWORD	Edi;
   DWORD	Esi;
   DWORD	Ebx;
   DWORD	Edx;
   DWORD	Ecx;
   DWORD	Eax;

   DWORD	Ebp;
   DWORD	Eip;
   DWORD	SegCs; 
   DWORD	EFlags;
   DWORD	Esp;
   DWORD	SegSs;
} CONTEXT_I386, *PCONTEXT_I386, *LPCONTEXT_I386;

/* #ifdef __i386__ */
#define FLOATING_SAVE_AREA		FLOATING_SAVE_AREA_I386	
#define CONTEXT				CONTEXT_I386
#define PCONTEXT			PCONTEXT_I386
#define LPCONTEXT			LPCONTEXT_I386
/* #endif  __i386__ */

typedef struct __LIST_ENTRY 
{ 
   struct __LIST_ENTRY	*Flink; 
   struct __LIST_ENTRY	*Blink; 
} LIST_ENTRY, *PLIST_ENTRY; 
 
typedef struct __CRITICAL_SECTION_DEBUG
{
   WORD		Type;
   WORD		CreatorBackTraceIndex;
   struct __CRITICAL_SECTION *CriticalSection;
   LIST_ENTRY	ProcessLocksList;
   DWORD	EntryCount;
   DWORD	ContentionCount;
   DWORD	Depth;
   PVOID	OwnerBackTrace[ 5 ];
} CRITICAL_SECTION_DEBUG, *PCRITICAL_SECTION_DEBUG;

typedef struct __CRITICAL_SECTION
{
    PCRITICAL_SECTION_DEBUG	DebugInfo;
    LONG			LockCount;
    LONG			RecursionCount;
    HANDLE			OwningThread;
    HANDLE			LockSemaphore;
    DWORD			Reserved;
} CRITICAL_SECTION, *PCRITICAL_SECTION, *LPCRITICAL_SECTION;

typedef struct __CREATE_PROCESS_DEBUG_INFO 
{
   HANDLE	hFile; 
   HANDLE	hProcess; 
   HANDLE	hThread; 
   LPVOID	lpBaseOfImage; 
   DWORD	dwDebugInfoFileOffset;  
   DWORD	nDebugInfoSize; 
   LPVOID	lpThreadLocalBase; 
   LPTHREAD_START_ROUTINE	lpStartAddress; 
   LPVOID	lpImageName; 
   WORD		fUnicode; 
} CREATE_PROCESS_DEBUG_INFO; 
 
typedef struct __CREATE_THREAD_DEBUG_INFO
{
   HANDLE	hThread; 
   LPVOID	lpThreadLocalBase; 
   LPTHREAD_START_ROUTINE	lpStartAddress; 
} CREATE_THREAD_DEBUG_INFO; 

struct __EXCEPTION_RECORD;
struct __EXCEPTION_FRAME;
/* also defined as __jump_unwind */
typedef struct __EXCEPTION_FRAME
{
  struct __EXCEPTION_FRAME 	*Prev;
  FARPROC			 Handler;
} EXCEPTION_FRAME, *PEXCEPTION_FRAME, *LPEXCEPTION_FRAME;

typedef struct __EXCEPTION_POINTERS 
{
  struct __EXCEPTION_RECORD	*ExceptionRecord;
  LPCONTEXT			ContextRecord;
} EXCEPTION_POINTERS, *PEXCEPTION_POINTERS, *LPEXCEPTION_POINTERS;

typedef struct __EXCEPTION_RECORD
{
    DWORD    ExceptionCode;
    DWORD    ExceptionFlags;
    struct __EXCEPTION_RECORD *ExceptionRecord;
    PVOID    ExceptionAddress;
    DWORD    NumberParameters;
    DWORD    ExceptionInformation[EXCEPTION_MAXIMUM_PARAMETERS];
} EXCEPTION_RECORD, *PEXCEPTION_RECORD, *LPEXCEPTION_RECORD;

typedef struct __EXCEPTION_RECORD_MIN
{
    DWORD    ExceptionCode;
    DWORD    ExceptionFlags;
    struct __EXCEPTION_RECORD *ExceptionRecord;

    PVOID    ExceptionAddress;
    DWORD    NumberParameters;
    DWORD    ExceptionInformation[0];
} EXCEPTION_RECORD_MIN;

/* function pointer to a UnhandledExceptionFilter(); */
typedef long (WINAPI *__PTOP_EXCFILTER)
                   (PEXCEPTION_POINTERS ExceptionInfo);
typedef __PTOP_EXCFILTER PTOP_LEVER_EXCEPTION_FILTER;
typedef __PTOP_EXCFILTER LPTOP_LEVEL_EXCEPTION_FILTER;
 
typedef struct __EXCEPTION_DEBUG_INFO 
{
   EXCEPTION_RECORD	ExceptionRecord; 
   DWORD		dwFirstChance; 
} EXCEPTION_DEBUG_INFO; 
 
typedef struct __EXIT_PROCESS_DEBUG_INFO 
{ 
  DWORD dwExitCode; 
} EXIT_PROCESS_DEBUG_INFO; 
 
typedef struct __EXIT_THREAD_DEBUG_INFO
{ 
  DWORD dwExitCode; 
} EXIT_THREAD_DEBUG_INFO; 
 
typedef struct __LOAD_DLL_DEBUG_INFO 
{ 
   HANDLE	hFile; 
   LPVOID	lpBaseOfDll; 
   DWORD	dwDebugInfoFileOffset; 
   DWORD	nDebugInfoSize; 
   LPVOID	lpImageName; 
   WORD		fUnicode; 
} LOAD_DLL_DEBUG_INFO; 
 
typedef struct __UNLOAD_DLL_DEBUG_INFO
{ 
   LPVOID	lpBaseOfDll; 
} UNLOAD_DLL_DEBUG_INFO; 
 
typedef struct __OUTPUT_DEBUG_STRING_INFO 
{ 
   LPSTR	lpDebugStringData; 
   WORD		fUnicode; 
   WORD		nDebugStringLength; 
} OUTPUT_DEBUG_STRING_INFO; 
 
typedef struct __RIP_INFO
{ 
  DWORD		dwError; 
  DWORD		dwType; 
} RIP_INFO; 
 
typedef struct __DEBUG_EVENT 
{ 
  DWORD dwDebugEventCode; 
  DWORD dwProcessId; 
  DWORD dwThreadId; 
  union 
  { 
    EXCEPTION_DEBUG_INFO Exception; 
    CREATE_THREAD_DEBUG_INFO CreateThread; 
    CREATE_PROCESS_DEBUG_INFO CreateProcessInfo; 
    EXIT_THREAD_DEBUG_INFO ExitThread; 
    EXIT_PROCESS_DEBUG_INFO ExitProcess; 
    LOAD_DLL_DEBUG_INFO LoadDll; 
    UNLOAD_DLL_DEBUG_INFO UnloadDll; 
    OUTPUT_DEBUG_STRING_INFO DebugString; 
    RIP_INFO RipInfo; 
  } u; 
} DEBUG_EVENT, *LPDEBUG_EVENT; 
 
typedef struct __LARGE_INTEGER 
{ 
  DWORD LowPart; 
  LONG  HighPart; 
} LARGE_INTEGER, *PLARGE_INTEGER; 
   
typedef struct __SYSTEMTIME
{  
   WORD wYear; 
   WORD wMonth; 
   WORD wDayOfWeek; 
   WORD wDay; 
   WORD wHour; 
   WORD wMinute; 
   WORD wSecond; 
   WORD wMilliseconds; 
} SYSTEMTIME, *PSYSTEMTIME, *LPSYSTEMTIME; 
 
typedef struct __LDT_ENTRY
{ 
   WORD LimitLow; 
   WORD BaseLow; 
   union
   { 
      struct
      { 
         BYTE BaseMid; 
         BYTE Flags1; 
         BYTE Flags2; 
         BYTE BaseHi; 
      } Bytes; 
      struct
      { 
         DWORD BaseMid : 8; 
         DWORD Type : 5; 
         DWORD Dpl : 2; 
         DWORD Pres : 1; 
         DWORD LimitHi : 4; 
         DWORD Sys : 1; 
         DWORD Reserved_0 : 1; 
         DWORD Default_Big : 1; 
         DWORD Granularity : 1; 
         DWORD BaseHi : 8; 
      } Bits; 
  } HighWord; 
} LDT_ENTRY, *PLDT_ENTRY, *LPLDT_ENTRY; 

typedef LARGE_INTEGER LUID, *PLUID;

typedef struct __LUID_AND_ATTRIBUTES
{ 
   LUID  Luid; 
   DWORD Attributes; 
} LUID_AND_ATTRIBUTES; 
 
typedef LUID_AND_ATTRIBUTES LUID_AND_ATTRIBUTES_ARRAY[ANYSIZE_ARRAY];
typedef LUID_AND_ATTRIBUTES_ARRAY *PLUID_AND_ATTRIBUTES_ARRAY;
 
typedef struct __MEMORY_BASIC_INFORMATION
{ 
   PVOID BaseAddress;            
   PVOID AllocationBase;         
   DWORD AllocationProtect;      
   DWORD RegionSize;             
   DWORD State;                  
   DWORD Protect;                
   DWORD Type;                   
} MEMORY_BASIC_INFORMATION, *PMEMORY_BASIC_INFORMATION, *LPMEMORY_BASIC_INFORMATION; 
 
typedef struct __MEMORYSTATUS
{ 
   DWORD dwLength;        
   DWORD dwMemoryLoad;    
   DWORD dwTotalPhys;     
   DWORD dwAvailPhys;     
   DWORD dwTotalPageFile; 
   DWORD dwAvailPageFile; 
   DWORD dwTotalVirtual;  
   DWORD dwAvailVirtual;  
} MEMORYSTATUS, *LPMEMORYSTATUS; 
 
typedef struct __OFSTRUCT
{ 
   BYTE cBytes; 
   BYTE fFixedDisk; 
   WORD nErrCode; 
   WORD Reserved1; 
   WORD Reserved2; 
   CHAR szPathName[OFS_MAXPATHNAME]; 
} OFSTRUCT, *LPOFSTRUCT; 
 
typedef struct __OSVERSIONINFOA
{ 
   DWORD dwOSVersionInfoSize; 
   DWORD dwMajorVersion; 
   DWORD dwMinorVersion; 
   DWORD dwBuildNumber; 
   DWORD dwPlatformId; 
   CHAR szCSDVersion[ 128 ]; 
} OSVERSIONINFOA, *POSVERSIONINFOA, *LPOSVERSIONINFOA; 
 
typedef struct __OSVERSIONINFOW
{ 
   DWORD dwOSVersionInfoSize; 
   DWORD dwMajorVersion; 
   DWORD dwMinorVersion; 
   DWORD dwBuildNumber; 
   DWORD dwPlatformId; 
   WCHAR szCSDVersion[ 128 ]; 
} OSVERSIONINFOW, *POSVERSIONINFOW, *LPOSVERSIONINFOW; 
  
typedef struct __OVERLAPPED
{ 
   DWORD  Internal; 
   DWORD  InternalHigh; 
   DWORD  Offset; 
   DWORD  OffsetHigh; 
   HANDLE hEvent; 
} OVERLAPPED, *POVERLAPPED, *LPOVERLAPPED; 

typedef struct __PREVENT_MEDIA_REMOVAL
{ 
   BOOLEAN PreventMediaRemoval; 
} PREVENT_MEDIA_REMOVAL ; 
  
typedef struct __PROCESS_HEAP_ENTRY
{  
   PVOID	lpData; 
   DWORD	cbData; 
   BYTE		cbOverhead; 
   BYTE		iRegionIndex; 
   WORD		wFlags; 
   DWORD	dwCommittedSize; 
   DWORD	dwUnCommittedSize; 
   LPVOID	lpFirstBlock; 
   LPVOID	lpLastBlock; 
   HANDLE	hMem; 
} PROCESS_HEAPENTRY, *LPPROCESS_HEAP_ENTRY; 
 
typedef struct __PROCESS_INFORMATION
{ 
   HANDLE	hProcess; 
   HANDLE	hThread; 
   DWORD	dwProcessId; 
   DWORD	dwThreadId; 
} PROCESS_INFORMATION, *LPPROCESS_INFORMATION; 
 
 
typedef struct __SECURITY_ATTRIBUTES
{ 
   DWORD	nLength; 
   LPVOID	lpSecurityDescriptor; 
   BOOL		bInheritHandle; 
} SECURITY_ATTRIBUTES, *LPSECURITY_ATTRIBUTES; 
 
typedef DWORD SECURITY_INFORMATION, *PSECURITY_INFORMATION; 
 
typedef struct __SINGLE_LIST_ENTRY 
{ 
   struct __SINGLE_LIST_ENTRY *Next; 
} SINGLE_LIST_ENTRY; 
 
typedef struct __STARTUPINFOA
{ 
   DWORD	cb; 
   LPSTR	lpReserved; 
   LPSTR	lpDesktop; 
   LPSTR	lpTitle; 
   DWORD	dwX; 
   DWORD	dwY; 
   DWORD	dwXSize; 
   DWORD	dwYSize; 
   DWORD	dwXCountChars; 
   DWORD	dwYCountChars; 
   DWORD	dwFillAttribute; 
   DWORD	dwFlags; 
   WORD		wShowWindow; 
   WORD		cbReserved2; 
   LPBYTE	lpReserved2; 
   HANDLE	hStdInput; 
   HANDLE	hStdOutput; 
   HANDLE	hStdError; 
} STARTUPINFOA, *PSTARTUPINFOA, *LPSTARTUPINFOA; 

typedef struct __STARTUPINFOW
{ 
   DWORD	cb; 
   LPWSTR	lpReserved; 
   LPWSTR	lpDesktop; 
   LPWSTR	lpTitle; 
   DWORD	dwX; 
   DWORD	dwY; 
   DWORD	dwXSize; 
   DWORD	dwYSize; 
   DWORD	dwXCountChars; 
   DWORD	dwYCountChars; 
   DWORD	dwFillAttribute; 
   DWORD	dwFlags; 
   WORD		wShowWindow; 
   WORD		cbReserved2; 
   LPBYTE	lpReserved2; 
   HANDLE	hStdInput; 
   HANDLE	hStdOutput; 
   HANDLE	hStdError; 
} STARTUPINFOW, *PSTARTUPINFOW, *LPSTARTUPINFOW; 
 
typedef struct __SYSTEM_INFO
{ 
   WORD		wProcessorArchitecture; 
   WORD		wReserved; 
   DWORD	dwPageSize; 
   LPVOID	lpMinimumApplicationAddress; 
   LPVOID	lpMaximumApplicationAddress; 
   DWORD	dwActiveProcessorMask; 
   DWORD	dwNumberOfProcessors; 
   DWORD	dwProcessorType; 
   DWORD	dwAllocationGranularity; 
   WORD		wProcessorLevel; 
   WORD		wProcessorRevision; 
} SYSTEM_INFO, *LPSYSTEM_INFO; 
 
typedef struct __SYSTEM_POWER_STATUS
{
   BYTE		ACLineStatus;           
   BYTE		BatteryFlag;           
   BYTE		BatteryLifePercent;    
   BYTE		Reserved1;             
   DWORD	BatteryLifeTime;      
   DWORD	BatteryFullLifeTime;  
} SYSTEM_POWER_STATUS;
typedef struct SYSTEM_POWER_STATUS *LPSYSTEM_POWER_STATUS;
 
 
typedef struct _TIME_ZONE_INFORMATION { 
   LONG       Bias; 
   WCHAR      StandardName[ 32 ]; 
   SYSTEMTIME StandardDate; 
   LONG       StandardBias; 
   WCHAR      DaylightName[ 32 ]; 
   SYSTEMTIME DaylightDate; 
   LONG       DaylightBias; 
} TIME_ZONE_INFORMATION, *LPTIME_ZONE_INFORMATION; 

typedef struct __ULARGE_INTEGER
{ 
  DWORD LowPart; 
  DWORD HighPart; 
} ULARGE_INTEGER, *PULARGE_INTEGER; 
 
typedef struct __WIN32_FIND_DATAA
{ 
   DWORD	dwFileAttributes; 
   FILETIME	ftCreationTime; 
   FILETIME	ftLastAccessTime; 
   FILETIME	ftLastWriteTime; 
   DWORD	nFileSizeHigh; 
   DWORD	nFileSizeLow; 
   DWORD	dwReserved0; 
   DWORD	dwReserved1; 
   CHAR		cFileName[ MAX_PATH ]; 
   CHAR		cAlternateFileName[ 14 ]; 
} WIN32_FIND_DATAA, *PWIN32_FIND_DATAA, *LPWIN32_FIND_DATAA; 

typedef struct __WIN32_FIND_DATAW
{ 
   DWORD	dwFileAttributes; 
   FILETIME	ftCreationTime; 
   FILETIME	ftLastAccessTime; 
   FILETIME	ftLastWriteTime; 
   DWORD	nFileSizeHigh; 
   DWORD	nFileSizeLow; 
   DWORD	dwReserved0; 
   DWORD	dwReserved1; 
   WCHAR	cFileName[ MAX_PATH ]; 
   WCHAR	cAlternateFileName[ 14 ]; 
} WIN32_FIND_DATAW, *PWIN32_FIND_DATAW, *LPWIN32_FIND_DATAW; 
 
typedef struct __WIN32_STREAM_ID
{ 
   DWORD	dwStreamId; 
   DWORD	dwStreamAttributes; 
   LARGE_INTEGER	Size; 
   DWORD	dwStreamNameSize; 
   WCHAR	*cStreamName ; 
} WIN32_STREAM_ID;

 
#ifndef UNICODE
#define ENUMRESTYPEPROC		ENUMRESTYPEPROCA 
#define ENUMRESNAMEPROC		ENUMRESNAMEPROCA
#define ENUMRESLANGPROC		ENUMRESLANGPROCA
#define OSVERSIONINFO		OSVERSIONINFOA
#define POSVERSIONINFO		POSVERSIONINFOA
#define LPOSVERSIONINFO		LPOSVERSIONINFOA
#define STARTUPINFO		STARTUPINFOA
#define PSTARTUPINFO		PSTARTUPINFOA
#define LPSTARTUPINFO		LPSTARTUPINFOA
#define WIN32_FIND_DATA		WIN32_FIND_DATAA
#define PWIN32_FIND_DATA	PWIN32_FIND_DATAA
#define LPWIN32_FIND_DATA	LPWIN32_FIND_DATAA
#else
#define ENUMRESTYPEPROC		ENUMRESTYPEPROCW
#define ENUMRESNAMEPROC		ENUMRESNAMEPROCW
#define ENUMRESLANGPROC		ENUMRESLANGPROCW
#define OSVERSIONINFO		OSVERSIONINFOW
#define POSVERSIONINFO		POSVERSIONINFOW
#define LPOSVERSIONINFO		LPOSVERSIONINFOW
#define STARTUPINFO		STARTUPINFOW
#define PSTARTUPINFO		PSTARTUPINFOW
#define LPSTARTUPINFO		LPSTARTUPINFOW
#define WIN32_FIND_DATA		WIN32_FIND_DATAW
#define PWIN32_FIND_DATA	PWIN32_FIND_DATAW
#define LPWIN32_FIND_DATA	LPWIN32_FIND_DATAW
#endif

ATOM
WINAPI
AddAtomA(
	 LPCSTR lpString
	 );
ATOM
WINAPI
AddAtomW(
	 LPCWSTR lpString
	 );
	 
BOOL
WINAPI
AreFileApisANSI( VOID );

BOOL
WINAPI
BackupRead(
	   HANDLE hFile,
	   LPBYTE lpBuffer,
	   DWORD nNumberOfBytesToRead,
	   LPDWORD lpNumberOfBytesRead,
	   BOOL bAbort,
	   BOOL bProcessSecurity,
	   LPVOID *lpContext
	   );


BOOL
WINAPI
BackupSeek(
	   HANDLE hFile,
	   DWORD  dwLowBytesToSeek,
	   DWORD  dwHighBytesToSeek,
	   LPDWORD lpdwLowByteSeeked,
	   LPDWORD lpdwHighByteSeeked,
	   LPVOID *lpContext
	   );


BOOL
WINAPI
BackupWrite(
	    HANDLE hFile,
	    LPBYTE lpBuffer,
	    DWORD nNumberOfBytesToWrite,
	    LPDWORD lpNumberOfBytesWritten,
	    BOOL bAbort,
	    BOOL bProcessSecurity,
	    LPVOID *lpContext
	    );

BOOL
WINAPI
Beep(
     DWORD dwFreq,
     DWORD dwDuration
     );

HANDLE
WINAPI
BeginUpdateResourceA(
		     LPCSTR pFileName,
		     BOOL bDeleteExistingResources
		     );

HANDLE
WINAPI
BeginUpdateResourceW(
		     LPCWSTR pFileName,
		     BOOL bDeleteExistingResources
		     );

BOOL
WINAPI
CallNamedPipeA(
    LPCSTR lpNamedPipeName,
    LPVOID lpInBuffer,
    DWORD nInBufferSize,
    LPVOID lpOutBuffer,
    DWORD nOutBufferSize,
    LPDWORD lpBytesRead,
    DWORD nTimeOut
    );

BOOL
WINAPI
CallNamedPipeW(
    LPCWSTR lpNamedPipeName,
    LPVOID lpInBuffer,
    DWORD nInBufferSize,
    LPVOID lpOutBuffer,
    DWORD nOutBufferSize,
    LPDWORD lpBytesRead,
    DWORD nTimeOut
    );

BOOL
WINAPI
CloseHandle(
	    HANDLE hObject
	    );

LONG
WINAPI
CompareFileTime(
		CONST FILETIME *lpFileTime1,
		CONST FILETIME *lpFileTime2
		);

BOOL
WINAPI
ConnectNamedPipe(
		 HANDLE hNamedPipe,
		 LPOVERLAPPED lpOverlapped
		 );

BOOL
WINAPI
ContinueDebugEvent(
		   DWORD dwProcessId,
		   DWORD dwThreadId,
		   DWORD dwContinueStatus
		   );

BOOL
WINAPI
CopyFileA(
    LPCSTR lpExistingFileName,
    LPCSTR lpNewFileName,
    BOOL bFailIfExists
    );

BOOL
WINAPI
CopyFileW(
    LPCWSTR lpExistingFileName,
    LPCWSTR lpNewFileName,
    BOOL bFailIfExists
    );

BOOL
WINAPI
CreateDirectoryA(
    LPCSTR lpPathName,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );

BOOL
WINAPI
CreateDirectoryW(
    LPCWSTR lpPathName,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );

BOOL
WINAPI
CreateDirectoryExA(
    LPCSTR lpTemplateDirectory,
    LPCSTR lpNewDirectory,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );

BOOL
WINAPI
CreateDirectoryExW(
    LPCWSTR lpTemplateDirectory,
    LPCWSTR lpNewDirectory,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );

HANDLE
WINAPI
CreateEventA(
	     LPSECURITY_ATTRIBUTES lpEventAttributes,
	     BOOL bManualReset,
	     BOOL bInitialState,
	     LPCSTR lpName
	     );

HANDLE
WINAPI
CreateEventW(
	     LPSECURITY_ATTRIBUTES lpEventAttributes,
	     BOOL bManualReset,
	     BOOL bInitialState,
	     LPCWSTR lpName
	     );

HANDLE
WINAPI
CreateFileA(
    LPCSTR lpFileName,
    DWORD dwDesiredAccess,
    DWORD dwShareMode,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    DWORD dwCreationDisposition,
    DWORD dwFlagsAndAttributes,
    HANDLE hTemplateFile
    );

HANDLE
WINAPI
CreateFileW(
    LPCWSTR lpFileName,
    DWORD dwDesiredAccess,
    DWORD dwShareMode,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    DWORD dwCreationDisposition,
    DWORD dwFlagsAndAttributes,
    HANDLE hTemplateFile
    );

HANDLE
WINAPI
CreateFileMappingA(
		   HANDLE hFile,
		   LPSECURITY_ATTRIBUTES lpFileMappingAttributes,
		   DWORD flProtect,
		   DWORD dwMaximumSizeHigh,
		   DWORD dwMaximumSizeLow,
		   LPCSTR lpName
		   );

HANDLE
WINAPI
CreateFileMappingW(
		   HANDLE hFile,
		   LPSECURITY_ATTRIBUTES lpFileMappingAttributes,
		   DWORD flProtect,
		   DWORD dwMaximumSizeHigh,
		   DWORD dwMaximumSizeLow,
		   LPCWSTR lpName
		   );

HANDLE
WINAPI
CreateIoCompletionPort(
		       HANDLE FileHandle,
		       HANDLE ExistingCompletionPort,
		       DWORD CompletionKey,
		       DWORD NumberOfConcurrentThreads
		       );

HANDLE
WINAPI
CreateMailslotA(
		LPCSTR lpName,
		DWORD nMaxMessageSize,
		DWORD lReadTimeout,
		LPSECURITY_ATTRIBUTES lpSecurityAttributes
		);

HANDLE
WINAPI
CreateMailslotW(
		LPCWSTR lpName,
		DWORD nMaxMessageSize,
		DWORD lReadTimeout,
		LPSECURITY_ATTRIBUTES lpSecurityAttributes
		);

HANDLE
WINAPI
CreateMutexA(
	     LPSECURITY_ATTRIBUTES lpMutexAttributes,
	     BOOL bInitialOwner,
	     LPCSTR lpName
	     );

HANDLE
WINAPI
CreateMutexW(
	     LPSECURITY_ATTRIBUTES lpMutexAttributes,
	     BOOL bInitialOwner,
	     LPCWSTR lpName
	     );

HANDLE
WINAPI
CreateNamedPipeA(
    LPCSTR lpName,
    DWORD dwOpenMode,
    DWORD dwPipeMode,
    DWORD nMaxInstances,
    DWORD nOutBufferSize,
    DWORD nInBufferSize,
    DWORD nDefaultTimeOut,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );

HANDLE
WINAPI
CreateNamedPipeW(
    LPCWSTR lpName,
    DWORD dwOpenMode,
    DWORD dwPipeMode,
    DWORD nMaxInstances,
    DWORD nOutBufferSize,
    DWORD nInBufferSize,
    DWORD nDefaultTimeOut,
    LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );

BOOL
WINAPI
CreatePipe(
	   PHANDLE hReadPipe,
	   PHANDLE hWritePipe,
	   LPSECURITY_ATTRIBUTES lpPipeAttributes,
	   DWORD nSize
	   );

BOOL
WINAPI
CreateProcessA(
    LPCSTR lpApplicationName,
    LPSTR lpCommandLine,
    LPSECURITY_ATTRIBUTES lpProcessAttributes,
    LPSECURITY_ATTRIBUTES lpThreadAttributes,
    BOOL bInheritHandles,
    DWORD dwCreationFlags,
    LPVOID lpEnvironment,
    LPCSTR lpCurrentDirectory,
    LPSTARTUPINFOA lpStartupInfo,
    LPPROCESS_INFORMATION lpProcessInformation
    );

BOOL
WINAPI
CreateProcessW(
    LPCWSTR lpApplicationName,
    LPWSTR lpCommandLine,
    LPSECURITY_ATTRIBUTES lpProcessAttributes,
    LPSECURITY_ATTRIBUTES lpThreadAttributes,
    BOOL bInheritHandles,
    DWORD dwCreationFlags,
    LPVOID lpEnvironment,
    LPCWSTR lpCurrentDirectory,
    LPSTARTUPINFOW lpStartupInfo,
    LPPROCESS_INFORMATION lpProcessInformation
    );

HANDLE
WINAPI
CreateRemoteThread(
		   HANDLE hProcess,
		   LPSECURITY_ATTRIBUTES lpThreadAttributes,
		   DWORD dwStackSize,
		   LPTHREAD_START_ROUTINE lpStartAddress,
		   LPVOID lpParameter,
		   DWORD dwCreationFlags,
		   LPDWORD lpThreadId
		   );

HANDLE
WINAPI
CreateSemaphoreA(
		 LPSECURITY_ATTRIBUTES lpSemaphoreAttributes,
		 LONG lInitialCount,
		 LONG lMaximumCount,
		 LPCSTR lpName
		 );

HANDLE
WINAPI
CreateSemaphoreW(
		 LPSECURITY_ATTRIBUTES lpSemaphoreAttributes,
		 LONG lInitialCount,
		 LONG lMaximumCount,
		 LPCWSTR lpName
		 );

HANDLE
WINAPI
CreateThread(
	     LPSECURITY_ATTRIBUTES lpThreadAttributes,
	     DWORD dwStackSize,
	     LPTHREAD_START_ROUTINE lpStartAddress,
	     LPVOID lpParameter,
	     DWORD dwCreationFlags,
	     LPDWORD lpThreadId
	     );

BOOL
WINAPI
DebugActiveProcess(
		   DWORD dwProcessId
		   );

VOID
WINAPI
DebugBreak( VOID ); 

BOOL
WINAPI
DefineDosDeviceA(
    DWORD dwFlags,
    LPCSTR lpDeviceName,
    LPCSTR lpTargetPath
    );
   
BOOL
WINAPI
DefineDosDeviceW(
    DWORD dwFlags,
    LPCWSTR lpDeviceName,
    LPCWSTR lpTargetPath
    );
   
ATOM
WINAPI
DeleteAtom(
	   ATOM nAtom
	   );

VOID
WINAPI
DeleteCriticalSection(
		      LPCRITICAL_SECTION lpCriticalSection
		      );

BOOL
WINAPI
DeleteFileA(
    LPCSTR lpFileName
    );

BOOL
WINAPI
DeleteFileW(
    LPCWSTR lpFileName
    );

BOOL
WINAPI
DeviceIoControl(
		HANDLE hDevice,
		DWORD dwIoControlCode,
		LPVOID lpInBuffer,
		DWORD nInBufferSize,
		LPVOID lpOutBuffer,
		DWORD nOutBufferSize,
		LPDWORD lpBytesReturned,
		LPOVERLAPPED lpOverlapped
		);
	
BOOL
WINAPI
DisableThreadLibraryCalls(
			  HMODULE hLibModule
			  );
			  
BOOL
WINAPI
DisconnectNamedPipe(
		    HANDLE hNamedPipe
		    );

BOOL
WINAPI
DosDateTimeToFileTime(
		      WORD wFatDate,
		      WORD wFatTime,
		      LPFILETIME lpFileTime
		      );

BOOL
WINAPI
DuplicateHandle(
		HANDLE hSourceProcessHandle,
		HANDLE hSourceHandle,
		HANDLE hTargetProcessHandle,
		LPHANDLE lpTargetHandle,
		DWORD dwDesiredAccess,
		BOOL bInheritHandle,
		DWORD dwOptions
		);		      

BOOL
WINAPI
EndUpdateResourceA(
		   HANDLE      hUpdate,
		   BOOL        fDiscard
		   );

BOOL
WINAPI
EndUpdateResourceW(
		   HANDLE      hUpdate,
		   BOOL        fDiscard
		   );

VOID
WINAPI
EnterCriticalSection(
		     LPCRITICAL_SECTION lpCriticalSection
		     );

BOOL
WINAPI
EnumResourceLanguagesA(
		       HMODULE hModule,
		       LPCSTR lpType,
		       LPCSTR lpName,
		       ENUMRESLANGPROCA lpEnumFunc,
		       LONG lParam
		       );

BOOL
WINAPI
EnumResourceLanguagesW(
		       HMODULE hModule,
		       LPCWSTR lpType,
		       LPCWSTR lpName,
		       ENUMRESLANGPROCW lpEnumFunc,
		       LONG lParam
		       );

BOOL
WINAPI
EnumResourceNamesA(
		   HMODULE hModule,
		   LPCSTR lpType,
		   ENUMRESNAMEPROCA lpEnumFunc,
		   LONG lParam
		   );

BOOL
WINAPI
EnumResourceNamesW(
		   HMODULE hModule,
		   LPCWSTR lpType,
		   ENUMRESNAMEPROCW lpEnumFunc,
		   LONG lParam
		   );

BOOL
WINAPI
EnumResourceTypesA(
		   HMODULE hModule,
		   ENUMRESTYPEPROCA lpEnumFunc,
		   LONG lParam
		   );

BOOL
WINAPI
EnumResourceTypesW(
		   HMODULE hModule,
		   ENUMRESTYPEPROCW lpEnumFunc,
		   LONG lParam
		   );

VOID
WINAPI
ExitProcess(
	    UINT uExitCode
	    );

VOID
WINAPI
ExitThread(
	   DWORD dwExitCode
	   );

DWORD
WINAPI
ExpandEnvironmentStringsA(
			  LPCSTR lpSrc,
			  LPSTR lpDst,
			  DWORD nSize
			  );

DWORD
WINAPI
ExpandEnvironmentStringsW(
			  LPCWSTR lpSrc,
			  LPWSTR lpDst,
			  DWORD nSize
			  );
			  
VOID
WINAPI
FatalAppExitA(
	      UINT uAction,
	      LPCSTR lpMessageText
	      );
VOID
WINAPI
FatalAppExitW(
	      UINT uAction,
	      LPCWSTR lpMessageText
	      );

VOID
WINAPI
FatalExit(
	  int ExitCode
	  );

BOOL
WINAPI
FileTimeToDosDateTime(
		      CONST FILETIME *lpFileTime,
		      LPWORD lpFatDate,
		      LPWORD lpFatTime
		      );

BOOL
WINAPI
FileTimeToLocalFileTime(
			CONST FILETIME *lpFileTime,
			LPFILETIME lpLocalFileTime
			);

BOOL
WINAPI
FileTimeToSystemTime(
		     CONST FILETIME *lpFileTime,
		     LPSYSTEMTIME lpSystemTime
		     );

VOID
WINAPI
FillMemory (
	    PVOID Destination,
	    DWORD Length,
	    BYTE  Fill
	     );

ATOM
WINAPI
FindAtomA(
	  LPCSTR lpString
	  );

ATOM
WINAPI
FindAtomW(
	  LPCWSTR lpString
	  );

BOOL
WINAPI
FindClose(
	  HANDLE hFindFile
	  );

BOOL
WINAPI
FindCloseChangeNotification(
			    HANDLE hChangeHandle
			    );

HANDLE
WINAPI
FindFirstChangeNotificationA(
    LPCSTR lpPathName,
    BOOL bWatchSubtree,
    DWORD dwNotifyFilter
    );

HANDLE
WINAPI
FindFirstChangeNotificationW(
    LPCWSTR lpPathName,
    BOOL bWatchSubtree,
    DWORD dwNotifyFilter
    );

HANDLE
WINAPI
FindFirstFileA(
    LPCSTR lpFileName,
    LPWIN32_FIND_DATAA lpFindFileData
    );

HANDLE
WINAPI
FindFirstFileW(
    LPCWSTR lpFileName,
    LPWIN32_FIND_DATAW lpFindFileData
    );

BOOL
WINAPI
FindNextChangeNotification(
			   HANDLE hChangeHandle
			   );
BOOL
WINAPI
FindNextFileA(
    HANDLE hFindFile,
    LPWIN32_FIND_DATAA lpFindFileData
    );

BOOL
WINAPI
FindNextFileW(
    HANDLE hFindFile,
    LPWIN32_FIND_DATAW lpFindFileData
    );

HRSRC
WINAPI
FindResourceA(
	      HINSTANCE hModule,
	      LPCSTR lpName,
	      LPCSTR lpType
	      );

HRSRC
WINAPI
FindResourceW(
	      HINSTANCE hModule,
	      LPCWSTR lpName,
	      LPCWSTR lpType
	      );

HRSRC
WINAPI
FindResourceExA(
		HINSTANCE hModule,
		LPCSTR lpType,
		LPCSTR lpName,
		WORD    wLanguage
		);

HRSRC
WINAPI
FindResourceExW(
		HINSTANCE hModule,
		LPCWSTR lpType,
		LPCWSTR lpName,
		WORD    wLanguage
		);
		
BOOL
WINAPI
FlushFileBuffers(
		 HANDLE hFile
		 );

BOOL
WINAPI
FlushInstructionCache(
		      HANDLE hProcess,
		      LPCVOID lpBaseAddress,
		      DWORD dwSize
		      );

BOOL
WINAPI
FlushViewOfFile(
		LPCVOID lpBaseAddress,
		DWORD dwNumberOfBytesToFlush
		);

DWORD
WINAPI
FormatMessageA(
	       DWORD dwFlags,
	       LPCVOID lpSource,
	       DWORD dwMessageId,
	       DWORD dwLanguageId,
	       LPSTR lpBuffer,
	       DWORD nSize,
	       va_list *Arguments
	       );

DWORD
WINAPI
FormatMessageW(
	       DWORD dwFlags,
	       LPCVOID lpSource,
	       DWORD dwMessageId,
	       DWORD dwLanguageId,
	       LPWSTR lpBuffer,
	       DWORD nSize,
	       va_list *Arguments
	       );

BOOL
WINAPI
FreeEnvironmentStringsA(
			LPSTR lpEnvironmentBlock
			);

BOOL
WINAPI
FreeEnvironmentStringsW(
			LPWSTR lpEnvironmentBlock
			);
	       		      
BOOL
WINAPI
FreeLibrary(
	    HINSTANCE hLibModule
	    );

VOID
WINAPI
FreeLibraryAndExitThread(
			 HMODULE hLibModule,
			 DWORD dwExitCode
			 );

BOOL
WINAPI
FreeResource(
	     HGLOBAL hResData
	     );

UINT
WINAPI
GetACP(void);

UINT
WINAPI
GetAtomNameA(
	     ATOM nAtom,
	     LPSTR lpBuffer,
	     int nSize
	     );

UINT
WINAPI
GetAtomNameW(
	     ATOM nAtom,
	     LPWSTR lpBuffer,
	     int nSize
	     );

BOOL
WINAPI
GetBinaryTypeA(
	       LPCSTR lpApplicationName,
	       LPDWORD lpBinaryType
	       );

BOOL
WINAPI
GetBinaryTypeW(
	       LPCWSTR lpApplicationName,
	       LPDWORD lpBinaryType
	       );

LPSTR
WINAPI
GetCommandLineA( VOID );

LPWSTR
WINAPI
GetCommandLineW( VOID );

DWORD
WINAPI
GetCompressedFileSizeA(
    LPCSTR lpFileName,
    LPDWORD lpFileSizeHigh
    );

DWORD
WINAPI
GetCompressedFileSizeW(
    LPCWSTR lpFileName,
    LPDWORD lpFileSizeHigh
    );

BOOL
WINAPI
GetComputerNameA (
    LPSTR lpBuffer,
    LPDWORD nSize
    );

BOOL
WINAPI
GetComputerNameW (
    LPWSTR lpBuffer,
    LPDWORD nSize
    );

DWORD
WINAPI
GetCurrentDirectoryA(
    DWORD nBufferLength,
    LPSTR lpBuffer
    );

DWORD
WINAPI
GetCurrentDirectoryW(
    DWORD nBufferLength,
    LPWSTR lpBuffer
    );

HANDLE
WINAPI
GetCurrentProcess( VOID );

DWORD
WINAPI
GetCurrentProcessId( VOID );

HANDLE
WINAPI
GetCurrentThread( VOID );


DWORD
WINAPI
GetCurrentThreadId( VOID );

BOOL
WINAPI
GetDiskFreeSpaceA(
    LPCSTR lpRootPathName,
    LPDWORD lpSectorsPerCluster,
    LPDWORD lpBytesPerSector,
    LPDWORD lpNumberOfFreeClusters,
    LPDWORD lpTotalNumberOfClusters
    );

BOOL
WINAPI
GetDiskFreeSpaceW(
    LPCWSTR lpRootPathName,
    LPDWORD lpSectorsPerCluster,
    LPDWORD lpBytesPerSector,
    LPDWORD lpNumberOfFreeClusters,
    LPDWORD lpTotalNumberOfClusters
    );

UINT
WINAPI
GetDriveTypeA(
    LPCSTR lpRootPathName
    );

UINT
WINAPI
GetDriveTypeW(
    LPCWSTR lpRootPathName
    );

LPSTR
WINAPI
GetEnvironmentStringsA( VOID );

LPWSTR
WINAPI
GetEnvironmentStringsW( VOID );

DWORD
WINAPI
GetEnvironmentVariableA(
			LPCSTR lpName,
			LPSTR lpBuffer,
			DWORD nSize
			);

DWORD
WINAPI
GetEnvironmentVariableW(
			LPCWSTR lpName,
			LPWSTR lpBuffer,
			DWORD nSize
			);

BOOL
WINAPI
GetExitCodeProcess(
		   HANDLE hProcess,
		   LPDWORD lpExitCode
		   );

BOOL
WINAPI
GetExitCodeThread(
		  HANDLE hThread,
		  LPDWORD lpExitCode
		  );

DWORD
WINAPI
GetFileAttributesA(
    LPCSTR lpFileName
    );

DWORD
WINAPI
GetFileAttributesW(
    LPCWSTR lpFileName
    );

BOOL
WINAPI
GetFileInformationByHandle(
			   HANDLE hFile,
			   LPBY_HANDLE_FILE_INFORMATION lpFileInformation
			   );

DWORD
WINAPI
GetFullPathNameA(
    LPCSTR lpFileName,
    DWORD nBufferLength,
    LPSTR lpBuffer,
    LPSTR *lpFilePart
    );

DWORD
WINAPI
GetFullPathNameW(
    LPCWSTR lpFileName,
    DWORD nBufferLength,
    LPWSTR lpBuffer,
    LPWSTR *lpFilePart
    );

DWORD
WINAPI
GetFileType(
	    HANDLE hFile
	    );


DWORD
WINAPI
GetFileSize(
	    HANDLE hFile,
	    LPDWORD lpFileSizeHigh
	    );


BOOL
WINAPI
GetFileTime(
	    HANDLE hFile,
	    LPFILETIME lpCreationTime,
	    LPFILETIME lpLastAccessTime,
	    LPFILETIME lpLastWriteTime
	    );

BOOL
WINAPI
GetHandleInformation(
		     HANDLE hObject,
		     LPDWORD lpdwFlags
		     );

DWORD
WINAPI
GetLastError(VOID);

VOID
WINAPI
GetLocalTime(
	     LPSYSTEMTIME lpSystemTime
	     );

DWORD
WINAPI
GetLogicalDrives(VOID);

DWORD
WINAPI
GetLogicalDriveStringsA(
			DWORD nBufferLength,
			LPSTR lpBuffer
			);

DWORD
WINAPI
GetLogicalDriveStringsW(
			DWORD nBufferLength,
			LPWSTR lpBuffer
			);

BOOL
WINAPI
GetMailslotInfo(
		HANDLE hMailslot,
		LPDWORD lpMaxMessageSize,
		LPDWORD lpNextSize,
		LPDWORD lpMessageCount,
		LPDWORD lpReadTimeout
		);

DWORD
WINAPI
GetModuleFileNameA(
		   HINSTANCE hModule,
		   LPSTR lpFilename,
		   DWORD nSize
		   );

DWORD
WINAPI
GetModuleFileNameW(
		   HINSTANCE hModule,
		   LPWSTR lpFilename,
		   DWORD nSize
		   );

HMODULE
WINAPI
GetModuleHandleA(
		 LPCSTR lpModuleName
		 );

HMODULE
WINAPI
GetModuleHandleW(
		 LPCWSTR lpModuleName
		 );

BOOL
WINAPI
GetNamedPipeHandleStateA(
    HANDLE hNamedPipe,
    LPDWORD lpState,
    LPDWORD lpCurInstances,
    LPDWORD lpMaxCollectionCount,
    LPDWORD lpCollectDataTimeout,
    LPSTR lpUserName,
    DWORD nMaxUserNameSize
    );

BOOL
WINAPI
GetNamedPipeHandleStateW(
    HANDLE hNamedPipe,
    LPDWORD lpState,
    LPDWORD lpCurInstances,
    LPDWORD lpMaxCollectionCount,
    LPDWORD lpCollectDataTimeout,
    LPWSTR lpUserName,
    DWORD nMaxUserNameSize
    );

BOOL
WINAPI
GetNamedPipeInfo(
		 HANDLE hNamedPipe,
		 LPDWORD lpFlags,
		 LPDWORD lpOutBufferSize,
		 LPDWORD lpInBufferSize,
		 LPDWORD lpMaxInstances
		 );

UINT
WINAPI
GetOEMCP(void);

BOOL
WINAPI
GetOverlappedResult(
		    HANDLE hFile,
		    LPOVERLAPPED lpOverlapped,
		    LPDWORD lpNumberOfBytesTransferred,
		    BOOL bWait
		    );

DWORD
WINAPI
GetPriorityClass(
		 HANDLE hProcess
		 );

FARPROC
WINAPI
GetProcAddress(
	       HINSTANCE hModule,
	       LPCSTR lpProcName
	       );

BOOL
WINAPI
GetProcessAffinityMask(
		       HANDLE hProcess,
		       LPDWORD lpProcessAffinityMask,
		       LPDWORD lpSystemAffinityMask
		       );

HANDLE
WINAPI
GetProcessHeap( VOID );


DWORD
WINAPI
GetProcessHeaps(
		DWORD NumberOfHeaps,
		PHANDLE ProcessHeaps
		);

BOOL
WINAPI
GetProcessShutdownParameters(
			     LPDWORD lpdwLevel,
			     LPDWORD lpdwFlags
			     );
		
BOOL
WINAPI
GetProcessTimes(
		HANDLE hProcess,
		LPFILETIME lpCreationTime,
		LPFILETIME lpExitTime,
		LPFILETIME lpKernelTime,
		LPFILETIME lpUserTime
		);


BOOL
WINAPI
GetProcessWorkingSetSize(
			 HANDLE hProcess,
			 LPDWORD lpMinimumWorkingSetSize,
			 LPDWORD lpMaximumWorkingSetSize
			);

UINT
WINAPI
GetPrivateProfileIntA(
    LPCSTR lpAppName,
    LPCSTR lpKeyName,
    INT nDefault,
    LPCSTR lpFileName
    );

UINT
WINAPI
GetPrivateProfileIntW(
    LPCWSTR lpAppName,
    LPCWSTR lpKeyName,
    INT nDefault,
    LPCWSTR lpFileName
    );

DWORD
WINAPI
GetPrivateProfileSectionA(
    LPCSTR lpAppName,
    LPSTR lpReturnedString,
    DWORD nSize,
    LPCSTR lpFileName
    );

DWORD
WINAPI
GetPrivateProfileSectionW(
    LPCWSTR lpAppName,
    LPWSTR lpReturnedString,
    DWORD nSize,
    LPCWSTR lpFileName
    );

DWORD
WINAPI
GetPrivateProfileStringA(
    LPCSTR lpAppName,
    LPCSTR lpKeyName,
    LPCSTR lpDefault,
    LPSTR lpReturnedString,
    DWORD nSize,
    LPCSTR lpFileName
    );

DWORD
WINAPI
GetPrivateProfileStringW(
    LPCWSTR lpAppName,
    LPCWSTR lpKeyName,
    LPCWSTR lpDefault,
    LPWSTR lpReturnedString,
    DWORD nSize,
    LPCWSTR lpFileName
    );

DWORD
WINAPI
GetProfileSectionA(
    LPCSTR lpAppName,
    LPSTR lpReturnedString,
    DWORD nSize
    );

DWORD
WINAPI
GetProfileSectionW(
    LPCWSTR lpAppName,
    LPWSTR lpReturnedString,
    DWORD nSize
    );

UINT
WINAPI
GetProfileIntA(
	       LPCSTR lpAppName,
	       LPCSTR lpKeyName,
	       INT nDefault
	       );

UINT
WINAPI
GetProfileIntW(
	       LPCWSTR lpAppName,
	       LPCWSTR lpKeyName,
	       INT nDefault
	       );

DWORD
WINAPI
GetProfileStringA(
		  LPCSTR lpAppName,
		  LPCSTR lpKeyName,
		  LPCSTR lpDefault,
		  LPSTR lpReturnedString,
		  DWORD nSize
		  );

DWORD
WINAPI
GetProfileStringW(
		  LPCWSTR lpAppName,
		  LPCWSTR lpKeyName,
		  LPCWSTR lpDefault,
		  LPWSTR lpReturnedString,
		  DWORD nSize
		  );

BOOL
WINAPI
GetQueuedCompletionStatus(
			  HANDLE CompletionPort,
			  LPDWORD lpNumberOfBytesTransferred,
			  LPDWORD lpCompletionKey,
			  LPOVERLAPPED *lpOverlapped,
			  DWORD dwMilliseconds
			  );

DWORD
WINAPI
GetShortPathNameA(
		  LPCSTR lpszLongPath,
		  LPSTR  lpszShortPath,
		  DWORD    cchBuffer
		  );

DWORD
WINAPI
GetShortPathNameW(
		  LPCWSTR lpszLongPath,
		  LPWSTR  lpszShortPath,
		  DWORD    cchBuffer
		  );

VOID
WINAPI
GetStartupInfoA(
    LPSTARTUPINFOA lpStartupInfo
    );

VOID
WINAPI
GetStartupInfoW(
    LPSTARTUPINFOW lpStartupInfo
    );

HANDLE
WINAPI
GetStdHandle(
	     DWORD nStdHandle
	     );

UINT
WINAPI
GetSystemDirectoryA(
    LPSTR lpBuffer,
    UINT uSize
    );

UINT
WINAPI
GetSystemDirectoryW(
    LPWSTR lpBuffer,
    UINT uSize
    );

VOID
WINAPI
GetSystemInfo(
	      LPSYSTEM_INFO lpSystemInfo
	      );

VOID
WINAPI
GetSystemTime(
	      LPSYSTEMTIME lpSystemTime
	      );

BOOL
WINAPI
GetSystemTimeAdjustment(
			PDWORD lpTimeAdjustment,
			PDWORD lpTimeIncrement,
			PBOOL  lpTimeAdjustmentDisabled
			);

DWORD
WINAPI
GetTempPathA(
    DWORD nBufferLength,
    LPSTR lpBuffer
    );

DWORD
WINAPI
GetTempPathW(
    DWORD nBufferLength,
    LPWSTR lpBuffer
    );

UINT
WINAPI
GetTempFileNameA(
    LPCSTR lpPathName,
    LPCSTR lpPrefixString,
    UINT uUnique,
    LPSTR lpTempFileName
    );

UINT
WINAPI
GetTempFileNameW(
    LPCWSTR lpPathName,
    LPCWSTR lpPrefixString,
    UINT uUnique,
    LPWSTR lpTempFileName
    );
			
DWORD
WINAPI
GetTickCount(VOID);

DWORD
WINAPI
GetTimeZoneInformation(
		       LPTIME_ZONE_INFORMATION lpTimeZoneInformation
		       );
			
BOOL
WINAPI
GetThreadContext(
		 HANDLE hThread,
		 LPCONTEXT lpContext
		 );

int
WINAPI
GetThreadPriority(
		  HANDLE hThread
		  );

BOOL
WINAPI
GetThreadSelectorEntry(
		       HANDLE hThread,
		       DWORD dwSelector,
		       LPLDT_ENTRY lpSelectorEntry
		       );

BOOL
WINAPI
GetThreadTimes(
	       HANDLE hThread,
	       LPFILETIME lpCreationTime,
	       LPFILETIME lpExitTime,
	       LPFILETIME lpKernelTime,
	       LPFILETIME lpUserTime
	       );

DWORD
WINAPI
GetVersion( VOID );


BOOL
WINAPI
GetVersionExA(
    LPOSVERSIONINFOA lpVersionInformation
    );

BOOL
WINAPI
GetVersionExW(
    LPOSVERSIONINFOW lpVersionInformation
    );

BOOL
WINAPI
GetVolumeInformationA(
    LPCSTR lpRootPathName,
    LPSTR lpVolumeNameBuffer,
    DWORD nVolumeNameSize,
    LPDWORD lpVolumeSerialNumber,
    LPDWORD lpMaximumComponentLength,
    LPDWORD lpFileSystemFlags,
    LPSTR lpFileSystemNameBuffer,
    DWORD nFileSystemNameSize
    );

BOOL
WINAPI
GetVolumeInformationW(
    LPCWSTR lpRootPathName,
    LPWSTR lpVolumeNameBuffer,
    DWORD nVolumeNameSize,
    LPDWORD lpVolumeSerialNumber,
    LPDWORD lpMaximumComponentLength,
    LPDWORD lpFileSystemFlags,
    LPWSTR lpFileSystemNameBuffer,
    DWORD nFileSystemNameSize
    );

UINT
WINAPI
GetWindowsDirectoryA(
    LPSTR lpBuffer,
    UINT uSize
    );

UINT
WINAPI
GetWindowsDirectoryW(
    LPWSTR lpBuffer,
    UINT uSize
    );

ATOM
WINAPI
GlobalAddAtomA(
	       LPCSTR lpString
	       );

ATOM
WINAPI
GlobalAddAtomW(
	       LPCWSTR lpString
	       );

HGLOBAL
WINAPI
GlobalAlloc(
	    UINT uFlags,
	    DWORD dwBytes
	    );

UINT
WINAPI
GlobalCompact(
	      DWORD dwMinFree
	      );

ATOM
WINAPI
GlobalDeleteAtom(
		 ATOM nAtom
		 );

HGLOBAL
GlobalDiscard(
	      HGLOBAL hglbMem
	      );

ATOM
WINAPI
GlobalFindAtomA(
		LPCSTR lpString
		);

ATOM
WINAPI
GlobalFindAtomW(
		LPCWSTR lpString
		);

VOID
WINAPI
GlobalFix(
	  HGLOBAL hMem
	  );

UINT
WINAPI
GlobalFlags(
	    HGLOBAL hMem
	    );

HGLOBAL
WINAPI
GlobalFree(
	   HGLOBAL hMem
	   );

UINT
WINAPI
GlobalGetAtomNameA(
		   ATOM nAtom,
		   LPSTR lpBuffer,
		   int nSize
		   );

UINT
WINAPI
GlobalGetAtomNameW(
		   ATOM nAtom,
		   LPWSTR lpBuffer,
		   int nSize
		   );

HGLOBAL
WINAPI
GlobalHandle(
	     LPCVOID pMem
	     );

VOID
WINAPI
GlobalMemoryStatus(
		   LPMEMORYSTATUS lpBuffer
		   );

LPVOID
WINAPI
GlobalLock(
	   HGLOBAL hMem
	   );

HGLOBAL
WINAPI
GlobalReAlloc(
	      HGLOBAL hMem,
	      DWORD dwBytes,
	      UINT uFlags
	      );

DWORD
WINAPI
GlobalSize(
	   HGLOBAL hMem
	   );

VOID
WINAPI
GlobalUnfix(
	    HGLOBAL hMem
	    );

BOOL
WINAPI
GlobalUnlock(
	     HGLOBAL hMem
	     );

BOOL
WINAPI
GlobalUnWire(
	     HGLOBAL hMem
	     );

LPVOID
WINAPI
GlobalWire(
	   HGLOBAL hMem
	   );

LPVOID
WINAPI
HeapAlloc(
	  HANDLE hHeap,
	  DWORD dwFlags,
	  DWORD dwBytes
	  );

HANDLE
WINAPI
HeapCreate(
	   DWORD flOptions,
	   DWORD dwInitialSize,
	   DWORD dwMaximumSize
	   );

UINT
WINAPI
HeapCompact(
	    HANDLE hHeap,
	    DWORD dwFlags
	    );

BOOL
WINAPI
HeapDestroy(
	    HANDLE hHeap
	    );

BOOL
WINAPI
HeapFree(
	 HANDLE hHeap,
	 DWORD dwFlags,
	 LPVOID lpMem
	 );

BOOL
WINAPI
HeapLock(
	 HANDLE hHeap
	 );

LPVOID
WINAPI
HeapReAlloc(
	    HANDLE hHeap,
	    DWORD dwFlags,
	    LPVOID lpMem,
	    DWORD dwBytes
	    );

DWORD
WINAPI
HeapSize(
	 HANDLE hHeap,
	 DWORD	dwFlags,
	 LPCVOID lpMem
	 );

BOOL
WINAPI
HeapUnlock(
	   HANDLE hHeap
	   );


BOOL
WINAPI
HeapValidate(
	     HANDLE hHeap,
	     DWORD dwFlags,
	     LPCVOID lpMem
	     );

BOOL
WINAPI
HeapWalk(
	 HANDLE hHeap,
	 LPPROCESS_HEAP_ENTRY lpEntry
	 );
	 
BOOL
WINAPI
InitAtomTable(
	      DWORD nSize
	      );

VOID
WINAPI
InitializeCriticalSection(
			  LPCRITICAL_SECTION lpCriticalSection
			  );

LONG
WINAPI
InterlockedDecrement(
		     LPLONG lpAddend
		     );

LONG
WINAPI
InterlockedExchange(
		    LPLONG Target,
		    LONG Value
		    );

LONG
WINAPI
InterlockedIncrement(
		     LPLONG lpAddend
		     );

BOOL
WINAPI
IsBadCodePtr(
	     FARPROC lpfn
	     );

BOOL
WINAPI
IsBadHugeReadPtr(
		 CONST VOID *lp,
		 UINT ucb
		 );


BOOL
WINAPI
IsBadHugeWritePtr(
		  LPVOID lp,
		  UINT ucb
		  );

BOOL
WINAPI
IsBadReadPtr(
	     CONST VOID *lp,
	     UINT ucb
	     );

BOOL
WINAPI
IsBadStringPtrA(
    LPCSTR lpsz,
    UINT ucchMax
    );

BOOL
WINAPI
IsBadStringPtrW(
    LPCWSTR lpsz,
    UINT ucchMax
    );

BOOL
WINAPI
IsBadWritePtr(
	      LPVOID lp,
	      UINT ucb
	      );

BOOL
WINAPI
IsTextUnicode(
	      CONST LPVOID lpBuffer,
	      int cb,
	      LPINT lpi
	      );	      
	      
VOID
WINAPI
LeaveCriticalSection(
		     LPCRITICAL_SECTION lpCriticalSection
		     );

HMODULE
WINAPI
LoadLibraryA(
	     LPCSTR lpLibFileName
	     );

HMODULE
WINAPI
LoadLibraryW(
	     LPCWSTR lpLibFileName
	     );

HMODULE
WINAPI
LoadLibraryExA(
	       LPCSTR lpLibFileName,
	       HANDLE hFile,
	       DWORD dwFlags
	       );

HMODULE
WINAPI
LoadLibraryExW(
	       LPCWSTR lpLibFileName,
	       HANDLE hFile,
	       DWORD dwFlags
	       );

DWORD
WINAPI
LoadModule(
	   LPCSTR lpModuleName,
	   LPVOID lpParameterBlock
	   );

HGLOBAL
WINAPI
LoadResource(
	     HINSTANCE hModule,
	     HRSRC hResInfo
	     );

HLOCAL
WINAPI
LocalAlloc(
	   UINT uFlags,
	   UINT uBytes
	   );

UINT
WINAPI
LocalCompact(
	     UINT uMinFree
	     );

HLOCAL
LocalDiscard(
	     HLOCAL hlocMem
	     );

BOOL
WINAPI
LocalFileTimeToFileTime(
			CONST FILETIME *lpLocalFileTime,
			LPFILETIME lpFileTime
			);

UINT
WINAPI
LocalFlags(
	   HLOCAL hMem
	   );


HLOCAL
WINAPI
LocalFree(
	  HLOCAL hMem
	  );

HLOCAL
WINAPI
LocalHandle(
	    LPCVOID pMem
	    );

LPVOID
WINAPI
LocalLock(
	  HLOCAL hMem
	  );

HLOCAL
WINAPI
LocalReAlloc(
	     HLOCAL hMem,
	     UINT uBytes,
	     UINT uFlags
	     );

UINT
WINAPI
LocalShrink(
	    HLOCAL hMem,
	    UINT cbNewSize
	    );

UINT
WINAPI
LocalSize(
	  HLOCAL hMem
	  );

BOOL
WINAPI
LocalUnlock(
	    HLOCAL hMem
	    );

BOOL
WINAPI
LockFile(
	 HANDLE hFile,
	 DWORD dwFileOffsetLow,
	 DWORD dwFileOffsetHigh,
	 DWORD nNumberOfBytesToLockLow,
	 DWORD nNumberOfBytesToLockHigh
	 );

BOOL
WINAPI
LockFileEx(
	   HANDLE hFile,
	   DWORD dwFlags,
	   DWORD dwReserved,
	   DWORD nNumberOfBytesToLockLow,
	   DWORD nNumberOfBytesToLockHigh,
	   LPOVERLAPPED lpOverlapped
	   );

LPVOID
WINAPI
LockResource(
	     HGLOBAL hResData
	     );

LPVOID
WINAPI
MapViewOfFile(
	      HANDLE hFileMappingObject,
	      DWORD dwDesiredAccess,
	      DWORD dwFileOffsetHigh,
	      DWORD dwFileOffsetLow,
	      DWORD dwNumberOfBytesToMap
	      );

LPVOID
WINAPI
MapViewOfFileEx(
		HANDLE hFileMappingObject,
		DWORD dwDesiredAccess,
		DWORD dwFileOffsetHigh,
		DWORD dwFileOffsetLow,
		DWORD dwNumberOfBytesToMap,
		LPVOID lpBaseAddress
		);

BOOL
WINAPI
MoveFileA(
    LPCSTR lpExistingFileName,
    LPCSTR lpNewFileName
    );

BOOL
WINAPI
MoveFileW(
    LPCWSTR lpExistingFileName,
    LPCWSTR lpNewFileName
    );

BOOL
WINAPI
MoveFileExA(
    LPCSTR lpExistingFileName,
    LPCSTR lpNewFileName,
    DWORD dwFlags
    );

BOOL
WINAPI
MoveFileExW(
    LPCWSTR lpExistingFileName,
    LPCWSTR lpNewFileName,
    DWORD dwFlags
    );

VOID
WINAPI
MoveMemory (
	    PVOID Destination,
	    CONST VOID *Source,
	    DWORD Length
	     );

int
WINAPI
MulDiv(
       int nNumber,
       int nNumerator,
       int nDenominator
       );

HANDLE
WINAPI
OpenEventA(
	   DWORD dwDesiredAccess,
	   BOOL bInheritHandle,
	   LPCSTR lpName
	   );
HANDLE
WINAPI
OpenEventW(
	   DWORD dwDesiredAccess,
	   BOOL bInheritHandle,
	   LPCWSTR lpName
	   );

HANDLE
WINAPI
OpenFile(
	 LPCSTR lpFileName,
	 LPOFSTRUCT lpReOpenBuff,
	 UINT uStyle
	 );

HANDLE
WINAPI
OpenFileMappingA(
		 DWORD dwDesiredAccess,
		 BOOL bInheritHandle,
		 LPCSTR lpName
		 );

HANDLE
WINAPI
OpenFileMappingW(
		 DWORD dwDesiredAccess,
		 BOOL bInheritHandle,
		 LPCWSTR lpName
		 );

HANDLE
WINAPI
OpenMutexA(
	   DWORD dwDesiredAccess,
	   BOOL bInheritHandle,
	   LPCSTR lpName
	   );

HANDLE
WINAPI
OpenMutexW(
	   DWORD dwDesiredAccess,
	   BOOL bInheritHandle,
	   LPCWSTR lpName
	   );
	   	 
HANDLE
WINAPI
OpenProcess(
	    DWORD dwDesiredAccess,
	    BOOL bInheritHandle,
	    DWORD dwProcessId
	    );
HANDLE
WINAPI
OpenSemaphoreA(
	       DWORD dwDesiredAccess,
	       BOOL bInheritHandle,
	       LPCSTR lpName
	       );

HANDLE
WINAPI
OpenSemaphoreW(
	       DWORD dwDesiredAccess,
	       BOOL bInheritHandle,
	       LPCWSTR lpName
	       );	    
	       
VOID
WINAPI
OutputDebugStringA(
		   LPCSTR lpOutputString
		   );

VOID
WINAPI
OutputDebugStringW(
		   LPWSTR lpOutputString
		   );

BOOL
WINAPI
PeekNamedPipe(
	      HANDLE hNamedPipe,
	      LPVOID lpBuffer,
	      DWORD nBufferSize,
	      LPDWORD lpBytesRead,
	      LPDWORD lpTotalBytesAvail,
	      LPDWORD lpBytesLeftThisMessage
	      );

BOOL
WINAPI
PulseEvent(
	   HANDLE hEvent
	   );

DWORD
WINAPI
QueryDosDeviceA(
    LPCSTR lpDeviceName,
    LPSTR lpTargetPath,
    DWORD ucchMax
    );

DWORD
WINAPI
QueryDosDeviceW(
    LPCWSTR lpDeviceName,
    LPWSTR lpTargetPath,
    DWORD ucchMax
    );

BOOL
WINAPI
QueryPerformanceCounter(
			LARGE_INTEGER *lpPerformanceCount
			);

BOOL
WINAPI
QueryPerformanceFrequency(
			  LARGE_INTEGER *lpFrequency
			  );

VOID
WINAPI
RaiseException(
	       DWORD dwExceptionCode,
	       DWORD dwExceptionFlags,
	       DWORD nNumberOfArguments,
	       CONST DWORD *lpArguments
	       );

BOOL
WINAPI
ReadFile(
	 HANDLE hFile,
	 LPVOID lpBuffer,
	 DWORD nNumberOfBytesToRead,
	 LPDWORD lpNumberOfBytesRead,
	 LPOVERLAPPED lpOverlapped
	 );

BOOL
WINAPI
ReadFileEx(
	   HANDLE hFile,
	   LPVOID lpBuffer,
	   DWORD nNumberOfBytesToRead,
	   LPOVERLAPPED lpOverlapped,
	   LPOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine
	   );

BOOL
WINAPI
ReadProcessMemory(
		  HANDLE hProcess,
		  LPCVOID lpBaseAddress,
		  LPVOID lpBuffer,
		  DWORD nSize,
		  LPDWORD lpNumberOfBytesRead
		  );

BOOL
WINAPI
ReleaseMutex(
	     HANDLE hMutex
	     );

BOOL
WINAPI
ReleaseSemaphore(
		 HANDLE hSemaphore,
		 LONG lReleaseCount,
		 LPLONG lpPreviousCount
		 );

BOOL
WINAPI
RemoveDirectoryA(
    LPCSTR lpPathName
    );

BOOL
WINAPI
RemoveDirectoryW(
    LPCWSTR lpPathName
    );

BOOL
WINAPI
ResetEvent(
	   HANDLE hEvent
	   );

DWORD
WINAPI
ResumeThread(
	     HANDLE hThread
	     );

DWORD
WINAPI
SearchPathA(
    LPCSTR lpPath,
    LPCSTR lpFileName,
    LPCSTR lpExtension,
    DWORD nBufferLength,
    LPSTR lpBuffer,
    LPSTR *lpFilePart
    );

DWORD
WINAPI
SearchPathW(
    LPCWSTR lpPath,
    LPCWSTR lpFileName,
    LPCWSTR lpExtension,
    DWORD   nBufferLength,
    LPWSTR  lpBuffer,
    LPWSTR *lpFilePart
    );
  
BOOL
WINAPI
SetComputerNameA (
    LPCSTR lpComputerName
    );

BOOL
WINAPI
SetComputerNameW (
    LPCWSTR lpComputerName
    );

BOOL
WINAPI
SetCurrentDirectoryA(
    LPCSTR lpPathName
    );

BOOL
WINAPI
SetCurrentDirectoryW(
    LPCWSTR lpPathName
    );

BOOL
WINAPI
SetEndOfFile(
	     HANDLE hFile
	     );

BOOL
WINAPI
SetEnvironmentVariableA(
			LPCSTR lpName,
			LPCSTR lpValue
			);

BOOL
WINAPI
SetEnvironmentVariableW(
			LPCWSTR lpName,
			LPCWSTR lpValue
			);

UINT
WINAPI
SetErrorMode(
	     UINT uMode
	     );

BOOL
WINAPI
SetEvent(
	 HANDLE hEvent
	 );

BOOL
WINAPI
SetFileAttributesA(
    LPCSTR lpFileName,
    DWORD dwFileAttributes
    );

BOOL
WINAPI
SetFileAttributesW(
    LPCWSTR lpFileName,
    DWORD dwFileAttributes
    );

UINT
WINAPI
SetHandleCount(
	       UINT uNumber
	       );

BOOL
WINAPI
SetHandleInformation(
		     HANDLE hObject,
		     DWORD dwMask,
		     DWORD dwFlags
		     );

VOID
WINAPI
SetFileApisToANSI( VOID );

VOID
WINAPI
SetFileApisToOEM( VOID );

DWORD
WINAPI
SetFilePointer(
	       HANDLE hFile,
	       LONG lDistanceToMove,
	       PLONG lpDistanceToMoveHigh,
	       DWORD dwMoveMethod
	       );

BOOL
WINAPI
SetFileTime(
	    HANDLE hFile,
	    CONST FILETIME *lpCreationTime,
	    CONST FILETIME *lpLastAccessTime,
	    CONST FILETIME *lpLastWriteTime
	    );
		     		     
VOID
WINAPI
SetLastError(
	     DWORD dwErrCode
	     );

BOOL
WINAPI
SetLocalTime(
	     CONST SYSTEMTIME *lpSystemTime
	     );

BOOL
WINAPI
SetMailslotInfo(
		HANDLE hMailslot,
		DWORD lReadTimeout
		);

BOOL
WINAPI
SetNamedPipeHandleState(
			HANDLE hNamedPipe,
			LPDWORD lpMode,
			LPDWORD lpMaxCollectionCount,
			LPDWORD lpCollectDataTimeout
			);

BOOL
WINAPI
SetPriorityClass(
		 HANDLE hProcess,
		 DWORD dwPriorityClass
		 );

BOOL
WINAPI
SetProcessShutdownParameters(
			     DWORD dwLevel,
			     DWORD dwFlags
			     );

BOOL
WINAPI
SetProcessWorkingSetSize(
			 HANDLE hProcess,
			 DWORD dwMinimumWorkingSetSize,
			 DWORD dwMaximumWorkingSetSize
			 );

BOOL
WINAPI
SetStdHandle(
	     DWORD nStdHandle,
	     HANDLE hHandle
	     );

BOOL
WINAPI
SetSystemTime(
	      CONST SYSTEMTIME *lpSystemTime
	      );

BOOL
WINAPI
SetSystemTimeAdjustment(
			DWORD dwTimeAdjustment,
			BOOL  bTimeAdjustmentDisabled
			);

BOOL
WINAPI
SetTimeZoneInformation(
		       CONST TIME_ZONE_INFORMATION *lpTimeZoneInformation
		       );
 	
DWORD
WINAPI
SetThreadAffinityMask(
		      HANDLE hThread,
		      DWORD dwThreadAffinityMask
		      );

BOOL
WINAPI
SetThreadContext(
		 HANDLE hThread,
		 CONST CONTEXT *lpContext
		 );

BOOL
WINAPI
SetThreadPriority(
		  HANDLE hThread,
		  int nPriority
		  );
		  
LPTOP_LEVEL_EXCEPTION_FILTER
WINAPI
SetUnhandledExceptionFilter(
	LPTOP_LEVEL_EXCEPTION_FILTER lpTopLevelExceptionFilter
	);

BOOL
WINAPI
SetVolumeLabelA(
    LPCSTR lpRootPathName,
    LPCSTR lpVolumeName
    );

BOOL
WINAPI
SetVolumeLabelW(
    LPCWSTR lpRootPathName,
    LPCWSTR lpVolumeName
    );

DWORD
WINAPI
SizeofResource(
	       HINSTANCE hModule,
	       HRSRC hResInfo
	       );

VOID
WINAPI
Sleep(
      DWORD dwMilliseconds
      );

DWORD
WINAPI
SleepEx(
	DWORD dwMilliseconds,
	BOOL bAlertable
	);

DWORD
WINAPI
SuspendThread(
	      HANDLE hThread
	      );

BOOL
WINAPI
SystemTimeToFileTime(
		     CONST SYSTEMTIME *lpSystemTime,
		     LPFILETIME lpFileTime
		     );

BOOL
WINAPI
SystemTimeToTzSpecificLocalTime(
				LPTIME_ZONE_INFORMATION lpTimeZoneInformation,
				LPSYSTEMTIME lpUniversalTime,
				LPSYSTEMTIME lpLocalTime
				);
				
BOOL
WINAPI
TerminateProcess(
		 HANDLE hProcess,
		 UINT uExitCode
		 );

BOOL
WINAPI
TerminateThread(
		HANDLE hThread,
		DWORD dwExitCode
		);

DWORD
WINAPI
TlsAlloc( VOID );

BOOL
WINAPI
TlsFree(
	DWORD dwTlsIndex
	);

LPVOID
WINAPI
TlsGetValue(
	    DWORD dwTlsIndex
	    );


BOOL
WINAPI
TlsSetValue(
	    DWORD dwTlsIndex,
	    LPVOID lpTlsValue
	    );

BOOL
WINAPI
TransactNamedPipe(
		  HANDLE hNamedPipe,
		  LPVOID lpInBuffer,
		  DWORD nInBufferSize,
		  LPVOID lpOutBuffer,
		  DWORD nOutBufferSize,
		  LPDWORD lpBytesRead,
		  LPOVERLAPPED lpOverlapped
		  );
		  	     
LONG
WINAPI
UnhandledExceptionFilter(
			 LPEXCEPTION_POINTERS pExceptionInfo
			 );

BOOL
WINAPI
UnlockFile(
	   HANDLE hFile,
	   DWORD dwFileOffsetLow,
	   DWORD dwFileOffsetHigh,
	   DWORD nNumberOfBytesToUnlockLow,
	   DWORD nNumberOfBytesToUnlockHigh
	   );

BOOL
WINAPI
UnlockFileEx(
	     HANDLE hFile,
	     DWORD dwReserved,
	     DWORD nNumberOfBytesToUnlockLow,
	     DWORD nNumberOfBytesToUnlockHigh,
	     LPOVERLAPPED lpOverlapped
	     );

BOOL
WINAPI
UnmapViewOfFile(
		LPVOID lpBaseAddress
		);

BOOL
WINAPI
UpdateResourceA(
		HANDLE      hUpdate,
		LPCSTR     lpType,
		LPCSTR     lpName,
		WORD        wLanguage,
		LPVOID      lpData,
		DWORD       cbData
		);

BOOL
WINAPI
UpdateResourceW(
		HANDLE      hUpdate,
		LPCWSTR     lpType,
		LPCWSTR     lpName,
		WORD        wLanguage,
		LPVOID      lpData,
		DWORD       cbData
		);		

LPVOID
WINAPI
VirtualAlloc(
	     LPVOID lpAddress,
	     DWORD dwSize,
	     DWORD flAllocationType,
	     DWORD flProtect
	     );


BOOL
WINAPI
VirtualFree(
	    LPVOID lpAddress,
	    DWORD dwSize,
	    DWORD dwFreeType
	    );

BOOL
WINAPI
VirtualLock(
	    LPVOID lpAddress,
	    DWORD dwSize
	    );



BOOL
WINAPI
VirtualProtect(
	       LPVOID lpAddress,
	       DWORD dwSize,
	       DWORD flNewProtect,
	       PDWORD lpflOldProtect
	       );

BOOL
WINAPI
VirtualProtectEx(
		 HANDLE hProcess,
		 LPVOID lpAddress,
		 DWORD dwSize,
		 DWORD flNewProtect,
		 PDWORD lpflOldProtect
		 );
		 
DWORD
WINAPI
VirtualQuery(
	     LPCVOID lpAddress,
	     PMEMORY_BASIC_INFORMATION lpBuffer,
	     DWORD dwLength
	     );

DWORD
WINAPI
VirtualQueryEx(
	       HANDLE hProcess,
	       LPCVOID lpAddress,
	       PMEMORY_BASIC_INFORMATION lpBuffer,
	       DWORD dwLength
	       );

BOOL
WINAPI
VirtualUnlock(
	      LPVOID lpAddress,
	      DWORD dwSize
	      );

BOOL
WINAPI
WaitForDebugEvent(
		  LPDEBUG_EVENT lpDebugEvent,
		  DWORD dwMilliseconds
		  );

DWORD
WINAPI
WaitForMultipleObjects(
		       DWORD nCount,
		       CONST HANDLE *lpHandles,
		       BOOL bWaitAll,
		       DWORD dwMilliseconds
		       );

DWORD
WINAPI
WaitForMultipleObjectsEx(
			 DWORD nCount,
			 CONST HANDLE *lpHandles,
			 BOOL bWaitAll,
			 DWORD dwMilliseconds,
			 BOOL bAlertable
			 );

DWORD
WINAPI
WaitForSingleObject(
		    HANDLE hHandle,
		    DWORD dwMilliseconds
		    );

DWORD
WINAPI
WaitForSingleObjectEx(
		      HANDLE hHandle,
		      DWORD dwMilliseconds,
		      BOOL bAlertable
		      );

BOOL
WINAPI
WaitNamedPipeA(
    LPCSTR lpNamedPipeName,
    DWORD nTimeOut
    );

BOOL
WINAPI
WaitNamedPipeW(
    LPCWSTR lpNamedPipeName,
    DWORD nTimeOut
    );

UINT
WINAPI
WinExec(
	LPCSTR lpCmdLine,
	UINT uCmdShow
	);

int
WINAPI
WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPSTR lpCmdLine,
	int nShowCmd
	);

BOOL
WINAPI
WriteFile(
	  HANDLE hFile,
	  LPCVOID lpBuffer,
	  DWORD nNumberOfBytesToWrite,
	  LPDWORD lpNumberOfBytesWritten,
	  LPOVERLAPPED lpOverlapped
	  );

BOOL
WINAPI
WriteFileEx(
	    HANDLE hFile,
	    LPCVOID lpBuffer,
	    DWORD nNumberOfBytesToWrite,
	    LPOVERLAPPED lpOverlapped,
	    LPOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine
	    );

BOOL
WINAPI
WritePrivateProfileSectionA(
    LPCSTR lpAppName,
    LPCSTR lpString,
    LPCSTR lpFileName
    );

BOOL
WINAPI
WritePrivateProfileSectionW(
    LPCWSTR lpAppName,
    LPCWSTR lpString,
    LPCWSTR lpFileName
    );

BOOL
WINAPI
WritePrivateProfileStringA(
    LPCSTR lpAppName,
    LPCSTR lpKeyName,
    LPCSTR lpString,
    LPCSTR lpFileName
    );

BOOL
WINAPI
WritePrivateProfileStringW(
    LPCWSTR lpAppName,
    LPCWSTR lpKeyName,
    LPCWSTR lpString,
    LPCWSTR lpFileName
    );

BOOL
WINAPI
WriteProcessMemory(
		   HANDLE hProcess,
		   LPVOID lpBaseAddress,
		   LPVOID lpBuffer,
		   DWORD nSize,
		   LPDWORD lpNumberOfBytesWritten
		   );

BOOL
WINAPI
WriteProfileSectionA(
    LPCSTR lpAppName,
    LPCSTR lpString
    );

BOOL
WINAPI
WriteProfileSectionW(
    LPCWSTR lpAppName,
    LPCWSTR lpString
    );

BOOL
WINAPI
WriteProfileStringA(
    LPCSTR lpAppName,
    LPCSTR lpKeyName,
    LPCSTR lpString
    );

BOOL
WINAPI
WriteProfileStringW(
    LPCWSTR lpAppName,
    LPCWSTR lpKeyName,
    LPCWSTR lpString
    );

VOID
WINAPI
ZeroMemory (
	    PVOID Destination,
	    DWORD Length
	     );

long
WINAPI
_hread(
       HANDLE hFile,
       LPVOID lpBuffer,
       long lBytes
       );


long
WINAPI
_hwrite(
	HANDLE hFile,
	LPCSTR lpBuffer,
	long lBytes
	);

HANDLE
WINAPI
_lclose(
	HANDLE hFile
	);

HANDLE
WINAPI
_lcreat(
	LPCSTR lpPathName,
	int  iAttribute
	);

HANDLE
WINAPI
_lopen(
       LPCSTR lpPathName,
       int iReadWrite
       );

UINT
WINAPI
_lread(
       HANDLE hFile,
       LPVOID lpBuffer,
       UINT uBytes
       );

LONG
WINAPI
_llseek(
	HANDLE hFile,
	LONG lOffset,
	int iOrigin
	);

UINT
WINAPI
_lwrite(
	HANDLE hFile,
	LPCSTR lpBuffer,
	UINT uBytes
	);

LPSTR
WINAPI
lstrcatA(
	 LPSTR lpString1,
	 LPCSTR lpString2
	 );

LPWSTR
WINAPI
lstrcatW(
	 LPWSTR lpString1,
	 LPCWSTR lpString2
	 );

int
WINAPI
lstrcmpA(
	 LPCSTR lpString1,
	 LPCSTR lpString2
	 );

int
WINAPI
lstrcmpW(
	 LPCWSTR lpString1,
	 LPCWSTR lpString2
	 );

int
WINAPI
lstrcmpiA(
	  LPCSTR lpString1,
	  LPCSTR lpString2
	  );

int
WINAPI
lstrcmpiW(
	  LPCWSTR lpString1,
	  LPCWSTR lpString2
	  );

LPSTR
WINAPI
lstrcpynA(
	  LPSTR lpString1,
	  LPCSTR lpString2,
	  int iMaxLength
	  );

LPWSTR
WINAPI
lstrcpynW(
	  LPWSTR lpString1,
	  LPCWSTR lpString2,
	  int iMaxLength
	  );

LPSTR
WINAPI
lstrcpyA(
	 LPSTR lpString1,
	 LPCSTR lpString2
	 );

LPWSTR
WINAPI
lstrcpyW(
	 LPWSTR lpString1,
	 LPCWSTR lpString2
	 );

int
WINAPI
lstrlenA(
	 LPCSTR lpString
	 );

int
WINAPI
lstrlenW(
	 LPCWSTR lpString
	 );

#ifndef UNICODE
#define AddAtom				AddAtomA
#define BeginUpdateResource		BeginUpdateResourceA
#define CallNamedPipe			CallNamedPipeA
#define CopyFile			CopyFileA
#define CreateDirectory			CreateDirectoryA
#define CreateDirectoryEx		CreateDirectoryExA
#define CreateEvent			CreateEventA
#define CreateFile			CreateFileA
#define CreateFileMapping		CreateFileMappingA
#define CreateMailslot			CreateMailslotA
#define CreateMutex			CreateMutexA
#define CreateNamedPipe			CreateNamedPipeA
#define CreateProcess			CreateProcessA
#define CreateSemaphore			CreateSemaphoreA
#define DefineDosDevice			DefineDosDeviceA
#define DeleteFile			DeleteFileA
#define EndUpdateResource		EndUpdateResourceA
#define EnumResourceLanguages		EnumResourceLanguagesA
#define EnumResourceNames		EnumResourceNamesA
#define EnumResourceTypes		EnumResourceTypesA
#define ExpandEnvironmentStrings	ExpandEnvironmentStringsA
#define FatalAppExit			FatalAppExitA
#define FindAtom			FindAtomA
#define FindFirstChangeNotification	FindFirstChangeNotificationA
#define FindNextFile			FindNextFileA
#define FindResource			FindResourceA
#define FindResourceEx			FindResourceExA
#define FormatMessage			FormatMessageA
#define FreeEnvironmentStrings		FreeEnvironmentStringsA
#define GetAtomName			GetAtomNameA
#define GetBinaryType			GetBinaryTypeA
#define GetCommandLine			GetCommandLineA
#define GetCompressedFileSize		GetCompressedFileSizeA
#define GetComputerName			GetComputerNameA
#define GetCurrentDirectory		GetCurrentDirectoryA
#define GetDiskFreeSpace		GetDiskFreeSpaceA
#define GetDriveType			GetDriveTypeA
#define GetEnvironmentStrings		GetEnvironmentStringsA
#define GetEnvironmentVariable		GetEnvironmentVariableA
#define GetFileAttributes		GetFileAttributesA
#define GetFullPathName			GetFullPathNameA
#define GetLogicalDriveStrings		GetLogicalDriveStringsA
#define GetModuleFileName		GetModuleFileNameA
#define GetModuleHandle			GetModuleHandleA
#define GetNamedPipeHandleState		GetNamedPipeHandleStateA
#define GetPrivateProfileInt		GetPrivateProfileIntA
#define GetPrivateProfileSection	GetPrivateProfileSectionA
#define GetPrivateProfileString		GetPrivateProfileStringA
#define GetProfileInt			GetProfileIntA
#define GetProfileSection		GetProfileSectionA
#define GetProfileString		GetProfileStringA
#define GetShortPathName		GetShortPathNameA
#define GetStartupInfo			GetStartupInfoA
#define GetSystemDirectory		GetSystemDirectoryA
#define GetTempPath			GetTempPathA
#define GetTempFileName			GetTempFileNameA
#define GetVolumeInformation		GetVolumeInformationA
#define GetWindowsDirectory		GetWindowsDirectoryA
#define GlobalAddAtom			GlobalAddAtomA
#define GlobalFindAtom			GlobalFindAtomA
#define GlobalGetAtomName		GlobalGetAtomNameA
#define LoadLibrary			LoadLibraryA
#define LoadLibraryEx			LoadLibraryExA
#define MoveFile			MoveFileA
#define MoveFileEx			MoveFileExA
#define OpenEvent  			OpenEventA
#define OpenFileMapping			OpenFileMappingA
#define OpenMutex			OpenMutexA
#define OpenSemaphore			OpenSemaphoreA
#define OutputDebugString		OutputDebugStringA
#define QueryDosDevice			QueryDosDeviceA
#define RemoveDirectory			RemoveDirectoryA
#define SearchPath			SearchPathA
#define SetComputerName			SetComputerNameA
#define SetCurrentDirectory		SetCurrentDirectoryA
#define SetEnvironmentVariable		SetEnvironmentVariableA
#define SetFileAttributes		SetFileAttributesA
#define SetVolumeLabel  		SetVolumeLabelA
#define UpdateResource			UpdateResourceA
#define WaitNamedPipe			WaitNamedPipeA
#define WritePrivateProfileString	WritePrivateProfileStringA
#define WritePrivateProfileSection	WritePrivateProfileSectionA
#define WriteProfileSection		WriteProfileSectionA
#define WriteProfileString		WriteProfileStringA
#define lstrcat				lstrcatA
#define lstrcmp				lstrcmpA
#define lstrcmpi			lstrcmpiA
#define lstrcpy				lstrcpyA
#define lstrcpyn			lstrcpynA
#define lstrlen				lstrlenA
#else
#define AddAtom				AddAtomW
#define BeginUpdateResource		BeginUpdateResourceW
#define CallNamedPipe			CallNamedPipeW
#define CopyFile			CopyFileW
#define CreateDirectory			CreateDirectoryW
#define CreateDirectoryEx		CreateDirectoryExW
#define CreateEvent			CreateEventW
#define CreateFile			CreateFileW
#define CreateFileMapping		CreateFileMappingW
#define CreateMailslot			CreateMailslotW
#define CreateMutex			CreateMutexW
#define CreateNamedPipe			CreateNamedPipeW
#define CreateProcess			CreateProcessW
#define CreateSemaphore			CreateSemaphoreW
#define DefineDosDevice			DefineDosDeviceW
#define DeleteFile			DeleteFileW
#define EndUpdateResource		EndUpdateResourceW
#define EnumResourceLanguages		EnumResourceLanguagesW
#define EnumResourceNames		EnumResourceNamesW
#define EnumResourceTypes		EnumResourceTypesW
#define ExpandEnvironmentStrings	ExpandEnvironmentStringsW
#define FatalAppExit			FatalAppExitW
#define FindAtom			FindAtomW
#define FindFirstChangeNotification	FindFirstChangeNotificationW
#define FindNextFile			FindNextFileW
#define FindResource			FindResourceW
#define FindResourceEx			FindResourceExW
#define FormatMessage			FormatMessageW
#define FreeEnvironmentStrings		FreeEnvironmentStringsW
#define GetAtomName			GetAtomNameW
#define GetBinaryType			GetBinaryTypeW
#define GetCommandLine			GetCommandLineW
#define GetCompressedFileSize		GetCompressedFileSizeW
#define GetComputerName			GetComputerNameW
#define GetCurrentDirectory		GetCurrentDirectoryW
#define GetDiskFreeSpace		GetDiskFreeSpaceW
#define GetDriveType			GetDriveTypeW
#define GetEnvironmentStrings		GetEnvironmentStringsW
#define GetEnvironmentVariable		GetEnvironmentVariableW
#define GetFileAttributes		GetFileAttributesW
#define GetFullPathName			GetFullPathNameW
#define GetLogicalDriveStrings		GetLogicalDriveStringsW
#define GetModuleFileName		GetModuleFileNameW
#define GetModuleHandle			GetModuleHandleW
#define GetNamedPipeHandleState		GetNamedPipeHandleStateW
#define GetPrivateProfileInt		GetPrivateProfileIntW
#define GetPrivateProfileSection	GetPrivateProfileSectionW
#define GetPrivateProfileString		GetPrivateProfileStringW
#define GetProfileInt			GetProfileIntW
#define GetProfileSection		GetProfileSectionW
#define GetProfileString		GetProfileStringW
#define GetShortPathName		GetShortPathNameW
#define GetStartupInfo			GetStartupInfoW
#define GetSystemDirectory		GetSystemDirectoryW
#define GetTempPath			GetTempPathW
#define GetTempFileName			GetTempFileNameW
#define GetVolumeInformation		GetVolumeInformationW
#define GetWindowsDirectory		GetWindowsDirectoryW
#define GlobalAddAtom			GlobalAddAtomW
#define GlobalFindAtom			GlobalFindAtomW
#define GlobalGetAtomName		GlobalGetAtomNameW
#define LoadLibrary			LoadLibraryW
#define LoadLibraryEx			LoadLibraryExW
#define MoveFile			MoveFileW
#define MoveFileEx			MoveFileExW
#define OpenEvent  			OpenEventW
#define OpenFileMapping			OpenFileMappingW
#define OpenMutex			OpenMutexW
#define OpenSemaphore			OpenSemaphoreW
#define OutputDebugString		OutputDebugStringW
#define QueryDosDevice			QueryDosDeviceW
#define RemoveDirectory			RemoveDirectoryW
#define SearchPath			SearchPathW
#define SetComputerName			SetComputerNameW
#define SetCurrentDirectory		SetCurrentDirectoryW
#define SetEnvironmentVariable		SetEnvironmentVariableW
#define SetFileAttributes		SetFileAttributesW
#define SetVolumeLabel  		SetVolumeLabelW
#define UpdateResource			UpdateResourceW
#define WaitNamedPipe			WaitNamedPipeW
#define WritePrivateProfileString	WritePrivateProfileStringW
#define WritePrivateProfileSection	WritePrivateProfileSectionW
#define WriteProfileSection		WriteProfileSectionW
#define WriteProfileString		WriteProfileStringW
#define lstrcat				lstrcatW
#define lstrcmp				lstrcmpW
#define lstrcmpi			lstrcmpiW
#define lstrcpy				lstrcpyW 
#define lstrcpyn			lstrcpynW
#define lstrlen				lstrlenW
#endif 
           
#ifdef __cplusplus
}
#endif

#endif  /* __WIN32_STD_H */