/*
 * win32/ntimage.h
 * Copyright 1996, Onno Hovers, All rights reserved
 * 
 * This is the file full with all these stupid 
 * incomprehensile structures Microsoft uses to 
 * describe it's "pe"-format
 */
#ifndef __WIN32_NTIMAGE_H
#define __WIN32_NTIMAGE_H

#include <win32/defines.h>

#define IMAGE_DOS_SIGNATURE      ('M'+('Z'<<8))
#define IMAGE_OS2_SIGNATURE      ('N'+('E'<<8))
#define IMAGE_OS2_SIGNATURE_LE   ('L'+('E'<<8))
#define IMAGE_NT_SIGNATURE       ('P'+('E'<<8))

typedef struct __IMAGE_FILE_HEADER
{
   UWORD Machine;
   UWORD NumberOfSections;
   ULONG TimeDateStamp;
   ULONG PointerToSymbolTable;
   ULONG NumberOfSymbols;
   UWORD SizeOfOptionalHeader;
   UWORD Characteristics;
} IMAGE_FILE_HEADER;

typedef struct __IMAGE_FILE_HEADER  *PIMAGE_FILE_HEADER;

/* These defines describe the meanings of the bits in the Characteristics
   field */
#define IMAGE_FILE_RELOCS_STRIPPED	1 /* No relocation info */
#define IMAGE_FILE_EXECUTABLE_IMAGE	2
#define IMAGE_FILE_LINE_NUMS_STRIPPED   4
#define IMAGE_FILE_LOCAL_SYMS_STRIPPED  8
#define IMAGE_FILE_16BIT_MACHINE	0x40
#define IMAGE_FILE_BYTES_REVERSED_LO	0x80
#define IMAGE_FILE_32BIT_MACHINE	0x100
#define IMAGE_FILE_DEBUG_STRIPPED	0x200
#define IMAGE_FILE_SYSTEM		0x1000
#define IMAGE_FILE_DLL			0x2000
#define IMAGE_FILE_BYTES_REVERSED_HI	0x8000

/* These are the settings of the Machine field. */
#define IMAGE_FILE_MACHINE_UNKNOWN	0
#define IMAGE_FILE_MACHINE_I860		0x14d
#define IMAGE_FILE_MACHINE_I386		0x14c
#define IMAGE_FILE_MACHINE_R3000	0x162
#define IMAGE_FILE_MACHINE_R4000	0x166
#define IMAGE_FILE_MACHINE_ALPHA	0x184

/* reasons a dll entry point is called */
#ifndef DLL_PROCESS_ATTACH
#define DLL_PROCESS_ATTACH   1
#define DLL_PROCESS_DETACH   0 
#define DLL_THREAD_ATTACH    2
#define DLL_THREAD_DETACH    3
#endif 

typedef struct __IMAGE_DATA_DIRECTORY
{
   ULONG VirtualAddress;
   ULONG Size;
} IMAGE_DATA_DIRECTORY;

typedef struct __IMAGE_DATA_DIRECTORY *PIMAGE_DATA_DIRECTORY;

/* Optional coff header - used by NT to provide additional information. */

typedef struct __IMAGE_OPTIONAL_HEADER
{
   UWORD Magic;  		/* Good old COFF magic 0413 */
   UBYTE MajorLinkerVersion;
   UBYTE MinorLinkerVersion;
   ULONG SizeOfCode;
   ULONG SizeOfInitializedData;
   ULONG SizeOfUninitializedData;
   ULONG AddressOfEntryPoint;
   ULONG BaseOfCode;
   ULONG BaseOfData;
   ULONG BaseOfImage;
   ULONG SectionAlignment;
   ULONG FileAlignment;
   UWORD MajorOperatingSystemVersion;
   UWORD MinorOperatingSystemVersion;
   UWORD MajorImageVersion;
   UWORD MinorImageVersion;
   UWORD MajorSubsystemVersion;
   UWORD MinorSubsystemVersion;
   ULONG Unknown1;
   ULONG SizeOfImage;
   ULONG SizeOfHeaders;
   ULONG CheckSum;
   UWORD Subsystem;
   UWORD DllCharacteristics;
   ULONG SizeOfStackReserve;
   ULONG SizeOfStackCommit;
   ULONG SizeOfHeapReserve;
   ULONG SizeOfHeapCommit;
   ULONG LoaderFlags;
   ULONG NumberOfRvaAndSizes;
   IMAGE_DATA_DIRECTORY DataDirectory[16];
} IMAGE_OPTIONAL_HEADER;

typedef struct __IMAGE_OPTIONAL_HEADER *PIMAGE_OPTIONAL_HEADER;

/* These are indexes into the DataDirectory array */
#define IMAGE_DIRECTORY_ENTRY_EXPORT           0
#define IMAGE_DIRECTORY_ENTRY_IMPORT           1
#define IMAGE_DIRECTORY_ENTRY_RESOURCE         2
#define IMAGE_DIRECTORY_ENTRY_EXCEPTION        3
#define IMAGE_DIRECTORY_ENTRY_SECURITY         4
#define IMAGE_DIRECTORY_ENTRY_BASERELOC        5
#define IMAGE_DIRECTORY_ENTRY_DEBUG            6
#define IMAGE_DIRECTORY_ENTRY_COPYRIGHT        7
#define IMAGE_DIRECTORY_ENTRY_GLOBALPTR        8   /* Mips */
#define IMAGE_DIRECTORY_ENTRY_TLS              9
#define IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG      10

typedef struct __IMAGE_NT_HEADERS
{
   ULONG                     Signature;
   IMAGE_FILE_HEADER         FileHeader;
   IMAGE_OPTIONAL_HEADER     OptionalHeader;
} IMAGE_NT_HEADERS;

typedef struct __IMAGE_NT_HEADERS *PIMAGE_NT_HEADERS;

typedef struct __IMAGE_SECTION_HEADER
{
   UBYTE   Name[8];
   ULONG   VirtualSize;
   ULONG   VirtualAddress;
   ULONG   SizeOfRawData;
   ULONG   PointerToRawData;
   ULONG   PointerToRelocations;
   ULONG   PointerToLinenumbers;
   UWORD   NumberOfRelocations;
   UWORD   NumberOfLinenumbers;
   ULONG   Characteristics;
} IMAGE_SECTION_HEADER;

typedef struct __IMAGE_SECTION_HEADER *PIMAGE_SECTION_HEADER;

/* These defines are for the Characteristics bitfield. */

#define IMAGE_SCN_CNT_CODE                         0x20
#define IMAGE_SCN_CNT_INITIALIZED_DATA             0x40
#define IMAGE_SCN_CNT_UNINITIALIZED_DATA           0x80
#define IMAGE_SCN_MEM_DISCARDABLE             0x2000000
#define IMAGE_SCN_MEM_SHARED 	             0x10000000
#define IMAGE_SCN_MEM_EXECUTE	             0x20000000
#define IMAGE_SCN_MEM_READ	             0x40000000
#define IMAGE_SCN_MEM_WRITE	             0x80000000

/*
 * Import module directory stuff
 */

typedef struct __IMAGE_IMPORT_DESCRIPTOR 
{
  ULONG  Characteristics;
  ULONG  TimeDateStamp;
  ULONG  ForwarderChain;
  ULONG  Name;
  LPVOID FirstThunk;
} IMAGE_IMPORT_DESCRIPTOR;

typedef struct __IMAGE_IMPORT_DESCRIPTOR  *PIMAGE_IMPORT_DESCRIPTOR;

typedef struct __IMAGE_IMPORT_BY_NAME
{
  UWORD Hint;
  CHAR  Name[1];
} IMAGE_IMPORT_BY_NAME;

typedef struct __IMAGE_IMPORT_BY_NAME *PIMAGE_IMPORT_BY_NAME;

/*
 * Export module directory stuff
 */

typedef struct __IMAGE_EXPORT_DIRECTORY
{
  ULONG  Characteristics;
  ULONG  TimeDateStamp;
  UWORD  MajorVersion;
  UWORD  MinorVersion;
  ULONG  Name;
  ULONG  Base;
  ULONG  NumberOfFunctions;
  ULONG  NumberOfNames;
  PULONG AddressOfFunctions;
  PULONG AddressOfNames;
  PUWORD AddressOfNameOrdinals;
/*  UCHAR ModuleName[1]; */
} IMAGE_EXPORT_DIRECTORY;

typedef struct __IMAGE_EXPORT_DIRECTORY *PIMAGE_EXPORT_DIRECTORY;

/*
 * Resource directory stuff
 */

typedef struct __IMAGE_RESOURCE_DIRECTORY
{
  ULONG Characteristics;
  ULONG TimeDateStamp;
  UWORD MajorVersion;
  UWORD MinorVersion;
  UWORD NumberOfNamedEntries;
  UWORD NumberOfIdEntries;
} IMAGE_RESOURCE_DIRECTORY;

typedef struct __IMAGE_RESOURCE_DIRECTORY *PIMAGE_RESOURCE_DIRECTORY;

typedef struct __IMAGE_RESOURCE_DIRECTORY_ENTRY
{
  ULONG Name;
  ULONG OffsetToData;
} IMAGE_RESOURCE_DIRECTORY_ENTRY;

typedef struct __IMAGE_RESOURCE_DIRECTORY_ENTRY 
                                *PIMAGE_RESOURCE_DIRECTORY_ENTRY;

typedef struct __IMAGE_RESOURCE_DIRECTORY_STRING
{
  UWORD Length;
  CHAR NameString[1];
} IMAGE_RESOURCE_DIRECTORY_STRING;

typedef struct __IMAGE_RESOURCE_DIRECTORY_STRING 
                                *PIMAGE_RESOURCE_DIRECTORY_STRING;

typedef struct __IMAGE_RESOURCE_DIR_STRING_U
{
  UWORD Length;
  UWORD NameString[1];
} IMAGE_RESOURCE_DIR_STRING_U ;

typedef struct __IMAGE_RESOURCE_DIR_STRING_U  *PIMAGE_RESOURCE_DIR_STRING_U;

typedef struct __IMAGE_RESOURCE_DATA_ENTRY
{
  ULONG OffsetToData;
  ULONG Size;
  ULONG CodePage;
  ULONG Reserved;
} IMAGE_RESOURCE_DATA_ENTRY;

typedef struct __IMAGE_RESOURCE_DATA_ENTRY  *PIMAGE_RESOURCE_DATA_ENTRY;

#define IMAGE_RESOURCE_NAME_IS_STRING    0x80000000
#define IMAGE_RESOURCE_DATA_IS_DIRECTORY 0x80000000

typedef struct __IMAGE_BASE_RELOCATION
{
   ULONG   VirtualAddress;       /* virtual address of page */
   ULONG   SymbolTableIndex;     /* actually the size of this block */
   USHORT  Type[1];              /* actually the relocations */
} IMAGE_RELOCATION;

typedef struct __IMAGE_BASE_RELOCATION *PIMAGE_BASE_RELOCATION;

#define IMAGE_REL_BASED_ABSOLUTE                0
#define IMAGE_REL_BASED_HIGH                    1
#define IMAGE_REL_BASED_LOW                     2
#define IMAGE_REL_BASED_HIGHLOW                 3
#define IMAGE_REL_BASED_HIGHADJ                 4
#define IMAGE_REL_BASED_MIPS_JMPADDR            5


typedef VOID (WINAPI * PIMAGE_TLS_CALLBACK)(
               PVOID hmodule, ULONG reason, PVOID reserved);

typedef struct __IMAGE_TLS_DIRECTORY
{
   ULONG                  StartAddressOfRawData;
   ULONG                  EndAddressOfRawData;
   PULONG                 AddressOfIndex;
   PIMAGE_TLS_CALLBACK    AddressOfCallBacks;
   ULONG                  SizeOfZeroFill;
   ULONG                  Characteristics;
} IMAGE_TLS_DIRECTORY;

typedef struct __IMAGE_TLS_DIRECTORY *PIMAGE_TLS_DIRECTORY;

#endif