/*
 * kernel/console.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * console functions
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */ 

#include <unistd.h>
#include <win32/types.h>
#include <win32/file.h>
#include "teb.h"
#include "imalloc.h"
#include "object.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef NOTYET
CONIN_OBJECT  __ConInput;
CONOUT_OBJECT __ConOutput;
CONOUT_OBJECT __ConError;

BOOL  __ConInit()
{
   PFILE_OBJECT pfile;
   
   if(isatty(stdin))
      HStdIn=__ConDescOpen(stdin);
   
   if(isatty(stdout))
      HStdOut=__ConDescOpen(stdout);
      
   if(isatty(stderr))
      HStdErr=__ConDescOpen(stderr);
      
   /* init stdin */
   pfile=imalloc(sizeof(FILE_OBJECT));
   if(!pfile)
      return __ErrorReturnFalse(ERROR_OUTOFMEMORY); 
   pfile->Kernel.Magic=MAGIC_FILE;
   pfile->Kernel.Class=(PKERNEL_CLASS) &__FileClass;
   pfile->Kernel.HandleCount=1;
   pfile->File.FileDesc=0; 
   pfile->File.Pointer=0;
   __HStdIn =__HandleAlloc((PKERNEL_OBJECT) pfile, FILE_GENERIC_READ );  
   if(!__HStdIn)
   { 
      ifree(pfile); 
      return __ErrorReturnFalse(ERROR_OUTOFMEMORY);
   };
   
   /* init stdout */
   pfile=imalloc(sizeof(FILE_OBJECT));
   if(!pfile)
      return __ErrorReturnFalse(ERROR_OUTOFMEMORY); 
   pfile->Kernel.Magic=MAGIC_FILE;
   pfile->Kernel.Class=(PKERNEL_CLASS) &__FileClass;
   pfile->Kernel.HandleCount=1;
   pfile->File.FileDesc=1; 
   pfile->File.Pointer=0;
   __HStdOut=__HandleAlloc((PKERNEL_OBJECT) pfile, FILE_GENERIC_WRITE );  
   if(!__HStdOut)
   { 
      ifree(pfile); 
      return __ErrorReturnFalse(ERROR_OUTOFMEMORY);
   };
   
   /* init stderr */
   pfile=imalloc(sizeof(FILE_OBJECT));
   if(!pfile)
      return __ErrorReturnFalse(ERROR_OUTOFMEMORY); 
   pfile->Kernel.Magic=MAGIC_FILE;
   pfile->Kernel.Class=(PKERNEL_CLASS) &__FileClass;
   pfile->Kernel.HandleCount=1;
   pfile->File.FileDesc=2; 
   pfile->File.Pointer=0;
   __HStdErr=__HandleAlloc((PKERNEL_OBJECT) pfile, FILE_GENERIC_WRITE );  
   if(!(__HStdErr))
   { 
      ifree(pfile); 
      return __ErrorReturnFalse(ERROR_OUTOFMEMORY);
   };

   return TRUE;                    
}

#ifdef NOT
BOOL WINAPI AllocConsole(VOID)
{   
   __ConinObject.Kernel.Magic=	MAGIC_CONIN;
   __ConinObject.Kernel.Class=	(PKERNEL_CLASS) &__ConinClass;
   __ConinObject.Kernel.Count=	0;
   __ConinObject.Kernel.Security=NULL;
   __ConinObject.File.FileDesc=	STDIN_FILENO;
   __ConinObject.File.Pointer=	0;
 
   /* XXX: should be allocated, as more output buffers are possible */
   __ConoutObject.Kernel.Magic=	MAGIC_CONOUT;
   __ConoutObject.Kernel.Class=	(PKERNEL_CLASS) &__ConoutClass;
   __ConoutObject.Kernel.Count=	0;
   __ConoutObject.Kernel.Security=NULL;
   __ConoutObject.File.FileDesc=STDOUT_FILENO;
   __ConoutObject.File.Pointer=	0;

   conerrObject.Kernel.Magic=	MAGIC_CONOUT;
   conerrObject.Kernel.Class=	(PKERNEL_CLASS) &__ConoutClass;
   conerrObject.Kernel.Count=	0;
   conerrObject.Kernel.Security=NULL;
   conerrObject.File.FileDesc=	STDERR_FILENO;
   conerrObject.File.Pointer=	0;    
   
   return TRUE;
}
#endif

BOOL  __ConinOpen(PHANDLE phand, DWORD access, LPSECURITY_ATTRIBUTES psa)
{
   HANDLE hand;
   
   hand=__HandleAlloc((PKERNEL_OBJECT)pconin, access);

   __ConinObject.Count++;   
   *phand=hand;   
}

BOOL  __ConinIsType(VOID)
{
   return ( ( Magic!=MAGIC_FILE ) ? TRUE : FALSE )
}

BOOL  __ConinClose(PFILE_OBJECT pfile)
{
   close(pfile->File.FileDesc);
}

BOOL  __ConinGetFileType
{
   return FILE_TYPE_CHAR;
}

BOOL conSeek(ULONG seek, PULONG pseekhi, ULONG whence)
{
   SetLastError(ERROR_SEEK_ON_DEVICE);
   return FALSE;
}

HANDLE GetStdHandle(DWORD nhandle)
{
   HANDLE         hnd;
   DWORD          access;
   PKERNEL_OBJECT pobj;
   
   switch(nhandle)
   {
      case STD_INPUT_HANDLE:
         pobj  =(PKERNEL_OBJECT) &__ConinObject;
         access=GENERIC_READ;
         break;
      case STD_OUTPUT_HANDLE:
         pobj  =(PKERNEL_OBJECT) &__ConoutObject;
         access=GENERIC_WRITE;
         break;            
      case STD_ERROR_HANDLE:
         pobj  =(PKERNEL_OBJECT) &conerrObject;
         access=GENERIC_WRITE;
         break;            
      default
         return __ErrorReturnInvalid(ERROR_INVALID_PARAMETER);
   }
}
#endif


/*********************************************************************
*                     GetStdHandle  --  KERNEL32                     *
*                                                                    *
* Get one of the standard handles                                    *
*                                                                    *
*********************************************************************/
                   
HANDLE WINAPI GetStdHandle(DWORD what)
{
   dprintf("GetStdHandle called\n");
   switch(what)
   {
      case STD_INPUT_HANDLE:
         return __HStdIn;
      case STD_OUTPUT_HANDLE:
         return __HStdOut;
      case STD_ERROR_HANDLE:
         return __HStdErr;
      default:
         SetLastError(ERROR_INVALID_PARAMETER);
         return (HANDLE) NULL;
   }
}
                    

#ifdef
}
#endif
