/*
 * kernel/envir.c
 * Copyright (C) 1996,  Onno Hovers, All rights reserved
 * 
 * environment functions
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <win32/defines.h>
#include <win32/kernel.h>
#include <win32/error.h>
#include <win32/console.h>
#include "imalloc.h"
#include "wfc.h"
#include "peload.h"
#include "process.h"

#define ENV_GRANULARITY  256     /* small enough to catch my errors, for now */

/*******************************************************************
*              GetCommandLineA   - KERNEL32.DLL                    *
*******************************************************************/
LPSTR WINAPI GetCommandLineA(VOID)
{
    aprintf("GetCommandLineA()\n");
    return __CommandLineA;
}

/*******************************************************************
*	     GetEnvironmentStringsA - KERNEL32.DLL                 *
*******************************************************************/
LPSTR WINAPI GetEnvironmentStringsA(VOID)
{
    aprintf("GetEnvironmentStringsA()\n");
    return __EnvironmentA;
}

/******************************************************************
*	FreeEnvironmentStringsA - KERNEL32.DLL                    *
******************************************************************/
BOOL WINAPI FreeEnvironmentStringsA(LPSTR penvblock)
{
   aprintf("FreeEnviromentStringsA()\n");
   return TRUE;
}

BOOL __EnvironmentInit(VOID)
{
   __EnvironmentA=imalloc(ENV_GRANULARITY);
   if(!__EnvironmentA) 
      return FALSE;
   memset(__EnvironmentA,0,ENV_GRANULARITY);
   __EnvironmentSize=1;
   __EnvironmentAlloc=ENV_GRANULARITY;
   if(getenv("WF_PATH"))
      SetEnvironmentVariableA("PATH",getenv("WF_PATH"));      
   
   return TRUE;
   
   /* initialize startup info */
   memset(&__StartupInfoA,0,sizeof(STARTUPINFOA));
   __StartupInfoA.cb=sizeof(STARTUPINFOA);
   __StartupInfoA.dwXCountChars=atoi(getenv("COLUMNS"));
   __StartupInfoA.dwYCountChars=atoi(getenv("LINES"));
   __StartupInfoA.dwFillAttribute= FOREGROUND_RED | FOREGROUND_GREEN |
                                   FOREGROUND_BLUE;
   __StartupInfoA.hStdInput  = __HStdIn;
   __StartupInfoA.hStdOutput = __HStdOut;
   __StartupInfoA.hStdError  = __HStdErr;
}

 
/**********************************************************************
*	GetEnvironmentVariableA - KERNEL32.DLL                        *
**********************************************************************/
DWORD WINAPI GetEnvironmentVariableA(LPCSTR key, LPSTR buf, DWORD bufsize)
{
   LPSTR envir;
   INT   keylen;

   aprintf("GetEnvironmentVariableA(%s, 0x%lX, %ld)\n",
           key, (ULONG) buf, bufsize);
   
   envir=__EnvironmentA;
   keylen=strlen(key);        
        
   while(*envir!='\0')
   {
      if((strncmp(key,envir,keylen)==0)&&(envir[keylen]=='='))
      {
         envir=envir+keylen+1;
         strncpy( buf, envir, bufsize);
         return strlen(envir);            
      }
      envir=envir+strlen(envir)+1;
   }
   SetLastError(ERROR_ENVVAR_NOT_FOUND);
   return 0;
}

LPSTR __EnvironmentShrink(LPSTR newenv, LPSTR oldenv)
{
   LPSTR startenv;   
   
   /* fixme: should free unnecessary memory */
   __EnvironmentSize=__EnvironmentSize+ (INT)(newenv-oldenv);   
   
   startenv=oldenv;   
   while(*oldenv!='\0')
   {
       while(*oldenv!='\0')
       {
           *newenv=*oldenv;
           oldenv++;
           newenv++;
       } 
       *newenv='\0';
       oldenv++;
       newenv++;
   }
   return startenv;
}

LPSTR __EnvironmentExpand(LPSTR newenv, LPSTR oldenv)
{   
   ULONG newalloc;
   ULONG newenvsize;
   LPSTR startenv;
   
   newenvsize=__EnvironmentSize+ (INT)(newenv-oldenv);
   if(newenvsize>__EnvironmentAlloc)
   {
       /* reallocate the environment */
       newalloc=ROUNDDOWN(newenvsize,ENV_GRANULARITY);
       startenv=irealloc(__EnvironmentA,newalloc);
       if(!startenv)
          return NULL;
          
       oldenv=startenv+(INT)(oldenv-__EnvironmentA);
       newenv=startenv+(INT)(newenv-__EnvironmentA);
       __EnvironmentA=startenv;
       __EnvironmentAlloc=newalloc;     
   }
   __EnvironmentSize=newenvsize;
   startenv=oldenv;
   
   while(*oldenv!='\0')
   {
       while(*oldenv!='\0')
       {       
           oldenv++;        
           newenv++;
       }      
       oldenv++;
       newenv++;
   }
   
   while(oldenv>startenv)
   {
      *newenv=*oldenv;
      newenv--;
      oldenv--;
   }
   return startenv;
}

/********************************************************************
*       SetEnvironmentVariableA - KERNEL32.DLL                      *
********************************************************************/
BOOL WINAPI SetEnvironmentVariableA(LPCSTR key, LPCSTR value)
{
   LPSTR oldenv;
   LPSTR newenv;
   INT   keylen;
   INT   oldvallen;
   INT   newvallen;
   INT   result;
   INT   diff;

   aprintf("SetEnvironmentVariableA(%s, %s)\n",key,value);
   
   oldenv=__EnvironmentA;
   keylen=strlen(key);
   
   if((value==NULL)||(value[0]=='\0'))
   {
      /* remove the variable from the environment */             
      while(strncmp(oldenv,key,keylen)&&(oldenv[keylen]!='='))
      {
         oldenv=oldenv+keylen+1;
         if(oldenv[0]=='\0')
            return TRUE;
      }
      oldvallen=strlen(oldenv);
      __EnvironmentShrink(oldenv,oldenv+oldvallen+1);
      return TRUE;
   }
   
   /* else */
   
   newvallen=strlen(value);
   while(*oldenv!='\0')
   {
      dprintf("%s\n",oldenv);
      result=strncmp(oldenv,key,keylen);
      newvallen=strlen(value);
      
      if((result==0)&&(oldenv[keylen]=='='))
      {
         oldenv=oldenv+keylen+1;
         
         /* replace value */
         oldvallen=strlen(oldenv);       
         diff=newvallen-oldvallen;
         if(diff==0)
         {
            strncpy(oldenv,value,newvallen);      
         }
         else if(diff<0)
         {           
            oldenv=oldenv+strlen(oldenv)+1;
            newenv=oldenv+diff;  
            oldenv=__EnvironmentShrink(newenv,oldenv);
            strncpy(oldenv,value,newvallen);
         }
         else if(diff>0)
         {         
            oldenv=oldenv+strlen(oldenv)+1; 
            newenv=oldenv+diff;
            oldenv=__EnvironmentExpand(newenv,oldenv);
            strncpy(oldenv,value,newvallen);
         }
         return TRUE;
      }
      else if(result>0)
         break;
      
      oldenv=oldenv+strlen(oldenv)+1;      
   }
   /* we have not found our key, so make a new one */   
   newenv=oldenv+keylen+newvallen+2;
   oldenv=__EnvironmentExpand(newenv,oldenv);
   strcpy(oldenv,key);
   strcat(oldenv,"=");
   strcat(oldenv,value);
   dprintf("%s\n",oldenv);
   return TRUE;  
}

/********************************************************************
*                GetStartupInfoA - KERNEL32.DLL                     *
********************************************************************/

VOID WINAPI GetStartupInfoA(LPSTARTUPINFOA psi)
{
   dprintf("GetStartupInfoA called\n");
   memcpy(psi,&__StartupInfoA,sizeof(STARTUPINFOA));
}

#ifdef 0
/* if we sense that we have been started by CreateProcess,
 * we have to copy some data from our dear parent:
 *
 * arg0: progname
 * arg1: --intern
 * arg2: cmdline
 * env: %X  = intern
 *      %%X = %X
 */
 
int __EnvirMainChild(int argc, char **argv, char **environ)
{
   LPSTR env;
   LPSTR str;
   INT   envcount;
   
   /* copy command line */
   __CommandLineA=imalloc(strlen(argv[2])+1);
   
   strcpy(__CommandLineA,argv[2]);
   /* convert environment */
   while(env)
   { 
      if(env[0]=='%')
      {
         if(env[1]!='%')
         {
            /* internal environment entry */
            if(strncmp(env,"%DRIVE_",7)==0)
            {
               str=env+8;
               SetEnvironmentVariable( ,str)
            }
            else if(strncmp(env,"%SIA_",5)==0)
            {
               /* startup info */
               str=env+5;
               if(strcmp(str,"DESKTOP=")==0)
               {
                  str=str+8;
                  __StartupInfoA->lpDesktop=imalloc(strlen(str)+1);
                  strcpy(__StartupInfoA->lpDesktop,str);
               }
               else if(strcmp(str,"TITLE=")==0)
               {
                  str=str+6;
                  __StartupInfoA->lpTitle=imalloc(strlen(str)+1);
                  strcpy(__StartupInfoA->lpTitle,str);              
               }
               else if(strcmp(str,"X=")==0)
                  __StartupInfoA->dwX=(DWORD) atoi(str+2);
               else if(strcmp(str,"Y=")==0)
                  __StartupInfoA->dwY=(DWORD) atoi(str+2);
               else if(strcmp(str,"XSIZE=")==0)
                  __StartupInfoA->dwXSize=(DWORD) atoi(str+6);
               else if(strcmp(str,"YSIZE=")==0)
                  __StartupInfoA->dwYSize=(DWORD) atoi(str+6);
               else if(strcmp(str,"XCOUNTCHARS=")==0)
                  __StartupInfoA->dwXCountChars=(DWORD) atoi(str+12);
               else if(strcmp(str,"YCOUNTCHARS=")==0)
                  __StartupInfoA->dwYCountChars=(DWORD) atoi(str+12);
               else if(strcmp(str,"FILLATTRIB=")==0)
                  __StartupInfoA->dwFillAttribute=(DWORD) atoi(str+11);
               else if(strcmp(str,"FLAGS=")==0)
                  __StartupInfoA->dwFlags=(DWORD) atoi(str+6);
               else if(strcmp(str,"SHOWWINDOW=")==0)
                  __StartupInfoA->wShowWindow=(WORD) atoi(str+11);
               else if(strcmp(str,"HSTDIN=")==0)
                  __StartupInfoA->hStdInput=(HANDLE) atoi(str+7);              
               else if(strcmp(str,"HSTDOUT=")==0)
                  __StartupInfoA->hstdOutput=(HANDLE) atoi(str+8);
               else if(strcmp(str,"HSTDERR=")==0)
                  __StartupInfoA->hstdError=(HANDLE) atoi(str+8);              
            }
            else
               envcount=envcount+strlen(env);        
         }  
      }
      else
         envcount=envcount+strlen(env)+1;
   }
}

#endif

