/*
 * error.c
 * Copyright 1996, Onno Hovers, All rights reserved  
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 

#include<stdlib.h>
#include<stdio.h>
#include<win32/defines.h>
#include "exe.h"
#include "peload.h"
#include "run.h"
#include "teb.h"

UINT __ErrorMode;

void _exp_unknown(ULONG stuff, LPSTR fname )
{
  printf("unknown function %s from %X\n", fname, 
         (unsigned)  ((PULONG)(&stuff))[-1]);
  exit(1);
}

void _exp_unimp(ULONG stuff, LPSTR name1, LPSTR name2)
{
  LPSTR name;
  
  if(stuff<1000)
     name=name2;
  else
     name=name1;   
     
  printf("unimplemented function %s\n", name);
  __TebLastError=120; 
}

VOID SetLastError(DWORD err)
{
   aprintf("SetLastError( %ld )\n", err);
   __TebLastError=err;
}

DWORD GetLastError(VOID)
{
   aprintf("GetLastError()\n");
   return __TebLastError;
}

UINT SetErrorMode(UINT errmode)
{
   UINT retval=__ErrorMode;
   
   aprintf("SetErrorMode( 0x%X )\n", errmode);
   __ErrorMode=errmode;
   
   return retval;
}

HANDLE __ErrorReturnInvalid(DWORD err)
{
    SetLastError(err);
    return INVALID_HANDLE_VALUE;
}

BOOL   __ErrorReturnFalse(DWORD err)
{
    SetLastError(err);
    return FALSE; 
}

PVOID  __ErrorReturnNull(DWORD err)
{
    SetLastError(err);
    return NULL;
}

DWORD  __ErrorFromErrno(int err)
{
   if(!err)
      return 0;
   switch(err)
   {
      /* from Linux /usr/include/asm/errno.h */
         
      case EPERM:        /* Operation not permitted */
         return  ERROR_ACCESS_DENIED; 
      case ENOENT:       /* No such file or directory */
         return  ERROR_FILE_NOT_FOUND; 
      case ESRCH:        /* No such process */
         return  ERROR_UNKNOWN; 
      case EINTR:        /* Interrupted system call */
         return  ERROR_UNKNOWN; 
      case EIO:          /* I/O error */
         return  ERROR_UNKNOWN; 
      case ENXIO:        /* No such device or address */
         return  ERROR_UNKNOWN; 
      case E2BIG:        /* Arg list too long */
         return  ERROR_UNKNOWN; 
      case ENOEXEC:      /* Exec format error */
      	 return  ERROR_BAD_EXE_FORMAT; 
      case EBADF:        /* Bad file number */
     	 return  ERROR_INVALID_HANDLE; 
      case ECHILD:       /* No child processes */
     	 return  ERROR_UNKNOWN; 
      case EAGAIN:       /* Try again */
     	 return  ERROR_UNKNOWN; 
      case ENOMEM:       /* Out of memory */
     	 return  ERROR_OUTOFMEMORY; 
      case EACCES:       /* Permission denied */
         return  ERROR_ACCESS_DENIED; 
      case EFAULT:       /* Bad address */
         return  ERROR_NOACCESS; 
      case ENOTBLK:      /* Block device required */
         return  ERROR_UNKNOWN; 
      case EBUSY:        /* Device or resource busy */
         return  ERROR_UNKNOWN; 
      case EEXIST:       /* File exists */
         return  ERROR_FILE_EXISTS; 
      case EXDEV:        /* Cross-device link */
         return  ERROR_UNKNOWN; 
      case ENODEV:       /* No such device */
         return  ERROR_UNKNOWN; 
      case ENOTDIR:      /* Not a directory */
         return  ERROR_DIRECTORY; 
      case EISDIR:       /* Is a directory */
     	 return  ERROR_DIRECTORY; 
      case EINVAL:       /* Invalid argument */
         return  ERROR_INVALID_PARAMETER;      	           	
      case ENFILE:       /* File table overflow */
         return  ERROR_NO_MORE_FILES;      	      
      case EMFILE:       /* Too many open files */
     	 return  ERROR_TOO_MANY_OPEN_FILES; 
      case ENOTTY:       /* Not a typewriter */
      case ETXTBSY:      /* Text file busy */
      case EFBIG:        /* File too large */
         return  ERROR_UNKNOWN; 
      case ENOSPC:       /* No space left on device */
         return  ERROR_DISK_FULL; 
      case ESPIPE:       /* Illegal seek */
         return  ERROR_SEEK_ON_DEVICE; 
      case EROFS:        /* Read-only file system */
         return  ERROR_WRITE_PROTECT; 
      case EMLINK:       /* Too many links */
         return  ERROR_UNKNOWN; 
      case EPIPE:        /* Broken pipe */
         return  ERROR_PIPE_NOT_CONNECTED; 
      case EDOM:         /* Math argument out of domain of func */
      case ERANGE:       /* Math result not representable */
         return  ERROR_UNKNOWN; 
      case EDEADLK:      /* Resource deadlock would occur */
         return  ERROR_POSSIBLE_DEADLOCK; 
      case ENAMETOOLONG: /* File name too long */
         return  ERROR_FILENAME_EXCED_RANGE; 
      case ENOLCK:       /* No record locks available */
         return  ERROR_UNKNOWN; 
      case ENOSYS:       /* Function not implemented */
         return  ERROR_CALL_NOT_IMPLEMENTED;
      case ENOTEMPTY:	 /* Directory not empty */
         return  ERROR_DIR_NOT_EMPTY; 
      case ELOOP:        /* Too many symbolic links encountered */
         return  ERROR_FILE_NOT_FOUND; 
      case ENOMSG:       /* No message of desired type */
      case EIDRM:        /* Identifier removed */
      case ECHRNG:       /* Channel number out of range */
      case EL2NSYNC:     /* Level 2 not synchronized */
      case EL3HLT:       /* Level 3 halted */
      case EL3RST:       /* Level 3 reset */
      case ELNRNG:       /* Link number out of range */
      case EUNATCH:      /* Protocol driver not attached */
      case ENOCSI:       /* No CSI structure available */
      case EL2HLT:       /* Level 2 halted */
      case EBADE:        /* Invalid exchange */
      case EBADR:        /* Invalid request descriptor */
      case EXFULL:       /* Exchange full */
      case ENOANO:       /* No anode */
      case EBADRQC:      /* Invalid request code */
      case EBADSLT:      /* Invalid slot */
         return  ERROR_UNKNOWN;      	
#ifdef NEVER
      case EDEADLOCK:	/* File locking deadlock error */
         return  ERROR_POSSIBLE_DEADLOCK; 
#endif         
      case EBFONT:	        /* Bad font file format */
      case ENOSTR:		/* Device not a stream */
      case ENODATA:	/* No data available */
      case ETIME:		/* Timer expired */
      case ENOSR:		/* Out of streams resources */
      case ENONET:		/* Machine is not on the network */
      case ENOPKG:		/* Package not installed */
      case EREMOTE:	/* Object is remote */
      case ENOLINK:        /* Link has been severed */
      case EADV:		/* Advertise error */
      case ESRMNT:		/* Srmount error */
      case ECOMM:		/* Communication error on send */
      case EPROTO:		/* Protocol error */
      case EMULTIHOP:	/* Multihop attempted */
      case EDOTDOT:	/* RFS specific error */
      case EBADMSG:	/* Not a data message */
      case EOVERFLOW:	/* Value too large for defined data type */
      case ENOTUNIQ:	/* Name not unique on network */
      case EBADFD:		/* File descriptor in bad state */
      case EREMCHG:	/* Remote address changed */
      case ELIBACC:	/* Can not access a needed shared library */
      case ELIBBAD:	/* Accessing a corrupted shared library */
      case ELIBSCN:	/* .lib section in a.out corrupted */
      case ELIBMAX:	/* Attempting to link in too many shared libraries */
      case ELIBEXEC:	/* Cannot exec a shared library directly */
      case EILSEQ:		/* Illegal byte sequence */
      case ERESTART:	/* Interrupted system call should be restarted */
      case ESTRPIPE:	/* Streams pipe error */
      case EUSERS:		/* Too many users */
      case ENOTSOCK:	/* Socket operation on non-socket */
      case EDESTADDRREQ:	/* Destination address required */
      case EMSGSIZE:	/* Message too long */
      case EPROTOTYPE:	/* Protocol wrong type for socket */
      case ENOPROTOOPT:	/* Protocol not available */
      case EPROTONOSUPPORT:	/* Protocol not supported */
      case ESOCKTNOSUPPORT:	/* Socket type not supported */
      case EOPNOTSUPP:	/* Operation not supported on transport endpoint */
      case EPFNOSUPPORT:	/* Protocol family not supported */
      case EAFNOSUPPORT:	/* Address family not supported by protocol */
      case EADDRINUSE:	/* Address already in use */
      case EADDRNOTAVAIL:	/* Cannot assign requested address */
      case ENETDOWN:	/* Network is down */
      case ENETUNREACH:	/* Network is unreachable */
      case ENETRESET:	/* Network dropped connection because of reset */
      case ECONNABORTED:	/* Software caused connection abort */
      case ECONNRESET:	/* Connection reset by peer */
      case ENOBUFS:		/* No buffer space available */
      case EISCONN:		/* Transport endpoint is already connected */
      case ENOTCONN:	/* Transport endpoint is not connected */
      case ESHUTDOWN:	/* Cannot send after transport endpoint shutdown */
      case ETOOMANYREFS:	/* Too many references: cannot splice */
      case ETIMEDOUT:	/* Connection timed out */
      case ECONNREFUSED:	/* Connection refused */
      case EHOSTDOWN:      /* Host is down */
      case EHOSTUNREACH:	/* No route to host */
      case EALREADY:	/* Operation already in progress */
      case EINPROGRESS:	/* Operation now in progress */
      case ESTALE:		/* Stale NFS file handle */
      case EUCLEAN:	/* Structure needs cleaning */
      case ENOTNAM:        /* Not a XENIX named type file */
      case ENAVAIL:	/* No XENIX semaphores available */
      case EISNAM:		/* Is a named type file */
      case EREMOTEIO:	/* Remote I/O error */
         return  ERROR_UNKNOWN;      	     	
      case EDQUOT:         /* Quota exceeded */
         return  ERROR_NOT_ENOUGH_QUOTA; 
      default: 
         return  0;   	
   }
}