/*
 * kernel/findfile.c
 * Copyright (C) 1996, Onno Hovers
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 *  
 * Find* functions
 *
 */

#include<win32/defines.h>
#include<win32/kernel.h>
#include<win32/error.h> 
#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<string.h>
#include<dirent.h>
#include<fnmatch.h>
#include"wfc.h"
#include"fs.h"
#include"object.h"
#include"handle.h"
#include"itime.h"
#include"error.h"
#include"rtlmem.h"

#define FIND_ONE	1
#define FIND_MATCH	2
#define FIND_ALL	3
#define FIND_LINUX_FATFS	1   /* use linux vfat ioctl */

typedef struct __FIND_OBJECT
{
   struct KERNEL_OBJECT_PART Kernel;
   struct FIND_OBJECT_PART
   {   
      UBYTE Type;
      UBYTE Flags;
      LPSTR Pattern;   
      LPSTR UnixName;
      DIR  *Dir;
   }  Find;
} FIND_OBJECT, *PFIND_OBJECT;

typedef struct __FIND_CLASS
{
   struct KERNEL_CLASS_PART Kernel;
#ifdef NOT   
   struct FIND_CLASS_PART
   {   
   } Find;
#endif   
} FIND_CLASS;


BOOL __FindFillA(LPWIN32_FIND_DATAA pfinddata, struct stat *pmystat)
{  
   pfinddata->dwFileAttributes=0;
   pfinddata->ftCreationTime=__TimeUnixToFile(min(pmystat->st_ctime,
                                                  pmystat->st_mtime));
   pfinddata->ftLastAccessTime=__TimeUnixToFile(pmystat->st_atime);
   pfinddata->ftLastWriteTime=__TimeUnixToFile(pmystat->st_mtime);
   pfinddata->nFileSizeHigh=0;
   pfinddata->nFileSizeLow=pmystat->st_size;
   return TRUE;
}   

BOOL __FindMakeDosNameA(LPSTR dest, LPSTR src)
{
   LPSTR dot;
   INT   namelen;
   INT   extlen;
   INT   i;
   
   extlen=0;
   namelen=strlen(src);
   dot=strrchr(src,'.');
   if(dot) 
   {
      extlen=min(src+namelen-dot,3);
      namelen=min(dot-src,8);
   }
   /* append dos name */
   for(i=0;i<namelen;i++)
   {
      *dest=toupper(*src);
      dest++; src++;
   }
   if(extlen)
   {
      *dest='.';
      dest++;
      
      for(i=0;i<extlen;i++)
      {
         *dest=toupper(*src);
         dest++; src++;
      }
   }
   return TRUE;
}

/* object functions */

BOOL __FindClose(PKERNEL_OBJECT pkern)
{
   PFIND_OBJECT pfind=(PFIND_OBJECT) pkern;
   
   if(pfind->Find.Type!=FIND_ONE)
   {
      closedir(pfind->Find.Dir);
      if(pfind->Find.Type==FIND_MATCH)
         ifree(pfind->Find.Pattern);
      ifree(pfind->Find.UnixName);
   }
   ifree(pfind);
   return TRUE;
}

BOOL __FindGetInfoHandle(PKERNEL_OBJECT pkern, PHANDLE_DATA phand,
                         PDWORD pinfo)
{
    SetLastError(ERROR_INVALID_HANDLE);
    return FALSE;
}                         

BOOL __FindSetInfoHandle(PKERNEL_OBJECT pkern, PHANDLE_DATA phand,
                         DWORD mask, DWORD info)
{
    SetLastError(ERROR_INVALID_HANDLE);
    return FALSE;
}                         
 
BOOL __FindIsType(PKERNEL_OBJECT pobj, ULONG typeid)
{
   if((pobj->Kernel.Magic==MAGIC_FIND)&&(typeid==TYPEID_FIND))
      return TRUE;
   else
      return __ObjectIsType(pobj, typeid);
}

FIND_CLASS __FindClass=
{ { { FILE_GENERIC_READ, FILE_GENERIC_WRITE, FILE_GENERIC_EXECUTE, 
      FILE_ALL_ACCESS }, 
      __FindClose, __FindClose, __FindIsType, __FindGetInfoHandle, 
      __FindSetInfoHandle, __ObjectDuplicate, __ObjectWait },
#ifdef NOT   
   {}
#endif   
};

/********************************************************************
*                 FindFirstFileA  -- KERNEL32                       *
********************************************************************/

HANDLE WINAPI FindFirstFileA(LPCSTR findname, LPWIN32_FIND_DATAA pfinddata)
{
   CHAR		 dirname[MAX_PATH];
   CHAR		 unixname[MAX_PATH];
   CHAR		 fullname[MAX_PATH];
   LPCSTR        sname;
   LPCSTR	 searchname;
   LPSTR	 found;
   struct dirent *pmydent;
   struct stat   mystat;
   DIR           *mydir;
   HANDLE        hfind;
   PFIND_OBJECT  pfind;  
   LPSTR	 pcertain;

   aprintf("FindFirstFileA( %s, 0x%lX )\n", findname, (ULONG) pfinddata);   
   /* allocate an object and a handle */
   pfind=imalloc(sizeof(FIND_OBJECT));
   if(!pfind)
      return __ErrorReturnInvalid(ERROR_OUTOFMEMORY);
   pfind->Kernel.Magic=MAGIC_FIND;
   pfind->Kernel.Class=(PKERNEL_CLASS) (&__FindClass);
   pfind->Kernel.HandleCount=1;
   hfind=__HandleAlloc((PKERNEL_OBJECT) pfind, GENERIC_READ);
   if(!hfind)
      { ifree(pfind); return __ErrorReturnInvalid(ERROR_OUTOFMEMORY); }
        
   /* find the filename part */
   sname=findname;
   searchname=NULL;
   while(*sname)
   {
     if((*sname=='\\')||(*sname=='/')||(*sname==':'))    
        searchname=sname+1;
     sname++;     
   }
   if(searchname)
   {
      istrcpy(dirname,findname,searchname-findname-1);
      dirname[searchname-findname-1]='\0';
   }
   else
   {
      searchname=findname;
      dirname[0]='.';dirname[1]='\0';
   }
   
   if(strchr(searchname,'*')||strchr(searchname,'?')||strchr(searchname,'['))
   {    
      /* open our directory */
      if(!__FsGetNameA(dirname, fullname, unixname, &pcertain, PATH_MAX))
         return FALSE;
 
      mydir=opendir(unixname);
      if(mydir==NULL)
      {
         if(!__FsFindNameA(dirname, unixname, pcertain, PATH_MAX,
            FINDNAME_DIR | FINDNAME_EXIST ))
            return FALSE;

         mydir=opendir(unixname);                
         if(mydir==NULL)
            return __ErrorReturnInvalid(__ErrorFromErrno(errno));
      }          
   
      pfind->Find.Dir=mydir;
      pfind->Find.UnixName=istrdup(unixname);
            
      /* read the files */
      if( (searchname[0]=='*') &&
          ( (searchname[1]=='\0') || 
            ( (searchname[1]=='.')   &&
              (searchname[2]=='*')   &&
              (searchname[3]=='\0') ) ) )
      {
         pfind->Find.Type=FIND_ALL;
         pfind->Find.Pattern="*";
         pmydent=readdir(mydir);
         if(pmydent==NULL)
            return __ErrorReturnInvalid(ERROR_FILE_NOT_FOUND);  
      }
      else
      {
         pfind->Find.Type=FIND_MATCH;
         pfind->Find.Pattern=istrdup(searchname);
         do
         {
            pmydent=readdir(mydir);
            if(pmydent==NULL)
               return __ErrorReturnInvalid(ERROR_FILE_NOT_FOUND);
         } while(fnmatch(searchname, pmydent->d_name, 
                 FNM_PATHNAME | FNM_NOESCAPE | FNM_CASEFOLD));
      }
      istrcat(unixname,"/",MAX_PATH);
      istrcat(unixname,pmydent->d_name, PATH_MAX);
      if(stat(unixname, &mystat)==-1)
         return __ErrorReturnInvalid(__ErrorFromErrno(errno));
      found=pmydent->d_name;
   }
   else
   {
      pfind->Find.Type=FIND_ONE;
      pfind->Find.Pattern="\0";
      
      /* open our file */
      if(!__FsGetNameA(findname, fullname, unixname, &pcertain, PATH_MAX))
         return FALSE;
 
      if(stat(unixname,&mystat)==-1)
      {
         if(!__FsFindNameA(fullname, unixname, pcertain, PATH_MAX,
            FINDNAME_DIR | FINDNAME_FILE | FINDNAME_EXIST ))
            return FALSE;
                
         if(stat(unixname, &mystat)==-1)
            return __ErrorReturnInvalid(__ErrorFromErrno(errno));
      }
      found=strrchr(fullname,'\\')+1;
   } 
    
   istrcpy(pfinddata->cFileName, found, PATH_MAX );
   __FindMakeDosNameA(pfinddata->cAlternateFileName, found);
   __FindFillA(pfinddata,&mystat );
   
   return hfind; 
}

/********************************************************************
*                    FindNextFileA  -- KERNEL32                     *
********************************************************************/

BOOL WINAPI FindNextFileA(HANDLE hfind, PWIN32_FIND_DATAA pfinddata)
{     
   PFIND_OBJECT   pfind;
   struct stat    mystat;
   struct dirent *pmydent;
   CHAR           unixname[MAX_PATH];

   aprintf("FindNextFileA( 0x%lX, 0x%lX )\n", (ULONG) hfind,
           (ULONG) pfinddata);   
   
   pfind=(PFIND_OBJECT) __HandleGetObject(hfind, 0, TYPEID_FIND);
   if(!pfind)
      return FALSE;
   
   if(pfind->Find.Type==FIND_ONE)
      return __ErrorReturnFalse(ERROR_NO_MORE_FILES); 
  
   /* read the files */
   if(pfind->Find.Type==FIND_MATCH)
   {
#ifdef NOT
      if(pfind->Flags& FIND_LINUX_FATFS)
      {
         do
         {
            if(ioctl(  , ,fatdent)==-1)
              return __ErrorReturnFalse(ERROR_NO_MORE_FILES);
         } while(fnmatch(pfind->Pattern, fatdent[1].d_name);
      }
      else
#endif      
      do
      {      
         pmydent=readdir(pfind->Find.Dir);
         if(pmydent==NULL)
            return __ErrorReturnFalse(ERROR_NO_MORE_FILES);
      } while(fnmatch(pfind->Find.Pattern, pmydent->d_name, 
              FNM_PATHNAME | FNM_NOESCAPE | FNM_CASEFOLD));
   }
   else
   {
#ifdef NOT   
      if(pfind->Flags& FIND_LINUX_FATFS)
      {
         if(ioctl(  , ,fatdent)==-1)
            return __ErrorReturnFalse(ERROR_NO_MORE_FILES);
      }
      else
#endif     
      pmydent=readdir(pfind->Find.Dir);
      if(pmydent==NULL)
         return __ErrorReturnFalse(ERROR_NO_MORE_FILES);  
   }
   istrcpy(unixname,pfind->Find.UnixName, MAX_PATH);
   istrcat(unixname,"/", MAX_PATH);
   istrcat(unixname, pmydent->d_name, MAX_PATH);
   if(stat(unixname, &mystat)==-1)
      return __ErrorReturnFalse(__ErrorFromErrno(errno));
  
   istrcpy(pfinddata->cFileName, pmydent->d_name, PATH_MAX );
   __FindMakeDosNameA(pfinddata->cAlternateFileName, pmydent->d_name);
   __FindFillA(pfinddata, &mystat);
   
   return TRUE;
}

/********************************************************************
*                    FindClose  -- KERNEL32                         *
*                                                                   *
* Note that this implementation is too forgiving: you can also      *
* close a Find Handle with CloseHandle                              *
********************************************************************/

BOOL WINAPI FindClose(HANDLE hfind)
{
   PFIND_OBJECT pfind;
  
   aprintf("FindClose( 0x%lX )\n", (ULONG) hfind);
   
   pfind=(PFIND_OBJECT) __HandleGetObject( hfind, 0, TYPEID_FIND);
   if(!pfind)
      return FALSE;

   if( PKERNEL_OBJECT_CALL(pfind, Close, ((PKERNEL_OBJECT) pfind)) )
   {   
      __HandleFree(hfind);
      return TRUE;
   }
   else
      return FALSE;  
}