/* 
 * kernel/handle.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * handle functions for kernel objects
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#include<stddef.h>
#include<stdlib.h>
#include<string.h>
#include<win32/defines.h>
#include<win32/error.h>
#include<win32/kernel.h>
#include<win32/security.h>
#include"imalloc.h"
#include"object.h"
 
HANDLE_DATA ***__HandleArray;

#define HANDLE_START 0xC0000000
#define HANDLE_END   0xC0007FFF

BOOL __HandleInit(VOID)
{
   __HandleArray=(HANDLE_DATA ***) imalloc(sizeof(LPVOID)*32);
   if(!__HandleArray)
      return FALSE;
 
   memset((PVOID) __HandleArray, 0, sizeof(PVOID) * 32);
   return TRUE;
}

HANDLE __HandleAlloc(PKERNEL_OBJECT object, DWORD access)
{
   INT i,j,k,h;
   
   h=0;  
     
   for(i=0;i<32;i++)
   {
      if(__HandleArray[i]==NULL)
      {      
         __HandleArray[i]=(HANDLE_DATA **) imalloc(sizeof(PHANDLE_DATA)*32);
         memset((PVOID)__HandleArray[i], 0, sizeof(PHANDLE_DATA)*32); 
      }
      for(j=0;j<32;j++)
      {
         if(__HandleArray[i][j]==NULL)
         {
            __HandleArray[i][j]=(HANDLE_DATA *) imalloc(sizeof(HANDLE_DATA)*32);
            memset((PVOID) __HandleArray[i][j], 0, sizeof(HANDLE_DATA)*32);\
         }
         for(k=0;k<32;k++)
         {         
            if(__HandleArray[i][j][k].Object==NULL)
            {
            
               __HandleArray[i][j][k].Object=object;
               
               MapGenericMask(&access,&(object->Kernel.Class->Kernel.GenericAccess));
               __HandleArray[i][j][k].Access=access;
               
               return (HANDLE) h+HANDLE_START;
               
            }
            h++;
         }
      }
   }
   return NULL;
}

BOOL __HandleFree(HANDLE handle)
{
   PHANDLE_DATA hdata;
   
   hdata=__HandleGet(handle);
   if(!hdata)
      return FALSE;
   hdata->Object=NULL;
   hdata->Access=0;  
   return TRUE;
}

BOOL __HandleCheckAccess(HANDLE handle, DWORD required)
{
   PHANDLE_DATA hdata;
   
   hdata=__HandleGet(handle);
   if(!hdata)
      return FALSE;
   if((hdata->Access&required)==required)
      return TRUE;
   SetLastError(ERROR_ACCESS_DENIED);
   return FALSE;
}


PHANDLE_DATA __HandleGet(HANDLE handle)
{
   INT i,j,k;
   
   if(( (ULONG) handle < HANDLE_START )||( (ULONG) handle >HANDLE_END ))
      { SetLastError(ERROR_INVALID_HANDLE); return FALSE; }
   
   i=(((unsigned) handle)>>10) &0x1F;
   j=(((unsigned) handle)>>5)  &0x1F;
   k= ((unsigned) handle)      &0x1F;
   
   if((__HandleArray[i]==NULL)&&(__HandleArray[i][j]==NULL))
      { SetLastError(ERROR_INVALID_HANDLE); return FALSE; }

   return  &(__HandleArray[i][j][k]);
}

PKERNEL_OBJECT __HandleGetIndirect(PHANDLE_DATA hdata, DWORD required, DWORD typeid)
{       
   if(!hdata)
      return NULL;
   if((hdata->Access&required)!=required)
      { SetLastError(ERROR_ACCESS_DENIED); return FALSE; }
   if(!hdata->Object->Kernel.Class->Kernel.IsType(hdata->Object,typeid))
      { SetLastError(ERROR_INVALID_HANDLE); return FALSE; }
   return hdata->Object;
}

PKERNEL_OBJECT __HandleGetObject(HANDLE handle, DWORD required, DWORD typeid)
{
   PHANDLE_DATA hdata;  
   
   hdata=__HandleGet(handle);
   return __HandleGetIndirect(hdata, required, typeid); 
}


BOOL __HandleExit(VOID)
{
   INT             i,j,k,h;
   PKERNEL_OBJECT  pobject;
      
   h=0;  
     
   for(i=0;i<32;i++)
   {
      if(__HandleArray[i]!=NULL)
      {         
         for(j=0;j<32;j++)
         {
            if(__HandleArray[i][j]!=NULL)
            {        
               for(k=0;k<32;k++)
               {         
                  if(__HandleArray[i][j][k].Object!=NULL)
                  {                    
                     pobject=__HandleArray[i][j][k].Object;
                     pobject->Kernel.Class->Kernel.Exit(pobject);
                     __HandleArray[i][j][k].Object=NULL;
                     __HandleArray[i][j][k].Access=0;
                  }                 
               }
               ifree(__HandleArray[i][j]);
            }            
         }
         ifree(__HandleArray[i]);
      }
   }
   return TRUE;
}