How to implement a fast and efficient heap algorithm?

I have no statistics of how wfc memory usage will be.
But browsing through the X sources I found this. It is more or less 
typical of the memory usage of large apps.

/*
 ***** New malloc approach for the X server *****
 * Pascal Haible 1995
 *
 * Some statistics about memory allocation of the X server
 * The test session included several clients of different size, including
 * xv, emacs and xpaint with a new canvas of 3000x2000, zoom 5.
 * All clients were running together.
 * A protocolling version of Xalloc recorded 318917 allocating actions
 * (191573 Xalloc, 85942 XNFalloc, 41438 Xrealloc, 279727 Xfree).
 * Results grouped by size, excluding the next lower size
 * (i.e. size=32 means 16<size<=32):
 *
 *    size   nr of alloc   max nr of blocks allocated together
 *  (    8	1114		287	= 2296      )
 *      16	17341		4104	= 65664       (1)
 *      32	147352		2068	= 66176       (2)
 *      64	59053		2518	= 161152      (3)
 *     128	46882		1230	= 157440      (4)
 *     256	20544		1217	= 311552      (5)
For sizes up to +/- 256 byte we must have a fast algorithm.
We use fixed buckets allocated from block of 1K up to 4k  

alloc. size     block size:	alloc. per block:
16		1K		64  
32		1K		32  
64		2K		32  
128		2K		16  
256		4K		16 

 *     512	6808		117	= 58.5 k
 *    1024	8254		171	= 117  k
 *    2048	4841		287     = 574  k   (1) 
 *    4096	2429		84      = 336  k   (2)
 *    8192	3364		85      = 680  k   (3)
 *   16384	573		22      = 352  k   (4)
For sizes up to +/- 16 kbyte we use a standard heap algorithm. 

 *   32768	49		7
 *   65536	45		5
 *  131072	48		2
 *  262144	209		2
 *  524288	7		4
 * 1048576	2		1
 * 8388608	2		2
For sizes >16 kbyte it is very beneficial to use direct memory mapping  
and keep a linked list of references on the heap. 

 * The most used sizes:
 * count size
 * 24	136267
 * 40	37055
 * 72	17278
 * 56	13504
 * 80	9372
 * 16	8966
 * 32	8411
 * 136	8399
 * 104	7690
 * 12	7630
 * 120	5512
 * 88	4634
 * 152	3062
 * 52	2881
 * 48	2736
 * 156	1569
 * 168	1487
 * 160	1483
 * 28	1446
 * 1608	1379
 * 184	1305
 * 552	1270
 * 64	934
 * 320	891
 * 8	754
 *
 * Conclusions: more than the half of all allocations are <= 32 bytes.
 * But of these about 150,000 blocks, only a maximum of about 6,000 are
 * allocated together (including memory leaks..).
 * On the other side, only 935 of the 191573 or 0.5% were larger than 8kB
 * (362 or 0.2% larger than 16k).
 *
 * What makes the server really grow is the fragmentation of the heap,
 * and the fact that it can't shrink.
 * To cure this, we do the following:
 * - large blocks (>=11k) are mmapped on xalloc, and unmapped on xfree,
 *   so we don't need any free lists etc.
 *   As this needs 2 system calls, we only do this for the quite
 *   infrequent () large blocks.
This seems like a nice idea
 * - instead of reinventing the wheel, we use system malloc for medium
 *   sized blocks (>256, <11k).
 * - for small blocks (<=256) we use an other approach:
 *   As we need many small blocks, and most ones for a short time,
 *   we don't go through the system malloc:
 *   for each fixed sizes a seperate list of free blocks is kept.
 *   to KISS (Keep it Small and Simple), we don't free them
 *   (not freeing a block of 32 bytes won't be worse than having fragmented
 *   a larger area on allocation).
 *   This way, we (almost) allways have a fitting free block right at hand,
 *   and don't have to walk any lists.
 */

We do it like this:

LAYOUT of a suballocated heap block
WORD  magic 
BYTE  allocations
BYTE  size
DWORD next suballoc block  

DWORD next block
DWORD prev block
[block]

LAYOUT of an allocated heap block

DWORD magic 
DWORD size

DWORD next block
DWORD prev block
[block]


LAYOUT of a memory mapping reference
DWORD    size
LPVOID   mapping
LPVOID   next ref
LPVOID   prev ref