/* 
 * kernel/kernel32.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 * 
 * function stubs
 * TODO: make API logging aware of structures and flags
 *  
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include "wfc.h"

#define THREAD_ENTER /* */
#define THREAD_LEAVE /* */


#ifdef __cplusplus
extern "C" {
#endif

/* HINT: don't make this inline, as it is included in a lot of functions */
static LPSTR BOOLSTR( BOOL val )
{ 
   return (val?"TRUE":"FALSE");
} 
 
BOOL WINAPI CloseHandle( HANDLE hobject )
{
   BOOL retval;
   
   THREAD_ENTER
      API_LOG( API_OUT, "CloseHandle( 0x%X )\n", (unsigned) hobject); 
      retval = __CloseHandle(hobject);
      API_LOG( API_OUT, "CloseHandle returns %s\n", BOOLSTR(retval));
   THREAD_LEAVE
   
   return retval;
}


BOOL WINAPI CopyFileA( LPCSTR poldname, LPCSTR pnewname,
				BOOL failifexist )
{
   BOOL retval;
   
   THREAD_ENTER
      API_LOG( API_OUT, "CopyFileA( %s, %s, %s )\n", poldname, pnewname, 
               BOOLSTR(failifexist)); 
      retval = __CopyFileA( poldname, pnewname, failifexist );
      API_LOG( API_OUT, "CopyFileA returns %s\n", BOOLSTR(retval));
   THREAD_LEAVE
   return retval;
}				
 
BOOL WINAPI CreateDirectoryA( LPCSTR dirname, 
					LPSECURITY_ATTRIBUTES lpsa)
{
   BOOL retval;
   
   THREAD_ENTER
      API_LOG( API_OUT, "CreateDirectoryA( %s, 0x%X )\n", dirname, 
                                          (unsigned) lpsa );                
      retval = __CreateDirectoryA( dirname, lpsa );      
      API_LOG( API_OUT, "CreateDirectoryA returns %s\n", BOOLSTR(retval));
   THREAD_LEAVE
   
   return retval;
}


HANDLE WINAPI CreateFileA( LPCSTR filename, DWORD access,
				DWORD share, LPSECURITY_ATTRIBUTES lpsa,
				DWORD create, DWORD attrib, HANDLE htemp)
{
   HANDLE retval;
   
   THREAD_ENTER

      API_LOG( API_OUT, 
              "CreateFileA( %s, 0x%X, 0x%X, 0x%X, 0x%X, 0x%X, 0x%X )\n", 
              filename, (unsigned) access, (unsigned) share, (unsigned) lpsa, 
              (unsigned) create, (unsigned) attrib, (unsigned) htemp);  

      retval = __CreateFileA( filename, access, share, lpsa, create, 
      			attrib, htemp );      

      API_LOG( API_OUT, "CreateFileA returns 0x%X\n", (unsigned) retval);

   THREAD_LEAVE
   
   return retval;
}				

HANDLE WINAPI CreateFileMappingA( HANDLE hfile, 
				LPSECURITY_ATTRIBUTES lpsa,
				DWORD protect, DWORD sizehigh, DWORD sizelow,
				LPCSTR name )
{
   HANDLE retval;
   
   THREAD_ENTER

      API_LOG( API_OUT, 
               "CreateFileMappingA( 0x%X, 0x%X, 0x%X, 0x%X, 0x%X, %s )\n", 
               (unsigned) hfile, (unsigned) lpsa, (unsigned) protect, 
               (unsigned) sizehigh, (unsigned) sizelow, name);  

      retval = __CreateFileMappingA( hfile, lpsa, protect,
                                   sizehigh, sizelow, name );      

      API_LOG( API_OUT, "CreateFileMappingA returns 0x%X\n", 
               (unsigned) retval);

   THREAD_LEAVE
   
   return retval;
}		   

BOOL WINAPI CreateProcessA( LPCSTR appname, LPSTR cmdline,
                                     LPSECURITY_ATTRIBUTES pprocsa,
                                     LPSECURITY_ATTRIBUTES pthreadsa,    
                                     BOOL inherit, DWORD create, LPVOID envir,
                                     LPCSTR curdir, LPSTARTUPINFOA lpsi,
                                     LPPROCESS_INFORMATION lpinfo )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"CreateProcessA( %s, %s, 0x%X, 0x%X,"
                       " %s, 0x%X, 0x%X, %s, 0x%X, 0x%X )\n", 
               appname, cmdline, (unsigned) pprocsa, (unsigned) pthreadsa, 
               BOOLSTR(inherit), (unsigned) create, (unsigned) envir, curdir,
               (unsigned) lpsi, (unsigned) lpinfo );  

      retval = __CreateProcessA( appname, cmdline, pprocsa, pthreadsa, inherit,
      				create, envir, curdir, lpsi, lpinfo );      

      API_LOG( API_OUT, "CreateProcess returns %s\n", BOOLSTR(retval));

   THREAD_LEAVE
   
   return retval;
}    

HANDLE WINAPI CreateThread( LPSECURITY_ATTRIBUTES lpsa, 
				DWORD stacksize,
				LPTHREAD_START_ROUTINE startfunc,
	     			LPVOID startparam, DWORD create, 
	     			LPDWORD pthreadid )	     
{
   HANDLE retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"CreateThread( 0x%X, 0x%X, 0x%X, 0x%X, 0x%X, 0x%X )\n", 
               (unsigned) lpsa, (unsigned) stacksize, (unsigned) startfunc, 
               (unsigned) startparam, (unsigned) create, (unsigned) pthreadid);

      retval = __CreateThread( lpsa, stacksize, startfunc, startparam, create, 
                             pthreadid );
 
      API_LOG( API_OUT, "CreateThread returns 0x%X\n", (unsigned) retval);

   THREAD_LEAVE
   
   return retval;
}

VOID WINAPI DeleteCriticalSection( LPCRITICAL_SECTION pcritical)
{
   THREAD_ENTER

      API_LOG( API_OUT,"DeleteCriticalSection( 0x%X )\n", 
               (unsigned) pcritical );

      __DeleteCriticalSection( pcritical );
 
   THREAD_LEAVE
}

BOOL WINAPI DeleteFileA( LPCSTR filename )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"DeleteFileA( %s )\n", filename );

      retval = __DeleteFileA( filename );
 
      API_LOG( API_OUT, "DeleteFileA returns %s", BOOLSTR(retval));
   THREAD_LEAVE

   return retval;
}

BOOL WINAPI DeviceIoControl( HANDLE hdevice, DWORD iocode, 
                                      LPVOID pinbuf, DWORD inbufsize,
		                      LPVOID poutbuf, DWORD outbufsize, 
		                      LPDWORD preturned, LPOVERLAPPED lpol)
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"DeviceIoControl( 0x%X, 0x%X, 0x%X, %d,"
                       " 0x%X, %d, 0x%X, 0x%X )\n", 
               (unsigned) hdevice, (unsigned) iocode, (unsigned) pinbuf, 
               (unsigned) inbufsize, (unsigned) poutbuf, 
               (unsigned) outbufsize, (unsigned) preturned, (unsigned) lpol);

      retval = __DeviceIoControl( hdevice, iocode, pinbuf, inbufsize, poutbuf, 
                                outbufsize, preturned, lpol);
 
      API_LOG( API_OUT, "DeviceIoControl returns %s\n", BOOLSTR(retval));

   THREAD_LEAVE
   
   return retval;

}	

BOOL WINAPI DuplicateHandle( HANDLE hsrcproc, HANDLE hsource,
		                      HANDLE hdestproc, LPHANDLE phdest,
		                      DWORD access, BOOL inherit, DWORD opts)
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"DuplicateHandle( 0x%X, 0x%X, 0x%X, 0x%X,"
                       " 0x%X, %s, 0x%X )\n", 
               (unsigned) hsrcproc, (unsigned) hsource, (unsigned) hdestproc, 
               (unsigned) phdest, (unsigned) access, BOOLSTR(inherit), 
               (unsigned) opts);

      retval = __DuplicateHandle( hsrcproc, hsource, hdestproc, phdest, 
                                  access, inherit, opts);
 
      API_LOG( API_OUT, "DuplicateHandle returns %s\n", BOOLSTR(retval));

   THREAD_LEAVE
   
   return retval;

}


VOID WINAPI EnterCriticalSection( LPCRITICAL_SECTION pcritical)
{
   THREAD_ENTER

      API_LOG( API_OUT,"EnterCriticalSection( 0x%X )\n", 
               (unsigned) pcritical );

      __EnterCriticalSection( pcritical );
 
   THREAD_LEAVE
}

VOID WINAPI ExitProcess( UINT exitcode )
{
   THREAD_ENTER

   API_LOG( API_OUT,"ExitProcess( %d )\n", (int) exitcode );

   __ExitProcess( exitcode );

   DBG_LOG( DBG_OUT, "ExitProcess returned, should not happen\n"); 
   
   while(1)
     /* dunno what next */;
}

VOID WINAPI ExitThread( DWORD exitcode )
{
   THREAD_ENTER

   API_LOG( API_OUT,"ExitThread( %d )\n", (int) exitcode );

   __ExitThread( exitcode );
 
   DBG_LOG( DBG_OUT, "ExitThread returned, should not happen\n");
   
   while(1)
      /* dunno what next */;
}

BOOL WINAPI FindClose( HANDLE hfind )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FindClose( 0x%X )\n", (unsigned) hfind );

      retval = __FindClose( hfind );
 
      API_LOG( API_OUT, "FindClose returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

HANDLE WINAPI FindFirstFileA( LPCSTR filename,
                                       LPWIN32_FIND_DATAA pfinddata )
{
   HANDLE retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FindFirstFileA( %s, 0x%X )\n", filename, 
               (unsigned) pfinddata );

      retval = __FindFirstFileA( filename, pfinddata );
 
      API_LOG( API_OUT, "FindFirstFileA returns 0x%X", (unsigned) retval);

   THREAD_LEAVE

   return retval;

}
                                       
BOOL WINAPI FindNextFileA( HANDLE hfind,
                                    LPWIN32_FIND_DATAA pfinddata )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FindNextFileA( 0x%X, 0x%X )\n", 
               (unsigned) hfind, (unsigned) pfinddata );

      retval = __FindNextFileA( hfind, pfinddata );
 
      API_LOG( API_OUT, "FindNextFileA returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI FlushFileBuffers( HANDLE hfile )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FlushFileBuffers( 0x%X )\n", (unsigned) hfile );

      retval = __FlushFileBuffers( hfile );
 
      API_LOG( API_OUT, "FlushFileBuffers returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI FlushViewOfFile( LPCVOID pbase, DWORD size )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FlushViewOfFile( 0x%X, 0x%X )\n", 
               (unsigned) pbase, (unsigned) size);

      retval = __FlushViewOfFile( pbase, size );
 
      API_LOG( API_OUT, "FlushViewOfFile returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI FreeEnvironmentStringsA( LPSTR penvblock )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FreeEnvironmentStringsA( 0x%X )\n", 
              (unsigned) penvblock );

      retval = __FreeEnvironmentStringsA( penvblock );
 
      API_LOG( API_OUT, "FreeEnvironmentStringsA returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}
	       		      
BOOL WINAPI FreeLibrary( HINSTANCE hmodule )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"FreeLibrary( 0x%X )\n", (unsigned) hmodule );

      retval = __FreeLibrary( hmodule );
 
      API_LOG( API_OUT, "FreeLibrary returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

VOID WINAPI FreeLibraryAndExitThread( HMODULE hmodule, 
						DWORD exitcode )
{
   THREAD_ENTER

   API_LOG( API_OUT,"FreeLibraryAndExitThread( 0x%X, %d )\n", 
           (unsigned) hmodule, (unsigned) exitcode );

   __FreeLibrary( hmodule );
   __ExitThread( exitcode );
 
   DBG_LOG( DBG_OUT, "ExitThread returned, should not happen\n");
  
   while(1)
      /* dunno */;
}

LPSTR WINAPI GetCommandLineA( VOID )
{
   LPSTR retval;
   
   THREAD_ENTER

      retval = __GetCommandLineA();
 
      API_LOG( API_OUT, "GetCommandLineA returns %s", retval);

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI GetComputerNameA ( LPSTR pbuffer, LPDWORD pbufsize )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetComputerNameA( 0x%X, 0x%X )\n", 
               (unsigned) pbuffer, (unsigned) pbufsize );

      retval = __GetComputerNameA( pbuffer, pbufsize );
 
      API_LOG( API_OUT, "GetComputerNameA returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}   

DWORD WINAPI GetCurrentDirectoryA( DWORD bufsize, LPSTR pbuffer )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetCurrentDirectoryA( %d, 0x%X )\n", 
              (int) bufsize, (unsigned) pbuffer );

      retval = __GetCurrentDirectoryA( bufsize, pbuffer );
 
      API_LOG( API_OUT, "GetCurrentDirectoryA returns %d", (int) retval);

   THREAD_LEAVE

   return retval;
}

HANDLE WINAPI GetCurrentProcess( VOID )
{
   HANDLE retval;
   
   THREAD_ENTER

      retval=__GetCurrentProcess();
 
      API_LOG( API_OUT, "GetCurrentProcess returns 0x%X\n",
               (unsigned) retval);
                
   THREAD_LEAVE

   return retval;
}

DWORD WINAPI GetCurrentProcessId( VOID )
{
   DWORD retval;
   
   THREAD_ENTER

      retval=__GetCurrentProcessId();
 
      API_LOG( API_OUT, "GetCurrentProcessId returns %d\n",
               (unsigned) retval);
                
   THREAD_LEAVE

   return retval;
}

HANDLE WINAPI GetCurrentThread( VOID )
{  
   HANDLE retval;
   
   THREAD_ENTER

      retval=__GetCurrentThread();
 
      API_LOG( API_OUT, "GetCurrentThread returns 0x%X\n",
               (unsigned) retval);
                
   THREAD_LEAVE

   return retval;
}

DWORD WINAPI GetCurrentThreadId( VOID )
{
   DWORD retval;
   
   THREAD_ENTER

      retval=__GetCurrentThreadId();
 
      API_LOG( API_OUT, "GetCurrentThreadId returns %d\n",
               (unsigned) retval);
                
   THREAD_LEAVE

   return retval;
}

UINT WINAPI GetDriveTy__ImageA( LPCSTR rootname )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetDriveTy__ImageA( %s )\n", rootname );

      retval = __GetDriveTy__ImageA( rootname );
 
      API_LOG( API_OUT, "GetDriveTy__ImageA returns %ul", retval);

   THREAD_LEAVE

   return retval;
}

LPSTR WINAPI GetEnvironmentStringsA( VOID )
{
   LPSTR retval;
   
   THREAD_ENTER

      retval=__GetEnvironmentStringsA();
 
      API_LOG( API_OUT, "GetEnvironmentStringsA returns %0xX\n",
               (unsigned) retval);
                
   THREAD_LEAVE

   return retval;
}

DWORD WINAPI GetEnvironmentVariableA( LPCSTR key, LPSTR pbuffer,
                                               DWORD bufsize )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetEnvironmentVariableA( %s, 0x%X, %u )\n", key, 
               (unsigned) pbuffer, (unsigned) bufsize );

      retval = __GetEnvironmentVariableA( key, pbuffer, bufsize );
 
      API_LOG( API_OUT, "GetEnvironmentVariableA returns %u", 
              (unsigned) retval);

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI GetExitCodeProcess( HANDLE hprocess, LPDWORD pexitcode )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetExitCodeProcess( 0x%X, 0x%X )\n", 
               (unsigned) hprocess, (unsigned) pexitcode );

      retval = __GetExitCodeProcess( hprocess, pexitcode );
 
      API_LOG( API_OUT, "GetExitCodeProcess returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI GetExitCodeThread( HANDLE hthread, LPDWORD pexitcode )
{
   BOOL retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetExitCodeThread( 0x%X, 0x%X )\n", 
               (unsigned) hthread, (unsigned) pexitcode );

      retval = __GetExitCodeThread( hthread, pexitcode );
 
      API_LOG( API_OUT, "GetExitCodeThread returns %u", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

DWORD WINAPI GetFileAttributesA( LPCSTR filename )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetFileAttributesA( %s )\n", 
               filename );

      retval = __GetFileAttributesA( filename );
 
      API_LOG( API_OUT, "GetFileAttributes returns 0x%Xl\n", retval);

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI GetFileInformationByHandle( HANDLE hfile,
                                         LPBY_HANDLE_FILE_INFORMATION pinfo )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetFileInformationByHandle( 0x%X, 0x%X )\n", 
               (unsigned) hfile, (unsigned) pinfo );

      retval = __GetFileInformationByHandle( hfile, pinfo );
 
      API_LOG( API_OUT, "GetFileInformationByHandle returns %s\n", 
               BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}			   

DWORD WINAPI GetFullPathNameA( LPCSTR filename, DWORD bufsize, 
                                        LPSTR pbuffer, LPSTR *plastpart)
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetFullPathNameA( %s, %ul, 0x%X, 0x%X )\n", 
               filename, bufsize, (unsigned) pbuffer, (unsigned) plastpart );

      retval = __GetFullPathNameA( filename, bufsize, pbuffer, plastpart );
 
      API_LOG( API_OUT, "GetFullPathNameA returns %ul\n", retval);

   THREAD_LEAVE

   return retval;
}                               


DWORD WINAPI GetFileType( HANDLE hfile )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetFileType( 0x%X )\n", 
               (unsigned) hfile );

      retval = __GetFileType( hfile );
 
      API_LOG( API_OUT, "GetFileAttributes returns 0x%Xl\n", retval);

   THREAD_LEAVE

   return retval;
}

DWORD WINAPI GetFileSize( HANDLE hfile, LPDWORD psizehigh )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetFileSize( 0x%X, 0x%X )\n", 
               (unsigned) hfile, (unsigned) psizehigh );

      retval = __GetFileSize( hfile, psizehigh );
 
      API_LOG( API_OUT, "GetFileSize returns 0x%Xl\n", retval);

   THREAD_LEAVE

   return retval;
}

BOOL WINAPI GetFileTime( HANDLE hfile, LPFILETIME pctime,
                                  LPFILETIME patime, LPFILETIME pmtime )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetFileTime( 0x%X, 0x%X, 0x%X, 0x%X )\n",
               (unsigned) hfile, (unsigned) pctime, (unsigned) patime, 
               (unsigned) pmtime);

      retval = __GetFileTime( hfile, pctime, patime, pmtime );
 
      API_LOG( API_OUT, "GetFileSize returns %s\n", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}                                  

BOOL WINAPI GetHandleInformation( HANDLE handle, LPDWORD pflags )
{
   DWORD retval;
   
   THREAD_ENTER

      API_LOG( API_OUT,"GetHandleInformation( 0x%X, 0x%X )\n",
               (unsigned) handle, (unsigned) pflags );

      retval = __GetHandleInformation( handle, pflags );
 
      API_LOG( API_OUT, "GetHandleInformation returns %s", BOOLSTR(retval));

   THREAD_LEAVE

   return retval;
}

DWORD WINAPI GetLastError(VOID)
{
   DWORD retval;
   
   THREAD_ENTER

      retval = __GetLastError();
 
      API_LOG( API_OUT, "GetLastError returns 0x%Xl\n", retval );

   THREAD_LEAVE

   return retval;
}

VOID WINAPI GetLocalTime( LPSYSTEMTIME psystime )
{
   THREAD_ENTER
      __GetLocalTime(psystime); 
   THREAD_LEAVE
}

DWORD WINAPI GetLogicalDrives(VOID)
{
   DWORD retval;   
   THREAD_ENTER
      retval = __GetLogicalDrives(); 
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI GetLogicalDriveStringsA( DWORD bufsize, LPSTR pbuffer )
{
   DWORD retval;   
   THREAD_ENTER
      retval = __GetLogicalDriveStringsA( bufsize, pbuffer ); 
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI GetModuleFileNameA( HINSTANCE hmodule, LPSTR filename,
                                          DWORD bufsize)		   			
{
   DWORD retval;   
   THREAD_ENTER
      retval = __GetModuleFileNameA( hmodule, filename, bufsize ); 
   THREAD_LEAVE
   return retval;
}

HMODULE WINAPI GetModuleHandleA( LPCSTR modulename )
{
   HMODULE retval;   
   THREAD_ENTER
      retval = __GetModuleHandleA( modulename ); 
   THREAD_LEAVE
   return retval;
}


FARPROC WINAPI GetProcAddress( HINSTANCE hmodule, LPCSTR procname)
{
   FARPROC retval;   
   THREAD_ENTER
      retval = __GetProcAddress( hmodule, procname ); 
   THREAD_LEAVE
   return retval;
}	      


DWORD WINAPI GetShortPathNameA( LPCSTR longpath, LPSTR  shortpath, 
                                         DWORD  bufsize )
{
   DWORD retval;   
   THREAD_ENTER
      retval = __GetShortPathNameA( longpath, shortpath, bufsize ); 
   THREAD_LEAVE
   return retval;
}

VOID WINAPI GetStartupInfoA( LPSTARTUPINFOA pstartupinfo )
{
   THREAD_ENTER
      __GetStartupInfoA( pstartupinfo ); 
   THREAD_LEAVE
}


HANDLE WINAPI GetStdHandle( DWORD what )
{
   HANDLE retval;   
   THREAD_ENTER
      retval = __GetStdHandle( what ); 
   THREAD_LEAVE
   return retval;
}

UINT WINAPI GetSystemInfo( LPSYSTEM_INFO psysinfo )
{
   UINT retval;
   THREAD_ENTER
      retval=__GetSystemInfo( psysinfo );
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI GetVersion( VOID )
{
   DWORD retval; 
   THREAD_ENTER
      retval = __GetVersion();       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI GetVersionExA( LPOSVERSIONINFOA pverinfo )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __GetVersionExA(pverinfo);        
   THREAD_LEAVE
   return retval;
}


VOID WINAPI InitializeCriticalSection( LPCRITICAL_SECTION pcritical )
{   
   THREAD_ENTER
      __InitializeCriticalSection( pcritical );       
   THREAD_LEAVE
}


/* the interlocked functions actually don't need any locking */

LONG WINAPI InterlockedDecrement( LPLONG pval )
{
   LONG retval;   
   THREAD_ENTER
      retval = __InterlockedDecrement( pval );       
   THREAD_LEAVE
   return retval;
}

LONG WINAPI InterlockedExchange( LPLONG ptarget, LONG value )
{
   LONG retval;   
   THREAD_ENTER
      retval = __InterlockedExchange( ptarget, value );       
   THREAD_LEAVE
   return retval;
}

LONG WINAPI InterlockedIncrement( LPLONG pval )
{
   LONG retval;   
   THREAD_ENTER
      retval = __InterlockedIncrement( pval );       
   THREAD_LEAVE
   return retval;
}

VOID WINAPI LeaveCriticalSection( LPCRITICAL_SECTION pcritical )
{
   THREAD_ENTER
      __LeaveCriticalSection( pcritical );       
   THREAD_LEAVE
}

HMODULE WINAPI LoadLibraryA( LPCSTR libname )
{
   HMODULE retval;   
   THREAD_ENTER
      retval = __LoadLibraryA( libname );       
   THREAD_LEAVE
   return retval;
}


HMODULE WINAPI LoadLibraryExA( LPCSTR libname, HANDLE hfile, 
                                        DWORD flags )
{
   HMODULE retval;   
   THREAD_ENTER
      retval = __LoadLibraryExA( libname, hfile, flags );      
   THREAD_LEAVE
   return retval;
}                                        

DWORD WINAPI LoadModule( LPCSTR modname, LPVOID pparamblock )
{
   DWORD retval;   
   THREAD_ENTER
      retval = __LoadModule( modname, pparamblock );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI LockFile( HANDLE hfile, DWORD offsetlow, 
                               DWORD offsethigh, DWORD sizelow, 
                               DWORD sizehigh )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __LockFile( hfile, offsetlow, offsethigh, sizelow, sizehigh );       
   THREAD_LEAVE
   return retval;
}                               

BOOL WINAPI LockFileEx( HANDLE hfile, DWORD flags, DWORD reserved,
	                         DWORD sizelow, DWORD sizehigh, 
	                         LPOVERLAPPED pol )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __LockFileEx( hfile, flags, reserved, sizelow, sizehigh, pol );       
   THREAD_LEAVE
   return retval;
}	                         

LPVOID WINAPI MapViewOfFile( HANDLE hmapping, DWORD access, 
                                      DWORD offsethigh, DWORD offsetlow,
                                      DWORD size )
{
   LPVOID retval;   
   THREAD_ENTER
      retval = __MapViewOfFile( hmapping, access, offsetlow, offsethigh, 
                                size );       
   THREAD_LEAVE
   return retval;
}

LPVOID WINAPI MapViewOfFileEx(	HANDLE hmapping, DWORD access,
                                        DWORD offsethigh, DWORD offsetlow,
                                        DWORD size, LPVOID address )
{
   LPVOID retval;   
   THREAD_ENTER
      retval = __MapViewOfFileEx( hmapping, access, offsetlow, offsethigh, 
                                  size, address );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI MoveFileA( LPCSTR oldname, LPCSTR newname )
{
   BOOL retval;  
   THREAD_ENTER
      retval = __MoveFileA( oldname, newname );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI MoveFileExA( LPCSTR oldname, LPCSTR newname, DWORD flags )
{
   BOOL retval;    
   THREAD_ENTER
      retval = __MoveFileExA( oldname, newname, flags );       
   THREAD_LEAVE
   return retval;
}


int WINAPI MulDiv( int number, int numerator, int denominator )
{
   int retval;   
   THREAD_ENTER
      retval = __MulDiv( number, numerator, denominator );       
   THREAD_LEAVE
   return retval;
}

HANDLE WINAPI OpenFile( LPCSTR filename, LPOFSTRUCT pofstruct,
                                 UINT style )
{
   HANDLE retval;   
   THREAD_ENTER
      retval = __OpenFile( filename, pofstruct, style );       
   THREAD_LEAVE
   return retval;
}

VOID WINAPI OutputDebugStringA( LPCSTR poutstr )
{   
   THREAD_ENTER
      __OutputDebugStringA( poutstr );       
   THREAD_LEAVE
} 

VOID WINAPI RaiseException( DWORD exccode, DWORD flags, 
                                     DWORD argcount, CONST DWORD *pargs )
{
   THREAD_ENTER
      __RaiseException( exccode, flags, argcount, pargs );       
   THREAD_LEAVE   
}

BOOL WINAPI ReadFile( HANDLE hfile, LPVOID pbuffer, DWORD toread,
                               LPDWORD pread, LPOVERLAPPED pol )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __ReadFile( hfile, pbuffer, toread, pread, pol );       
   THREAD_LEAVE
   return retval;   
}

BOOL WINAPI ReadFileEx( HANDLE hfile, LPVOID pbuffer, DWORD toread,
                                 LPOVERLAPPED pol,
                                 LPOVERLAPPED_COMPLETION_ROUTINE polcr )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __ReadFileEx( hfile, pbuffer, toread, pol, polcr );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI RemoveDirectoryA( LPCSTR dirname )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __RemoveDirectoryA( dirname );       
   THREAD_LEAVE
   return retval;
}

#ifdef 0  /* already defined */
VOID WINAPI RtlFillMemory ( PVOID pdest, DWORD len, BYTE  fill )
{
   /* XXX: should use memset directly */
   RtlFillMemory( pdest, len, fill ); 
}

VOID WINAPI RtlMoveMemory( PVOID pdest, CONST VOID *psrc, DWORD len )
{
   THREAD_ENTER
      RtlMoveMemory(pdest, psrc, len);       
   THREAD_LEAVE
}

VOID WINAPI RtlZeroMemory ( PVOID pdest, DWORD len )
{
   THREAD_ENTER
      RtlZeroMemory(pdest, len);       
   THREAD_LEAVE
}
#endif

BOOL WINAPI SetCurrentDirectoryA( LPCSTR dirname )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __SetCurrentDirectoryA( dirname );       
   THREAD_LEAVE
   return retval;
}


BOOL WINAPI SetEndOfFile( HANDLE hfile )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __SetEndOfFile( hfile );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI SetEnvironmentVariableA( LPCSTR key, LPCSTR value )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __SetEnvironmentVariableA( key, value );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI SetFileAttributesA( LPCSTR filename, DWORD attrib )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __SetFileAttributesA( filename, attrib );       
   THREAD_LEAVE
   return retval;
}

UINT WINAPI SetHandleCount( UINT count )
{
   UINT retval;   
   THREAD_ENTER
      retval = __SetHandleCount( count );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI SetHandleInformation( HANDLE handle, DWORD mask, 
                                           DWORD flags )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __SetHandleInformation( handle, mask, flags );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI SetFilePointer( HANDLE hfile, LONG offset,  
                                      PLONG poffsethigh, DWORD whence )
{
   DWORD retval;   
   THREAD_ENTER
      retval = __SetFilePointer( hfile, offset, poffsethigh, whence );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI SetFileTime( HANDLE hfile, CONST FILETIME *pctime,
                                  CONST FILETIME *patime, 
                                  CONST FILETIME *pmtime )
{
   BOOL retval;   
   THREAD_ENTER
      retval = __SetFileTime( hfile, pctime, patime, pmtime );       
   THREAD_LEAVE
   return retval;
}
		     		     
VOID WINAPI SetLastError( DWORD error )
{
   THREAD_ENTER
      __SetLastError( error );       
   THREAD_LEAVE
}

BOOL WINAPI SetLocalTime( CONST SYSTEMTIME *ptime )
{
   BOOL retval;
   THREAD_ENTER
      retval = __SetLocalTime( ptime );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI SetStdHandle( DWORD what, HANDLE handle )
{
   BOOL retval;
   THREAD_ENTER
      retval = __SetStdHandle( what, handle );       
   THREAD_LEAVE
   return retval;
}
		  
LPTOP_LEVEL_EXCEPTION_FILTER WINAPI  SetUnhandledExceptionFilter(
   LPTOP_LEVEL_EXCEPTION_FILTER ptopexc )
{
   LPTOP_LEVEL_EXCEPTION_FILTER retval;
   THREAD_ENTER
      retval = __SetUnhandledExceptionFilter( ptopexc );       
   THREAD_LEAVE
   return retval;
}

VOID WINAPI Sleep( DWORD useconds )
{
   THREAD_ENTER
      __Sleep( useconds );       
   THREAD_LEAVE
}

DWORD WINAPI SleepEx( DWORD useconds, BOOL alertable )
{
   DWORD retval;
   THREAD_ENTER
      retval=__SleepEx( useconds, alertable );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI TlsAlloc( VOID )
{
   DWORD retval;
   THREAD_ENTER
      retval=TlsAlloc();       
   THREAD_LEAVE
   return retval; 
}

BOOL WINAPI TlsFree( DWORD index )
{
   BOOL retval;
   THREAD_ENTER
      retval=TlsFree(index);       
   THREAD_LEAVE
   return retval;
}

LPVOID WINAPI TlsGetValue( DWORD index )
{
   LPVOID retval;
   THREAD_ENTER
      retval=TlsGetValue(index);       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI TlsSetValue( DWORD index, LPVOID pvalue)
{
   BOOL retval;
   THREAD_ENTER
      retval=TlsSetValue( index, pvalue);       
   THREAD_LEAVE
   return retval;
}

LONG WINAPI UnhandledExceptionFilter( LPEXCEPTION_POINTERS pexcept )
{
   LONG retval;
   THREAD_ENTER
      retval=UnhandledExceptionFilter( pexcept );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI UnlockFile( HANDLE hfile, DWORD offsetlow,
                                 DWORD offsethigh, DWORD sizelow,
                                 DWORD sizehigh )
{
   BOOL retval;
   THREAD_ENTER
      retval=UnlockFile( hfile, offsetlow, offsethigh, sizelow, sizehigh );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI UnlockFileEx( HANDLE hfile, DWORD reserved, 
                                   DWORD sizelow, DWORD sizehigh, 
                                   LPOVERLAPPED pol )
{
   BOOL retval;
   THREAD_ENTER
      retval=UnlockFileEx( hfile, reserved, sizelow, sizehigh, pol );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI UnmapViewOfFile( LPVOID address )
{
   BOOL retval;
   THREAD_ENTER
      retval=UnmapViewOfFile( address );       
   THREAD_LEAVE
   return retval;
}

LPVOID WINAPI VirtualAlloc( LPVOID address, DWORD size, DWORD type,
	                             DWORD protect )
{
   LPVOID retval;
   THREAD_ENTER
      retval=VirtualAlloc( address, size, type, protect );       
   THREAD_LEAVE
   return retval;
}	                             


BOOL WINAPI VirtualFree( LPVOID address, DWORD size, DWORD type )
{
   BOOL retval;
   THREAD_ENTER
      retval=VirtualFree( address, size, type );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI VirtualLock( LPVOID address, DWORD size )
{
   BOOL retval;
   THREAD_ENTER
      retval=VirtualLock( address, size );       
   THREAD_LEAVE
   return retval;
}


BOOL WINAPI VirtualProtect( LPVOID address, DWORD size, 
                                     DWORD newprot, PDWORD poldprot )
{
   BOOL retval;
   THREAD_ENTER
      retval=VirtualProtect( address, size, newprot, poldprot );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI VirtualProtectEx( HANDLE hprocess, LPVOID address,
                                       DWORD size, DWORD newprot,
                                       PDWORD poldprot )
{
   BOOL retval;
   THREAD_ENTER
      retval=VirtualProtectEx( hprocess, address, size, newprot, poldprot );       
   THREAD_LEAVE
   return retval;
}
		 
DWORD WINAPI VirtualQuery( LPCVOID address, 
                                    PMEMORY_BASIC_INFORMATION pbuffer,
                                    DWORD length )
{
   DWORD retval;
   THREAD_ENTER
      retval=VirtualQuery( address, pbuffer, length );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI VirtualQueryEx( HANDLE hprocess, LPCVOID address,
                                      PMEMORY_BASIC_INFORMATION pbuffer,
                                      DWORD length )
{
   DWORD retval;
   THREAD_ENTER
      retval=VirtualQueryEx( hprocess, address, pbuffer, length );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI VirtualUnlock( LPVOID address, DWORD size )
{
   BOOL retval;
   THREAD_ENTER
      retval=VirtualUnlock( address, size );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI WaitForDebugEvent( LPDEBUG_EVENT pevent, DWORD useconds )
{
   BOOL retval;
   THREAD_ENTER
      retval=WaitForDebugEvent( pevent, useconds );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI WaitForMultipleObjects( DWORD count,
                                              CONST HANDLE *phandles,
                                              BOOL waitall, DWORD useconds )
{
   DWORD retval;
   THREAD_ENTER
      retval=WaitForMultipleObjects( count, phandles, waitall, useconds );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI WaitForMultipleObjectsEx( DWORD count,
                                                CONST HANDLE *phandles,
                                                BOOL  waitall, DWORD useconds,
                                                BOOL  alertable )
{
   DWORD retval;
   THREAD_ENTER
      retval=WaitForMultipleObjectsEx( count, phandles, waitall, 
                                       useconds, alertable );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI WaitForSingleObject( HANDLE handle, DWORD useconds )
{
   DWORD retval;
   THREAD_ENTER
      retval=WaitForSingleObject( handle, useconds );       
   THREAD_LEAVE
   return retval;
}

DWORD WINAPI WaitForSingleObjectEx( HANDLE handle, DWORD useconds,
		                             BOOL alertable )
{
   DWORD retval;
   THREAD_ENTER
      retval=WaitForSingleObjectEx( handle, useconds, alertable );       
   THREAD_LEAVE
   return retval;
}

UINT WINAPI WinExec( LPCSTR cmdline, UINT cmdshow )
{
   UINT retval;
   THREAD_ENTER
      retval=WinExec( cmdline, cmdshow );       
   THREAD_LEAVE
   return retval;
}


BOOL WINAPI WriteFile( HANDLE hfile, LPCVOID pbuffer, DWORD towrite,
                                LPDWORD pwritten, LPOVERLAPPED pol )
{
   BOOL retval;
   THREAD_ENTER
      retval=WriteFile( hfile, pbuffer, towrite, pwritten, pol );       
   THREAD_LEAVE
   return retval;
}

BOOL WINAPI WriteFileEx( HANDLE hfile, LPCVOID pbuffer, 
                                  DWORD towrite, LPOVERLAPPED pol,
                                  LPOVERLAPPED_COMPLETION_ROUTINE polcr)
{
   BOOL retval;
   THREAD_ENTER
      retval=WriteFileEx( hfile, pbuffer, towrite, pol, polcr );       
   THREAD_LEAVE
   return retval;
}


long WINAPI _hread( HANDLE hfile, LPVOID pbuffer, long size )
{
   long retval;
   THREAD_ENTER
      retval=_hread( hfile, pbuffer, size );       
   THREAD_LEAVE
   return retval;
}

long WINAPI _hwrite( HANDLE hfile, LPCSTR pbuffer, long size )
{
   long retval;
   THREAD_ENTER
      retval=_hwrite( hfile, pbuffer, size );       
   THREAD_LEAVE
   return retval;
}

HANDLE WINAPI _lclose(	HANDLE hfile )
{
   HANDLE retval;
   THREAD_ENTER
      retval=_lclose( hfile );       
   THREAD_LEAVE
   return retval;
}

HANDLE WINAPI _lcreat(	LPCSTR filename, int attrib )
{
   HANDLE retval;
   THREAD_ENTER
      retval=_lcreat( filename, attrib );       
   THREAD_LEAVE
   return retval;
}

HANDLE WINAPI _lopen( LPCSTR filename, int mode )
{
   HANDLE retval;
   THREAD_ENTER
      retval=_lopen( filename, mode );       
   THREAD_LEAVE
   return retval;
}

UINT WINAPI KERNEl32__lread( HANDLE hfile, LPVOID pbuffer, UINT size )
{
   UINT retval;
   THREAD_ENTER
      retval=_lread( hfile, pbuffer, size );       
   THREAD_LEAVE
   return retval;
}

LONG WINAPI _llseek( HANDLE hfile, LONG offset, int whence )
{
   LONG retval;
   THREAD_ENTER
      retval=_llseek( hfile, offset, whence );       
   THREAD_LEAVE
   return retval;
}

UINT WINAPI _lwrite( HANDLE hfile, LPCSTR pbuffer, UINT size )
{
   UINT retval;
   THREAD_ENTER
      retval=_lwrite( hfile, pbuffer, size );       
   THREAD_LEAVE
   return retval;
}

LPSTR WINAPI lstrcatA( LPSTR dest, LPCSTR src )
{
   LPSTR retval;
   THREAD_ENTER
      retval=lstrcatA( dest, src );       
   THREAD_LEAVE
   return retval;
}

LPWSTR WINAPI lstrcatW( LPWSTR dest, LPCWSTR src )
{
   LPWSTR retval;
   THREAD_ENTER
      retval=lstrcatW( dest, src );       
   THREAD_LEAVE
   return retval;
}

int WINAPI lstrcmpA( LPCSTR str1, LPCSTR str2 )
{
   int retval;
   THREAD_ENTER
      retval=lstrcmpA( str1, str2 );       
   THREAD_LEAVE
   return retval;
}

int WINAPI lstrcmpW( LPCWSTR str1, LPCWSTR str2 )
{
   int retval;
   THREAD_ENTER
      retval=lstrcmpW( str1, str2 );       
   THREAD_LEAVE
   return retval;
}

int WINAPI lstrcmpiA( LPCSTR str1, LPCSTR str2 )
{
   int retval;
   THREAD_ENTER
      retval=lstrcmpiA( str1, str2 );       
   THREAD_LEAVE
   return retval;
}

int WINAPI lstrcmpiW( LPCWSTR str1, LPCWSTR str2 )
{
   int retval;
   THREAD_ENTER
      retval=lstrcmpiW( str1, str2 );       
   THREAD_LEAVE
   return retval;
}

LPSTR WINAPI lstrcpynA( LPSTR dest, LPCSTR src, int len )
{
   LPSTR retval;
   THREAD_ENTER
      retval=lstrcpynA( dest, src, len );       
   THREAD_LEAVE
   return retval;
}

LPWSTR WINAPI lstrcpynW( LPWSTR dest, LPCWSTR src, int len )
{
   LPWSTR retval;
   THREAD_ENTER
      retval=lstrcpynW( dest, src, len );       
   THREAD_LEAVE
   return retval;
}

LPSTR WINAPI lstrcpyA( LPSTR dest,  LPCSTR src )
{
   LPSTR retval;
   THREAD_ENTER
      retval=lstrcpyA( dest, src );       
   THREAD_LEAVE
   return retval;
}

LPWSTR WINAPI lstrcpyW( LPWSTR dest, LPCWSTR src )
{
   LPWSTR retval;
   THREAD_ENTER
      retval=lstrcpyW( dest, src );       
   THREAD_LEAVE
   return retval;
}

int WINAPI lstrlenA( LPCSTR str )
{
   int retval;
   THREAD_ENTER
      retval=lstrlenA( str );       
   THREAD_LEAVE
   return retval;
}

int WINAPI lstrlenW( LPCWSTR str )
{
   int retval;
   THREAD_ENTER
      retval=lstrlenW( str );       
   THREAD_LEAVE
   return retval;
}

#ifdef __cplusplus
}
#endif
