/*
 *  kernel/module.c
 *  Copyright (c) 1996, Onno Hovers, All rights reserved
 *
 *  module (DLL and EXE) functions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <win32/defines.h>
#include <win32/ntimage.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include <i386/segment.h>
#include "peload.h"
#include "wfc.h"
#include "teb.h"
#include "run.h"
#include "rtlmem.h"

HMODULE LoadLibraryA(LPCSTR filename)
{
   PPELOAD_FILE ppefile;

   dprintf("LoadLibraryA called\n");
   ppefile=__ImageLoadModule(filename);
   if(!ppefile)
      return 0;
   
   if(ppefile->Usage==1)
   {
      if(!__CallDll(ppefile,DLL_PROCESS_ATTACH,0))
      {
         __CallDll(ppefile,DLL_PROCESS_DETACH,0);
         __ImageFreeModule(ppefile);
      }
   }
   return (HMODULE) ppefile->HModule;
}

FARPROC GetProcAddress(HMODULE hmodule, LPCSTR pproc)
{     
   dprintf("GetProcAddress called, hmodule=0x%X, ",(unsigned) hmodule);
#ifdef DEBUG
   if(HIWORD((int) pproc))
      dprintf("function=%s\n",pproc);
   else
      dprintf("ordinal=%d\n",(int) pproc);
   fflush(stdout);
#endif   
   return __ImageGetProcAddress(hmodule, pproc);
}

DWORD GetModuleFileNameA(HMODULE hmodule, LPSTR path, DWORD size)
{
   INT          retval;
   PPELOAD_FILE ppefile;
   
   aprintf("GetModuleFileNameA( 0x%lX, 0x%lX, %ld )\n", 
           (ULONG) hmodule, (ULONG) path, size );
   if(hmodule)
      ppefile=__ImageGetModuleInfo(hmodule);
   else
      ppefile=__ImageExecutable;
      
   if(!ppefile)
      { SetLastError(ERROR_INVALID_PARAMETER); return 0;}
      
   retval=istrcpy(path,ppefile->WinName,size);
   if(retval>size)
      SetLastError(ERROR_INSUFFICIENT_BUFFER);
   return retval;
}

HMODULE GetModuleHandleA(LPCSTR filename)
{ 
   PPELOAD_FILE ppefile;
 
   dprintf("GetModuleHandleA called\n");
   if(filename==NULL)
   {
      dprintf("returning 0x%X\n",(unsigned) __ImageExecutable->HModule);
      return __ImageExecutable->HModule;
   }
   dprintf("filename=%s\n",filename);   

   ppefile=__ImageLoadModule(filename);
   dprintf("HMODULE=0x%X\n",(unsigned) ppefile->HModule);
   return (HMODULE) ppefile->HModule;
}

BOOL FreeLibrary(HMODULE hmodule)
{
   PPELOAD_FILE ppefile;
   
   dprintf("FreeLibrary called\n");
   ppefile=__ImageGetModuleInfo(hmodule);
   __CallDll(ppefile, DLL_PROCESS_DETACH, 0);
   return __ImageFreeModule(ppefile);
}