/* 
 * kernel/object.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * kernel object functions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#include<stddef.h>
#include<win32/defines.h>
#include<win32/error.h>
#include<win32/kernel.h>
#include"imalloc.h"
#include"object.h"
#include"handle.h"

BOOL CloseHandle(HANDLE handle)
{
   PKERNEL_OBJECT pobject; 
   
   pobject=__HandleGetObject(handle,0,TYPEID_OBJECT);
   if(pobject==NULL)
      return FALSE;
      
   if( PKOBJ_CALL(pobject,Kernel.Close,(pobject)) )
   {
      __HandleFree(handle);
      return TRUE;
   }
   else
      return FALSE;
}

BOOL GetHandleInformation(HANDLE handle, LPDWORD presult)
{
   PHANDLE_DATA   pdata;
   PKERNEL_OBJECT pobject; 
   
   pdata=__HandleGet(handle);
   pobject=__HandleGetIndirect(pdata,0,TYPEID_OBJECT);
   if(pobject==NULL)
      return FALSE;
   
   return PKERNEL_OBJECT_CALL(pobject, GetInfoHandle, 
                              ( pobject, pdata, presult ));
}


BOOL SetHandleInformation(HANDLE handle, DWORD mask, DWORD info)
{
   PHANDLE_DATA pdata;
   PKERNEL_OBJECT pobject; 
   
   pdata=__HandleGet(handle);
   pobject=__HandleGetIndirect(pdata,0,TYPEID_OBJECT);
   if(pobject==NULL)
      return FALSE;
   
   return PKERNEL_OBJECT_CALL(pobject, SetInfoHandle, 
                              ( pobject, pdata, mask, info ));
}



UINT SetHandleCount(UINT count)
{
   return count;
}

BOOL __ObjectExit(PKERNEL_OBJECT pobj)
{
    return TRUE;
}

BOOL __ObjectClose(PKERNEL_OBJECT pobj)
{
    return TRUE;
}

BOOL __ObjectSetInfoHandle(PKERNEL_OBJECT pobj, PHANDLE_DATA phand, 
                      DWORD mask, DWORD info)
{  
   phand->Access = (info&mask) >>24;
   return TRUE;
}

BOOL __ObjectGetInfoHandle(PKERNEL_OBJECT pobj, PHANDLE_DATA phand,
                      PDWORD pinfo)
{
   *pinfo= (phand->Access) >> 24;
   return TRUE;
}

BOOL __ObjectWait(HANDLE hobj, PKERNEL_OBJECT pobj)
{
   return FALSE;
}

BOOL __ObjectDuplicate(HANDLE hsrcproc, HANDLE hsrc, HANDLE hdestproc, 
                  PHANDLE phdest, DWORD access, BOOL inherit, DWORD options)
{
   return FALSE;
}

BOOL __ObjectIsType(PKERNEL_OBJECT pobj, ULONG typeid)
{
    if((pobj->Kernel.Magic==MAGIC_OBJECT)&&(typeid==TYPEID_OBJECT))
       return TRUE;
    else
       return FALSE;   
}
