/* 
 *  kernel/peload.c
 *  Copyright (c) 1996, Onno Hovers, All rights reserved
 *
 *  based on wine's loader/pe_image.c
 *  Copyright	1994	Eric Youngdale & Erik Bos
 *  Copyright	1995	Martin von Loewis
 *
 *  Win32 Portable Executable loader
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include <win32/ntimage.h>
#include "wfc.h"
#include "imalloc.h"
#include "rtlmem.h" 
#include "fs.h"
#include "virtual.h"
#include "peload.h"
#include "exe.h"
#include "error.h"

#ifdef __cplusplus
extern "C" {
#endif

PPELOAD_FILE __ImageLoadModule(LPCSTR filename);

PPELOAD_FILE __ImageFiles         = NULL;
PPELOAD_FILE __ImageExecutable    = NULL;
INT          __ImageBuiltinModule = 16;

VOID __ImageInitBuiltin(PIMAGE_EXPORT_DIRECTORY pexports, LPSTR modname)
{
   PPELOAD_FILE ppefile;
      
   ppefile=imalloc(sizeof(PELOAD_FILE));
   memset(ppefile,0,sizeof(PELOAD_FILE));
   
   ppefile->Exports=pexports;   
   ppefile->WinName=modname;
   ppefile->HModule=(HMODULE) __ImageBuiltinModule;
   ppefile->Next=__ImageFiles;
   __ImageFiles=ppefile;
   
   __ImageBuiltinModule++;
}

/* XXX: these should be in a header file */
extern IMAGE_EXPORT_DIRECTORY KERNEL32_Exports;
extern IMAGE_EXPORT_DIRECTORY ADVAPI32_Exports;
extern IMAGE_EXPORT_DIRECTORY USER32_Exports;
extern IMAGE_EXPORT_DIRECTORY NTDLL_Exports; 

BOOL __ImageInit(VOID)
{
   __ImageInitBuiltin(&KERNEL32_Exports,"KERNEL32.DLL");
   __ImageInitBuiltin(&ADVAPI32_Exports,"ADVAPI32.DLL");
   __ImageInitBuiltin(&USER32_Exports,"USER32.DLL");  
   __ImageInitBuiltin(&NTDLL_Exports,"NTDLL.DLL");
   return TRUE;
}

#ifdef 0
VOID __ImageDumpExports(PIMAGE_EXPORT_DIRECTORY pexports, LPVOID loadptr)
{ 
  LPSTR  Module;
  INT   i;
  UWORD * ordinal;
  ULONG * function;
  UBYTE ** name, *ename;

  Module = ((char *) load_addr) + pe_exports->Name;
  dprintf("\n*******EXPORT DATA*******\n"
          "Module name is %s, %ld functions, %ld names\n", 
	   Module,
           pexports->Number_Of_Functions,
           pexports->Number_Of_Names);

  ordinal  = (UWORD *) (((char *) load_addr) + 
                         (int) pexports->AddressOfNameOrdinals);
  function = (ULONG *)  (((char *) load_addr) + 
                         (int) pexports->AddressOfFunctions);
  name     = (UBYTE **)  (((char *) load_addr) + 
                         (int) pexports->AddressOfNames);

  dprintf("%-32s Ordinal Virt Addr\n", "Function Name");
  for(i=0; i< pe_exports->Number_Of_Functions; i++)
  {
      ename =  (char *) (((char *) load_addr) + (int) *name++);
      dprintf(stddeb,"%-32s %4d    %8.8lx\n", ename, *ordinal++, *function++);
  }
}
#endif

/*********************************************************************
*                       __ImageGetExportedAddress                    *
*                                                                    *
*  Find an exported function from the export directory               *
*                                                                    *
*********************************************************************/

LPVOID __ImageGetExportedAddress(LPVOID loadptr, 
                            PIMAGE_EXPORT_DIRECTORY pexports,
                            LPCSTR funcname)
{
   PUWORD                   ordinal;
   PULONG                   function;
   LPSTR                   *name;
   LPSTR                    ename;
   INT                      i;
      
   if(!pexports)
      return NULL;
	   
   ordinal  = (UWORD *) (loadptr + (int) pexports->AddressOfNameOrdinals);
   function = (ULONG *) (loadptr + (int) pexports->AddressOfFunctions);
   name     = (LPSTR *) (loadptr + (int) pexports->AddressOfNames);
   
   if(HIWORD(funcname))
   {	                        
      for(i=0; i < pexports->NumberOfNames; i++)
      {       
         ename =  loadptr + (int) *name;         
         if(strcmp(ename,funcname)==0)
         {
            dprintf(" - %d - \n",*ordinal);        
            return loadptr + function[ *ordinal - pexports->Base];
         }
         
         name++;
         ordinal++;
      }
      return NULL;
   }       
   else
   {      
      /* the easy case (tm) */
      return  loadptr + 
              function[(int) LOWORD((int) funcname)-(int) pexports->Base];
   }
}

/*********************************************************************
*                       __ImageGetProcAddress                        *
*                                                                    *
*  Find an exported function of a module                             *
*                                                                    *
*********************************************************************/

LPVOID __ImageGetProcAddress(HMODULE hmodule, LPCSTR funcname)
{
   PPELOAD_FILE             ppefile;
   PIMAGE_NT_HEADERS        pheader;
   PIMAGE_EXPORT_DIRECTORY  pexports;
   LPVOID                   loadptr;
   
   if((ULONG) hmodule>=0x10000)
   {              
      /* get export directory from mapped file */
      loadptr  =  (LPVOID) hmodule;      
            
      pheader  =  (PIMAGE_NT_HEADERS) 
                     (loadptr + ((PMZEXE_HEADER) loadptr)->NewExe);
                       
      pexports =  (PIMAGE_EXPORT_DIRECTORY) 
                     (loadptr + pheader->OptionalHeader.DataDirectory
                        [IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress);                        
   }
   else
   {               
      /* get export directory from internal information */
      ppefile=__ImageFiles;
      while(ppefile)
      {
         if(ppefile->HModule==hmodule) 
            break;
         ppefile=ppefile->Next;
      }
   
      if(!ppefile)
         return NULL;      
      
      pexports =  ppefile->Exports;
      loadptr  =  0; 
   }
   
   return __ImageGetExportedAddress(loadptr, pexports, funcname);
}

/********************************************************************
*                  __ImageFixupImports                              *
*                                                                   *
*  Fixup imports for a loaded module                                *
********************************************************************/

BOOL __ImageFixupImports(PPELOAD_FILE ppefile)
{ 
   PIMAGE_IMPORT_DESCRIPTOR  pimports;  
   PPELOAD_FILE              pimpfile;
   LPVOID                    loadptr;
   PIMAGE_IMPORT_BY_NAME     pname; 
   PULONG                    importlist;
   PULONG                    thunklist;
   INT                       i;

   loadptr=(LPVOID) ppefile->LoadAddress;
   pimports=        ppefile->Imports;
   i=0;
   
   while(pimports->Name)
   {
      dprintf("Imports from module %s:\n",(char *)(loadptr+pimports->Name));

      pimpfile=__ImageLoadModule(loadptr +  pimports->Name);
      
      if(!pimpfile)
      {         
         return FALSE;
      }
      
      importlist  = loadptr + (int) (pimports->Characteristics);
      thunklist   = loadptr + (int) (pimports->FirstThunk);
           
      while(*thunklist) 
      {   
         /* try thunklist first */      
         if( ( *thunklist >= 0x80000000) && 
             ( *thunklist <  0x80010000) )
         {
            /* import by ordinal */
            dprintf("import by ordinal: %ld\n", *thunklist & 0xFFFF);

            (LPVOID) *thunklist = __ImageGetExportedAddress(
                                     (LPVOID) pimpfile->LoadAddress, 
                                     pimpfile->Exports, 
                                     (LPCSTR) (*thunklist & 0xFFFF));
         }
         else if( ((int) loadptr + *thunklist >  ppefile->LoadAddress) &&
                  ((int) loadptr + *thunklist <  ppefile->LoadAddress+
                         ppefile->LoadSize) )
         {
            /* import by name */
             pname = (PIMAGE_IMPORT_BY_NAME) (loadptr + *thunklist);
             dprintf("import by name: %s\n",(char *)&(pname->Name));

             (LPVOID) *thunklist = __ImageGetExportedAddress(
                                      (LPVOID) pimpfile->LoadAddress,
                                      pimpfile->Exports,
                                      (LPSTR) &(pname->Name));                                      
         }
         
         /*
          * this file has been cursed by "BIND", try the inofficial
          * importlist 
          */
          
         else if( ( *importlist >= 0x80000000) && 
                  ( *importlist <  0x80010000) )
         {
             /* import by ordinal */             
             dprintf("bind used, import by ordinal: %d\n",
                     (int) (*importlist & 0xFFFF));
             
             (LPVOID) *thunklist = __ImageGetExportedAddress( 
                                      (LPVOID) pimpfile->LoadAddress, 
                                      pimpfile->Exports, 
                                      (LPCSTR) (*importlist & 0xFFFF));
                                                                                               
         }
         else if( ((int) loadptr + *importlist >  ppefile->LoadAddress) && 
                  ((int) loadptr + *importlist <  ppefile->LoadAddress+
                         ppefile->LoadSize) )
         {
            /* import by name */
             pname = (PIMAGE_IMPORT_BY_NAME) (loadptr + *importlist);
             dprintf("bind used, import by name: %s\n",
                     (char *) &(pname->Name));
             
             (LPVOID) *thunklist = __ImageGetExportedAddress(
                                       (LPVOID) pimpfile->LoadAddress,
                                       pimpfile->Exports,
                                       (LPSTR) &(pname->Name));                                                  
         }
         else
         {
            /* last resort */
            dprintf("bind used, no name/ordinal: %X\n",(int) *thunklist);

            *thunklist = *thunklist+pimpfile->LoadAddress-
                          pimpfile->BaseAddress;
         }
         dprintf("found function at %lX\n", *thunklist);

         thunklist++;
         importlist++;
     }
     i++;
     pimports++;
   }
   return TRUE;
}

/************************************************************************
*  __ImageRelocate()                                                    *
*  NOTE:                                                                *
*  The names in the structure IMAGE_BASE_RELOCATION are very confusing  *
*   - VirtualAddress is the Relative Virtual Address of the page to be  *
*     relocated                                                         *
*   - SymbolTableIndex is the size of this relocation block (HUH!!!)    *
*   - Type has the relocation type in the 4 high bits and the offset    *
*     into the page in the 12 low bits                                  *
*  There is one IMAGE_BASE_RELOCATION per page                          *
************************************************************************/
 
BOOL __ImageRelocate(PPELOAD_FILE ppefile)
{
   PIMAGE_BASE_RELOCATION  prelocblock;   
   PVOID                   ppage;
   ULONG                   loadaddr;
   INT                     delta;
   INT                     count;   
   INT                     type;
   INT                     offset;
   INT                     i; 
   
   loadaddr    = ppefile->LoadAddress;
   delta       = loadaddr - ppefile->BaseAddress;
   prelocblock = ppefile->Relocations;
	
   if(delta == 0)
      return TRUE;
   if(prelocblock == NULL)
      return FALSE;

   /* do the base relocations for a file */
   while(prelocblock->VirtualAddress)
   {
      ppage  = (PVOID)(ppefile->LoadAddress + prelocblock->VirtualAddress);
      count = (prelocblock->SymbolTableIndex - 8)/2;
			
      /* do the base relocations for a page */
      for(i=0;i<count;i++)
      {
         offset = prelocblock->Type[i] & 0xFFF;
         type   = prelocblock->Type[i] >> 12;

#ifdef NEVER
         dprintf("patching page %x offset %x type %x\n", ppage, offset, type);
#endif         
         switch(type)
         {
              case IMAGE_REL_BASED_ABSOLUTE: 
                 /* a NOP */
                 break;
              case IMAGE_REL_BASED_HIGH:
                 *(PUWORD)(ppage+offset) += HIWORD(delta);
                 break;
              case IMAGE_REL_BASED_LOW:
                 *(PUWORD)(ppage+offset) += LOWORD(delta);
                 break;
              case IMAGE_REL_BASED_HIGHLOW:
                 *(PULONG)(ppage+offset) += delta;
                 break;
              default:
                 fprintf(stderr, "Unknown fixup type\n");
                 break;
         }
      }
      prelocblock = (PIMAGE_BASE_RELOCATION) ((PBYTE)prelocblock + 
                     prelocblock->SymbolTableIndex);
   }
   return TRUE;		                    
}

/********************************************************************
*                      __ImageOpenFile                              *
*                                                                   *
* opens a file                                                      *
********************************************************************/

static INT __ImageOpenFile(LPCSTR filename, PPELOAD_FILE ppefile)
{

   LPSTR pcertain;
   CHAR  fullname[MAX_PATH];
   CHAR  unixname[MAX_PATH];
   int   fd;
   
   if(!__FsGetNameA(filename, fullname, unixname, &pcertain, MAX_PATH))
      return -1;
   
   dprintf("Open image, first try: %s\n",unixname);
   fd=open(unixname, O_RDONLY, 0666);
   if(fd==-1)
   {
      if(!__FsFindNameA(fullname, unixname, pcertain, MAX_PATH, 
                        FINDNAME_FILE| FINDNAME_EXIST))
         return -1;
      
      fd=open(unixname, O_RDONLY, 0666);
      if(fd==-1)
         return __ErrorReturnFalse(__ErrorFromErrno(errno));
   }
   ppefile->WinName=istrdup(fullname);
   ppefile->ShortName=strrchr(fullname,'\\');
   ppefile->UnixName=istrdup(unixname); 
   ppefile->Usage=1;
   return fd;
}

/********************************************************************
*               __ImageSearchAndOpenFile                            *
*                                                                   *
* searches for a file in our path                                   *
********************************************************************/

static INT __ImageSearchAndOpenFile(LPCSTR filename, PPELOAD_FILE ppefile)
{
   LPSTR curpath;
   LPSTR nextpath;
   CHAR  path[1024];
   CHAR  fullname[MAX_PATH];
   int   fd;
   
   dprintf("__ImageSearchAndOpenFile(%s)\n",filename );
   
   /* XXX: search PATH environment if just a filename */
   if((!strchr(filename,'\\'))&&(!strchr(filename,':'))&&(!strchr(filename,'/')))
   {
      if(GetEnvironmentVariableA("PATH",path,1024))
      {  
         dprintf("%s\n",path);
         curpath=path;
         nextpath=strchr(curpath,';')+1;
         while(nextpath!=(char *) 1) /* NULL+1 */
         { 
             istrcpy(fullname, curpath, min(nextpath-curpath-1,MAX_PATH));
             istrcat(fullname, "/", MAX_PATH);
             istrcat(fullname, filename, MAX_PATH);
             curpath=nextpath;            
             nextpath=strchr(curpath,';')+1;              
             fd=__ImageOpenFile(fullname, ppefile);
             if(fd!=-1)
                return fd;
         }
         istrcpy(fullname, curpath, min(strlen(curpath)+1,MAX_PATH));
         istrcat(fullname, "/", MAX_PATH);
         istrcat(fullname, filename, MAX_PATH);
         fd=__ImageOpenFile(fullname, ppefile);
         if(fd!=-1)
            return fd;         
      }
      else
         SetLastError(0);
   }   
   return __ImageOpenFile(filename, ppefile);
}


/********************************************************************
*               __ImageOpenLoadedModule                             *
*  checks if this module is already loaded and if this is the case, *
*  opens it                                                         *
********************************************************************/

static PPELOAD_FILE __ImageOpenLoadedModule(LPCSTR filename)
{
   LPCSTR       shortname;
   PPELOAD_FILE ppefile;
   
   dprintf("__ImageOpenLoadedModule(%s)\n",filename);
   shortname=strrchr(filename,'\\');
   if(!shortname)
      shortname=filename;
   
   /* look in the internal filename table */  
   ppefile=__ImageFiles;
   while(ppefile)
   {  
      fflush(stdout);    
      if(strcasecmp(ppefile->WinName,shortname)==0)
      {
         dprintf("found loaded file %s\n", ppefile->WinName);
         ppefile->Usage=ppefile->Usage+1;
         return ppefile;
      }
      ppefile=ppefile->Next;
   } 
   return NULL;
}

/********************************************************************
*               __ImageLoadFile                                     *
*  loads all sections of this image in memory                       *
********************************************************************/
LPVOID __ImageLoadFile(INT fd, PPELOAD_FILE ppefile)
{
   MZEXE_HEADER          mzheader;
   IMAGE_NT_HEADERS      header;
   INT                   i,result;
   INT                   headsize;
   INT                   dirsize;
   INT                   flags;
   ULONG                 rawdatasize;
   ULONG                 virtualsize;
   ULONG                 winprot;
   ULONG                 dirrva;
   LONG                  diff;
   LPVOID                loadptr;
   LPVOID                section;

   dprintf("__ImageLoadFile( %d, %#X ) \n", fd, (INT) ppefile );
   /* read PE header */ 
   read(  fd, &mzheader, sizeof(MZEXE_HEADER));
   if(mzheader.Magic!=MZEXE_MAGIC)
      { dprintf("Invalid dos header\n"); return NULL; }
      
   lseek( fd, mzheader.NewExe, SEEK_SET);
   read(  fd, &header, sizeof(IMAGE_NT_HEADERS));
   
   if(header.Signature != IMAGE_NT_SIGNATURE ) 
      { dprintf("Invalid PE header\n"); return NULL; }      

   /* reserve memory for our image */
   ppefile->BaseAddress=header.OptionalHeader.BaseOfImage;
   ppefile->LoadSize=header.OptionalHeader.SizeOfImage;

   loadptr=__VirtualAllocImage((LPVOID) header.OptionalHeader.BaseOfImage,
                               header.OptionalHeader.SizeOfImage, FALSE);

   if(!loadptr)
     { dprintf("__ImageLoadModule: could not reserve memory\n"); return NULL; }

   ppefile->LoadAddress=(ULONG) loadptr;
   
   /* commit memory for file headers and section headers */
   headsize= mzheader.NewExe + header.OptionalHeader.SizeOfHeaders;

   if(!__VirtualCommit(loadptr, headsize, PAGE_READWRITE))
      { dprintf("__ImageLoadModule: could not commit memory\n"); return NULL; }

   if( header.OptionalHeader.FileAlignment>=PAGE_SIZE)
   {
      /* we can mmap, now */
      if(mmap(loadptr, ROUNDUP(headsize,PAGE_SIZE), 
              PROT_READ | PROT_WRITE,
              MAP_FILE | MAP_PRIVATE | MAP_FIXED, fd, 0)==(void *)-1)
      {
          dprintf("__ImageLoadModule: could not map headers\n"); 
          return NULL; 
      }    
   }
   else
   {
      /* get memory for file headers */
      if(mmap(loadptr, ROUNDUP(headsize,PAGE_SIZE), 
              PROT_READ | PROT_WRITE,
              MAP_ANON | MAP_PRIVATE | MAP_FIXED, __DevZero, 0)==(void *)-1)
      {
          dprintf("__ImageLoadModule: could not get memory for headers\n"); 
          return NULL; 
      }    
      /* read all file headers */   
      lseek(fd, 0, SEEK_SET);
      read(fd, loadptr, headsize);
   } 
   /* set pointers */
   ppefile->Headers  = loadptr + (int) mzheader.NewExe;
   ppefile->Sections = loadptr + (int) mzheader.NewExe +
                        sizeof(IMAGE_NT_HEADERS);
   
   /* load sections */   
   for(i=0; i < header.FileHeader.NumberOfSections; i++)
   {           
      dprintf("Section[%d].Name           = %s\n", i,
              ppefile->Sections[i].Name);
      dprintf("Section[%d].VirtualAddress = %X\n", i,
               (unsigned) ppefile->Sections[i].VirtualAddress);
      dprintf("Section[%d].SizeOfRawData  = %X\n", i,
               (unsigned) ppefile->Sections[i].SizeOfRawData);
      dprintf("Section[%d].VirtualSize    = %X\n", i,
               (unsigned) ppefile->Sections[i].VirtualSize);
    
      section=loadptr+ppefile->Sections[i].VirtualAddress;
      /* commit memory for this section (does nothing but bookkeeping) */
      if(!__VirtualCommit(section, ROUNDUP(ppefile->Sections[i].VirtualSize,
                          PAGE_SIZE), PAGE_READWRITE))                                                        
         { return NULL; }
      
      /*
       * The SizeOfRawData is the size of data that has to be loaded 
       * into memory. 
       * If the VirtualSize is greater than the SizeOfRawData
       * the rest is BSS. 
       * Suprisingly, some apps have a VirtualSize == 0
       */
                             
      flags      =ppefile->Sections[i].Characteristics;      
      rawdatasize=ppefile->Sections[i].SizeOfRawData;
      virtualsize=ppefile->Sections[i].VirtualSize;
      
      /* load this section */
      if((flags & IMAGE_SCN_CNT_UNINITIALIZED_DATA)==0)
      {
         if(header.OptionalHeader.FileAlignment>=PAGE_SIZE)
         {
            /* if we have 4 K filealign we can just mmap everything */           
            
            dprintf("__ImageLoadModule: mapping section %d\n",i);
            if(mmap(section, ROUNDUP(rawdatasize,PAGE_SIZE), 
                   PROT_READ | PROT_WRITE,
                   MAP_FILE | MAP_PRIVATE | MAP_FIXED, fd,                 
                   ppefile->Sections[i].PointerToRawData)==(void *)-1)
            {
               dprintf("__ImageLoadModule: could not get map file\n"); 
               return NULL; 
            } 
         }
         else 
         {
            dprintf("__ImageLoadModule: reading in section %d\n",i);
                    
            if(mmap(section, ROUNDUP(rawdatasize, PAGE_SIZE), 
               PROT_READ | PROT_WRITE, 
               MAP_ANON | MAP_PRIVATE | MAP_FIXED, __DevZero, 0)==(void *)-1)
            { 
                dprintf("__ImageLoadModule:"
                        " could not get memory for section\n"); 
                return NULL; 
            }
            
            if(lseek(fd,ppefile->Sections[i].PointerToRawData,SEEK_SET) == -1)
              { dprintf("__ImageLoadModule: could not seek\n"); return NULL; }
            
            if(read(fd, section, rawdatasize) == -1)
              { dprintf("__ImageLoadModule: could not read\n"); return NULL; }
         }
#ifdef NOT         
         if(rawdatasize % PAGE_SIZE)
         {
            /* zero out the rest */
            memset(section+rawdatasize, 0, rawdatasize % PAGE_SIZE );
         }
#endif           
         diff=ROUNDUP(virtualsize, PAGE_SIZE) - 
              ROUNDUP(rawdatasize, PAGE_SIZE);
         if(diff>0)
         {                 
            dprintf("The rest of this section is loaded as bss\n");
            if( mmap(section + ROUNDUP(rawdatasize, PAGE_SIZE), diff, 
                     PROT_READ | PROT_WRITE, 
                     MAP_FILE | MAP_PRIVATE | MAP_FIXED, 
                     __DevZero, 0)==(void *)-1)
            { 
                dprintf("__ImageLoadModule:"
                        " could not get memory for rest of section\n"); 
                return NULL; 
            }                        
         }  

      }
      else
      {
          if(mmap(section, ROUNDUP(virtualsize, PAGE_SIZE), 
             PROT_READ | PROT_WRITE, 
             MAP_ANON | MAP_PRIVATE | MAP_FIXED, __DevZero, 0)==(void *)-1)
          { 
              dprintf("__ImageLoadModule:"
                      " could not get memory for section\n"); 
              return NULL; 
          }
          dprintf("zeroing out bss\n");
          memset(section, 0, ppefile->Sections[i].SizeOfRawData);
      }      
   }
   
   /* get interesting RVAs */
   dirsize=header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].Size;
   dirrva =header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress;
   if(dirsize)
      ppefile->Exports = loadptr+dirrva;
   
   dirsize=header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].Size;
   dirrva =header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress;
   if(dirsize)
      ppefile->Imports = loadptr+dirrva;
   else
      dprintf("no imports\n");
    
   dirsize=header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].Size;
   dirrva =header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].VirtualAddress;
   if(dirsize)
      ppefile->Resources = loadptr+dirrva;
      
   dirsize=header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size;
   dirrva =header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress;
   if(dirsize)
      ppefile->Relocations = loadptr+dirrva;

   dirsize=header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_TLS].Size;
   dirrva =header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_TLS].VirtualAddress;
   if(dirsize)
   {
      PULONG tlssect;
      tlssect=loadptr+dirrva;
      printf("TLS start    =%X\n",(unsigned) tlssect[0]);
      printf("TLS end      =%X\n",(unsigned) tlssect[1]);
      printf("TLS index    =%X\n",(unsigned) tlssect[2]); 
      printf("TLS callbacks=%X\n",(unsigned) tlssect[3]);      
      printf("TLS zerosize =%X\n",(unsigned) tlssect[4]);       
      printf("TLS flags    =%X\n",(unsigned) tlssect[5]);
   }

   if(header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXCEPTION].Size)
      dprintf("Exception directory ignored\n");

   if(header.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY].Size)
      dprintf("Security directory ignored\n");

   if(header.OptionalHeader.DataDirectory [IMAGE_DIRECTORY_ENTRY_GLOBALPTR].Size)
      dprintf("Machine Value ignored\n");

   if(header.OptionalHeader.DataDirectory [IMAGE_DIRECTORY_ENTRY_TLS].Size)  
      dprintf("Thread local storage ignored\n");
  
   if(header.OptionalHeader.DataDirectory [IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG].Size)
      dprintf("Load configuration directory ignored\n");
   
   if(ppefile->Imports)      
      __ImageFixupImports(ppefile); 
      
#ifdef 0     
   if(ppefile->Export)       
      __ImageDumpExports(ppefile->Exports,loadptr);
#endif      
   if(ppefile->Relocations)  
      __ImageRelocate(ppefile);
  
   /* now give the sections the protection they need */     
   for(i=0; i < header.FileHeader.NumberOfSections; i++)
   {           
      flags      =ppefile->Sections[i].Characteristics;      
      virtualsize=ROUNDUP(ppefile->Sections[i].VirtualSize, PAGE_SIZE);
            
      /*  get the right protection for this segment */
      switch(flags & ( IMAGE_SCN_MEM_READ | IMAGE_SCN_MEM_WRITE | 
                       IMAGE_SCN_MEM_EXECUTE ))
      {
         case 0:
            winprot=PAGE_NOACCESS;
            break;   
         case IMAGE_SCN_MEM_EXECUTE:
            winprot=PAGE_EXECUTE;
            break;
         case IMAGE_SCN_MEM_READ:
            winprot=PAGE_READONLY;
            break;
         case IMAGE_SCN_MEM_READ | IMAGE_SCN_MEM_EXECUTE :
            winprot=PAGE_EXECUTE_READ;
            break;
         case IMAGE_SCN_MEM_WRITE:
            winprot=PAGE_READWRITE;
            break;
         case IMAGE_SCN_MEM_WRITE | IMAGE_SCN_MEM_EXECUTE:
            winprot=PAGE_EXECUTE_READWRITE;
            break;
         case IMAGE_SCN_MEM_WRITE | IMAGE_SCN_MEM_READ:
            winprot=PAGE_READWRITE;
            break;
         case IMAGE_SCN_MEM_WRITE | IMAGE_SCN_MEM_READ | IMAGE_SCN_MEM_EXECUTE:
            winprot=PAGE_EXECUTE_READWRITE;
            break;           
         default:
            winprot=PAGE_NOACCESS; 
#ifdef TEST
            winprot=PAGE_EXECUTE_READWRITE;
#endif          
      }
      
      if(mprotect( loadptr + ppefile->Sections[i].VirtualAddress,
                 virtualsize, __VirtualWinProtToUnix(winprot))==-1)
      {
          dprintf("__ImageLoadModule:"
                  " could not change protection of section: %d\n", errno);
          return NULL; 
      }

      result=VirtualProtect( loadptr+ ppefile->Sections[i].VirtualAddress,
                             virtualsize, winprot, NULL );                              
      if(!result)
         return NULL;
       
   }  
   ppefile->LoadAddress = (ULONG)   loadptr;
   ppefile->HModule     = (HMODULE) loadptr;
   return loadptr;
}


/********************************************************************
*                    __ImageLoadModule                              *
*                                                                   *
*  Load one PE format file into memory, called by LoadExe           *
*                                                                   *
********************************************************************/
 
PPELOAD_FILE __ImageLoadModule(LPCSTR filename)
{
   PPELOAD_FILE          ppefile;
   LPVOID                loadptr;
   INT                   fd;

   /* look in the internal filename table */
   ppefile=__ImageOpenLoadedModule(filename);
   if(ppefile)
      return ppefile;

   ppefile=imalloc(sizeof(PELOAD_FILE));
   memset(ppefile, 0, sizeof(PELOAD_FILE));
      
   /* open file */
   fd=__ImageSearchAndOpenFile(filename,ppefile);
   if(fd==-1)
   {
      dprintf("cannot open file, error: %d\n",errno);
      return __ErrorReturnNull(ERROR_FILE_NOT_FOUND);
   }
 
   loadptr=__ImageLoadFile(fd, ppefile);
   if(!loadptr)
      return NULL;     
      
   ppefile->LoadAddress = (ULONG)   loadptr;
   ppefile->HModule     = (HMODULE) loadptr;
   ppefile->Next        = __ImageFiles;
   __ImageFiles         = ppefile;
   return ppefile;  
}

BOOL __ImageFreeModule(PPELOAD_FILE ppefile)
{
   if(ppefile->LoadAddress<PAGE_SIZE)
       return FALSE;
      
   /* free the image and module */
   VirtualFree((LPVOID) ppefile->LoadAddress, 0, MEM_RELEASE );
   ifree(ppefile);
   return TRUE;
}

PPELOAD_FILE __ImageGetModuleInfo(HMODULE hmod)
{
   PPELOAD_FILE ppefile;
   
   ppefile=__ImageFiles; 
   while(ppefile)
   {
      if(ppefile->HModule==hmod)
         return ppefile;
         
      ppefile=ppefile->Next;   
   }
   return NULL;
} 

#ifdef __cplusplus
}
#endif