/*
 * kernel[6~/process.c
 * Copyright (C) 1996,  Onno Hovers, All rights reserved
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include "imalloc.h"
#include "process.h"
#include "peload.h"
#include "virtual.h"
#include "wfc.h"
#include "run.h"

/* global variables */
#ifndef __WindowsEmulate
ULONG 		__WindowsEmulate=VER_PLATFORM_WIN32_FREE;
#endif
ULONG 		__WindowsVersion=0x400;
ULONG		__WindowsBuild=1000;
STARTUPINFOA	__StartupInfoA;
LPSTR		__CommandLineA;
LPSTR		__EnvironmentA;
CHAR		__UnixCurrentDirectory[MAX_PATH];   /* unix cwd */
CHAR		__CurrentDirectoryA[MAX_PATH];      /* current directory */
ULONG		__EnvironmentSize;  /*  size of the environment block */
ULONG		__EnvironmentAlloc; /*allocated for environment block*/
HANDLE		__HStdIn;
HANDLE		__HStdOut;
HANDLE		__HStdErr;
HANDLE		__Heap;			/* process heap */

/* debugging and spying */
BOOL		__DbgLog=FALSE;
FILE *		__DbgOut=stdout;
BOOL		__ApiLog=FALSE;
FILE *		__ApiOut=stdout;

/********************************************************************
*                       __ProcessExit                               *
*                                                                   *
* exits a process                                                   *
*                                                                   *
********************************************************************/

VOID NORETURN __ProcessExit(INT exitcode)
{
   PPELOAD_FILE ppefile;
   PPELOAD_FILE pnext;
   
   ppefile=__ImageFiles;

   /* Call all DLLs */
   while(ppefile)
   {
      if((ppefile!=__ImageExecutable)&&
         (ppefile->LoadAddress!=0)&&
         (ppefile->Headers!=NULL)&&
         (ppefile->Headers->OptionalHeader.AddressOfEntryPoint!=0))
      {
         if(!__CallDll(ppefile, DLL_PROCESS_DETACH, 0))
         {
             /* big hack ! */
            fprintf(stderr,"Unable to unload a dll\n");
            exit(1);
         }
      }
      ppefile=ppefile->Next;
   }

   ppefile=__ImageFiles;

   while(ppefile)
   {
      dprintf(" ppefile->WinName=%s\n", ppefile->WinName);
      pnext=ppefile->Next;
      if(ppefile->LoadAddress>=PAGE_SIZE)
         __ImageFreeModule(ppefile);
      ppefile=pnext;
   }
   
   exit(exitcode);
   
   /* just to keep gcc happy */
   while(1) /* nothing */;
}


/********************************************************************
*                       __ProcessStart                              *
*                                                                   *
*                                                                   *
********************************************************************/

VOID NORETURN __ProcessStart(VOID)
{
   PPELOAD_FILE ppefile;

   ppefile= __ImageFiles;
    
   /* Call all DLLs */
   while(ppefile)
   {
      if((ppefile!=__ImageExecutable)&&
         (ppefile->LoadAddress!=0)&&
         (ppefile->Headers!=NULL)&&
         (ppefile->Headers->OptionalHeader.AddressOfEntryPoint!=0))
      {
         if(!__CallDll(ppefile, DLL_PROCESS_ATTACH, 0))
         {
             /* big hack ! */
            fprintf(stderr,"Unable to load a dll\n");
            __ProcessExit(1);
         }
      }
      ppefile=ppefile->Next;
   }
#ifdef 0
   if(ppefile->Tls)
   {
      /* initialize tls */
   
   }
#endif   
   __ProcessExit(__CallExe());
}

VOID WINAPI ExitProcess(UINT exitcode)
{
   aprintf("ExitProcess( %d )\n",exitcode);
   
   __ProcessExit(exitcode);
}

HANDLE WINAPI GetCurrentProcess(VOID)
{
   aprintf("GetCurrentProcess returns 0x%X\n",(UINT) PROC_HSELF);
   return PROC_HSELF;
}

DWORD WINAPI GetCurrentProcessId(VOID)
{
   return (DWORD) getpid();
}
