/* 
 * kernel/rtlmem.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * runtime library memory functions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#include<win32/defines.h>
#include<win32/kernel.h>
#include<string.h>
#include"wcstring.h"

/* internal functions */

int istrcpy(char *dest, const char *src, int count)
{
   int len;
   
   len=0;   
   while(count>0)
   {

      *dest=*src;
      
      if(*src=='\0')
         return len;
         
      src++;
      dest++;      
      len++;
      count--;
   } 
   
   do
   {
      len++;
      src++;
   }
   while(*src!='\0');
   return len;
}

int istrcat(char *dest, const char *src, int count)
{
   int len;

   len=0;   
   while((*dest!='\0')&&(count>0))
   {
      len++;
      count--;
      dest++;
   }
   
   while(count>0)
   {

      *dest=*src;
      if(*src=='\0')
         return len;
      src++;
      dest++;
      len++;
      count--;
      
   }
    
   do
   {
      len++;
      src++;
   }
   while(*src!='\0');
   return len;
}

/*******************************************************************
*                 RtlFillMemory  - KERNEL32.DLL                    *
*******************************************************************/
VOID WINAPI RtlFillMemory(PVOID dest, DWORD length, BYTE fill)
{
   memset(dest, fill, length);
}

/********************************************************************
*                 RtlMoveMemory  - KERNEL32.DLL                     *
********************************************************************/ 
VOID WINAPI RtlMoveMemory(PVOID dest, CONST VOID * src, DWORD length)
{
   memmove(dest,src,length);
}

/********************************************************************
*                 RtlZeroMemory - KERNEL32.DLL                      *
********************************************************************/
VOID WINAPI RtlZeroMemory(PVOID dest, DWORD length)
{
   memset(dest, 0, length);
}

/********************************************************************
*                    lstrcatA - KERNEL32.DLL                        *
********************************************************************/
PSTR WINAPI lstrcatA(PSTR dest, PCSTR add)
{
   return strcat(dest,add);      
}

/********************************************************************
*                    lstrcatW - KERNEL32.DLL                        *
********************************************************************/
PWSTR WINAPI lstrcatW(PWSTR dest, PCWSTR add)
{
   return wcscat(dest,add);      
}

/********************************************************************
*                    lstrcmpA - KERNEL32.DLL                        *
********************************************************************/
INT WINAPI lstrcmpA(PCSTR str1, PCSTR str2)
{
   return strcmp(str1,str2); 
}

/********************************************************************
*                    lstrcmpW - KERNEL32.DLL                        *
********************************************************************/
INT WINAPI lstrcmpW(PCWSTR str1, PCWSTR str2)
{
   return wcscmp(str1,str2); 
}

/********************************************************************
*                    lstrcmpiA - KERNEL32.DLL                       *
********************************************************************/
INT WINAPI lstrcmpiA(PCSTR str1, PCSTR str2)
{
   return strcasecmp(str1,str2);
}

/********************************************************************
*                    lstrcpyA - KERNEL32.DLL                        *
********************************************************************/
PSTR WINAPI lstrcpyA(PSTR dest, PCSTR src)
{
   return strcpy(dest,src);
}

/********************************************************************
*                    lstrcpyW - KERNEL32.DLL                        *
********************************************************************/
PWSTR WINAPI lstrcpyW(PWSTR dest, PCWSTR src)
{
   return wcscpy(dest,src);
}

/********************************************************************
*                    lstrcpynA - KERNEL32.DLL                       *
********************************************************************/
PSTR WINAPI lstrcpynA(PSTR dest, PCSTR src, INT n)
{
   return strncpy(dest,src,n);
}

/********************************************************************
*                    lstrcpynW - KERNEL32.DLL                       *
********************************************************************/
PWSTR WINAPI lstrcpynW(PWSTR dest, PCWSTR src, INT n)
{
   return wcsncpy(dest,src,n);
}

/********************************************************************
*                    lstrlenA - KERNEL32.DLL                        *
********************************************************************/
INT WINAPI lstrlenA(PCSTR str)
{
    return strlen(str);
}

/********************************************************************
*                    lstrlenW - KERNEL32.DLL                        *
********************************************************************/
INT WINAPI lstrlenW(PCWSTR str)
{
    return wcslen(str);
}
