/*
 *  kernel/run.c
 *  Copyright (c) 1996, Onno Hovers, All rights reserved
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include <win32/ntimage.h>
#include <i386/segment.h>
#include <i386/stdcall.h>
#include "peload.h"
#include "imalloc.h"
#include "virtual.h"
#include "teb.h"
#include "wfc.h"

/* 
 * calls a thread, assuming the TEB has been initialized
 * and the function never returns 
 */
VOID NORETURN __CallInitFirstThread(PVOID proc, ULONG param )
{
   LPVOID  pstack;
   ULONG   stacksize;
   
   stacksize=__ImageExecutable->Headers->OptionalHeader.SizeOfStackReserve;

   if( stacksize>1024*1024 )
      stacksize=1024*1024;
   else if( stacksize < 64*1024 )
      stacksize=64*1024;
      
   pstack=__VirtualAllocStack( stacksize );
                      
   __TebStackHigh=pstack+stacksize;
   __TebStackLow =pstack;
   __TebTlsArray =NULL; 
                 
   __asm__ __volatile__ (
      "\t fs			\n"
      "\t movl  4,%%esp		\n"
      "\t subl  $4,%%esp        \n" 
      "\t pushl %1		\n"
      "\t call  %0		\n"     
      :: "r" (proc), "r" (param)
   );
   while(1)
     /* nothing */;   
} 

typedef INT (*PEXE_ENTRY)(VOID);
	
INT __CallExe(VOID)	
{     
   /* what should this be ?? */
   return __stdcall3( 
      (LPVOID) (__ImageExecutable->LoadAddress + 
                __ImageExecutable->Headers->OptionalHeader.AddressOfEntryPoint),
      (ULONG) __ImageExecutable->HModule,
      0, 0);      
}

INT __CallDll(PPELOAD_FILE ppefile, INT reason, INT reserved)
{      
   return __stdcall3( 
      (LPVOID) (ppefile->Headers->OptionalHeader.AddressOfEntryPoint+
                ppefile->LoadAddress), 
      (ULONG)   ppefile->HModule, 
      reason, 
      reserved); 
} 
