/*
 * Copyright 1996 Onno Hovers, All rights reserved
 *
 * We only need segments for Thread Environment Blocks, now
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <win32/defines.h>
#include <win32/error.h>
#include <win32/kernel.h>
#include <i386/segment.h>
#include "wfc.h"

#ifdef linux
#include <linux/unistd.h>
#include <linux/head.h>
#include <linux/ldt.h>

_syscall3(int, modify_ldt, int, func, void *, ptr, unsigned long, bytecount)

#endif  /* linux */

#define LDT_SIZE 8182
#define LDT_FIRST_ENTRY      6
#define LDT_LAST_ENTRY       (LDT_SIZE-1)
#define LDT_ENTRY_TO_SEL(i)  (((i)<<3)|7)
#define LDT_SEL_TO_ENTRY(i)  ((i)>>3)
#define IS_LDT_ENTRY_FREE(i) (__SegmentLdtCopy[(i)].Size==0)

typedef struct __LDT_COPY
{
   LPVOID Address;
   DWORD  Size;
   DWORD  Flags;
} LDT_COPY;

typedef struct __LDT_COPY *PLDT_COPY;

LDT_COPY __SegmentLdtCopy[LDT_SIZE];

/***********************************************************************
*                        __SegmentInit                                 *
*                                                                      *
***********************************************************************/

BOOL __SegmentInit(VOID)
{
   memset(__SegmentLdtCopy,0,LDT_SIZE * sizeof (LDT_COPY));
   return TRUE;
}
        
/***********************************************************************
*                        __SegmentAlloc                                *
*                                                                      *
* Allocate a LDT entry.                                                *
***********************************************************************/
INT __SegmentAlloc( LPVOID addr, DWORD size, DWORD flags)
{
    INT    entry;
    INT    retval;
    DWORD  limit;
    BOOL   biglimit;

    retval=0;
    if(!size) return 0;

    entry = LDT_FIRST_ENTRY;
    while((entry < LDT_SIZE) && (!IS_LDT_ENTRY_FREE(entry)))
       entry++;
       
    if (entry == LDT_SIZE) 
    {  
       dprintf("__SegmentAllocSegment: out of selectors\n");
       return 0;
    }
    
    if(size>=1024*1024)
    {
      limit=size/4096;
      biglimit=1;
    }
    else
    {
      limit=size;
      biglimit=0;  
    }
#ifdef linux    
    {
        struct modify_ldt_ldt_s ldt_info;

        /* Clear all unused bits (like seg_not_present) */
        memset( &ldt_info, 0, sizeof(ldt_info) );
        ldt_info.entry_number   = entry;
        ldt_info.base_addr      = (ULONG) addr;
        ldt_info.limit          = limit;
        ldt_info.seg_32bit      = (flags & SEGMENT_16BIT) != 1;
        ldt_info.contents       = flags  & SEGMENT_TYPE;
        ldt_info.read_exec_only = (flags & SEGMENT_READONLY) != 0;
        ldt_info.limit_in_pages = biglimit;
        retval = modify_ldt(1, &ldt_info, sizeof(ldt_info));
    }
#endif

    if (retval < 0) return 0;
    __SegmentLdtCopy[entry].Address  = addr;
    __SegmentLdtCopy[entry].Size     = size;
    __SegmentLdtCopy[entry].Flags    = flags;
    return (entry*8)+7;
}


BOOL __SegmentFree(INT i)
{
    INT entry;
    INT retval;
     
    entry=LDT_SEL_TO_ENTRY(i);
    
    if(!__SegmentLdtCopy[entry].Size)
       return FALSE;
       
    __SegmentLdtCopy[entry].Address=0;
    __SegmentLdtCopy[entry].Size   =0;
    __SegmentLdtCopy[entry].Flags  =0;
    
#ifdef linux    
    {
        struct modify_ldt_ldt_s ldt_info;

        /* Clear all unused bits (like seg_not_present) */
        memset( &ldt_info, 0, sizeof(ldt_info) );
        retval = modify_ldt(1, &ldt_info, sizeof(ldt_info));
        return (retval==0);        
    }
#else
    return FALSE;    
#endif
}