/*
 * kernel/signal.c
 * Copyright (C) 1996, Onno Hovers, All Rights reserved
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <setjmp.h>
#include <syscall.h>
#include <win32/defines.h>
#include <win32/kernel.h>
#include <win32/console.h>
#include <win32/error.h>
#include "wfc.h"
#include "error.h"

typedef struct __CTRLHANDLER
{
   struct __CTRLHANDLER *Next;
   PHANDLER_ROUTINE      Handler;
} CTRLHANDLER, *PCTRLHANDLER;

PCTRLHANDLER __ControlHandler=NULL;

BOOL WINAPI SetConsoleCtrlHandler(PHANDLER_ROUTINE phandler, BOOL add)
{
   PCTRLHANDLER pnew;
   PCTRLHANDLER pprev;
   
   aprintf("SetConsoleCtrlHandler( 0x%lX, %s )\n", (ULONG) phandler, 
           add?"TRUE":"FALSE");
   
   if(add)
   {
      pnew=imalloc(sizeof(CTRLHANDLER));   
      if(!pnew)
         return __ErrorReturnFalse(ERROR_OUTOFMEMORY);

      pnew->Handler=phandler;
      pnew->Next=NULL;
      
      /* insert in list */
      if(!__ControlHandler)
      {     
         __ControlHandler=pnew;        
      }
      else
      {
         pprev=__ControlHandler;
         while(pprev->Next)
            pprev=pprev->Next;
         pprev->Next=pnew;
      }
   }
   else
   {
      if(!__ControlHandler)
         return __ErrorReturnFalse(ERROR_INVALID_PARAMETER);

      /* remove from list */
      if(__ControlHandler->Handler==phandler)
      {
         pnew=__ControlHandler;
         __ControlHandler=__ControlHandler->Next;
         ifree(pnew);
      }
      else
      {
         pprev=__ControlHandler;
         while((pprev->Next)&&(pprev->Next->Handler!=phandler))
             pprev=pprev->Next;
         if(!pprev->Next)
             return __ErrorReturnFalse(ERROR_INVALID_PARAMETER);
         pnew=pprev->Next;    
         pprev->Next=pprev->Next->Next;
         ifree(pnew);
      }
   }  
   return TRUE;    
}


#ifdef NOTYET
#ifdef LINUX_BROKEN_TRAP_NO
/********************************************************************
*                       __SignalBrokenTrapNo                        *
*                                                                   *
* signal handler for the division by zero we do after a signal to   *
* set the trap_no back to zero                                      *     
********************************************************************/

static void __SignalBrokenTrapNo(int signal, sigcontext_struct context)
{
   context.eip=context.eip+2;
}
#endif

/********************************************************************
*                      __SignalException                            *
*                                                                   *
* handle exceptions                                                 *
********************************************************************/

static void __SignalException(int signal, sigcontext_struct context)
{
    I386_CONTEXT         wcontext;
    EXCEPTION_RECORD     excrecord;
    
    wcontext.ContextFlags=CONTEXT_FULL;
    wcontext.SegGs  = context.gs;
    wcontext.SegFs  = context.fs;
    wcontext.SegEs  = context.es;
    wcontext.SegDs  = context.ds;
    wcontext.Edi    = context.edi;
    wcontext.Esi    = context.esi;
    wcontext.Ebx    = context.ebx;
    wcontext.Edx    = context.edx;
    wcontext.Ecx    = context.ecx;
    wcontext.Eax    = context.eax;
    wcontext.Ebp    = context.ebp; 
    wcontext.Eip    = context.eip;
    wcontext.SegCs  = context.cs;
    wcontext.EFlags = context.eflags;
    wcontext.Esp    = context.esp;
    wcontext.SegSs  = context.ss;
    
    switch(signal)
    {
       case SIGSEGV:
          switch(context.trap_no)
          {
             case 0: /* vague */
                excrec.ExceptionCode=EXCEPTION_ACCESS_VIOLATION;
                break;
             case 4: /* OVERFLOW */
                excrec.ExceptionCode=EXCEPTION_INT_OVERFLOW;
                break;
             case 5: /* BOUNDS CHECK */
                excrec.ExceptionCode=EXCEPTION_ARRAY_BOUNDS_EXCEEDED;
                break;
             case 7: /* FPU NOT AVAILABLE */             
                excrec.ExceptionCode=EXCEPTION_FPU_INVALID_OPERATION;
                break;
             case 8: /* DOUBLE FAULT: we should not get this! */
                excrec.ExceptionCode=STATUS_NONCONTINUABLE_EXCEPTION;
                break;
             case 10: /* INVALID TSS: we should not get this! */
                excrec.ExceptionCode=STATUS_NONCONTINUABLE_EXCEPTION;
                break;
             case 13: /* GENERAL PROTECTION FAILURE */
                excrec.ExceptionCode=EXCEPTION_PRIV_INSTRUCTION;
                break;
             case 14: /* PAGE FAULT */
                excrec.ExceptionCode=EXCEPTION_ACCESS_VIOLATION;
                break;
             case 15: /* RESERVED */             
                excrec.ExceptionCode=EXCEPTION_PRIV_INSTRUCTION;
                break;
             case 17: /* ALIGNMENT CHECK */
                excrec.ExceptionCode=EXCEPTION_DATATYPE_MISALIGNMENT;
                break;                
             default: /* probably an explicit kill */
                return;             
          }
          break;
       case SIGTRAP:
          switch(context.trap_no)
          {
             case 1: /* DEBUG */
                excrec.ExceptionCode=EXCEPTION_SINGLE_STEP ;
                break;
             case 3: /* INT 3 */
                excrec.ExceptionCode=EXCEPTION_BREAKPOINT ;
                break;
             default: /* probably an explicit kill */
                return;           
          }
          break;
       case SIGILL:
          if(context.trap_no!=6)
             return;
          excrec.ExceptionCode=EXCEPTION_ILLEGAL_INSTRACTION;
          break;             
       case SIGFPE:  /* FLOATING POINT EXCEPTION */
          switch(context.trap_no)
          {
             case 0: /* INTEGER (???) division by zero */
             {
                BYTE op1,op2;
                
                /* test if this is from a "div"/"idiv" instruction */
                op1= ((PBYTE)context.eip) [0];
                op2= ((PBYTE)context.eip) [1];
                if(((op1&0xFE)==0xF6)&&((op2&0x30)==0x30))
                   excrec.ExceptionCode=EXCEPTION_INT_DIVISION_BY_ZERO;
                else /* what else ??? */
                   excrec.ExceptionCode=EXCEPTION_PRIV_INSTRUCTION;  
                break;
             }
             case 9: /* FPU segment overrun */
                excrec.ExceptionCode=EXCEPTION_FLT_STACK_CHECK;
                break;
             case 16: /* FPU EXCEPTION */
             
                break;
          }
          break;
       case SIGBUS: /* BUS ERROR (????) */
          switch(context.trap_no)
          {
             case 0:   /* access past the end of a shared mmap'ed file ?? */
                excrec.ExceptionCode=EXCEPTION_ACCESS_VIOLATION
                break;                
             case 11:  /* SEGMENT NOT PRESENT */
                excrec.ExceptionCode=EXCEPTION_PRIV_INSTRUCTION; 
                break;
             case 12:  /* STACK SEGMENT CHECK */
                excrec.ExceptionCode=EXCEPTION_PRIV_INSTRUCTION;
                break;
             default:
          }
       default:
          return;    
    } 
#ifdef LINUX_BROKEN_TRAP_NO
    /* reset the trapno to 0 by a division by zero */
    if(context.trap_no!=0)
    {       
       __SignalSetHandler(SIGFPE, (void(*)()) __SignalBrokenTrapNo);
       /* assembler statement for division */
       volatile __asm__("\tdivb %%0\n"::"a"(0)); 
       __SignalSetHandler(SIGFPE, (void(*)()) __SignalException);
    }
#endif        
    __iRaiseExceptionContext( &excrec, &wcontext );

    context.gs     = wcontext.SegGs;
    context.fs     = wcontext.SegFs;
    context.es     = wcontext.SegEs;
    context.ds     = wcontext.SegDs;
    context.edi    = wcontext.Edi;
    context.esi    = wcontext.Esi;
    context.ebx    = wcontext.Ebx;
    context.edx    = wcontext.Edx;
    context.ecx    = wcontext.Ecx;
    context.eax    = wcontext.Eax;
    context.ebp    = wcontext.Ebp;     
    context.eip    = wcontext.Eip;    
    context.cs     = wcontext.SegCs; 
    context.eflags = wcontext.EFlags;
    context.esp    = wcontext.Esp;
    context.ss     = wcontext.SegSs;           
}


/*********************************************************************
*		          __SignalSetHandler                         *
*********************************************************************/

static BOOL __SignalSetHandler( int sig, void (*func)() )
{
    int ret;
    struct sigaction sigact;
    
    sigact.sa_handler = func;
    sigact.sa_flags = SA_RESTART | SA_NOMASK;
    sigact.sa_restorer = NULL;
    ret = sigaction( sig, &sigact, NULL );   

    if (ret < 0)
    {
        perror( "sigaction" );
        exit(1);
    }
}



/**********************************************************************
*	                    __SignalInit                              *
*                                                                     *
* initialize signal handling                                          *
**********************************************************************/

void __SignalInit(void)
{
   __SignalSetHandler( SIGALRM, (void (*)()) __SignalAlarm );    
   __SignalSetHandler( SIGSEGV, (void (*)()) __SignalException );
   __SignalSetHandler( SIGILL,  (void (*)()) __SignalException );
   __SignalSetHandler( SIGFPE,  (void (*)()) __SignalException );
   __SignalSetHandler( SIGTRAP, (void (*)()) __SignalException ); 
#ifdef SIGBUS
   __SignalSetHandler( SIGBUS,  (void (*)()) __SignalException );
#endif    
   __SignalSetHandler( SIGHUP,  (void (*)()) __SignalControl );
   __SignalSetHandler( SIGINT,  (void (*)()) __SignalControl );
   __SignalSetHandler( SIGQUIT, (void (*)()) __SignalControl );
   __SignalSetHandler( SIGTSTP, (void (*)()) __SignalControl );
#ifdef NOTYET
   __SignalSetHandler( SIGUSR1, (void (*)()) __SignalIPC1 );
   __SignalSetHandler( SIGUSR2, (void (*)()) __SignalIPC2 ); /* For IPC */   
#endif   
}


BOOL __SignalSendIPC1(INT pid, ULONG timeout )
{   
   __tpTimeOut->tv_sec  = timeout / 1000 ;
   __tpTimeOut->tv_usec = timeout % 1000 ;
  
   kill(pid, SIGUSR1);   
   /* 
    * trick: we wait until the signal is delivered 
    * if a signal is delivered, timeout will be zero   
    */  
    
   select(0 , NULL, NULL, NULL, __tpTimeOut);       
}

BOOL __SignalReplyIPC1(INT pid)
{
   thread->TimeOut->tv_sec=0;
   thread->TimeOut->tv_usec=0;
   
   kill(pid, SIGUSR1); 
}
#endif