/*
 * kernel/sysinfo.c
 * Copyright 1996, Onno Hovers, All rights reserved
 * 
 * functions for getting system information
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */
 
 
#include<string.h>
#include<sys/utsname.h>
#include<win32/kernel.h>
#include<win32/error.h> 
#include "wfc.h"

static BOOL		__UtsInitialized=FALSE;
static struct utsname	__UtsBuffer;

/********************************************************************
*                      GetVersion -- KERNEL32                       *
********************************************************************/

DWORD WINAPI GetVersion(VOID)
{
  DWORD retval;
  
  aprintf("GetVersion()\n");
  retval=HIBYTE(__WindowsVersion)+
         (LOBYTE(__WindowsVersion)<<8)+
         (__WindowsBuild<<16);
  
  if(!NATIVE)
  {
     if(WIN95_ISMS)
        retval=retval|0xC0000000;
     else if(WIN32S_ISMS)
        retval=retval|0xE0000000;
  }
  return retval;
}

/********************************************************************
*                    GetVersionEx -- KERNEL32                       *
********************************************************************/

BOOL WINAPI GetVersionExA(LPOSVERSIONINFOA pinfo)
{
   aprintf("GetVersionEx( 0x%lX)\n", (ULONG) pinfo);
   pinfo->dwOSVersionInfoSize=sizeof(OSVERSIONINFOA);
   pinfo->dwMajorVersion=HIBYTE(__WindowsVersion);
   pinfo->dwMinorVersion=LOBYTE(__WindowsVersion);
   pinfo->dwBuildNumber=__WindowsBuild;
   pinfo->dwPlatformId=__WindowsEmulate;
   strcpy(pinfo->szCSDVersion,"WinFree Console");
   return TRUE;
}

/********************************************************************
*                    GetSystemInfo -- KERNEL32                      *
********************************************************************/

#ifndef PROCESSOR_INTEL_PENTIUM 
#define PROCESSOR_INTEL_PENTIUM		586
#endif            
#ifndef PROCESSOR_INTEL_PENTIUM_PRO 
#define PROCESSOR_INTEL_PENTIUM_PRO	686
#endif            

VOID WINAPI GetSystemInfo(LPSYSTEM_INFO pinfo)
{     
   aprintf("GetSystemInfo( 0x%lX)\n", (ULONG) pinfo);
   memset(pinfo,0,sizeof(SYSTEM_INFO));
   
   pinfo->wProcessorArchitecture=PROCESSOR_ARCHITECTURE_INTEL;
   pinfo->dwPageSize=PAGE_SIZE;
   
   if(NATIVE)
   {
      pinfo->lpMinimumApplicationAddress=(LPVOID) 0x00010000;
      pinfo->lpMaximumApplicationAddress=(LPVOID) 0xBFFF0000;   
   }
   else if(WIN32S_ISMS)
   {
      pinfo->lpMinimumApplicationAddress=(LPVOID) 0x80000000;
      pinfo->lpMaximumApplicationAddress=(LPVOID) 0xC0000000;
   }
   else if(WIN95_ISMS)
   {
      pinfo->lpMinimumApplicationAddress=(LPVOID) 0x400000;
      pinfo->lpMaximumApplicationAddress=(LPVOID) 0x80000000;
   }
   else if(NT_ISMS)
   {
      pinfo->lpMinimumApplicationAddress=(LPVOID) 0x00010000;
      pinfo->lpMaximumApplicationAddress=(LPVOID) 0x7FFF0000;    
   }
   
   /* if we are running Linux SMP this is incorrect */
   pinfo->dwActiveProcessorMask=1;
   pinfo->dwNumberOfProcessors=1;
   pinfo->dwAllocationGranularity=0x10000;
   
   if(!__UtsInitialized)
   {
      uname(&__UtsBuffer);
      __UtsInitialized=TRUE;
   }

   if(strcmp(__UtsBuffer.machine,"i386")==0)
   {
      pinfo->dwProcessorType=PROCESSOR_INTEL_386;
   }
   else if(strcmp(__UtsBuffer.machine,"i486")==0)
   {
      pinfo->dwProcessorType=PROCESSOR_INTEL_486;
   }
   else if(strcmp(__UtsBuffer.machine,"i586")==0)
   {
      pinfo->dwProcessorType=PROCESSOR_INTEL_PENTIUM;
   }
   else if(strcmp(__UtsBuffer.machine,"i686")==0)
   {
      pinfo->dwProcessorType=PROCESSOR_INTEL_PENTIUM_PRO;
   }
}