/* 
 * kernel/time.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 * time functions
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; see the file COPYING.LIB. If
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 * Cambridge, MA 02139, USA.  
 */

#ifdef linux
#define __USE_MISC
#endif

#include<string.h>
#include<time.h>
#include<sys/time.h>
#include<win32/defines.h>
#include<win32/kernel.h>
#include"wfc.h"
#include"itime.h"

time_t __TimeFileToUnix (FILETIME ftime)
{
   ULONGLONG wtime;
   
   wtime=((ULONGLONG) ftime.dwHighDateTime << 32) +
         ((ULONGLONG) ftime.dwLowDateTime);
   
   return  (wtime-FACTOR)/NSPERSEC;
}

FILETIME  __TimeUnixToFile (time_t utime)
{
   ULONGLONG wtime;
   FILETIME  ftime;

   wtime=((ULONGLONG) utime*NSPERSEC)+FACTOR;
   
   ftime.dwHighDateTime=HILONG(wtime);
   ftime.dwLowDateTime =LOLONG(wtime);

   return  ftime;
}

/* XXX: Does DOS date/time use local time or GMT ?? */
BOOL WINAPI FileTimeToDosDateTime(CONST FILETIME *pftime,
                                  PWORD pdostime, PWORD pdosdate)
{
   time_t     utime;  
   struct tm *mytm;
   PDOSTIME  pdtime=(PDOSTIME) pdostime;
   PDOSDATE  pddate=(PDOSDATE) pdosdate;
 
   aprintf("FileTimeToDosDateTime()\n");  
   
   utime=__TimeFileToUnix(*pftime);
   mytm=gmtime(&utime);
   
   pddate->Year=   mytm->tm_year-80;
   pddate->Month=  mytm->tm_mon+1;
   pddate->Day=    mytm->tm_mday;
   pdtime->Hour=   mytm->tm_hour;
   pdtime->Minute= mytm->tm_min;
   pdtime->Second= mytm->tm_sec/2;
   return TRUE;
}

BOOL WINAPI DosDateTimeToFileTime(WORD dostime, WORD dosdate,
                                  LPFILETIME pftime)
{
   time_t     utime;  
   struct tm  mytm;
   PDOSTIME  pdtime = (PDOSTIME) &dostime;
   PDOSDATE  pddate = (PDOSDATE) &dosdate;

   aprintf("DosDateTimeToFileTime()\n");     
   
   mytm.tm_year  = pddate->Year+80;
   mytm.tm_mon   = pddate->Month-1;
   mytm.tm_mday  = pddate->Day;
   mytm.tm_hour  = pdtime->Hour;
   mytm.tm_min   = pdtime->Minute;
   mytm.tm_sec   = pdtime->Second*2;
   utime	  = timegm(&mytm);
   *pftime=__TimeUnixToFile(utime);
   return TRUE;
}

LONG WINAPI CompareFileTime(CONST FILETIME *ptime1, CONST FILETIME *ptime2)
{
   LONGLONG comp;
   
   aprintf("CompareFileTime()\n");
   
   comp=MAKELONGLONG(ptime1->dwLowDateTime, ptime1->dwHighDateTime)-
        MAKELONGLONG(ptime2->dwLowDateTime, ptime2->dwHighDateTime);

   if(comp<0)
      return -1;
   else if(comp==0)
      return 0;
   else
      return 1;
}

VOID WINAPI GetSystemTimeAsFileTime(PFILETIME pftime)
{
   ULONGLONG wtime;
   struct timeval mytv;
   
   aprintf("GetSystemTimeAsFileTime()\n");
   
   gettimeofday(&mytv, NULL);
   wtime=((ULONGLONG) mytv.tv_sec * 1000000LL+ (ULONGLONG) mytv.tv_usec) *
         NSPERUSEC + FACTOR;
            
   pftime->dwLowDateTime  = LOLONG(wtime);
   pftime->dwHighDateTime = HILONG(wtime);
}

BOOL WINAPI SystemTimeToFileTime(CONST SYSTEMTIME *psystime, PFILETIME pftime)
{
   ULONGLONG      wtime;
   struct tm      mytm;
   
   aprintf("SystemTimeToFileTime()\n");
   
   mytm.tm_year  = psystime->wYear-1900;;
   mytm.tm_mon   = psystime->wMonth-1;
   mytm.tm_wday  = psystime->wDayOfWeek;
   mytm.tm_mday  = psystime->wDay;
   mytm.tm_hour  = psystime->wHour;
   mytm.tm_min   = psystime->wMinute;
   mytm.tm_sec   = psystime->wSecond;
   
   wtime = ((ULONGLONG) timegm(&mytm)*1000 + 
            (ULONGLONG) psystime->wMilliseconds) * 10000 
           + FACTOR; 

   pftime->dwLowDateTime  = LOLONG(wtime);
   pftime->dwHighDateTime = HILONG(wtime);
   return TRUE;
}

BOOL WINAPI FileTimeToLocalTime(CONST FILETIME * pftime, PSYSTEMTIME psystime)
{
   ULONGLONG     wtime;
   time_t        secs;
   int           usecs;
   struct tm     *mytm;
   
   aprintf("FileTimeToLocalTime()\n");
   
   wtime=MAKELONGLONG(pftime->dwLowDateTime, pftime->dwHighDateTime);
   secs=wtime/NSPERSEC;
   usecs=(wtime%NSPERSEC)/NSPERUSEC;
   
   mytm=localtime(&secs);   
   psystime->wYear=        mytm->tm_year+1900;
   psystime->wMonth=       mytm->tm_mon+1;
   psystime->wDayOfWeek=   mytm->tm_wday;
   psystime->wDay=         mytm->tm_mday;
   psystime->wHour=        mytm->tm_hour;
   psystime->wMinute=	 mytm->tm_min;
   psystime->wSecond=      mytm->tm_sec;  
   psystime->wMilliseconds=usecs;

   return TRUE;
}

BOOL WINAPI FileTimeToSystemTime(CONST FILETIME * pftime, PSYSTEMTIME psystime)
{
   ULONGLONG     wtime;
   time_t        secs;
   int           usecs;
   struct tm     *mytm;
   
   aprintf("FileTimeToSystemTime()\n");
   
   wtime=MAKELONGLONG(pftime->dwLowDateTime, pftime->dwHighDateTime);
   secs=wtime/NSPERSEC;
   usecs=(wtime%NSPERSEC)/NSPERUSEC;
   
   mytm=gmtime(&secs);   
   psystime->wYear=        mytm->tm_year+1900;
   psystime->wMonth=       mytm->tm_mon+1;
   psystime->wDayOfWeek=   mytm->tm_wday;
   psystime->wDay=         mytm->tm_mday;
   psystime->wHour=        mytm->tm_hour;
   psystime->wMinute=      mytm->tm_min;
   psystime->wSecond=      mytm->tm_sec;  
   psystime->wMilliseconds=usecs;

   return TRUE;
}

BOOL WINAPI FileTimeToLocalFileTime(CONST FILETIME *pftime, PFILETIME pltime)
{
    time_t     utime;
    struct tm *pmytm;
    
    aprintf("FileTimeToLocalFileTime()\n");
      
    /* First convert to struct tm, then convert that to local */
    utime=__TimeFileToUnix(*pftime);
    pmytm=localtime(&utime); /* It's GMT, make it local */
    utime=timegm(pmytm);      /* and pretend that's GMT */
    *pltime=__TimeUnixToFile(utime);
    
    return TRUE;
}

BOOL WINAPI LocalFileTimeToFileTime(CONST FILETIME *pltime, PFILETIME pftime)
{
    time_t utime;
    struct tm *pmytm;      

    aprintf("LocalFileTimeToFileTime()\n");

    /* First convert to struct tm, then convert that to local */
    utime=__TimeFileToUnix(*pltime);
    pmytm=gmtime(&utime);   /* It's local, but convert like it's GMT */
    utime=mktime(pmytm);    /* and pretend that's local */
    *pftime=__TimeUnixToFile(utime);
    
    return TRUE;
}

VOID WINAPI GetLocalTime(LPSYSTEMTIME pwtime)
{
   struct timeval mytv;
   struct tm     *mytm;
   time_t         mytime;

   aprintf("GetLocalTime()\n");

   gettimeofday(&mytv,NULL);
   mytime=mytv.tv_sec;   
   mytm=localtime(&mytime);   
   pwtime->wYear=        mytm->tm_year+1900;
   pwtime->wMonth=       mytm->tm_mon+1;
   pwtime->wDayOfWeek=   mytm->tm_wday;
   pwtime->wDay=         mytm->tm_mday;
   pwtime->wHour=        mytm->tm_hour;
   pwtime->wMinute=	 mytm->tm_min;
   pwtime->wSecond=      mytm->tm_sec;  
   pwtime->wMilliseconds=(mytv.tv_usec/1000)%1000;
}

VOID WINAPI GetSystemTime(LPSYSTEMTIME pwtime)
{
   struct timeval mytv;
   struct tm     *mytm;
   time_t	  mytime;
   
   aprintf("GetSystemTime()\n");
   
   gettimeofday(&mytv,NULL);
   mytime=mytv.tv_sec;
   mytm=gmtime(&mytime);    
   pwtime->wYear         = mytm->tm_year+1900;
   pwtime->wMonth        = mytm->tm_mon+1;
   pwtime->wDayOfWeek    = mytm->tm_wday;
   pwtime->wDay          = mytm->tm_mday;
   pwtime->wHour         = mytm->tm_hour;
   pwtime->wMinute       = mytm->tm_min;
   pwtime->wSecond       = mytm->tm_sec;
   pwtime->wMilliseconds = (mytv.tv_usec/1000)%1000;
}

BOOL WINAPI SetLocalTime(CONST SYSTEMTIME *pwtime)
{
   struct timeval mytv;
   struct tm      mytm;
   
   aprintf("SetLocalTime()\n");
   
   mytm.tm_year  = pwtime->wYear-1900;
   mytm.tm_mon   = pwtime->wMonth-1;
   mytm.tm_wday  = pwtime->wDayOfWeek;
   mytm.tm_mday  = pwtime->wDay;
   mytm.tm_hour  = pwtime->wHour;
   mytm.tm_min   = pwtime->wMinute;
   mytm.tm_sec   = pwtime->wSecond;
   
   mytv.tv_sec  = mktime(&mytm);
   mytv.tv_usec = pwtime->wMilliseconds*1000;
   settimeofday(&mytv,NULL);
   return TRUE;
}

BOOL WINAPI SetSystemTime(CONST SYSTEMTIME *pwtime)
{
   struct timeval mytv;
   struct tm      mytm;
   
   aprintf("SetSystemTime()\n");
   
   mytm.tm_year  = pwtime->wYear-1900;
   mytm.tm_mon   = pwtime->wMonth-1;
   mytm.tm_wday  = pwtime->wDayOfWeek;
   mytm.tm_mday  = pwtime->wDay;
   mytm.tm_hour  = pwtime->wHour;
   mytm.tm_min   = pwtime->wMinute;
   mytm.tm_sec   = pwtime->wSecond;
   
   mytv.tv_sec  = timegm(&mytm);
   mytv.tv_usec = pwtime->wMilliseconds*1000;
   settimeofday(&mytv,NULL);
   return TRUE;
} 

DWORD WINAPI GetTimeZoneInformation(LPTIME_ZONE_INFORMATION  ptz)
{
  aprintf("GetTimeZoneInformation()\n");
   
  memset(ptz, 0, sizeof(TIME_ZONE_INFORMATION));
  
  return TIME_ZONE_ID_UNKNOWN;
}