/*
 * nls/locale.c
 *
 */

#include<stdlib.h>
#include<string.h>
#include<win32/defines.h>
#include<win32/error.h>
#include<win32/nls.h>
#include"config.h"
#include"lctable.h"
#include"teb.h"
#include"error.h"

PLOCALE __UserLocale;

BOOL __LocaleInit(VOID)
{
   PSTR		locstr;   
   LCID		lcid;
   PLOCALE	plocale=NULL;
   
   locstr=getenv("WF_LOCALE");
   if(locstr)
   {  
      plocale=__Locale;
      while((plocale->Id)&&(strcasecmp(locstr,plocale->AbbrName)))
         plocale++;
   }
   /* if we do not have a locale, default */
   if(!plocale)
   {
      lcid=MAKELCID(MAKELANGID(SYSTEM_DEFAULT_LANGUAGE,
                               SYSTEM_DEFAULT_SUBLANGUAGE),
                      SYSTEM_DEFAULT_SORTORDER);
      plocale=__Locale;
      while((plocale->Id)&&(lcid!=plocale->Id))
         plocale++;                                                   
   }
   /* if this does not work, use our disaster plan */
   if(!plocale)
      plocale=__Locale;

   __UserLocale=plocale;
   __TebLocale=plocale;
   return TRUE;
}

LANGID WINAPI GetUserDefaultLangID(VOID)
{
   return LANGIDFROMLCID(__UserLocale->Id);
}

LCID WINAPI GetUserDefaultLCID(VOID)
{
   return __UserLocale->Id;
}

LANGID WINAPI GetSystemDefaultLangID(VOID)
{
   return MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US);
}

LCID WINAPI GetSystemDefaultLCID(VOID)
{
   return MAKELCID(MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US), 
                   SORT_DEFAULT);
}

LCID WINAPI GetThreadLocale(VOID)
{
   return __TebLocale->Id;
}

BOOL WINAPI SetThreadLocale(LCID lcid)
{
   PLOCALE plocale;

   plocale=__Locale;   
   /* find locale */
   while((plocale->Id)&&(lcid!=plocale->Id))
      plocale++;      
   if(!plocale)
      return __ErrorReturnFalse(ERROR_INVALID_PARAMETER);      
   __TebLocale=plocale;
   return TRUE;   
}

BOOL WINAPI IsValidLocale(LCID lcid, DWORD flags)
{
   PLOCALE plocale;

   plocale=__Locale;   
   
   /* find locale */
   while((plocale->Id)&&(lcid!=plocale->Id))
      plocale++;     
   /* is it valid ?? */    
   if(!plocale)
      return FALSE;
   else
      return TRUE;   
}

LPSTR static __xtoa(LPSTR str, DWORD val)
{
   LPSTR retstr=str;
   DWORD hex;
   
   do
   {
      hex=val%16;
      if(hex<10)
         *str=val + '0';
      else
         *str=val + 'A';
      val=val/16;
      str++;
   }
   while(val);
   return retstr;
}

LPWSTR static __xtow(LPWSTR str, DWORD val)
{
   LPWSTR retstr=str;
   DWORD hex;
   
   do
   {
      hex=val%16;
      if(hex<10)
         *str=val + '0';
      else
         *str=val + 'A';
      val=val/16;
      str++;
   }
   while(val);
   return retstr;
}


BOOL WINAPI EnumSystemLocalesA(LOCALE_ENUMPROCA enumproc, DWORD flags)
{
   CHAR		locstr[10];
   BOOL		retval;
   PLOCALE	plocale;   
   
   if(!enumproc)
      return __ErrorReturnFalse(ERROR_INVALID_PARAMETER);
      
   plocale=__Locale;
   retval=TRUE;
   while((plocale->Id)&&(retval))
   {
      __xtoa(locstr,plocale->Id); 
      retval=enumproc(locstr);
   }
   return TRUE;
}

BOOL WINAPI EnumSystemLocalesW(LOCALE_ENUMPROCW enumproc, DWORD flags)
{
   WCHAR	locstr[10];
   BOOL		retval=TRUE;
   PLOCALE	plocale=__Locale;   
   
   if(!enumproc)
      return __ErrorReturnFalse(ERROR_INVALID_PARAMETER);
      
   while((plocale->Id)&&(retval))
   {
      __xtow(locstr,plocale->Id);
      retval=enumproc(locstr);
   }
   return TRUE;
}

INT WINAPI GetLocaleInfoW(LCID lcid, LCTYPE lctype, LPWSTR buffer, INT bufsize)
{
   INT		retcnt=0;
   PLOCALE	plocale=__Locale;  
   LPWSTR	infostr; 
   
   while(plocale->Id!=lcid)
      plocale++;
   
   if(lctype<89)
      infostr=plocale->Info0[lctype];
   else if((lctype>0x1000)&&(lctype<0x1011))
      infostr=plocale->Info1[lctype-0x1000];
   else
      { SetLastError(ERROR_INVALID_PARAMETER); return 0; }
      
   if(bufsize)
   {   
      /* I really need a wide string copy, here */
      do      
      {
         *buffer=*infostr;
         infostr++;
         buffer++;
         bufsize--;
         retcnt++;
      }
      while((*infostr)&&(bufsize));
   }
   else
   {
      /* I really need a wide string length, here */
      do
      {
         infostr++;
         retcnt++;
      }
      while(*infostr);
   }
   return retcnt;
}

INT WINAPI GetLocaleInfoA(LCID lcid, LCTYPE lctype, LPSTR buffer, INT bufsize)
{
   INT		retcnt=0;
   PLOCALE	plocale=__Locale;  
   LPWSTR	infostr; 
   
   while(plocale->Id!=lcid)
      plocale++;
   
   if(lctype<89)
      infostr=plocale->Info0[lctype];
   else if((lctype>0x1000)&&(lctype<0x1011))
      infostr=plocale->Info1[lctype-0x1000];
   else
      { SetLastError(ERROR_INVALID_PARAMETER); return 0; }
      
   if(bufsize)
   {   
      /* I really need a WideCharToMultiByte here */
      do      
      {
         *buffer=*infostr;
         infostr++;
         buffer++;
         bufsize--;
         retcnt++;
      }
      while((*infostr)&&(bufsize));
   }
   else
   {
      /* I really need a wide string length, here */
      do
      {
         infostr++;
         retcnt++;
      }
      while(*infostr);
   }
   return retcnt;
}
