/*
 *  kernel/wsprintf.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 *  Changes are copyright (C) 1996, Onno Hovers
 *
 *  vsprintf.c -- Lars Wirzenius & Linus Torvalds.
 *  Wirzenius wrote this portably, Torvalds fucked it up :-)
 *  
 *  Rewrote this function so it can 'sprintf' unicode and floats
 *  and cleaned it up -- Onno
 */

#include <stdarg.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/ctype.h>

/* we use this so that we can do without the ctype library */
#define is_digit(c)    ((c) >= '0' && (c) <= '9')

/* simple unicode to ansi conversion and back */
static LPSTR strutoa(LPSTR buf, LPWSTR str)
{
   while(*str)
   {
      *buf=*str;
      str++;
      buf++;
   }
}

static LWPSTR stratou(LPWSTR buf, LPSTR str)
{
   while(*str)
   {
      *buf=*str;
      str++;
      buf++;
   }
}


static int skip_atoiA(const char **s)
{
    int i=0;

    while (is_digit(**s))
        i = i*10 + *((*s)++) - '0';
    return i;
}

static int skip_atoiW(const WCHAR **s)
{
    int i=0;

    while (is_digit(**s))
        i = i*10 + *((*s)++) - '0';
    return i;
}


#define ZEROPAD	1	/* pad with zero */
#define SIGN	2	/* unsigned/signed long */
#define PLUS	4	/* show plus */
#define SPACE	8	/* space if plus */
#define LEFT	16	/* left justified */
#define SPECIAL	32	/* 0x */
#define LARGE	64	/* use 'ABCDEF' instead of 'abcdef' */

#define TYPE_NONE    0
#define TYPE_NUMBER  1
#define TYPE_POINTER 2
#define TYPE_STRING  3
#define TYPE_CHAR    4
#define TYPE_BUFSIZE 5
#define TYPE_FLOAT   6

#define do_div(n,base) ({ \
int __res; \
__res = ((ULONGLONG) n) % (unsigned) base; \
n     = ((ULONGLONG) n) / (unsigned) base; \
__res; })

static char * numberA(char * str, long num, int base, int size, int precision
    ,int type)
{
    char c,sign,tmp[66];
    const char *digits="0123456789abcdefghijklmnopqrstuvwxyz";
    int i;

    if (type & LARGE)
        digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    if (type & LEFT)
        type &= ~ZEROPAD;
    if (base < 2 || base > 36)
        return 0;
    c = (type & ZEROPAD) ? '0' : ' ';
    sign = 0;
    if (type & SIGN) {
        if (num < 0) {
            sign = '-';
            num = -num;
            size--;
        } else if (type & PLUS) {
            sign = '+';
            size--;
        } else if (type & SPACE) {
            sign = ' ';
            size--;
        }
    }
    if (type & SPECIAL) {
        if (base == 16)
            size -= 2;
        else if (base == 8)
            size--;
    }
    i = 0;
    if (num == 0)
        tmp[i++]='0';
    else while (num != 0)
        tmp[i++] = digits[do_div(unum,base)];
    if (i > precision)
        precision = i;
    size -= precision;
    if (!(type&(ZEROPAD+LEFT)))
        while(size-->0)
            *str++ = ' ';
    if (sign)
        *str++ = sign;
    if (type & SPECIAL)
        if (base==8)
            *str++ = '0';
        else if (base==16) {
            *str++ = '0';
            *str++ = digits[33];
        }
    if (!(type & LEFT))
        while (size-- > 0)
            *str++ = c;
    while (i < precision--)
        *str++ = '0';
    while (i-- > 0)
        *str++ = tmp[i];
    while (size-- > 0)
        *str++ = ' ';
    return str;
}

static LPWSTR numberW(LPWSTR str, LONGLONG num, INT base, INT size, 
                       INT precision, INT type)
{
    WCHAR      c,sign,tmp[66];
    ULONGLONG  unum;
    
    const char *digits="0123456789abcdefghijklmnopqrstuvwxyz";
    INT i;

    if (type & LARGE)
        digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    if (type & LEFT)
        type &= ~ZEROPAD;
    if (base < 2 || base > 36)
        return 0;
    c = (type & ZEROPAD) ? '0' : ' ';
    sign = 0;
    unum=num;
    if (type & SIGN) 
    {
        if ( num < 0) 
        {
            sign = '-';
            unum = -num;
            size--;
        } 
        else if (type & PLUS) 
        {
            sign = '+';
            size--;
        } 
        else if (type & SPACE)
        {
            sign = ' ';
            size--;
        }
    }
    if (type & SPECIAL)
    {
        if (base == 16)
            size -= 2;
        else if (base == 8)
            size--;
    }
    i = 0;
    if (unum == 0)
        tmp[i++]='0';
    else while (unum != 0)
        tmp[i++] = digits[do_div(unum,base)];
    if (i > precision)
        precision = i;
    size -= precision;
    if (!(type&(ZEROPAD+LEFT)))
        while(size-->0)
            *str++ = ' ';
    if (sign)
        *str++ = sign;
    if (type & SPECIAL)
        if (base==8)
            *str++ = '0';
        else if (base==16) {
            *str++ = '0';
            *str++ = digits[33];
        }
    if (!(type & LEFT))
        while (size-- > 0)
            *str++ = c;
    while (i < precision--)
        *str++ = '0';
    while (i-- > 0)
        *str++ = tmp[i];
    while (size-- > 0)
        *str++ = ' ';
    return str;
}

static inline int getflagsA(LPCSTR *ppformat)
{
   int       flags;
   LPCSTR    format;
   
   flags = 0;
   format= *ppformat;
   
   while(1)
   {
      format++;        /* this also skips first '%' */
      switch (*format) 
      {
          case '-': flags |= LEFT; break;
          case '+': flags |= PLUS; break;
          case ' ': flags |= SPACE; break;
          case '#': flags |= SPECIAL; break;
          case '0': flags |= ZEROPAD; break;
          default: *ppformat=format; return flags;
      }
   }
}

static inline int getflagsW(LPCWSTR *ppformat)
{
   int       flags;
   LPCWSTR    format;
   
   flags = 0;
   format= *ppformat;
   
   while(1)
   {
      format++;        /* this also skips first '%' */
      switch (*format) 
      {
          case '-': flags |= LEFT; break;
          case '+': flags |= PLUS; break;
          case ' ': flags |= SPACE; break;
          case '#': flags |= SPECIAL; break;
          case '0': flags |= ZEROPAD; break;
          default: *ppformat=format; return flags;
      }
   }
}

int wvsprintfA(char *buf, const char *fmt, va_list args)
{
    int len;
    unsigned long num;
    int i, base;
    char * str;
    const char *s;

    int flags;        /* flags to number() */

    int field_width;    /* width of output field */
    int precision;        /* min. # of digits for integers; max
                   number of chars for from string */
    int qualifier;        /* 'h', 'l', or 'L' for integer fields */

    for (str=buf ; *fmt ; ++fmt) 
    {
        if (*fmt != '%') 
        {
            *str++ = *fmt;
            continue;
        }
            
        /* process flags */
        flags = 0;
        repeat:
            ++fmt;        /* this also skips first '%' */
            switch (*fmt) {
                case '-': flags |= LEFT; goto repeat;
                case '+': flags |= PLUS; goto repeat;
                case ' ': flags |= SPACE; goto repeat;
                case '#': flags |= SPECIAL; goto repeat;
                case '0': flags |= ZEROPAD; goto repeat;
                }
        
        /* get field width */
        field_width = -1;
        if (is_digit(*fmt))
        {
            field_width = skip_atoiA(&fmt);
        }
        else if (*fmt == '*') 
        {
            ++fmt;
            /* it's the next argument */
            field_width = va_arg(args, int);
            if (field_width < 0) {
                field_width = -field_width;
                flags |= LEFT;
            }
        }

        /* get the precision */
        precision = -1;
        if (*fmt == '.') 
        {
            ++fmt;    
            if (is_digit(*fmt))
            {
                precision = skip_atoiA(&fmt);
            }
            else if (*fmt == '*') 
            {
                ++fmt;
                /* it's the next argument */
                precision = va_arg(args, int);
            }
            if (precision < 0)
                precision = 0;
        }

        /* get the conversion qualifier */
        qualifier = -1;
        if (*fmt == 'h' || *fmt == 'l' || *fmt == 'L') 
        {
            qualifier = *fmt;
            ++fmt;
        }

        /* default base */
        base = 10;

        switch (*fmt) 
        {
            case 'C': /* unicode */
            case 'c':
                if(!(flags & LEFT))
                    while (--field_width > 0)
                        *str++ = ' ';
                *str++ = (unsigned char) va_arg(args, int);
                while (--field_width > 0)
                    *str++ = ' ';
                continue;
        
            case 'S': /* unicode */                        
            case 's':
                s = va_arg(args, char *);
                if(!s)
                    s = "<NULL>";

                
                len = strnlenA(s, precision);

                if(!(flags & LEFT))
                   while (len < field_width--)
                       *str++ = ' ';
                for(i = 0; i < len; ++i)
                    *str++ = *s++;
                while (len < field_width--)
                    *str++ = ' ';
                continue;
              
        case 'P': /* own extension */
            flags|=LARGE;              
        case 'p':
            if (field_width == -1) {
                field_width = 2*sizeof(void *);
                flags |= ZEROPAD;
            }
            str = number(str,
                (unsigned long) va_arg(args, void *), 16,
                field_width, precision, flags);
            continue;


        case 'n':
            if (qualifier == 'l') {
                long * ip = va_arg(args, long *);
                *ip = (str - buf);
            } else {
                int * ip = va_arg(args, int *);
                *ip = (str - buf);
            }
            continue;

        /* integer number formats - set up the flags and "break" */
        case 'o':
            base = 8;
            break;

        case 'X':
            flags |= LARGE;
        case 'x':
            base = 16;
            break;

        case 'd':
        case 'i':
            flags |= SIGN;
        case 'u':
            break;

        default:
            if (*fmt != '%')
                *str++ = '%';
            if (*fmt)
                *str++ = *fmt;
            else
                --fmt;
            continue;
        }
        if (qualifier == 'l')
            num = va_arg(args, unsigned long);
        else if (qualifier == 'h')
            if (flags & SIGN)
                num = va_arg(args, short);
            else
                num = va_arg(args, unsigned short);
        else if (flags & SIGN)
            num = va_arg(args, int);
        else
            num = va_arg(args, unsigned int);
        str = numberA(str, num, base, field_width, precision, flags);
    }
    *str = '\0';
    return str-buf;
}

/* just about the same thing, except for the wide chars */
INT wvsprintfW(LPWSTR buf, LPCWSTR format, va_list args)
{
    INT		len;
    ULONG	num;
    INT		i, base;
    LPWSTR	str;
    LPCWSTR	s;

    INT		flags;		/* flags to number() */
    INT		field_width;	/* width of output field */
    INT		precision;	/* min. # of digits for integers; max
                                   number of chars for from string */
    INT         qualifier;      /* 'h', 'l', or 'L' for integer fields */
    INT         type;           /* type of field */
    BOOL        ascii;        /* if we use the 'S' or 'C' field */
    
    for (str=buf ; *format ; ++format) 
    {
        if (*format != '%') 
        {
            *str = *format;
            str++;
            continue;
        }
            
        /* process flags */
        flags = getflagsW(&format);

        /* get field width */        
        field_width = -1;
        if (is_digit(*format))
            field_width = skip_atoiW(&format);
        else if (*format == '*') 
        {
            ++format;
            /* it's the next argument */
            field_width = va_arg(args, int);
            if (field_width < 0) 
            {
                field_width = -field_width;
                flags |= LEFT;
            }
        }

        /* get the precision */
        precision = -1;
        if (*format == '.') 
        {
            ++format;    
            if (is_digit(*format))
                precision = skip_atoiW(&format);
            else if (*format == '*') 
            {
                ++format;
                /* it's the next argument */
                precision = va_arg(args, int);
            }
            if (precision < 0)
                precision = 0;
        }

        /* get the conversion qualifier */
        qualifier = -1;
        if((*format == 'h') || (*format == 'l') ||
           (*format == 'L') || (*format == 'q') ) 
        {
            if((*format=='l') && (qualifier=='l'))
                qualifier = 'q';
            else
                qualifier = *format;
            ++format;
        }
        
        /* set defaults */
        base  = 10;
        ascii = false;     

        /* get the type code */
        switch (*format) 
        {                
             case 'C': /* ascii */
                ascii=true;
             case 'c':
                type = TYPE_CHAR;
                break;

             case 'S': /* ascii */
                ascii=true;                    
             case 's':            
                type = TYPE_STRING;
                break;
                                  
             case 'P': /* own extension */
                flags|=LARGE;              
             case 'p':
                type = TYPE_POINTER;
                break;

             case 'n':          
                type = TYPE_BUFSIZE;
                break;

             case 'o':
                type = TYPE_NUMBER;
                base = 8;
                break;

             case 'X':
                flags |= LARGE;
             case 'x':
                type = TYPE_NUMBER;
                base = 16;
                break;

             case 'd':
             case 'i':
                type = TYPE_NUMBER;
                flags |= SIGN;
             case 'u':
                break;

             default:
                type = TYPE_NONE;            
        }
        
        if(type==TYPE_NUMBER)
        {        
            if((qualifier == 'q')||(qualifier == 'L'))
            {
               LONGLONG num;
               
               if (flags & SIGN)
                   num = va_arg(args, LONGLONG );
               else
                   num = va_arg(args, ULONGLONG );
               
               str = numberW(str, num, base, field_width, precision, flags);                       
            }
            if (qualifier == 'l')
            {
               LONG num;
               
               if (flags & SIGN)
                   num = va_arg(args, LONG );
               else
                   num = va_arg(args, ULONG );
               
               str = numberW(str, num, base, field_width, precision, flags);
            }
            else if (qualifier == 'h')
            {
               if (flags & SIGN)
                   num = va_arg(args, SHORT );
               else
                   num = va_arg(args, USHORT );
            }
            else 
            {
               if (flags & SIGN)
                  num = va_arg(args, INT);
               else
                  num = va_arg(args, UINT);
            }
            str = numberW(str, num, base, field_width, precision, flags);
        }
        else if(type==TYPE_POINTER)
        {
            if (field_width == -1) 
            {
                field_width = 2*sizeof(LPVOID);
                flags |= ZEROPAD;
            }
            str = numberW(str,
                (unsigned long) va_arg(args, LPVOID), 16,
                field_width, precision, flags);                
        }       
        else if(type==TYPE_CHAR)
        {
            if (!(flags & LEFT))
                while (--field_width > 0)
                    *str++ = ' ';
            *str++ = (unsigned char) va_arg(args, int);                        
            while (--field_width > 0)
                *str++ = ' ';
        }
        else if(type==TYPE_STRING)
        {
            s = va_arg(args, char *);
            if (!s)
                s = "<NULL>";

            if(qualifier=='h')
               ascii=FALSE;
            else if(qualifier=='l')
               ascii=TRUE;
             
            if(ascii)  
               len = strnlen(s, precision);
               s = va_arg(args, LPSTR );
            else 
               len = strnlenW(s, precision);
               s = va_arg(args, LPWSTR );

            if (!(flags & LEFT))
                while (len < field_width--)
                    *str++ = ' ';
            for (i = 0; i < len; ++i)
                *str++ = *s++;
            while (len < field_width--)
                *str++ = ' ';
        }
        else if( type == TYPE_BUFSIZE )
        {
            if (qualifier == 'l') 
            {
                long * ip = va_arg(args, long *);
                *ip = (str - buf);
            } 
            else 
            {
                int * ip = va_arg(args, int *);
                *ip = (str - buf);
            }
        }
        else if( type == TYPE_NONE )
        {            
            if (*format != '%')
                *str++ = '%';
            if (*format)
                *str++ = *format;
            else
                --format;                
        }
    }
    *str = '\0';
    return str-buf;
}

int wsprintfA(char * buf, const char *format, ...)
{
    va_list args;
    int i;

    va_start(args, format);
    i=wvsprintfA( buf,format,args);
    va_end(args);
    return i;
}

int wsprintfW(char * buf, const char *format, ...)
{
    va_list args;
    int i;

    va_start(args, format);
    i=wvsprintfW( buf,format,args);
    va_end(args);
    return i;
}