/*
 * tools/exprelay.c
 * Copyright (C) 1996, Onno Hovers, All rights reserved
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <win32/defines.h>

#define TYPE_UNUSED      0
#define TYPE_INVALID     1
#define TYPE_STDCALL     2
#define TYPE_CDECL       3
#define TYPE_DIRECT      4
#define TYPE_RETURN      5
#define TYPE_PTR         6
#define TYPE_REGS        7
#define TYPE_UNKNOWN     8
#define IS_UNIMP         1
#define IS_RETURN        2
#define MAX_ORDINAL	1299

#ifndef MAX_PATH
#define MAX_PATH 256
#endif

#define TOKEN_EOLN '\0'

typedef struct __EXPORT_DEFINITION
{
    INT    Type;
    INT    Flags;
    DWORD  Offset;
    INT    Args;
    INT    Return;     
    CHAR   ExportName[80];
    CHAR   InternalName[80];
} EXPORT_DEFINITION;

EXPORT_DEFINITION ExportDefinitions[MAX_ORDINAL];

LPSTR Prefix="";
BOOL  Debug=FALSE;
BOOL  Spy=FALSE;
BOOL  Masm=FALSE;
FILE *InputFile;
FILE *RelayFile;
INT   UnusedOrdinal=0;
INT   LargestOrdinal=0;
INT   OrdinalBase=0;
INT   ExportCount=0;
INT   ExportByNameCount=0;
INT   LineNumber=0;
CHAR  Module [MAX_PATH];
CHAR  ParseBuffer [512];
PCHAR ParseChar;

CONST CHAR  UnimpFunction[]=  "_exp_unimp";
CONST CHAR  UnknownFunction[]="_exp_unknown";
CONST CHAR  SpyFunction[]=    "_exp_spy";

INT StringToNumber(LPSTR  str);
BOOL GetInputLine(VOID);
BOOL GetToken(LPSTR token);
VOID InitExports(LPSTR filename);
VOID DoneExports(VOID);
VOID ParseExports(VOID);
VOID MakeExportsTable(VOID);
VOID MakeExportsNames(VOID);
VOID MakeExportsStubs(VOID);
VOID StubStdcallUnimp(INT noparms, LPSTR exportname, LPSTR inname);
VOID StubStdcall(INT noparms, LPSTR exportname, LPSTR inname);
VOID StubCdeclUnimp(LPSTR exportname, LPSTR inname);
VOID StubCdecl(LPSTR exportname, LPSTR inname);
VOID StubDirect(LPSTR exportname, LPSTR inname);
VOID StubReturn(LPSTR exportname, LPSTR inname);
VOID StubUnknown(LPSTR exportname);
VOID ExpRelay(LPSTR filename);
void usage(void);


INT StringToNumber(LPSTR  str)
{
    INT retval;

    retval=0;
        
    while (str[0] != '\0')
    {
	if (!isdigit(str[0]))	
	    return -1;
	retval=retval*10+str[0]-'0';
	str++;		    
    }
    
    return retval;
}

BOOL GetInputLine(VOID)
{

   INT i;  
    
   do
   {
       if (feof(InputFile))
       {
          ParseBuffer[0]='\0';
          return FALSE;       
       }
       
       LineNumber++;
       
       if (fgets(ParseBuffer, 511, InputFile) == NULL)
       {
          ParseBuffer[0]='\0';
          return FALSE;
       }
       
       for(i=0;isspace(ParseBuffer[i]);i++)
          /* nothing */;      
   }
   while((ParseBuffer[i] == '#')||(ParseBuffer[i]== '\n'));

   ParseChar=ParseBuffer;  
   return TRUE;
}

BOOL GetToken(LPSTR token)
{

   /*
    * Remove initial white space.
    */
   while(isspace(ParseChar[0]))
      ParseChar++;
       
   if((ParseChar[0] == '\0') || (ParseChar[0] == '#') || (ParseChar[0]=='\n'))
   {
      token[0]='\0';      
      return TRUE;
   }
   
   /*
    * Find end of token.
    */
   while( (ParseChar[0] != '\0')  && 
          (ParseChar[0] != '#' )  && 
          (!isspace(ParseChar[0])) )
   {
      token[0]=ParseChar[0];
      ParseChar++;
      token++;
   }   
   token[0]='\0';
     
   return TRUE;
}

VOID InitExports(LPSTR filename)
{
   LPSTR suffix;
   LPSTR modname;
   LPSTR modbuf;
   
   /* open exports file */
   InputFile=fopen(filename,"rt");
   if(!InputFile)
   {
     fprintf(stderr,"fatal error: cannot open exports file\n");
     exit(1);
   }
   
   /* get name of relay file and open it */
   suffix=strrchr(filename,'.');
   if(!suffix)
   {
      fprintf(stderr,"fatal: file %s is not an .exp file\n", filename);
      exit(1);
   }   

   if(!Masm)
      strcpy(suffix,".s");
   else
      strcpy(suffix,".asm");
            
   RelayFile=fopen(filename,"wt");
   if(!RelayFile)
   {
      fprintf(stderr,"fatal error: cannot open relay file\n");
      exit(1);
   }
     
   /* get a module name */   
   suffix[0]='\0';
   modname=strrchr(filename,'/');
   if(!modname)
      modname=filename;
   else
      modname++;
   
   modbuf=Module;
   while(modname[0])
   {  
      if(!islower(modname[0]))
          modbuf[0]=modname[0];
      else
          modbuf[0]=toupper(modname[0]);
     
     modname++;
     modbuf++;     
   }

   
   /* initialize data structures */
   memset(ExportDefinitions,0,sizeof(EXPORT_DEFINITION)*MAX_ORDINAL);  
   UnusedOrdinal=0;
   LargestOrdinal=0;
   ExportCount=0;
   ExportByNameCount=0;
   LineNumber=0;
}

VOID DoneExports(VOID)
{
   fclose(InputFile);
   fclose(RelayFile);
}

VOID ParseExports(VOID)
{
   CHAR   token[80];
   CHAR   exportname[80];
   INT    ordinal;
   INT    nparms;
  
   while(GetInputLine())
   { 
      /* get name */  
      GetToken(exportname);     
      
      if(exportname[0]=='*')
         exportname[0]='\0';
      else    
         ExportByNameCount++;
           
      ExportCount++;
      /* get ordinal */
      GetToken(token);
      if((token[0]=='*')||(token[0]==TOKEN_EOLN))
      {
         /* get a valid ordinal */
         while(ExportDefinitions[UnusedOrdinal].Type)
         {
            UnusedOrdinal++;
            if(UnusedOrdinal>MAX_ORDINAL)
            {
               fprintf(stderr,"fatal error: too many exports\n");
               exit(1);
            }
         }
         ordinal=UnusedOrdinal;
      }
      else
      {
         ordinal=StringToNumber(token);
         if(ordinal==-1)
         {
            fprintf(stderr,"fatal error: not an ordinal\n");        
         }
         /* check this ordinal */
         if(ExportDefinitions[ordinal].Type)
         {
            fprintf(stderr,"fatal error: ordinal already used\n");
            exit(1);
         }
      }
      /* set the largest ordinal */
      LargestOrdinal=max(LargestOrdinal,ordinal);
     
      /* set the export name of this export definition */
      strncpy(ExportDefinitions[ordinal].ExportName,exportname,79);
      
      if(token[0]==TOKEN_EOLN)
      {
         ExportDefinitions[ordinal].Type=TYPE_UNKNOWN;
         continue;
      }
      
      /* get function type */  
      GetToken(token);
      if(token[0]==TOKEN_EOLN)
      {
        /* create general stub function */
        ExportDefinitions[ordinal].Type=TYPE_UNKNOWN;
        continue;
      }
 
      if((strcmp(token,"?")==0)||(strcmp(token,"unknown")==0))     
      {    
         ExportDefinitions[ordinal].Type=TYPE_UNKNOWN;
         ExportDefinitions[ordinal].Args=0;
      }
      else if(strcmp(token,"cdecl")==0)
      {
         ExportDefinitions[ordinal].Type=TYPE_CDECL;
         ExportDefinitions[ordinal].Args=0;
      }        
      else if(strcmp(token,"return")==0)
         ExportDefinitions[ordinal].Type=TYPE_RETURN;
 
      else if(strcmp(token,"direct")==0)
         ExportDefinitions[ordinal].Type=TYPE_DIRECT;  
 
      else if(strcmp(token,"regs")==0)
         ExportDefinitions[ordinal].Type=TYPE_REGS;
 
      else if(strcmp(token,"ptr")==0)
         ExportDefinitions[ordinal].Type=TYPE_PTR;
      else
      {       
         nparms=StringToNumber(token);
         ExportDefinitions[ordinal].Type=TYPE_STDCALL;
         ExportDefinitions[ordinal].Args=nparms;
      }
 
      /* get real function name */
      GetToken(token);
      if(token[0]==TOKEN_EOLN)
      {
         /* stub function */
         ExportDefinitions[ordinal].Flags=IS_UNIMP;
         strcpy(ExportDefinitions[ordinal].InternalName,"0");
         continue;
      }
      else if(strcmp(token,"unimp")==0)
      {
         ExportDefinitions[ordinal].Flags=IS_UNIMP;
         GetToken(token);
         if(token[0]!=TOKEN_EOLN)
         {
            /* stub function */
            ExportDefinitions[ordinal].Flags=IS_UNIMP;
            strncpy(ExportDefinitions[ordinal].InternalName,token,80);
         } 
         else
         {
            strcpy(ExportDefinitions[ordinal].InternalName,"0");
         }
      }
      else
      {
         strncpy(ExportDefinitions[ordinal].InternalName,token,80);
      }
   }
}

/*
 * We simply make our export table that looks just like 
 * one, that a pe-file would normally have 
 */
    
VOID MakeExportsTable(VOID)
{
   INT    ordinal;   
   
   fprintf(RelayFile,"#\n");
   fprintf(RelayFile,"# relay file, produced automatically, do not edit \n");
   fprintf(RelayFile,"#\n");
   fprintf(RelayFile,"# exports table \n");
   fprintf(RelayFile,"#\n");
   
   fprintf(RelayFile,"\t.globl %s%s_Exports\n",Prefix,Module);
   fprintf(RelayFile,"%s%s_Exports:\n",Prefix,Module);
      
   /* make export directory */
   fprintf(RelayFile,"\t.long 0\n");  /* characteristics */
   fprintf(RelayFile,"\t.long 0\n");  /* time/date stamp */
   fprintf(RelayFile,"\t.long 0\n");  /* version  */
   fprintf(RelayFile,"\t.long %s_nameof_%s\n", Prefix, Module); /* name */
   fprintf(RelayFile,"\t.long 0\n");  /* OrdinalBase */
   fprintf(RelayFile,"\t.long %d\n", ExportCount );
   fprintf(RelayFile,"\t.long %d\n", ExportByNameCount );
   fprintf(RelayFile,"\t.long %s_%s_ExportPointers\n", Prefix, Module);
   fprintf(RelayFile,"\t.long %s_%s_ExportNames\n", Prefix, Module);
   fprintf(RelayFile,"\t.long %s_%s_ExportOrdinals\n", Prefix, Module);
      
   /* make function pointer array */
   fprintf(RelayFile,"\n%s_%s_ExportPointers:\n", Prefix, Module);
   for(ordinal=0;ordinal<=LargestOrdinal;ordinal++)
   {
      switch(ExportDefinitions[ordinal].Type)
      {      
        case TYPE_UNUSED:
           fprintf(RelayFile,"\t.long 0\n" );
           break;
           
        case TYPE_PTR: 
           fprintf(RelayFile,"\t.long %s%s\n", Prefix, 
                    ExportDefinitions[ordinal].InternalName);
           break;
        case TYPE_UNKNOWN:
           fprintf(RelayFile,"\t.long %s%s\n", Prefix,                 
                          ExportDefinitions[ordinal].ExportName);
           break;               
        default:
           if(ExportDefinitions[ordinal].Flags & IS_UNIMP)
              fprintf(RelayFile,"\t.long %s%s\n", Prefix,                 
                      ExportDefinitions[ordinal].ExportName);
           else
              fprintf(RelayFile,"\t.long %s%s\n", Prefix,                 
                      ExportDefinitions[ordinal].InternalName);

              
      }         
   }
   
   /* make function name array */
   fprintf(RelayFile,"\n%s_%s_ExportNames:\n", Prefix, Module);
   for(ordinal=0;ordinal<=LargestOrdinal;ordinal++)
   {
      if(ExportDefinitions[ordinal].ExportName[0])
      {
         fprintf(RelayFile,"\t.long _nameof_%s\n",
                       ExportDefinitions[ordinal].ExportName);
      }
   }   
   /* make function ordinal array */
   fprintf(RelayFile,"\n%s_%s_ExportOrdinals:\n", Prefix, Module);
   for(ordinal=0;ordinal<=LargestOrdinal;ordinal++)
   {
      if(ExportDefinitions[ordinal].ExportName[0])
         fprintf(RelayFile,"\t.word %d\n",ordinal);    
   }
}

VOID MakeExportsNames(VOID)
{
   INT   ordinal;
   LPSTR exportname; 
   
   fprintf(RelayFile,"#\n");
   fprintf(RelayFile,"# function names\n");
   fprintf(RelayFile,"#\n");

   fprintf(RelayFile,"%s_nameof_%s:\n", 
                 Prefix,Module);
   fprintf(RelayFile,"\t.asciz \"%s\"\n\n",
                 Module);
   
   for(ordinal=0;ordinal<=LargestOrdinal;ordinal++)      
      if(ExportDefinitions[ordinal].Type)
      {
         exportname=ExportDefinitions[ordinal].ExportName;
         fprintf(RelayFile,"%s_nameof_%s:\n", 
                 Prefix,exportname);
         fprintf(RelayFile,"\t.asciz \"%s\"\n\n",
                 exportname);
      }
}

VOID MakeExportsStubs(VOID)
{
   INT    ordinal;
   INT    noparms;
   INT    flags;
   LPSTR  exportname;
   LPSTR  inname;   
   
   fprintf(RelayFile,"#\n");
   fprintf(RelayFile,"# function stubs\n");
   fprintf(RelayFile,"#\n\n");
   for(ordinal=0;ordinal<=LargestOrdinal;ordinal++)
      if(ExportDefinitions[ordinal].Type)
      {
         exportname=ExportDefinitions[ordinal].ExportName;     
         inname    =ExportDefinitions[ordinal].InternalName;     
         noparms   =ExportDefinitions[ordinal].Args;     
         flags     =ExportDefinitions[ordinal].Flags;     
                  
         switch(ExportDefinitions[ordinal].Type)
         {            
            case TYPE_STDCALL:
                 if(flags & IS_UNIMP)
                    StubStdcallUnimp(noparms, exportname, inname );
                 else
                    StubStdcall(noparms, exportname, inname); 
                 break;
                 
            case TYPE_CDECL:
                 if(flags & IS_UNIMP)
                    StubCdeclUnimp(exportname, inname);
                 else
                    StubCdecl(exportname, inname);
                 break;
                 
            case TYPE_DIRECT:
                 if(flags & IS_UNIMP)
                    StubUnknown(exportname);
                 else
                    StubDirect(exportname, inname);
                 break; 
            
            case TYPE_RETURN:
                 if(flags & IS_UNIMP)
                    StubUnknown(exportname);
                 else
                    StubReturn(exportname, inname);
                 break; 
            
            case TYPE_PTR:
                 /* no stub is needed for ptr */
                 break;
            
            case TYPE_REGS:
                 StubUnknown(exportname);
                        
            case TYPE_UNKNOWN:
                 StubUnknown(exportname);
         }  
      }
}


VOID StubStdcallUnimp(INT noparms, LPSTR exportname, LPSTR inname)
{
   if(!Masm)
   {
      fprintf(RelayFile,"%s%s:\n", 
                 Prefix, exportname);
      if(Debug||Spy)
      {
         fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
         fprintf(RelayFile,"\tleal   4(%%esp),%%eax\n");
         fprintf(RelayFile,"\tpushl  %%eax\n");
         fprintf(RelayFile,"\tpushl  $%d\n",noparms);
         fprintf(RelayFile,"\tcall   %s%s\n",Prefix,UnimpFunction);
         fprintf(RelayFile,"\taddl   $12,%%esp\n");
      }
      else
      {
         fprintf(RelayFile,"\tpushl  $120\n");
         fprintf(RelayFile,"\tcall   %sSetLastError\n",Prefix);
         fprintf(RelayFile,"\tpopl   %%eax\n");         
      }
      fprintf(RelayFile,"\tmovl   %s%s,%%eax\n",Prefix,inname);
      fprintf(RelayFile,"\tret    $%d\n\n\n",noparms*4);
   }
}


VOID StubStdcall(INT noparms, LPSTR exportname, LPSTR inname)
{ 
#ifdef NOTNOW
   INT i;
   
   if(!Masm)
   {
      fprintf(RelayFile,"\t.globl\t%s%s_%s\n\n", Prefix, Module, exportname);
      fprintf(RelayFile,"%s%s_%s:\n",  Prefix, Module, exportname);                   
      fprintf(RelayFile,"\tpushl  %%ebp\n");
      fprintf(RelayFile,"\tmovl   %%esp,%%ebp\n");
      if(Spy)
      {
         fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
         fprintf(RelayFile,"\tleal   8(%%esp),%%eax\n");
         fprintf(RelayFile,"\tpushl  %%eax\n");
         fprintf(RelayFile,"\tpushl  $%d\n",noparms);        
         fprintf(RelayFile,"\tcall   %s%s\n",Prefix,SpyFunction);
         fprintf(RelayFile,"\taddl   $12,%%esp\n");
      }   
      for(i=noparms-1;i>=0;i--)
         fprintf(RelayFile,"\tpushl  %d(%%ebp)\n",i*4+8);
            
      fprintf(RelayFile,"\tcall   %s%s\n", Prefix, inname);
      fprintf(RelayFile,"\taddl   $%d,%%esp\n",noparms*4);
      fprintf(RelayFile,"\tpopl   %%ebp\n");
      fprintf(RelayFile,"\tret    $%d\n\n\n",noparms*4);
  }   
#endif    
}


VOID StubCdeclUnimp(LPSTR exportname, LPSTR inname)
{
   if(!Masm)
   {
      fprintf(RelayFile,"%s%s:\n", 
                 Prefix, exportname);
      if(Debug||Spy)
      {
         fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
         fprintf(RelayFile,"\tleal   4(%%esp),%%eax\n");
         fprintf(RelayFile,"\tpushl  %%eax\n");
         fprintf(RelayFile,"\tcall   %s%s\n",Prefix,UnimpFunction);
         fprintf(RelayFile,"\taddl   $8,%%esp\n");
      }
      else
      {
         fprintf(RelayFile,"\tpushl  $120\n");
         fprintf(RelayFile,"\tcall   %sSetLastError\n",Prefix);
         fprintf(RelayFile,"\tpopl   %%eax\n");         
      }
      fprintf(RelayFile,"\tmovl   %s%s,%%eax\n",Prefix, inname);      
      fprintf(RelayFile,"\tret\n\n\n");
   }
}


VOID StubCdecl(LPSTR exportname, LPSTR inname)
{        
#ifdef NOTNOW
   if(!Masm)
   {
      fprintf(RelayFile,"\t.globl\t%s%s_%s\n\n", Prefix, Module, exportname);
      fprintf(RelayFile,"%s%s_%s:\n",  Prefix, Module, exportname);                   
      if(Spy)
      {
         fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
         fprintf(RelayFile,"\tleal   8(%%esp),%%eax\n");
         fprintf(RelayFile,"\tpushl  %%eax\n");         
         fprintf(RelayFile,"\tcall   %s%s\n",Prefix,SpyFunction);
         fprintf(RelayFile,"\taddl   $8,%%esp\n");
      }   
      fprintf(RelayFile,"\tjmp    %s%s\n\n\n", Prefix, inname);
   }
#endif         
}


VOID StubDirect(LPSTR exportname, LPSTR inname)
{        
   if(!Masm)
   {
      fprintf(RelayFile,"\t.globl\t%s%s\n\n", Prefix, exportname);
      fprintf(RelayFile,"%s%s:\n",  Prefix, exportname);                   
      if(Spy)
      {
         fprintf(RelayFile,"\tpushal \n");
         fprintf(RelayFile,"\tpushfl \n");
         fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
         fprintf(RelayFile,"\tleal   4(%%esp),%%eax\n");
         fprintf(RelayFile,"\tpushl  %%eax\n");
         fprintf(RelayFile,"\tcall   %s%s\n",Prefix,SpyFunction);
         fprintf(RelayFile,"\taddl   $8,%%esp\n");
         fprintf(RelayFile,"\tpopfl  \n");
         fprintf(RelayFile,"\tpopal  \n");
      }   
      fprintf(RelayFile,"\tjmp    %s%s\n\n\n", Prefix, inname);
  }      
}

VOID StubReturn(LPSTR exportname, LPSTR inname)
{        
   if(!Masm)
   {
      fprintf(RelayFile,"\t.globl\t%s%s_%s\n\n", Prefix, Module, exportname);
      fprintf(RelayFile,"%s%s:\n",  Prefix, exportname);                   
      if(Spy)
      {
         fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
         fprintf(RelayFile,"\tleal   4(%%esp),%%eax\n");
         fprintf(RelayFile,"\tpushl  %%eax\n");
         fprintf(RelayFile,"\tcall   %s%s\n",Prefix,SpyFunction);
         fprintf(RelayFile,"\taddl   $8,%%esp\n");
      } 
      fprintf(RelayFile,"\tmovl   %s%s,%%eax\n", Prefix, inname);
      fprintf(RelayFile,"\tret\n\n\n");
   }      
}

VOID StubUnknown(LPSTR exportname)
{     
   if(!Masm)
   {
      fprintf(RelayFile,"%s%s:\n",  Prefix, exportname);                   
      fprintf(RelayFile,"\tpushl  $_nameof_%s\n",exportname);
      fprintf(RelayFile,"\tleal   8(%%esp),%%eax\n");
      fprintf(RelayFile,"\tpushl  %%eax\n");
      fprintf(RelayFile,"\tcall   %s%s\n",Prefix,UnknownFunction);
      fprintf(RelayFile,"\taddl   $8,%%esp\n");
      fprintf(RelayFile,"\txorl   %%eax,%%eax\n");
      fprintf(RelayFile,"\tincl   %%eax\n");
      fprintf(RelayFile,"\tret\n\n\n");
   }      
}


VOID ExpRelay(LPSTR filename)
{
   InitExports(filename);
   ParseExports();
   MakeExportsTable();
   MakeExportsNames();
   MakeExportsStubs();
   DoneExports();
}


void usage(void)
{
   fprintf(stderr,"usage: exprelay [options] file.exp\n");
   fprintf(stderr,"options: -e   elf format relay\n"
                  "         -d   debug stubs\n"
                  "         -s   spy stubs\n"
                  "         -u   unix format relay\n");
   exit(1);
}

int main(int argc, char **argv)
{
    int   i,j;
    char *suffix;

    if (argc <= 1) usage();

    /* check options */
    for(i=1;i<argc;i++)
    {
       if(argv[i][0]=='-')
       {
          for(j=1;argv[i][j]!='\0';j++)
          {
             switch(argv[i][j])
             {
               case 'e':         /* no underscores */
                 Prefix="_";
                 break;
               case 'u':         /* underscores    */
                 Prefix="";
                 break;
               case 'm':         /* MASM mode (not yet implemented) */ 
                 Masm=TRUE;
                 break;
               case 'd':         /* debug stubs    */
                 Debug=TRUE;
                 break;
               case 's':         /* spy stubs      */
                 Spy=TRUE;
                 break;
               default:
                 usage();
             }   
          }          
       }   
    }
    
    /* check filenames */
    for(i=1;i<argc;i++)
    {
       if(argv[i][0]!='-')
       {
          /* check file name */
          if(strlen(argv[i])>MAX_PATH-4)
          {
                fprintf(stderr,"fatal error: filename too long\n");
                exit(1);
          }

          suffix=strrchr(argv[i],'.');
          if(!suffix)
             usage();
          if(strcmp(suffix,".exp"))
             usage();
       }
    }
    
    /* do our work */
    for(i=1;i<argc;i++)
    {
       if(argv[i][0]!='-')
       {
          ExpRelay(argv[i]);         
       }
    }  
    return 0;
}